package WO::Client;

use strict;
use warnings;
use FindBin qw( $Bin );

use base qw( WO::Class );

use Data::Dumper;

sub init {
  my $s = shift;
  my %a = ( dir => undef, manual => undef, @_ );

  # don't remove - integrate_client_debug needs this
  if (defined $ENV{WECK_PROCESS_LIST}) {
    $s->debug("FO MANUAL CHOICES: #".$ENV{WECK_PROCESS_LIST}."#");
  }

  $s->debug("init (architecture is: ".$^O.") (manual is: ".$a{manual}.")");

  my $net_info = &_network_decode();
  $s->debug("Network Information:");
  $s->debug($net_info);

  die "Missing 'dir' parameter..." unless defined $a{dir};
}

sub run {
  my $s = shift;

  $s->debug("run()");

  my $cfg_info = $s->config();
  my $txt = $s->config_txt();

  $cfg_info->{tray_application}->{fo_multiple_choices} ||= "";

  my $proc_list = $ENV{'WECK_PROCESS_LIST'};
  my %proc_list;

  if (defined $proc_list) {
    my @r = split /,/, $proc_list;
    map { $proc_list{$_} = 1; } @r;
  }

  $cfg_info->{frontoffice} ||= [];

  my @fos = @{$cfg_info->{frontoffices}->{frontoffice}};

  my $total_active = 0;

  for (my $i=0; $i < scalar(@fos); $i++) {
    my $fo_obj = $fos[$i];
    $total_active++ if $fo_obj->{active};
  }

  for (my $i=0; $i < scalar(@fos); $i++) {
    my $fo_obj = $fos[$i];

    if (! $fo_obj->{active}) {
      $s->debug("Inactive <frontoffice>, skipping...");
      next;
    }

    if ($cfg_info->{tray_application}->{fo_multiple_choices}) {
      if (! exists $proc_list{$i}) {
        $s->debug("<fontoffice> $i isn't selected, ignoring...");
        next;
      }
    }
    else {
      if (scalar %proc_list && ! exists $proc_list{$i}) {
        $s->debug("No order to execute FO $i, skipping... store: #".$fo_obj->{weckoffice}->{store}."#");
        next;
      }
    }

    $s->debug($fo_obj);

    $fo_obj->{active} ||= 0;
    $fo_obj->{type} ||= undef;

    if (! defined $fo_obj->{type}) {
      die "You've an active <frontoffice> without a type, please correct that";
    }

    # add lib folder to perl's include path
    if (exists $ENV{WECK_CLIENT_PATH} && defined $ENV{WECK_CLIENT_PATH}) {
      $s->debug("WECK_CLIENT_PATH: $ENV{WECK_CLIENT_PATH}");
      use lib "$ENV{WECK_CLIENT_PATH}/lib";
    }

    $s->debug("Will load a <frontoffice> of type ".$fo_obj->{type});

    eval "require ".$fo_obj->{type};
    die "Error loading ".$fo_obj->{type}.": $@" if $@;

    $s->debug("Will start ".$fo_obj->{type}." (".$fo_obj->{weckoffice}->{company}." / ".$fo_obj->{weckoffice}->{store}.")");

    my $v = exists $cfg_info->{tray_application}->{visible} ? $cfg_info->{tray_application}->{visible} : 1;

    my $fo = $fo_obj->{type}->new( dir => $s->{dir}, info => $fo_obj, manual => $s->{manual}, total_active => $total_active, visible => $v  );

    $s->debug("Done.");
  }
}

sub _network_decode {
  my %a = ( routes => undef, ifconfig => undef, @_ );

  my %net;

  if ($^O eq 'MSWin32') {
    my $network = `ipconfig /all`;
    $network =~ s/\r//g;
    my @lines = split /\n/, $network;

    my $c = 0;
    my $eths = 0;

    for (@lines) {
      next unless $_ ne '';

      if ($_ !~ /^\s+/) {
        $eths++;

        next if $eths == 1;
        $c = 0;
      }
      elsif ($eths > 1) {
        my( $k, $v ) = split /\s+:\s+/;
        $k =~ s/^\s+//;
        $k =~ s/\s+\.(.*)$//;

        if ($c == 5) {
          $v =~ s/^\s+//;
          $net{$eths}->{ip} = $v;
        } elsif ($c == 2) {
          $net{$eths}->{mac} = $v;
        } elsif ($c == 6) {
          $net{$eths}->{netmask} = $v;
        } elsif ($c == 7) {
          $net{$eths}->{gateway} = $v;
        } elsif ($c == 1) {
          $net{$eths}->{description} = $v;          
        }

        $c++;
      }
    }
    
  } else {
    my %routes;

    my $routes;

    if (defined $a{routes}) {
      $routes = $a{routes};
    } else {
      $routes = `/sbin/route -n | grep UG`;
    }

    my @lines = split /\n/, $routes;
    
    for (@lines) {
      my @f = split /\s+/;
      $routes{$f[7]} = $f[1] if defined $f[7] && defined $f[1];
    }

    my $interfaces;

    if (defined $a{ifconfig}) {
      $interfaces = $a{ifconfig};
    } else {
      if (-e "/sbin/ifconfig") {
        $interfaces = `/sbin/ifconfig`;
      } elsif (-e "/bin/ifconfig") {
        $interfaces = `/bin/ifconfig`;
      }
    }

    @lines = split /\n/, $interfaces;

    my $cur_name;
    my $c = 0;
    my $read_content = 0;
    my $read_header = 0;
    
    for (@lines) {
      next unless $_ ne '';

      if ($_ !~ /^\s+/) {
        $read_header = 1;
        $read_content = 0;
        $c++;
      } else {
        $read_content = 1;
        $read_header = 0;
      }

      if ($read_header) {
        my( $name, $m1, $m2, $m3, $m4, $m5, $m6 ) = $_ =~ /^(.*?)\s.*(..)[\:|\-](..)[\:|\-](..)[\:|\-](..)[\:|\-](..)[\:|\-](..)/;
        
        $cur_name = $name;
                
        if (defined $m1) {
          $net{$c}->{mac} = "$m1-$m2-$m3-$m4-$m5-$m6";
          $net{$c}->{description} = $name;
          $net{$c}->{gateway} = $routes{$cur_name} if exists $routes{$cur_name};          
        }
      }       

      if ($read_content && $cur_name) {
        my $l = $_;
        $l =~ s/^\s+//;          
        my @toks = split /\s\s/, $l;

        next if exists $net{$c}->{ip};
          
        if (exists $toks[0] && $toks[0] =~ /:/) {
          my @a = split /:/, $toks[0];
          $net{$c}->{ip} = $a[1];
        }

        if (exists $toks[2] && $toks[2] =~ /:/) {
          my @a = split /:/, $toks[2];
          $net{$c}->{netmask} = $a[1];
        }
      }
    }  
  }

  my %info;

  foreach my $k (keys %net) {
    next unless exists $net{$k}->{ip} && $net{$k}->{ip} ne '255.255.255.0';
    $info{$k} = $net{$k};
  }

  my $str = "";

  foreach my $k1 (keys %info) {
    my $h = $info{$k1};
    foreach my $k2 (keys %$h) {
      $str .= "$k2=".$h->{$k2}.",";
    }
    chop($str);
    $str .= "\n";
  }

  chop($str) if $str;
  
  return $str;
}

1;
