package WO::Client::Integrators::CSV;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime ceil );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use Text::CSV_XS;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use Data::Dumper;

sub sync_fo_information {
  my $s = shift;
}

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  $s->debug("Encoded FO type is: $fo_type");

  my $csv = Text::CSV_XS->new({ binary => 1 });

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  my $dir = $s->vfh($xml, 'dir');

  my $sync_articles = $s->vfh($xml, 'sync_articles');
  my $sync_documents = $s->vfh($xml, 'sync_documents');
  my $sync_static_tables = $s->vfh($xml, 'sync_static_tables');

  if (! defined $dir) {
    $s->abort("Missing the 'dir' parameter...");
  }

  my $str;
  my $VAR1;

  if (defined $sync_documents) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'UMAGetDocuments', } );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $first = 0;

      my $out_file = $dir.$dsep."documents.csv";

      open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

      if (scalar @$VAR1) {
        my $first_elem = $VAR1->[0];
        my @headers = sort keys %$first_elem;
        my $status = $csv->combine(@headers);
        print F $csv->string()."\n";

        foreach my $a (@$VAR1) {
          my $i = [];

          for (@headers) {
            push @$i, $a->{$_};
          }

          my $status = $csv->combine(@$i);
          print F $csv->string()."\n";
        }
      }

      close(F);
    }
  }

  if (defined $sync_articles) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetArticlesWithoutStoreCheck', } );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $first = 0;

      my $out_file = $dir.$dsep."products.csv";

      open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

      if (scalar @$VAR1) {
        my $first_elem = $VAR1->[0];
        my @headers = sort keys %$first_elem;
        my $status = $csv->combine(@headers);
        print F $csv->string()."\n";

        foreach my $a (@$VAR1) {
          my $i = [];

          for (@headers) {
            push @$i, $a->{$_};
          }

          my $status = $csv->combine(@$i);
          print F $csv->string()."\n";
        }
      }

      close(F);
    }
  }

  if (defined $sync_static_tables) {
    my @tables = split /,/, $sync_static_tables;

    foreach my $table_name (@tables) {
      eval {
        $str = $s->weckoffice_send_data( {
          action => 'BackofficeGetDirectTable',
          table => $table_name,
        } );
      };

      if ($@) {
        die("Error: $@");
      }

      eval $str;

      if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
        my $first = 0;

        my $out_file = $dir.$dsep.$table_name.".csv";

        open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

        if (scalar @$VAR1) {
          my $first_elem = $VAR1->[0];
          my @headers = sort keys %$first_elem;
          my $status = $csv->combine(@headers);
          print F $csv->string()."\n";

          foreach my $a (@$VAR1) {
            my $i = [];

            for (@headers) {
              push @$i, $a->{$_};
            }

            my $status = $csv->combine(@$i);
            print F $csv->string()."\n";
          }
        }

        close(F);
      }
    }
  }
}

1;


