package WO::Client::Integrators::Customers::MGE;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use WO::DB::ODBC;
use Text::CSV_XS;
use FindBin qw( $Bin );
use File::Slurp qw( read_file );
use File::Basename qw( fileparse fileparse_set_fstype );

&fileparse_set_fstype('MSWin32');

#
# FO to weckOFFICE
#




sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  my $_sqls_host = $xml->{sql_server_addr};
  my $_sqls_port = $xml->{sql_server_port};
  my $_sqls_login = $xml->{sql_server_login};
  my $_sqls_pass = $xml->{sql_server_pass};

  my $sql_server_sale_db = $xml->{sql_server_sale_db};
  my $sql_server_merc_db = $xml->{sql_server_merc_db};
  my $sql_server_merc_db_prefix = $xml->{sql_server_merc_db_prefix};

  my $days = $xml->{back_days} || 7;

  my $check_dates_fast = $xml->{check_dates_fast} || 0;

  my $file_sale_families = $tmp_dir.'/__sale_families__.txt';
  my $file_sale_subfamilies = $tmp_dir.'/__sale_subfamilies__.txt';
  my $file_sale_articles = $tmp_dir.'/__sale_articles__.txt';

  my $file_merc_families = $tmp_dir.'/__merc_families__.txt';
  my $file_merc_subfamilies = $tmp_dir.'/__merc_subfamilies__.txt';
  my $file_merc_articles = $tmp_dir.'/__merc_articles__.txt';
  my $file_merc_suppliers = $tmp_dir.'/__merc_suppliers__.txt';

  my $file_details = $tmp_dir.'/__details__.txt';
  my $file_footers = $tmp_dir.'/__footers__.txt';
  my $file_locals = $tmp_dir.'/__fo_locals__.txt';
  my $file_users = $tmp_dir.'/__fo_users__.txt';

  my $file_units = $tmp_dir.'/__units__.txt';

  my $csv = Text::CSV_XS->new({ binary => 1 });

  my @zip_files;

  if ($xml->{sync_sales}) {
    my $dsn = "DBI:ODBC:Driver={SQL Server};Server=$_sqls_host;Port=$_sqls_port;Database=$sql_server_sale_db;UID=$_sqls_login;PWD=$_sqls_pass";
    my $dbh = DBI->connect($dsn) or $s->abort("Can't connect to the server $dsn@\n");
    my $db = WO::DB::ODBC->new( dbh => $dbh );

    my $VAR1 = {};

    my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration' } );
    eval "$company_config";
    my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

    my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDates';
    my $existent_dates = $s->weckoffice_send_data( { action => $act } );

    eval "$existent_dates";
    $existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? $VAR1 : {};

    $s->debug("FETCHING SALES FOR THE LAST $days days");

    # GET FAMILIES

    my $sth = $db->dbh()->prepare(qq{
      SELECT DISTINCT f.code, f.name
      FROM dbo.t_hpos_items i
      LEFT JOIN dbo.t_hpos_groups f ON (i.group_id = f.id)
    });
    $sth->execute();

    open(F, "> $file_sale_families") or die "Can't write to $file_sale_families: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    # GET SUBFAMILIES

    $sth = $db->dbh()->prepare(qq{
      SELECT DISTINCT sf.code, sf.name, f.code as family_id
      FROM dbo.t_hpos_items i
      LEFT JOIN dbo.t_hpos_groups f ON (i.group_id = f.id)
      LEFT JOIN dbo.t_hpos_families sf ON (i.family_id = sf.id)
    });
    $sth->execute();

    open(F, "> $file_sale_subfamilies") or die "Can't write to $file_sale_subfamilies: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    # GET WHOUSES

    $sth = $db->dbh()->prepare(qq{
      SELECT DISTINCT code, name
      FROM dbo.t_hpos_sector 
    });
    $sth->execute();

    open(F, "> $file_locals") or die "Can't write to $file_locals: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    # GET ARTICLES

    $sth = $db->dbh()->prepare(qq{
      SELECT i.code, i.name, f.code as family_code, sf.code as subfamily_code, i.price1, i.price2, i.price3, i.price4, i.price5, t1.rate as iva1, t2.rate as iva2
      FROM dbo.t_hpos_items i
      LEFT JOIN dbo.t_hpos_groups f ON (i.group_id = f.id)
      LEFT JOIN dbo.t_hpos_families sf ON (i.family_id = sf.id)
      LEFT JOIN dbo.t_hpos_taxcode t1 ON (i.tax1_id = t1.id)
      LEFT JOIN dbo.t_hpos_taxcode t2 ON (i.tax2_id = t2.id)
    });
    $sth->execute();

    open(F, "> $file_sale_articles") or die "Can't write to $file_sale_articles: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    #GET SALES

    #DETAILS
    $sth = $db->dbh()->prepare(qq{
      SELECT md.document_id, md.document_number, s.code as sector_id, i.code, md.user_id, md.time_b, md.time_e, md.table_id, 
      md.quantity, md.vat_tax, md.line_type, md.original_value, md.discount, md.offer, 0 as terminal_id, i.menu, md.n_clients,
      md.table_id, dt.code as document_type, md.officercheck, md.company_id
      FROM dbo.v_hpos_sales md 
      LEFT JOIN dbo.t_hpos_items i ON (md.item_id = i.id)
      LEFT JOIN dbo.t_hpos_sector s ON (md.sector_id = s.id)
      LEFT JOIN dbo.t_hpos_documenttype dt ON (md.document_id = dt.id)
      WHERE md.time_b >= CONVERT(VARCHAR(10),GETDATE()-$days,111)
      ORDER BY md.time_b
    });
    $sth->execute();

    #CHANGED

    open(F, "> $file_details") or die "Can't write to $file_details: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    #FOOTERS
    $sth = $db->dbh()->prepare(qq{
      SELECT md.document_id, md.document_number, md.payment_id, md.total, s.code, md.end_time, pt.code as payment_type
      FROM dbo.v_hpos_sales_payments md
      LEFT JOIN dbo.t_hpos_sector s ON (md.sector_id = s.id)
      LEFT JOIN dbo.t_hpos_paymenttypes pt ON (md.payment_id = pt.id)
      WHERE end_time >= CONVERT(VARCHAR(10),GETDATE()-$days,111)
      ORDER BY end_time
    });
    $sth->execute();

    open(F, "> $file_footers") or die "Can't write to $file_footers: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    # GET  USERS

    $sth = $db->dbh()->prepare(qq{
      SELECT DISTINCT [ID],[Company_ID],[Nr],[Code],[PassWord],[FirstName],[LastName] 
      FROM dbo.T_USER
    });
    $sth->execute();

    open(F, "> $file_users") or die "Can't write to $file_users: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    $dbh->disconnect;
  }

  if ($xml->{sync_mercs}) {
    my $dsn = "DBI:ODBC:Driver={SQL Server};Server=$_sqls_host;Port=$_sqls_port;Database=$sql_server_merc_db;UID=$_sqls_login;PWD=$_sqls_pass";
    my $dbh = DBI->connect($dsn) or $s->abort("Can't connect to the server $dsn@\n");
    my $db = WO::DB::ODBC->new( dbh => $dbh );
    my $sth = undef;

    #UNITS
    $sth = $db->dbh()->prepare("
      SELECT [Code],[Description]
      FROM [dbo].[".$sql_server_merc_db_prefix."Unit of Measure]
    ");
    $sth->execute();

    open(F, "> $file_units") or die "Can't write to $file_footers: $@";

#    print STDERR "$_sqls_host $sql_server_merc_db\n";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    #FAMILIES
    $sth = $db->dbh()->prepare(qq"
      SELECT DISTINCT i.[Inventory Posting Group] as Code, i.[Inventory Posting Group] as Description
      FROM [dbo].[".$sql_server_merc_db_prefix."Item] i
    ");
    $sth->execute();

    open(F, "> $file_merc_families") or die "Can't write to $file_merc_families: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }


    close(F);

    #SUBFAMILIAS
    $sth = $db->dbh()->prepare("
      SELECT [Code],[Description],[Def_ Inventory Posting Group]
      FROM [dbo].[".$sql_server_merc_db_prefix."Item Category]
    ");
    $sth->execute();

    open(F, "> $file_merc_subfamilies") or die "Can't write to $file_merc_subfamilies: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    #ARTICLES
    $sth = $db->dbh()->prepare(qq"
      SELECT i.[No_],i.[Description],i.[Base Unit of Measure],i.[Inventory Posting Group],c.Code
      FROM [dbo].[".$sql_server_merc_db_prefix."Item] i
      LEFT JOIN [dbo].[".$sql_server_merc_db_prefix."Item Category] c ON (i.[Item Category Code] = c.Code)
    ");
    $sth->execute();

    open(F, "> $file_merc_articles") or die "Can't write to $file_merc_articles: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    #FORNECEDORES
    $sth = $db->dbh()->prepare(qq"
      SELECT [No_],[Name],[Address],[Address 2],[City],[Contact],[VAT Registration No_],[Phone No_],[Post Code],[E-Mail] 
      FROM [dbo].[".$sql_server_merc_db_prefix."Vendor]
    ");
    $sth->execute();

    open(F, "> $file_merc_suppliers") or die "Can't write to $file_merc_suppliers: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    $dbh->disconnect();
  }

  if ($xml->{sync_sales}) {
    push @zip_files, $file_footers if -e $file_footers;
    push @zip_files, $file_details if -e $file_details;
    push @zip_files, $file_locals if -e $file_locals;
    push @zip_files, $file_sale_families if -e $file_sale_families;
    push @zip_files, $file_sale_subfamilies if -e $file_sale_subfamilies;
    push @zip_files, $file_sale_articles if -e $file_sale_articles;
    push @zip_files, $file_users if -e $file_users;
  }

  if ($xml->{sync_mercs}) {
    push @zip_files, $file_units if -e $file_units;
    push @zip_files, $file_merc_families if -e $file_merc_families;
    push @zip_files, $file_merc_subfamilies if -e $file_merc_subfamilies;
    push @zip_files, $file_merc_articles if -e $file_merc_articles;
    push @zip_files, $file_merc_suppliers if -e $file_merc_suppliers;
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $s->tmp_dir().$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData', file_upload => $zipFile, });
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      for (@zip_files) {
        $s->debug("Will delete file '$_'");
        $s->own_remove($_);
      }
    }
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }

  if ($xml->{get_inventories}) {
    $s->get_inventories();
  }

  if ($xml->{get_receptions}) {
    $s->get_receptions();
  }
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  $s->get_inventories();
}

sub _check_ask_date_script {
  my $s = shift;

  $s->debug("_check_ask_date_script()");

  my $ds = $s->directory_separator();

  my $f = $Bin.$ds."ask_dates.vbs";

  my $date_from = undef;
  my $date_to = undef;

  if (-e $Bin.$ds."date_from.txt") {
    $date_from = read_file($Bin.$ds."date_from.txt");
    chop($date_from);
  }

  if (-e $Bin.$ds."date_to.txt") {
    $date_to = read_file($Bin.$ds."date_to.txt");
    chop($date_to);
  }

  my $dateFromStr = defined $date_from ? '"'.$date_from.'"' : "dateStr";
  my $dateToStr = defined $date_to ? '"'.$date_to.'"' : "dateStr";

  if (! -e $f) {
    my $str =<<END;
Function RegExpValidator(patrn, strng)
    Dim regEx, Match, Matches, RetStr
    Set regEx = new RegExp ' Create a regular expression.
    regEx.Pattern = patrn ' Set pattern.
    regEx.IgnoreCase = True' Set case insensitivity.
    regEx.Global = false' Set global applicability.
    Set Matches = regEx.Execute(strng)' Execute search.
    if Matches.count > 0 then
        RetStr = true
    else
        RetStr = false
    end if
    set Matches = nothing
    set regEx = nothing
    RegExpValidator = RetStr
End Function

function ValidDate(dtDate, sErrorText)
    dim sPatrn, sMonth, sDay, sYear, aDate, bValid, bIsLeap, bPatrnMatch
    dtDate = trim(dtDate)
    sPatrn = "(\\d{4})(\\-)(\\d{2})(\\-)(\\d{2})"
    bPatrnMatch = RegExpValidator(sPatrn,dtDate) '// is the format ok?
    bValid = true
    if bPatrnMatch = true then
        aDate = split(dtDate,"-")
        if ubound(aDate) = 2 then
            sYear = aDate(0)
            sMonth = aDate(1)
            sDay = aDate(2)
            
            if len(sYear) = 4 then
                if sYear < 1753 then ' //dates less than January 1 1753 cause an exception in sql server
                    bValid = false
                    sErrorText = "Year must be greater than 1752"
                else
                    if sMonth < 1 or sMonth > 12 then  '// check month range
                        bValid = false
                        sErrorText = "Month must be between 1 and 12."
                    else
                        if sDay < 1 or sDay > 31 then
                            bValid = false  
                            sErrorText = "Day must be between 1 and 31."
                        else
                            if ((sMonth=4 or sMonth=6 or sMonth=9 or sMonth=11) and sDay=31) then
                                bValid = false
                                sErrorText ="Month " & sMonth & " does not have 31 days!"
                            else
                                if sMonth = 2 then '// check for february 29th
                                    If (sYear Mod 4 = 0 And sYear Mod 100 <> 0) Or sYear Mod 400 = 0 Then
                                        bIsLeap = True
                                    Else
                                        bIsLeap = False
                                    End If
                                    'Response.Write bIsLeap
                                    'Response.Write sDay
                                    
                                    if sDay > 29 or (sDay = 29 and not bIsLeap) then
                                        bValid = false
                                        sErrorText = "February " & sYear & " does not have " & sDay & " days!"
                                    end if
                                end if
                            end if
                        end if
                    end if
                end if
            else
                bValid = false
                sErrorText = "Ano tem que ter 4 digitos"
            end if
        else
            bValid = false
            sErrorText = "Formato para datas deve ser yyyy-mm-dd"
        end if        
    else
        bValid = false
        sErrorText = "Formato para datas deve ser yyyy-mm-dd"  
    end if
    
    ValidDate = bValid
end function

Dim Stuff, myFSO, WriteStuff, dateStamp, dateStr, dateOk, error

dateOk = True

dateStamp = Date()
dateStr = DatePart("yyyy", dateStamp) & "-" & DatePart("m", dateStamp) & "-" & DatePart("d", dateStamp)

Do
  dateFrom = InputBox("De:", "Data - De", $dateFromStr)
  dateOK = ValidDate(dateFrom,erro)
  if (dateFrom = "") Then WScript.Quit
  if (dateOk = False) Then MsgBox("Data Invalida " & erro)
Loop While (dateOK = False)

dateOk = True

Do
  dateTo = InputBox("Ate:", "Data - Ate", $dateToStr)
  dateOK = ValidDate(dateTo,erro) 
  if (dateTo = "") Then WScript.Quit
  if (dateOk = False) Then MsgBox("Data Invalida " & erro)

  if (dateTo < dateFrom) Then dateOK = False
  if (dateTo < dateFrom) Then MsgBox("Data - Ate maior que Data - De")

Loop While (dateOK = False)

Set myFSO = CreateObject("Scripting.FileSystemObject")
Set WriteStuff = myFSO.OpenTextFile("$Bin/date_from.txt", 2, True)
WriteStuff.WriteLine(dateFrom)
WriteStuff.Close

Set WriteStuff = myFSO.OpenTextFile("$Bin/date_to.txt", 2, True)
WriteStuff.WriteLine(dateTo)
WriteStuff.Close

SET WriteStuff = Nothing
SET myFSO = Nothing
END

    open(F, "> $f");
    print F $str;
    close(F);
  }
}

sub write_to_file {
  my( $fname, $cnt ) = @_;

  open(N, ">$fname") or die $!;
  print N $cnt."\r\n";
  close(N);
}

sub get_receptions {
  my $s = shift;

  $s->debug("get_receptions()");

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $xml = $s->info();

  my $dsep = $s->directory_separator();
  my $tmp_dir = $s->tmp_dir();
  my $dir = $s->vfh($xml, 'dir');

  $s->abort("Missing 'dir' parameter...") unless defined $dir;

  my $str;
  my $VAR1;

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_MGE_GetReceptions' } );
  };

  $s->abort($@) if $@;

  if ($str) {
    $s->debug("NEW DATA WRITING");

    my $filename = $dir.$dsep."compras.csv";

    $s->debug("## $filename");
    open(FH,">$filename");
    print FH $str;
    close FH;
  }
}

sub get_inventories {
  my $s = shift;

  $s->debug("get_inventories()");

  $s->_check_ask_date_script();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $xml = $s->info();

  # directory separator
  my $dsep = $s->directory_separator();

  # temporary folder
  my $tmp_dir = $s->tmp_dir();

  # folder where the data will be stored to
  my $dir = $s->vfh($xml, 'dir');

  $s->abort("Missing 'dir' parameter...") unless defined $dir;

  #my $file_sale_headers = $dir.$dsep."sale_headers.csv";
  #my $file_sale_details = $dir.$dsep."sale_details.csv";

  my $str;
  my $VAR1;

  my $script = $Bin.$dsep."ask_dates.vbs";

  system($script) if -e $script;

  my $date_from = undef;
  my $date_to = undef;

  if (-e $Bin.$dsep."date_from.txt") {
    $date_from = read_file($Bin.$dsep."date_from.txt");
    chop($date_from);
    $date_from = undef unless $date_from =~ /^\d\d\d\d\-\d\d\-\d\d$/;
  }

  if (-e $Bin.$dsep."date_to.txt") {
    $date_to = read_file($Bin.$dsep."date_to.txt");
    chop($date_to);
    $date_to = undef unless $date_to =~ /^\d\d\d\d\-\d\d\-\d\d$/;
  }

  $s->own_remove($script);

  my %done_sales = ();

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_MGE_GetInventories', date_from => $date_from, date_to => $date_to } );
  };

  $s->abort($@) if $@;

  eval $str;

  my $csv = Text::CSV_XS->new({ binary => 1, sep_char => ';' });

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $inventories = $VAR1;

    foreach my $inventory (@{ $inventories || [] }) {
      my $filename = $dir.$dsep."INV_".$inventory->{store_name}."_".$inventory->{date}.".txt";
      print STDERR "### $filename\n";
      open(FH, ">$filename");

      $csv->combine('Cod Artigo', 'Conta Navision', 'Nome Artigo', 'Quantidade', 'Unidade Defeito', 'PCU', 'PCM');
      print FH $csv->string();
      print FH "\n";

      foreach my $article ( @{$inventory->{_vn_articles} || []} ) {
        $article->{qtty} =~ s/\./\,/g;
        $article->{pcu} =~ s/\./\,/g;
        $article->{pcm} =~ s/\./\,/g;

        $csv->combine( 
          $article->{article_fo_id}, 
          $article->{barcode2},
          $article->{article_name},
          $article->{qtty},
          $article->{unidade_default_name} || 'UNIDADE',
          $article->{pcu},
          $article->{pcm}
        );
        print FH $csv->string();
        print FH "\n";
      }
      close FH;
    }
  }

}

1;



