package WO::Client::Integrators::Customers::PD;
# Comograma

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime ceil );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use Text::CSV_XS;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use File::Path qw( mkpath );
use Data::Dumper;

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();
  
  my $dir = $s->vfh($xml, 'dir');
  my $dir_products = $s->vfh($xml, 'dir_products');
  
  my $VAR1;
  my $str;

  if (! defined $dir) {
    $s->abort("Missing the 'dir' parameter...");
  }

  my $cache_dir = $dir.$dsep.'dias_ja_integrados';
  mkpath($cache_dir) unless -d $cache_dir;

  my $sync_sales = $s->vfh($xml, 'sync_sales');
  my $sync_articles = $s->vfh($xml, 'sync_articles');
  
  #$s->manual();
  
  if (defined $sync_sales && $sync_sales) {
    $s->debug("sync_sales()");

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetPaoDouro_SalesNew',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'HASH') {
      my $sum = $VAR1;

      my %control = ();

      foreach my $date (keys %$sum) {
        foreach my $store (keys %{ $sum->{$date} }) {
          foreach my $mov_type (keys %{ $sum->{$date}->{$store} }) {          
			foreach my $article (keys %{ $sum->{$date}->{$store}->{$mov_type} }) {
			  foreach my $vat (keys %{ $sum->{$date}->{$store}->{$mov_type}->{$article} }) {
                my $h = $sum->{$date}->{$store}->{$mov_type}->{$article}->{$vat};
                my $file = $dir.$dsep.'sales_'.strftime("%Y%m%d", localtime).".csv";
                my $docs_file = $dir.$dsep.'docs_'.strftime("%Y%m%d", localtime).".csv";

                next if -e $cache_dir.$dsep.$date."#".$store;
                $h->{customer} ||= "";

                open(F, ">> $file") or die "Can't write to $file: $!";
                print F $date."#".$store."#".$article."#".$vat."#".$mov_type."#".sprintf("%.3f", $h->{qttys})."#".sprintf("%.6f", $h->{sales})."#".sprintf("%.3f", $h->{qttys_no_sales})."#".$h->{customer}."\n";
                close(F);
              }
            }
		  }

          open(F, "> ".$cache_dir.$dsep.$date."#".$store);
        }
        
        close(F);
      }
    }
  }
  
  if (defined $sync_articles && $sync_articles) {
    $s->debug("sync_articles()");
     
    my $file = $dir_products.$dsep."pdouro_artigos.csv";
    
    if (! -e $file) {
      $s->debug("Won't send articles... can't find $file");
    }
    else {
      my $zip_fname = $dir.$dsep.$fo_type.'_FO_DATA.zip';
      my $zip = Archive::Zip->new();
      $zip->addFile( $file );
      die "write error $zip_fname" unless $zip->writeToFileNamed( $zip_fname ) == AZ_OK;
        
      eval {
        $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                    file_upload => $zip_fname,
                                  }
                                );
      };

      if (! $@) {
        $s->debug("$file sent... will remove it...");
        unlink $file;
      }
    }
  }
  
  return 1;
}

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");
}

1;

