package WO::Client::Integrators::Customers::PL_PDA;
# PDA PL

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime ceil );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use Text::CSV_XS;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use Data::Dumper;

sub sync_fo_information {
  my $s = shift;
  
  $s->debug("sync_fo_information");
}

sub sync_bk_information {
  my $s = shift;
  
  $s->debug("sync_bk_information");
  
  my $str = "";
  my $VAR1 = undef;
  
  my $xml = $s->info();
  my $dsep = $s->directory_separator();
  
  my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));
  
  my $prefix          = $s->vfh($xml, 'prefix');
  my $export_to_pda   = $s->vfh($xml, 'export_to_pda');
  my $import_from_pda = $s->vfh($xml, 'import_from_pda');
  my $email           = $s->vfh($xml, 'email');
  $email ||= "";
  
  my %warehouses_mapping = ();
  
  if (exists $xml->{warehouses_mapping}) {
    my $ref = exists $xml->{warehouses_mapping}->{when} ? $xml->{warehouses_mapping}->{when} : undef;
    
    if (defined $ref) {
      my @mappings = ref($ref) eq 'HASH' ? ( $ref ) : @$ref;
      
      foreach my $map (@mappings) {
        next unless exists $map->{store} && exists $map->{then};
        $warehouses_mapping{ $map->{store} } = $map->{then};
      }
    }
  }
  
  $prefix = 'SERNET' unless defined $prefix;
  
  my $csv = Text::CSV_XS->new({ binary => 1, sep_char => "|", quote_char => '' });
  
  #
  # From NetBO to PDA
  #
  
  if (defined $export_to_pda && $export_to_pda) {
    $s->debug("export_to_pda");
    
    my $file_to_pda = $s->vfh($xml, 'file_to_pda');
    my $file_to_pda_append = $s->vfh($xml, 'file_to_pda_append');
    $s->abort("Missing 'file_to_pda' parameter...") unless defined $file_to_pda; 
    
    $file_to_pda_append = 0 unless defined $file_to_pda_append;
      
    my @fields = ( 'A', $prefix, 'TBL_Importacao' );

    eval {
      $str = $s->weckoffice_send_data( { action => 'NetBO_PL_PDA_GetArticles',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $sig = $file_to_pda_append ? '>>' : '>';
      open(F, $sig." $file_to_pda") or die "Can't write to $file_to_pda: $!";
      binmode(F, ":utf8");

      for (@$VAR1) {
        $_->{stock} ||= 0;
        $_->{unit} ||= "";
        $_->{barcode} ||= "";
        $_->{barcode2} ||= "";
        
        #next unless $_->{stock} > 0;
        
        my $store_fo_id = $_->{store_fo_id};
        #$store_fo_id = $warehouses_mapping{ $store_fo_id } if exists $warehouses_mapping{ $store_fo_id };   
        
        if ($_->{barcode}) {
          my @row = @fields;
          push @row, ( $_->{fo_id}, $_->{name}, $_->{barcode}, $store_fo_id, $_->{store_name}, $_->{unit}, sprintf("%.3f", $_->{stock}), "" );
          my $status = $csv->combine(@row);
          print F chr(hex("0x02")).$csv->string().chr(hex("0x03")).chr(hex("0x0a"));
        }

=pod
        if ($_->{barcode2}) {
          my @row = @fields;
          push @row, ( $_->{fo_id}, $_->{name}, "", $store_fo_id, $_->{store_name}, $_->{barcode2}, $_->{unit}, sprintf("%.3f", $_->{stock}) );
          my $status = $csv->combine(@row);
          print F chr(hex("0x02")).$csv->string().chr(hex("0x03")).chr(hex("0x0a"));
        }
        
        if (! $_->{barcode} && ! $_->{barcode2}) {
          my @row = @fields;
          push @row, ( $_->{fo_id}, $_->{name}, "", $store_fo_id, $_->{store_name}, "", $_->{unit}, sprintf("%.3f", $_->{stock}) );
          my $status = $csv->combine(@row);
          print F chr(hex("0x02")).$csv->string().chr(hex("0x03")).chr(hex("0x0a"));
        }
=cut
      }
      
      close(F);
    }
  }

  #
  # From PDA to NetBO
  #

  if (defined $import_from_pda && $import_from_pda) {
    $s->debug("export_from_pda");
  
    my $file_from_pda = $s->vfh($xml, 'file_from_pda');
    $s->abort("Missing 'file_from_pda' parameter...") unless defined $file_from_pda;
    
    my @fields = ( 'A', $prefix, 'TBL_Exportacao' );

    if (open(F, "< $file_from_pda")) {
      binmode(F);

      my $buffer = "";
      my $content = "";
      
      while (my $rv = read(F, $buffer, 4096)) {
        $buffer =~ s/\xFF//g;
        $buffer =~ s/\x00//g;
        $buffer =~ s/\xFE//g;
        $buffer =~ s/\x02//g;
        $buffer =~ s/\x03//g;      
        $content .= $buffer;
      }
      
      close(F);
      
      $s->debug("$tmp_file");
      
      open(N, "> $tmp_file");
      print N $content;
      close(N);
      
      my $map_str = "";
      
      foreach my $k (keys %warehouses_mapping) {
        $map_str .= "$k=".$warehouses_mapping{$k}."#";
      }
      
      $map_str =~ s/#$//;

      eval {
        $s->weckoffice_send_data( { action => 'NetBO_PL_PDA_ReceiveStocktake',
                                    email => $email,
                                    map_str => $map_str,
                                    file_upload => $tmp_file,
                                  }
                                );
      };
      
      if ($@) {
        die("Error: $@");
      }
    }
  }

  if (-e $tmp_file) {
    unlink $tmp_file;
  }
}

1;

