package WO::Client::Integrators::FS3700;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use File::Remove qw( remove );
use File::Copy qw( move copy );
use File::Spec;
use File::Slurp qw( read_file read_dir write_file );
use File::Path qw( rmtree mkpath );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Basename;
use File::Find qw( find );
use Data::Dumper;
use DBI qw(:sql_types);

use WO::DB;
use WO::DB::SQLite;

&File::Basename::fileparse_set_fstype('MSWin32');

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  # MDB file
  my $file = $xml->{file} || "";

  my $VAR1;
  my $str;

  # RSYNC sync of images
  my ($file_name,$file_path,$file_suffix) = fileparse($file,());

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepScreenMiniData', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  my $company_id = $VAR1->{company_id};
  my $keyboard_id = $VAR1->{keyboard_id};

  if (! -e $file) {
    die "MDB file '$file' don't exists...";
  }

  my $dsn = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$file;
  my $dbh = DBI->connect($dsn, { LongTruncOk => 1, LongReadLen => 65536, odbc_async_exec => 0, PrintError => 1, RaiseError => 1 })
    or die "Can't connect to database: $@";
  my $db = WO::DB->new( dbh => $dbh );

  my @tables_tmp = $dbh->tables();
  my @tables = ();

  foreach my $table (@tables_tmp) {
    my( $name ) = $table =~ /`.`(.*?)`$/;
    push @tables, $name;
  }

  my %to_ignore = (
    'MSysAccessObjects' => 1,
    'MSysAccessXML' => 1,
    'MSysACEs' => 1,
    'MSysObjects' => 1,
    'MSysQueries' => 1,
    'MSysRelationships' => 1,
  );

  my @to_delete = ();

  foreach my $table (@tables) {
    next if exists $to_ignore{ $table };

#    next unless $table eq 'Buttons';

    $s->debug("Will get all from '$table'...");

    my $idx = 0;

    my $sth = $dbh->prepare("SELECT * FROM $table");
    $sth->execute();

    my @fields = ();
    my $fields_num = $sth->{NUM_OF_FIELDS};

    my $create_str = "DROP TABLE IF EXISTS bleep_$table;\nCREATE TABLE bleep_$table (\n";

#    $s->debug("####> $fields_num");

    for ( my $i = 0 ; $i < $fields_num; $i++ ) {
      my $name = $sth->{NAME}->[$i];
      push @fields, $name;

      $create_str .= "\t$name VARCHAR(70),\n";
    }

    $create_str = substr($create_str, 0, length($create_str)-2);
    $create_str .= "\n);\n";

    #eval { $sql_lite->do($create_str); };

    if ($@) {
      $s->debug("ERROR: $create_str");
    }

    my $tbl = $s->tmp_dir().$dsep."$table.data";
    my $schema = $s->tmp_dir().$dsep."$table.schema";
    push @to_delete, $tbl;
    push @to_delete, $schema;

    &write_file( $schema, $create_str );

    open(N, "> $tbl") or die "Can't write to $tbl: $!";

    while (my $row = $sth->fetchrow_arrayref) {
      my @values = ();

      for( my $i=0; $i < scalar @$row; $i++) {
        my $v = ! defined $row->[$i] ? "" : $row->[$i];

        $v =~ s/\r//g;
        $v =~ s/\n//g;

        push @values, $v;
      }

#      $s->debug("######### ".scalar(@values));

      print N join("\t", @values) . "\n";
    }

    close(N);
  }

#  $s->abort("aqui");

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $s->tmp_dir().$dsep.$fo_type.'_FO_DATA.zip';
  &remove( $zipFile ) if -e $zipFile;

  for (@to_delete) {
    my $member = $zip->addFile( $_ );
  }

  die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

  #
  # Send ZIP file to the server
  #

  eval {
    $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                file_upload => $zipFile,
                              }
                            );
  };

  &remove($zipFile) if -e $zipFile;

  if ($@) {
    die "Error sending file: $@";
  }

  for (@to_delete) {
    &remove($_);
  }

  $s->debug("Done.");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  $s->debug("Done.");
}

1;

