package WO::Client::Integrators::FirstPOS;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use File::Copy qw( move copy );
use File::Spec;
use File::Slurp qw( read_file read_dir );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Basename;
use File::Find qw( find );
use Data::Dumper;
use POSIX qw( strftime );

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  if (! -d $dir) {
    die "Frontoffice dir '$dir' don't exists, where is the frontoffice?";
  }

  # directory separator
  my $dsep = $s->directory_separator();

  my $file_articles = $dir.$dsep.'produtos.csv';
  my $file_clients = $dir.$dsep.'clientes.csv';
  my $file_users = $dir.$dsep.'workers.csv';
  my $file_pages = $dir.$dsep.'config'.$dsep.'pages.csv';
  my $file_geral = $dir.$dsep.'config'.$dsep.'geral.csv';

  # date_from filtering
  my $date_from = $xml->{date_from} || undef;
  $date_from =~ s/\///g if defined $date_from;

  my @possible_remove = ();

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $xml->{dir}.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  $s->common_sync_fo_and_sync_bk( $xml, $zip, \@possible_remove );

  # articles
  if ($xml->{sync_articles} && -e $file_articles) {
    $zip->addFile( $file_articles );
  }

  # clients
  if ($xml->{sync_clients} && -e $file_clients) {
    $zip->addFile( $file_clients );
  }

  # users
  if ($xml->{sync_users} && -e $file_users) {
    $zip->addFile( $file_users );
  }

  # users
  if ($xml->{sync_families} && -e $file_pages && -e $file_geral) {
    $zip->addFile( $file_pages );
    $zip->addFile( $file_geral );
  }

  if ($xml->{sync_sells}) {
    my $found_something = $s->sync_movements($zip, \@possible_remove, $date_from);
  }

  die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

  #
  # Send ZIP file to the server
  #

  $s->dump(\@possible_remove);

  eval {
    $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                file_upload => $zipFile,
                              }
                            );
  };

  $s->own_remove($zipFile) if -e $zipFile;

  if ($@) {
    die "Error sending file: $@";
  }

  if (scalar @possible_remove) {
    for (@possible_remove) {
      $s->debug("Will remove file '$_'");
      $s->own_remove( $_ );
    }
  }

  $s->debug("Done.");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  if (! -d $dir) {
    die "Frontoffice dir '$dir' don't exists, where is the frontoffice?";
  }

  if (! -d $dir.$dsep.'import') {
    die "Import dir ".$dir.$dsep."import  don't exists...";
  }

  $s->debug("manual: #".$s->manual()."#");

  my @color_base = ( '00', '33','66', '99', 'CC', 'FF' );
  my @red = @color_base;
  my @blue = @color_base;
  my @green = @color_base;
  my $count = 256;
  my %colors = ();

  foreach my $red (@red) {
    foreach my $green (@green) {
      foreach my $blue (@blue) {
        $colors{ $count++ } = $red.$green.$blue;
      }
    }
  }

  my $parameters_file   = $dir.$dsep."config".$dsep."geral.csv";
  my $articles_file     = $dir.$dsep."import".$dsep."produtos.csv";
  my $families_file     = $dir.$dsep."import".$dsep."fam.csv";
  my $sub_families_file = $dir.$dsep."import".$dsep."subfam.csv";
  my $waitresses_file   = $dir.$dsep."import".$dsep."workers.csv";
  my $clients_file      = $dir.$dsep."import".$dsep."clientes.csv";
  my $menus_file         = $dir.$dsep."import".$dsep."prom.csv";

  # retrieve files associated with a tree
  $s->receive_backoffice_tree_files();

  # default values are 1
  $xml->{sync_users} = exists $xml->{sync_users} ? $xml->{sync_users} : 1;
  $xml->{sync_articles} = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_families} = exists $xml->{sync_families} ? $xml->{sync_families} : 1;
  $xml->{sync_clients} = exists $xml->{sync_clients} ? $xml->{sync_clients} : 1;
  $xml->{sync_sells} = exists $xml->{sync_sells} ? $xml->{sync_sells} : 1;
  $xml->{sync_menus} = exists $xml->{sync_menus} ? $xml->{sync_menus} : 1;

  my $str;
  my $VAR1 = undef;

  #
  # Integration control
  #

  my %control_integrations = ();

  eval {
    $str = $s->weckoffice_send_data( { action => 'BackofficeGetControlIntegrations',
                                     }
                                   );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    for (@$VAR1) {
      $control_integrations{ $_->{content_type} } = $_;
    }
  }

  #
  # Sync articles
  #

  if ($s->manual() && $xml->{sync_articles}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetFirstPOSArticles',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      $s->debug("Will process ".scalar(@$VAR1)." articles...");
      $s->debug("THIS IS PROBABLY A DEMO COMPANY!!!!") if scalar(@$VAR1) == 30;

      my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));

      foreach my $a (sort { $a->{fo_seetings}->{_vord} <=> $a->{fo_seetings}->{_vord} } @$VAR1) {
        my %settings;
        map { my $k=$_; $_ =~ s/^mart_//; $settings{$_} = $a->{fo_seetings}->{$k} } keys %{$a->{fo_seetings}};
        $a->{fo_seetings} = \%settings;

        my $price1 = defined $a->{article_price1} ? $a->{article_price1}*100 : 0;
        my $price2 = defined $a->{article_price2} ? $a->{article_price2}*100 : 0;
        my $price3 = defined $a->{article_price3} ? $a->{article_price3}*100 : 0;
        my $price4 = defined $a->{article_price4} ? $a->{article_price4}*100 : 0;
        my $price5 = defined $a->{article_price5} ? $a->{article_price5}*100 : 0;

        my $tax1 = $a->{article_tax0} || 0;
        my $tax2 = $a->{article_tax1} || 0;

        my $family = $a->{family_fo_id} || 0;
        my $code = $a->{article_fo_id} || "";
        my $barcode = $a->{barcode} || 0;
        my $name = $a->{article_name} || "";
        $name =~ s/,/./g;

        my $cbot_tmp = $a->{fo_seetings}->{_cbot} || "";
        my $cbot = $cbot_tmp;

        my $icon = $a->{fo_seetings}->{icon} || "NULL";
        #my $printer = $a->{fo_seetings}->{printer} || "0";
        my $printer = defined $a->{fo_seetings}->{printer} ? $a->{fo_seetings}->{printer} : $a->{fo_seetings}->{zimp} || "0";
        my $type = $a->{fo_seetings}->{type} || "0";
        my $open_price = $a->{fo_seetings}->{open_price} || "0";
        my $linked_page = $a->{fo_seetings}->{linked_page} || "-1";

        if ($linked_page ne '-1' && $linked_page =~ /_/) {
          my( $top, $sub ) = $linked_page =~ /^(\d+)_(\d+)$/;
          $linked_page = $top*100 + $sub;
        }

        $cbot = "" unless defined $cbot;
        $cbot = exists $colors{$cbot_tmp} ? $colors{$cbot_tmp} : 'FFFFFF';

        my $ncur = $a->{fo_seetings}->{_ncur} || "";
        my $nbot = $a->{fo_seetings}->{_nbot} || "";
        my $medcook = $a->{fo_seetings}->{medcook} || "0";
        my $maxcook = $a->{fo_seetings}->{maxcook} || "0";

        my $use_weight = $a->{fo_seetings}->{use_weight} || 0;
        my $is_daily = $a->{fo_seetings}->{is_daily} || 0;
        my $extras = $a->{fo_seetings}->{extras} || 0;

        $code =~ s/,/./g;
        $name =~ s/,/./g;
        $ncur =~ s/,/./g;
        $nbot =~ s/,/./g;
        $price1 =~ s/,/./g;
        $price2 =~ s/,/./g;
        $price3 =~ s/,/./g;
        $price4 =~ s/,/./g;
        $price5 =~ s/,/./g;
        $tax1 =~ s/,/./g;
        $family =~ s/,/./g;
        $cbot =~ s/,/./g;
        $icon =~ s/,/./g;
        #$printer =~ s/,/./g;
        $type =~ s/,/./g;
        $open_price =~ s/,/./g;
        $linked_page =~ s/,/./g;
        $barcode =~ s/,/./g;
        $is_daily =~ s/,/./g;
        $extras =~ s/,/./g;
        $use_weight =~ s/,/./g;
        $maxcook =~ s/,/./g;
        $medcook =~ s/,/./g;

        if ($printer) {
          my $r = 0;
          my @t = split /,/, $printer;
          map { $r += 2 ** ($_+2) } @t;
          $printer = $r;
        }

        # codigo,nome,nomecurto,preco0,preco1,preco2,preco3,preco4,iva,familia,cor,icon,printers,tipo,varpvp,nextpage,barcode,rapido,extras,peso
        open(F, ">> $tmp_file") or die "Can't write to '$tmp_file': $!";
        print F "$code,$ncur,$nbot,$price1,$price2,$price3,$price4,$price5,$tax1,$family,$cbot,".
                "$icon,$printer,$type,$open_price,$linked_page,$barcode,$is_daily,$extras,$use_weight,$medcook,$maxcook\n";
        close(F);
      }

      &move( $tmp_file, $articles_file ) if -e $tmp_file;
    } else {
      die("Couldn't get articles from weckOFFICE server");
    }
  }

  $VAR1 = undef;

  my %family_families = ();

  #
  # Sync Families
  #

  if ($s->manual() && $xml->{sync_families}) {
    if (! -f $families_file) {
      #die("Can't find families file '$families_file'");
    }

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetFirstPOSFamilies',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $tmp_file_f  = $s->tmp_dir() . $dsep . int(rand(1000000));
      my $tmp_file_sf = $s->tmp_dir() . $dsep . int(rand(1000000));

      $s->debug("Will process ".scalar(@$VAR1)." families...");

      my %sub_invisible;

      foreach my $f (@$VAR1) {
        my $code = $f->{family_fo_id} || "";
        my $name = $f->{family_name} || "";
        $name =~ s/,/./g;
        my $parent_family = $f->{top_family_fo_id} || 0;

        my %settings;
        map { my $k=$_; $_ =~ s/^mfam_//; $settings{$_} = $f->{fo_seetings}->{$k} } keys %{$f->{fo_seetings}};
        $f->{fo_seetings} = \%settings;

        my $cbot_tmp = $f->{fo_seetings}->{_cbot} || "";
        my $cbot = $cbot_tmp;

        $cbot = "" unless defined $cbot;
        $cbot = exists $colors{$cbot_tmp} ? $colors{$cbot_tmp} : 'FFFFFF';

        $code =~ s/,/./g;
        $name =~ s/,/./g;
        $cbot =~ s/,/./g;

        # sub families
        if (defined $f->{top_family_id}) {
          my $invisible = $f->{fo_seetings}->{visible} || 0;
  
          # Codigo, Nome, Cor, art0, art1, art2, artn
          my $str = "$code,$name,$cbot,".$f->{articles};

          $sub_invisible{$code} = 1 if $invisible;

          push @{ $family_families{$parent_family}}, $code;

          #$s->debug("SUB: $name");

          open(F, ">> $tmp_file_sf") or die "Can't write to '$tmp_file_sf': $!";
          print F "$str\n";
          close(F);
        }
      }

      # top families
      foreach my $f (@$VAR1) {
        my $code = $f->{family_fo_id} || "";
        my $name = $f->{family_name} || "";
        $name =~ s/,/./g;
        my $parent_family = $f->{top_family_fo_id} || 0;

        my $cbot_tmp = defined $f->{fo_seetings}->{_cbot} ? $f->{fo_seetings}->{_cbot} : "";
        my $cbot = exists $colors{$cbot_tmp} ? $colors{$cbot_tmp} : 'FFFFFF';

        my $invisible = $f->{fo_seetings}->{visible} || 0;
        next if $invisible;

        $code =~ s/,/./g;
        $name =~ s/,/./g;
        $cbot =~ s/,/./g;

        # top family
        if (! defined $f->{top_family_id}) {
          # Codigo, Nome, Cor, subfamilia0, subfamilia1, subfamiliaN
          my $str = "$code,$name,$cbot";
          $family_families{$code} ||= [];

          my @final = ();

          for (@{$family_families{$code}}) {
            push @final, $_ unless exists $sub_invisible{$_};
          }

          $str = $str . "," . join(",", @final) if scalar @final;

          #$s->debug("TOP: $name");

          open(F, ">> $tmp_file_f") or die "Can't write to '$tmp_file_f': $!";
          print F "$str\n";
          close(F);
        }
      }

      &move( $tmp_file_f, $families_file ) if -e $tmp_file_f;
      &move( $tmp_file_sf, $sub_families_file ) if -e $tmp_file_sf;
    }
  }

  $VAR1 = undef;

  #
  # Sync Waitresses
  #

  if ($s->manual() && $xml->{sync_users}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetWaitresses',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      $s->debug("Will process ".scalar(@$VAR1)." waitresses...");
      $s->debug("THIS IS PROBABLY A DEMO COMPANY!!!!") if scalar(@$VAR1) == 3;

      my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));

      my %done;

      foreach my $w (@$VAR1) {
        next if exists $done{$w->{id}};
        my $code = $w->{fo_id} || "";
        my $name = $w->{name} || "";
        $name =~ s/,/./g;
        my $passwd = $w->{fo_passwd};
        next if $passwd eq '';
        my $access_type = $w->{fo_access_type} || "";

        my $cbot = $w->{fo_settings}->{_cbot} || "FFFFFF";

        # codigo,nome,niveldeacesso,cor,passwd
        open(F, ">> $tmp_file") or die "Can't write to '$tmp_file': $!";
        print F "$code,$name,$access_type,$cbot,$passwd\n";
        close(F);
        $done{$w->{id}} = 1;
      }

      &move( $tmp_file, $waitresses_file ) if -e $tmp_file;
    } else {
      die("Couldn't get waitresses from weckOFFICE server");
    }
  }

  $VAR1 = undef;

  #
  # Sync Clients
  #

  if ($s->manual() && $xml->{sync_clients}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetStoreClients',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      $s->debug("Will process ".scalar(@$VAR1)." clients...");

      my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));

      foreach my $c (@$VAR1) {
        my $pricei = 0;
        my $pricen = 0;
        my $code = $c->{fo_id} || 0;
        my $name = $c->{name} || 0;
        $name =~ s/,/./g;
        my $card_number = $c->{card_number} || 0;
        my $balance = $c->{balance} || 0;
        my $credit_limit = $c->{credit_limit} || 0;
        my $contrib = $c->{nif} || 0;
        my $category = 0;
        my $telephone = $c->{telephone} || 0;

#NEW STUFF
        my $address = $c->{address};
        my $local = $c->{localidade};
        my $postal_code = $c->{postal_code};
        $address =~ s/\,//g;
        $local =~ s/\,//g;
        $postal_code =~ s/\,//g;

        $credit_limit = $credit_limit*100 if $credit_limit > 0;
        $balance = $balance*100 if $balance > 0;

        # codigo,nome,desconto,cartao,categoria,telefone,precoi,precon
        open(F, ">> $tmp_file") or die "Can't write to '$tmp_file': $!";
        print F "$code,$name,$credit_limit,$card_number,$category,$telephone,$pricei,$pricen,$contrib,$balance,$address,$local,$postal_code\n";
        close(F);
      }

      &move( $tmp_file, $clients_file ) if -e $tmp_file;
    } else {
      die("Couldn't get waitresses from weckOFFICE server");
    }
  }

  #
  # Sync Menus
  #

  if ($s->manual() && $xml->{sync_menus}) {
    if (! -f $menus_file) {
      #die("Can't find clients file '$menus_file'");
    }

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetMenuArticles',
                                       }
                                     );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      if (scalar @$VAR1) {

      $s->debug("Will process ".scalar(@$VAR1)." menu records...");

      my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));

      my %control;

      foreach my $w (@$VAR1) {
        if ($w->{fo_id} !~ /^\d+$/) {
          next;
        }

        if ($w->{composite_fo_id} !~ /^\d+$/) {
          next;
        }

        next if exists $control{ $w->{top_internal_code} . $w->{comp_internal_code} . $w->{option} };

        my %menu_main_props = (
          parent_article => $w->{top_internal_code}  || "",
          step_number    => $w->{option}             || "",
          article        => $w->{comp_internal_code} || "",
          xtra_cost      => $w->{xtra_cost}          || "",
        );

        $w->{xtra_cost} = $w->{xtra_cost}*1;
        $w->{xtra_cost} = $w->{xtra_cost}*100 if $w->{xtra_cost} > 0;

        open(F, ">> $tmp_file") or die "Can't write to $tmp_file";
        print F $w->{top_internal_code}.",".$w->{comp_internal_code}.",".$w->{option}.",".$w->{xtra_cost}."\n";
        close(F);

        $control{ $w->{top_internal_code} . $w->{comp_internal_code} } = 1;
      }

      if (-s $tmp_file) {
        &move( $tmp_file, $menus_file );
      }
      } else {
        $s->debug("Received 0 menu records, ignoring...");
      }
    } else {
      die("Couldn't get menus from weckOFFICE server");
    }
  }

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $xml->{dir}.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  my @possible_remove = ();

  $s->common_sync_fo_and_sync_bk( $xml, $zip, \@possible_remove );

  #
  # Sync Printers
  #

  my $printers_file  = $s->tmp_dir() . $dsep . 'printers.txt';
  
  if (-e $parameters_file) {
    if (open(F, "< $parameters_file")) {
      my @printers;
      my $start = 0;
      my $printer_num = 0;

      while (<F>) {
        chop;

        if ($_ eq '[IMPRESSORAS]') {
          $start = 1;
          next;
        }

        push @printers, $_ if $start && $printer_num++ < 14;
      }
      close(F);
      
      if (scalar @printers) {
        if (open(F, "> $printers_file")) {
          print F Dumper(\@printers);
          close(F);
          $zip->addFile( $printers_file );
        }
      }
    }
  }

  #
  # Sync Sells
  #

  if ($xml->{sync_sells}) {
    # date_from filtering
    my $date_from = $xml->{date_from} || undef;
    $date_from =~ s/\///g if defined $date_from;

    my $found_something = $s->sync_movements($zip, \@possible_remove, $date_from);
  }

  die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

  #
  # Send ZIP file to the server
  #

  eval {
    $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                file_upload => $zipFile,
                              }
                            );
  };

  if ($@) {
    die "Error sending file: $@";
  }
    
  $s->own_remove($zipFile) if -e $zipFile;
  $s->own_remove($printers_file) if -e $printers_file;

  if (scalar @possible_remove) {
    for (@possible_remove) {
      $s->debug("Will remove file '$_'");
      $s->own_remove( $_ );
    }
  }

  $s->debug("Done.");
}

#
# Sync movements
#

sub sync_movements {
  my $s = shift;
  my $zip = shift;
  my $remove = shift;
  my $date_from = shift;

  $s->debug("sync_movements()");

  my $found_something = undef;

  # directory separator
  my $dsep = $s->directory_separator();

  # service xml
  my $xml = $s->info();

  my $prefix_doc_type = $s->vfh( $xml, 'prefix_doc_type' );
  my $prefix_op_type = $s->vfh( $xml, 'prefix_op_type' );

  if (defined $prefix_doc_type || defined $prefix_op_type) {
    my $misc_prefixes_file = $s->tmp_dir() . $dsep . "misc_prefixes.txt";

    open(F, "> $misc_prefixes_file") or die "Can't write to '$misc_prefixes_file': $!\n";

    print F "prefix_doc_type=$prefix_doc_type\n" if defined $prefix_doc_type;
    print F "prefix_op_type=$prefix_op_type\n" if defined $prefix_op_type;

    close(F);

    $zip->addFile( $misc_prefixes_file );

    push @$remove, $misc_prefixes_file;
  }

  my $check_dates_fast = $xml->{check_dates_fast} || 0;

  my $fo_type = $s->encode_fo_type();

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  # winrest sells dir
  my $sells_dir = $xml->{sells_dir} || "";

  # movements dir
  my $dir_movements = $sells_dir || $dir.$dsep.'dados';

  # remove sells after import
  my $remove_sells_after_import = $xml->{remove_sells_after_import} || 0;

  my $number_of_days_to_sync = $xml->{number_of_days_to_sync} || 0;

  my @files = &read_dir($dir_movements);

  my $VAR1 = {};

  my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration',
                                                 }
                                               );

  eval "$company_config";
  my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDates';
  my $existent_dates = $s->weckoffice_send_data( { action => $act,
                                                 }
                                               );

  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $today_d = strftime("%y%m%d", localtime(time));

  foreach my $file (@files) {
    next unless $file =~ /\.csv$/i || $file =~ /\.wrk$/i;

    my( $y, $m, $d ) = $file =~ /^.(\d\d)(\d\d)(\d\d)/;
    next unless defined $y && defined $m && defined $d;

    # ignore current day
    next if $today_d eq $y.$m.$d;

    $y = '20'.$y;

    if (defined $date_from && $y.$m.$d < $date_from) {
#      $s->debug("Ignoring date $y/$m/$d with the 'date_from' clause...");
      next;
    }

    if ($company_config{import_sells_date_dupplicate} eq 'ignore' && exists $existent_dates{"$y-$m-$d"}) {
#      $s->debug("Will ignore file '$file' because that date already exists");
      next;
    }

    push @$remove, $dir_movements.$dsep.$file if $remove_sells_after_import;

    my $member = $zip->addFile( $dir_movements.$dsep.$file );
    $found_something = 1;
  }

  return $found_something;
}

sub common_sync_fo_and_sync_bk {
  my $s = shift;
  my( $xml, $zip, $to_remove ) = @_;

  $s->debug("common_sync_fo_and_sync_bk()");

  my $dsep = $s->directory_separator();

  my $consumption_price_number = $s->vfh( $xml, 'consumption_price_number' );

  if (defined $consumption_price_number) {
    my $consumption_price_number_file = $s->tmp_dir() . $dsep . "consumption_price_number.txt";

    $s->debug("####> $consumption_price_number_file - $consumption_price_number");

    open(F, "> $consumption_price_number_file") or die "Can't write to '$consumption_price_number_file': $!\n";
    print F "$consumption_price_number\n";
    close(F);

    $zip->addFile( $consumption_price_number_file );

    push @$to_remove, $consumption_price_number_file;
  }

  #
  # use sale locals to map things
  #

  $xml->{map_sale_terminals} ||= undef;

  if (defined $xml->{map_sale_terminals}) {
    my $file_map_sale_terminals = $s->tmp_dir() . $dsep . "map_sale_terminals.txt";

    my @mappings = ();

    if (exists $xml->{map_sale_terminals}->{map}) {
      if (ref $xml->{map_sale_terminals}->{map} eq 'HASH') {
        push @mappings, $xml->{map_sale_terminals}->{map};
      } elsif (ref $xml->{map_sale_terminals}->{map} eq 'ARRAY') {
        @mappings = @{ $xml->{map_sale_terminals}->{map} };
      }
    }

    if (scalar @mappings) {
      open(F, "> $file_map_sale_terminals") or die "Can't write to '$file_map_sale_terminals': $!\n";
      print F Dumper(\@mappings);
      close(F);
      $zip->addFile( $file_map_sale_terminals );
      push @$to_remove, $file_map_sale_terminals;
    }
  }
}

1;


