package WO::Client::Integrators::GAS_BP1;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use File::Copy qw( move copy );
use File::Spec;
use File::Slurp qw( read_file read_dir );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Basename;
use File::Find qw( find );
use Data::Dumper;
use POSIX qw( strftime );

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  if (! -d $dir) {
    die "Dir '$dir' doesn't exists...";
  }

  # directory separator
  my $dsep = $s->directory_separator();

  my @possible_remove = ();

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $xml->{dir}.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  my @files = &read_dir($dir);

  foreach my $file (@files) {
    next unless $file =~ /\.log$/i;
    my $member = $zip->addFile( $dir.$dsep.$file );
    push @possible_remove, $dir.$dsep.$file;
  }

  die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

  #
  # Send ZIP file to the server
  #

  eval {
    $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                file_upload => $zipFile,
                              }
                            );
  };

  $s->own_remove($zipFile) if -e $zipFile;

  if ($@) {
    die "Error sending file: $@";
  }

  if (scalar @possible_remove) {
    for (@possible_remove) {
      $s->debug("Will remove file '$_'");
      $s->own_remove( $_ );
    }
  }

  $s->debug("Done.");
}


#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");
  $s->debug("Done.");
}

#
# Sync movements
#

sub sync_movements {
  my $s = shift;
  my $zip = shift;
  my $remove = shift;
  my $date_from = shift;

  $s->debug("sync_movements()");

  my $found_something = undef;

  # directory separator
  my $dsep = $s->directory_separator();

  # service xml
  my $xml = $s->info();

  my $prefix_doc_type = $s->vfh( $xml, 'prefix_doc_type' );
  my $prefix_op_type = $s->vfh( $xml, 'prefix_op_type' );

  if (defined $prefix_doc_type || defined $prefix_op_type) {
    my $misc_prefixes_file = $s->tmp_dir() . $dsep . "misc_prefixes.txt";

    open(F, "> $misc_prefixes_file") or die "Can't write to '$misc_prefixes_file': $!\n";

    print F "prefix_doc_type=$prefix_doc_type\n" if defined $prefix_doc_type;
    print F "prefix_op_type=$prefix_op_type\n" if defined $prefix_op_type;

    close(F);

    $zip->addFile( $misc_prefixes_file );

    push @$remove, $misc_prefixes_file;
  }

  my $check_dates_fast = $xml->{check_dates_fast} || 0;

  my $fo_type = $s->encode_fo_type();

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  # winrest sells dir
  my $sells_dir = $xml->{sells_dir} || "";

  # movements dir
  my $dir_movements = $sells_dir || $dir.$dsep.'dados';

  # remove sells after import
  my $remove_sells_after_import = $xml->{remove_sells_after_import} || 0;

  my $number_of_days_to_sync = $xml->{number_of_days_to_sync} || 0;

  my @files = &read_dir($dir_movements);

  my $VAR1 = {};

  my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration',
                                                 }
                                               );

  eval "$company_config";
  my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDates';
  my $existent_dates = $s->weckoffice_send_data( { action => $act,
                                                 }
                                               );

  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $today_d = strftime("%y%m%d", localtime(time));

  foreach my $file (@files) {
    next unless $file =~ /\.csv$/i;

    my( $y, $m, $d ) = $file =~ /^.(\d\d)(\d\d)(\d\d)/;
    next unless defined $y && defined $m && defined $d;

    # ignore current day
    next if $today_d eq $y.$m.$d;

    $y = '20'.$y;

    if (defined $date_from && $y.$m.$d < $date_from) {
#      $s->debug("Ignoring date $y/$m/$d with the 'date_from' clause...");
      next;
    }

    if ($company_config{import_sells_date_dupplicate} eq 'ignore' && exists $existent_dates{"$y-$m-$d"}) {
#      $s->debug("Will ignore file '$file' because that date already exists");
      next;
    }

    my $member = $zip->addFile( $dir_movements.$dsep.$file );
    $found_something = 1;
  }

  return $found_something;
}

sub common_sync_fo_and_sync_bk {
  my $s = shift;
  my( $xml, $zip, $to_remove ) = @_;

  $s->debug("common_sync_fo_and_sync_bk()");

  my $dsep = $s->directory_separator();

  my $consumption_price_number = $s->vfh( $xml, 'consumption_price_number' );

  if (defined $consumption_price_number) {
    my $consumption_price_number_file = $s->tmp_dir() . $dsep . "consumption_price_number.txt";

    $s->debug("####> $consumption_price_number_file - $consumption_price_number");

    open(F, "> $consumption_price_number_file") or die "Can't write to '$consumption_price_number_file': $!\n";
    print F "$consumption_price_number\n";
    close(F);

    $zip->addFile( $consumption_price_number_file );

    push @$to_remove, $consumption_price_number_file;
  }
}

1;

