package WO::Client::Integrators::Infogenesis;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Slurp qw( write_file );
use Data::Dumper;
use FileHandle;
use FindBin qw( $Bin );
use POSIX qw( strftime );

use DBI;
use WO::DB::ODBC;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  $xml->{odbc_name_business}  ||= "";
  $xml->{odbc_name_dimension} ||= "";
  $xml->{odbc_name_cfg}       ||= "";
  $xml->{odbc_login}          ||= "";
  $xml->{odbc_passwd}         ||= "";

  my $days = $xml->{back_days} || 30;
  my $day2sec = 24*60*60;
  my $date_from = strftime("%Y-%m-%d", localtime(time()-$days*($day2sec)));

  my @zip_files;

  $s->debug("Will connect to - ".$xml->{odbc_name_business});

  my $dbh1 = DBI->connect('dbi:ODBC:'.$xml->{odbc_name_business}, $xml->{odbc_login}, $xml->{odbc_passwd})
    or die("Can't connect to the ODBC! source name: ".$xml->{odbc_name_business}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

  $s->debug("Will connect to - ".$xml->{odbc_name_dimension});

  my $dbh2 = DBI->connect('dbi:ODBC:'.$xml->{odbc_name_dimension}, $xml->{odbc_login}, $xml->{odbc_passwd})
    or die("Can't connect to the ODBC! source name: ".$xml->{odbc_name_dimension}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

  my $dbh3 = DBI->connect('dbi:ODBC:'.$xml->{odbc_name_cfg}, $xml->{odbc_login}, $xml->{odbc_passwd})
    or die("Can't connect to the ODBC! source name: ".$xml->{odbc_name_cfg}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

  my $db_business = WO::DB::ODBC->new( dbh => $dbh1 );
  my $db_dimension = WO::DB::ODBC->new( dbh => $dbh2 );
  my $db_cfg = WO::DB::ODBC->new( dbh => $dbh3 );

  my $file_articles_master    = $Bin.'/__articles_master__.txt';     # Menu_Item_Master
  my $file_families           = $Bin.'/__families__.txt';     # Menu_Item_Master
  my $file_subfamilies        = $Bin.'/__subfamilies__.txt';     # Menu_Item_Master
  my $file_warehouses1        = $Bin.'/__warehouses_dimension__.txt'; # Profit_Center_Dimension
  my $file_warehouses2        = $Bin.'/__warehouses_master__.txt';   # Profit_Center_Master
  my $file_articles           = $Bin.'/__articles__.txt';            # Menu_Item_Dimension
  my $file_users              = $Bin.'/__users__.txt';               # Employee_Dimension
  my $file_pay_methods        = $Bin.'/__pay_methods__.txt';         # Tender_Dimension
  my $file_terminals          = $Bin.'/__terminals__.txt';           # Terminal_Dimension
  my $file_movements          = $Bin.'/__movements__.txt';           # Check_Sales_Detail
  my $file_movement_details   = $Bin.'/__movement_details__.txt';    # Check_Item_Detail
  my $file_movement_payments  = $Bin.'/__movement_payments__.txt';   # Check_Tender_Detail
  my $file_movement_customers = $Bin.'/__movement_customers__.txt';  # Check_Account_Detail

  my $sub;

  # stores
  $sub = $db_dimension->select_iter("SELECT * FROM Profit_Center_Dimension");

  if ($sub) {
    $s->write_csv_from_iter( $file_warehouses1, $sub );
    push @zip_files, $file_warehouses1;
  }

  # pay methods
  $sub = $db_dimension->select_iter("SELECT * FROM Tender_Dimension");

  if ($sub) {
    $s->write_csv_from_iter( $file_pay_methods, $sub );
    push @zip_files, $file_pay_methods;
  }


  # terminals
  $sub = $db_cfg->select_iter("SELECT * FROM Terminal_Master");

  if ($sub) {
    $s->write_csv_from_iter( $file_terminals, $sub );
    push @zip_files, $file_terminals;
  }

  # stores
  $sub = $db_cfg->select_iter("SELECT * FROM Profit_Center_Master");

  if ($sub) {
    $s->write_csv_from_iter( $file_warehouses2, $sub );
    push @zip_files, $file_warehouses2;
  }

  # articles
  $sub = $db_dimension->select_iter(qq{
    SELECT *
    FROM Menu_Item_Dimension
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_articles, $sub );
    push @zip_files, $file_articles;
  }

  # articles_master
  $sub = $db_cfg->select_iter(qq{
    SELECT *
    FROM Menu_Item_Master
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_articles_master, $sub );
    push @zip_files, $file_articles_master;
  }

  # families
  $sub = $db_cfg->select_iter(qq{
    SELECT *
    FROM Revenue_Cat_Master
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_families, $sub );
    push @zip_files, $file_families;
  }

  # sub-families
  $sub = $db_cfg->select_iter(qq{
    SELECT *
    FROM Product_Class_Master
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_subfamilies, $sub );
    push @zip_files, $file_subfamilies;
  }

  # users
  $sub = $db_dimension->select_iter(qq{
    SELECT *
    FROM Employee_Dimension
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_users, $sub );
    push @zip_files, $file_users;
  }

  my $VAR1 = {};
  
  my $existent_dates = $s->weckoffice_send_data( { action => 'FrontofficeExistentSellDatesNoStore',
                                                 }
                                               );
  eval "$existent_dates";

  my $exclude_str = "";

  if (scalar keys %$VAR1) {
    for (keys %$VAR1) {
      $exclude_str .= "'$_', ";
    }

    $exclude_str =~ s/, $//;
  }

print STDERR "###### $date_from\n";

  $sub = $db_business->select_iter(qq[
    SELECT *
    FROM Check_Sales_Detail csd
    WHERE csd.opened_date_time >= ?
  ], $date_from);

  if ($sub) {
    $s->write_csv_from_iter( $file_movements, $sub );
    push @zip_files, $file_movements;
  }

  # sale details
  $sub = $db_business->select_iter(qq[
    SELECT cid.*
    FROM Check_Item_Detail cid
    LEFT JOIN Check_Sales_Detail csd ON (cid.transaction_data_id = csd.transaction_data_id)
    WHERE csd.opened_date_time >= ?
  ], $date_from);
    #WHERE csd.opened_date_time >= CONVERT(varchar(10), DATEADD(day, - $days, GETDATE()), 101) + ' 05:00:00' AND csd.opened_date_time <= { fn CURDATE() } + ' 05:00:00' AND csd.opened_date_time >= ? AND csd.opened_date_time <= ?
  #], '2010-06-15', '2010-06-30');
    #WHERE opened_date_time >= DATEADD(day, -$days, GETDATE()) + ' 05:00:00' AND opened_date_time <= { fn CURDATE() } + ' 05:00:00'
    #WHERE csd.opened_date_time >= '2009-12-01'
    #WHERE csd.opened_date_time >= DATEADD(day, -4, GETDATE()) + ' 05:00:00'

  if ($sub) {
    $s->write_csv_from_iter( $file_movement_details, $sub );
    push @zip_files, $file_movement_details;
  }

  # sale footers
  $sub = $db_business->select_iter(qq[
    SELECT ctd.*
    FROM Check_Tender_Detail ctd
    LEFT JOIN Check_Sales_Detail csd ON (ctd.transaction_data_id = csd.transaction_data_id)
    WHERE csd.opened_date_time >= ?
  ], $date_from);
    #WHERE csd.opened_date_time >= CONVERT(varchar(10), DATEADD(day, - $days, GETDATE()), 101) + ' 05:00:00' AND csd.opened_date_time <= { fn CURDATE() } + ' 05:00:00' AND csd.opened_date_time >= ? AND csd.opened_date_time <= ?
  #], '2010-06-15', '2010-06-30');
    #WHERE csd.opened_date_time >= DATEADD(day, -$days, GETDATE()) + ' 05:00:00' AND csd.opened_date_time <= { fn CURDATE() } + ' 05:00:00'
    #WHERE csd.opened_date_time >= '2009-12-01'
    #WHERE csd.opened_date_time >= DATEADD(day, -4, GETDATE()) + ' 05:00:00'

  if ($sub) {
    $s->write_csv_from_iter( $file_movement_payments, $sub );
    push @zip_files, $file_movement_payments;
  }

  # sale customers
  $sub = $db_business->select_iter(qq[
    SELECT cad.*
    FROM Check_Account_Detail cad
    LEFT JOIN Check_Sales_Detail csd ON (cad.transaction_data_id = csd.transaction_data_id)
    WHERE csd.opened_date_time >= ?
  ], $date_from);
    #WHERE csd.opened_date_time >= CONVERT(varchar(10), DATEADD(day, - $days, GETDATE()), 101) + ' 05:00:00' AND csd.opened_date_time <= { fn CURDATE() } + ' 05:00:00' AND csd.opened_date_time >= ? AND csd.opened_date_time <= ?
  #], '2010-06-15', '2010-06-30');
    #WHERE csd.opened_date_time >= DATEADD(day, -$days, GETDATE()) + ' 05:00:00' AND csd.opened_date_time <= { fn CURDATE() } + ' 05:00:00'
    #WHERE csd.opened_date_time >= '2009-12-01'
    #WHERE csd.opened_date_time >= DATEADD(day, -4, GETDATE()) + ' 05:00:00'

  if ($sub) {
    $s->write_csv_from_iter( $file_movement_customers, $sub );
    push @zip_files, $file_movement_customers;
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $Bin.$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      for (@zip_files) {
        $s->debug("Will remove $_");
        $s->own_remove($_);
      }
    }

    $s->debug("Done.");
  } else {
    $s->debug("Nothing to send to...");
  }
}

#
# weckOFFICE to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  die "Not implemented..."
}

1;

