package WO::Client::Integrators::KTerm;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use FindBin qw( $Bin );
use POSIX qw( strftime );
use Encode qw( encode decode );
use File::Path qw( mkpath );
use File::Find qw( find );
use File::Slurp qw( read_dir );
use Digest::MD5 qw( md5_hex );
use IO::Socket::INET;
use Text::CSV_XS;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;
  $s->debug("sync_fo_information()");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");
  
  my $csv = Text::CSV_XS->new({ binary => 1, sep_char => ",", quote_char => '' });

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $xml = $s->info();

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  my $dir = $s->vfh($xml, 'dir');

  if (! defined $dir) {
    $s->abort("Missing the 'dir' parameter...");
  }

  my $cache_file = $dir.$dsep.'data.cache';
  my $export_dir = $dir.$dsep.'export';
  mkpath($export_dir) unless -d $export_dir;
  
  # Manual execution
  if ($s->manual()) {
    $s->debug("Will call NetBO_KTerm_GetArticles");

    my $str;
    my $VAR1;

    eval {
      $str = $s->weckoffice_send_data( { action => 'NetBO_KTerm_GetArticles' } );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      $s->debug("Received ".(scalar @$VAR1)." records...");
      open(F, "> $dir/Products.csv") or die "Error writing to file: $!";

      my %unit_control = ();

      my @units = ();

      for (@$VAR1) {
        $_->{button_name} = $_->{name} unless defined $_->{button_name} && $_->{button_name};
      }
      
      for (sort { $a->{button_name} cmp $b->{button_name} } @$VAR1) {
        #$_->{name} =~ s/\n|\r|,//g;
        $_->{default_unit} ||= "";
        
        $_->{name} =~ s/,//g;
        $_->{button_name} =~ s/,//g;

        my $art_type = "";
        
        $art_type = 'Vend' if $_->{is_sale};
        $art_type = 'Merc' if $_->{is_merc};
        $art_type = 'Gene' if $_->{is_bgeneric};
        $art_type = 'Prod' if $_->{is_prod};
        
        if (! $art_type) {
          #$s->debug($_->{name}." -> ignoring because can't determine product type (is_sale, is_merc, is_bgeneric, is_prod)");
          next;
        }

        if (! $_->{default_unit}) {
          #$s->debug($_->{name}." -> ignoring because don't have a default unit...");
          next;
        }
                
        #my @line = ( $_->{name}, $_->{button_name}, '', '', $_->{id}, $art_type, $_->{default_unit} );
        #my $status = $csv->combine(@line);
        #print F $csv->string()."\n";

        print F $_->{name}.",".$_->{button_name}.",,,".$_->{id}.",".$art_type.",".$_->{default_unit}."\n";
      }

      close(F);
    }
  }
  # automatic execution
  else {
    $s->debug("Will check export dir - $export_dir");

    my $file_to_send = undef;

    my @files = &read_dir($export_dir);
    my %f = ();

    for (@files) {
      next if $_ eq 'Run';
      my $full_p = $export_dir.$dsep.$_;
      my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($full_p);
      $f{$full_p} = defined $mtime ? $mtime : 0;
    }

    my $younger = undef;

    foreach my $file (reverse sort { $f{$a} cmp $f{$b} } keys %f) {
      if (! defined $younger) {
        $younger = $file;
      }
      else {
        #unlink $file;
      }
    }
    
    my $md5 = defined $younger ? $s->_md5_from_file($younger) : undef;

    if (defined $md5 && defined $younger) {
      my $cache = $s->_cache_read($cache_file);

      if (! exists $cache->{$younger} || $cache->{$younger} ne $md5) {
        my $ip = '109.74.194.95';
        my $port = '9001';
        my $file_name = "";
        
        my $sock = IO::Socket::INET->new(
          PeerAddr => $ip,
          PeerPort => $port,
          Proto    => 'tcp',
          Timeout  => 10,
        ) or die "ERROR CONNECTING TO $ip:$port - $@\n";
        
        my $fname = "";
        
        if ($^O eq 'linux') {
          ( $fname ) = $younger =~ /^.*\/(.*?)$/;
        }
        else {
          ( $fname ) = $younger =~ /^.*\\(.*?)$/;
        }
                
        if (open(F, "< $younger")) {
          my $dev_id = "";
          my $c = 0;
          
          while(<F>) {
            chop;
            
            if ($c++ == 0) {
              my $tmp = $fname;
              $tmp =~ s/\.csv$//i;

              my $store = $xml->{weckoffice}->{store};
              $store =~ s/^CODE://i;

              $file_name = $xml->{weckoffice}->{company}."_KTERM_".$tmp."_".$store.".csv";

              $dev_id = $_;

              print STDERR "###########> ".length($dev_id)."\n";

              print $sock "ACTION=BackofficeSendFileStart#FILENAME=".$file_name."#COMPANY=".$xml->{weckoffice}->{company}."#DEVICE_ID=".$dev_id."\r\n";              
            }
            else {
              if ($c == 2) {
                print $sock $dev_id."\r\n";
              }
              
              print $sock $_."\r\n";
            }
          }
          close(F);

          #my( $host, $kind, $date, $store ) = $file =~ /^(.*?)_(.*?)_(.*?)_(.*?)\./;
          
          print $sock "ACTION=BackofficeSendFileEnd#FILENAME=".$file_name."#COMPANY=".$xml->{weckoffice}->{company}."#DEVICE_ID=".$dev_id."#DONT_DECODE=1\r\n";
        }
        
        $s->debug("SYNC! $file_name");

        $cache->{$younger} = $md5;
        $s->_cache_save($cache_file, $cache);
        
        undef $sock;
      }
      else {
        $s->debug("NOT SYNC!!!!!!!!!!!!!!!!");
      }
    }

    my $f = $export_dir.$dsep.'Run';
    unlink $f if -e $f;
  }
}

sub _cache_read {
  my $s = shift;
  my $f = shift;
  
  my $r = {};
  
  if (open(F, "< $f")) {
    while (<F>) {
      chop;
      my( $k, $v ) = $_ =~ /^(.*?)=(.*?)$/;
      $r->{$k} = $v;
    }
    close(F);
  }
  
  return $r;
}

sub _cache_save {
  my $s = shift;
  my $f = shift;
  my $r = shift;
  
  if (open(F, "> $f")) {
    foreach my $k (keys %$r) {
      print F $k."=".$r->{$k}."\n";
    }
    close(F);
  }  
}

sub _md5_from_file {
  my $s = shift;
  my $f = shift;
  
  my $md5 = undef;
  
  if (open(F, "< $f")) {
    while (<F>) {
      $md5 .= &md5_hex($_);
    }
    
    close(F);
  }
  
  return $md5;
}

1;

