package WO::Client::Integrators::MiPOS;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use WO::DB::ODBC;
use Text::CSV_XS;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  $xml->{odbc_name}   ||= "";
  $xml->{odbc_login}  ||= "";
  $xml->{odbc_passwd} ||= "";

  my $dbh = DBI->connect('dbi:ODBC:'.$xml->{odbc_name}, $xml->{odbc_login}, $xml->{odbc_passwd})
    or $s->abort("Can't connect to the ODBC! source name: ".$xml->{odbc_name}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

  my $check_dates_fast = $xml->{check_dates_fast} || 0;

  my $VAR1 = {};

  my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration',
                                                 }
                                               );

  eval "$company_config";
  my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDates';
  my $existent_dates = $s->weckoffice_send_data( { action => $act,
                                                 }
                                               );

  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $db = WO::DB::ODBC->new( dbh => $dbh );

  my %articles = ();

  my $file_articles = $tmp_dir.'/__articles__.txt';
  my $file_headers = $tmp_dir.'/__headers__.txt';
  my $file_details = $tmp_dir.'/__details__.txt';
  my $file_footers = $tmp_dir.'/__footers__.txt';

  my $csv = Text::CSV_XS->new({ binary => 1 });

  my @zip_files;

  my $dbh_db = $db->dbh();

  my $sth = $dbh_db->prepare("SELECT * FROM TranHeader");
  $sth->execute();

  my $done_headers = 0;
  my $done_details = 0;
  my $done_footers = 0;

  open(F, "> $file_headers") or die "Can't write to $file_articles: $@";

  while( my $row = $sth->fetchrow_arrayref() ) {
    $done_headers = 1;
    my $status = $csv->combine(@$row);
    my $line = $csv->string();

    print F $line."\n";
  }

  close(F);

  $sth = $dbh_db->prepare("SELECT * FROM TranItem");
  $sth->execute();

  open(F, "> $file_details") or die "Can't write to $file_articles: $@";

  while( my $row = $sth->fetchrow_arrayref() ) {
    $done_details = 1;
    my $status = $csv->combine(@$row);
    my $line = $csv->string();

    print F $line."\n";
  }

  close(F);

  $sth = $dbh_db->prepare("SELECT * FROM TranPayment");
  $sth->execute();

  open(F, "> $file_footers") or die "Can't write to $file_articles: $@";

  while( my $row = $sth->fetchrow_arrayref() ) {
    $done_footers = 1;
    my $status = $csv->combine(@$row);
    my $line = $csv->string();

    print F $line."\n";
  }

  close(F);

  push @zip_files, $file_headers if $done_headers;
  push @zip_files, $file_details if $done_details;
  push @zip_files, $file_footers if $done_footers;

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $s->tmp_dir().$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      for (@zip_files) {
        $s->debug("Will delete file '$_'");
        $s->own_remove($_);
      }
    }
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;
}

1;



