package WO::Client::Integrators::OnlinePOSPg;

use strict;
no warnings;

use base qw ( WO::Client::Common );

use File::Copy qw( move copy );
use File::Spec;
use POSIX qw( strftime );
use File::Slurp qw( read_file read_dir );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Basename qw( fileparse );
use File::Find qw( find );
use Data::Dumper;
use FindBin qw( $Bin );
use Encode qw( encode );
use WO::DB;
use DBI;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # frontoffice directory
  my $dir = $xml->{dir} || "";

  my $db = &_db();

  my $tmp;

  #
  # Groups
  #

  $s->debug("groups");

  $tmp = $db->select("SELECT id, name, parent, deleted FROM groups");

  my %did_families = ();

  open(F, "> /tmp/".$$."_groups.csv");

  foreach my $p (@$tmp) {
    next if exists $did_families{$p->{id}};
    $did_families{$p->{id}} = 1;
    print F $p->{id}."\t".$p->{name}."\t".$p->{parent}."\t".$p->{deleted}."\n";
  }

  close(F);

  #
  # Products
  #

  $s->debug("products");

  $tmp = $db->select(qq{
    SELECT p.id, p.name, pg.group_id AS parent, p.deleted
    FROM product_groups pg
    LEFT JOIN products p ON (pg.product_id = p.id)
  });

  my %did_products = ();

  open(F, "> /tmp/".$$."_products.csv");

  foreach my $p (@$tmp) {
    next if exists $did_products{$p->{id}};
    $did_products{$p->{id}} = 1;
    print F $p->{id}."\t".$p->{name}."\t".$p->{parent}."\t".$p->{deleted}."\n";

  }

  close(F);

  #
  # Customers
  #

  $s->debug("customers");

  $tmp = $db->select("SELECT id, name, nif, deleted FROM customers");

  my %did_customers = ();

  open(F, "> /tmp/".$$."_customers.csv");

  foreach my $p (@$tmp) {
    next if exists $did_customers{$p->{id}};
    $did_customers{$p->{id}} = 1;
    print F $p->{id}."\t".$p->{name}."\t".$p->{nif}."\t".$p->{deleted}."\n";

  }

  close(F);

  #
  # Clerks
  #

  $s->debug("clerks");

  $tmp = $db->select("SELECT id, login, deleted FROM clerks");

  my %did_clerks = ();

  open(F, "> /tmp/".$$."_clerks.csv");

  foreach my $p (@$tmp) {
    next if exists $did_clerks{$p->{id}};
    $did_clerks{$p->{id}} = 1;
    print F $p->{id}."\t".$p->{login}."\t".$p->{deleted}."\n";

  }

  close(F);

  $s->sync_movements([
    "/tmp/".$$."_clerks.csv",
    "/tmp/".$$."_products.csv",
    "/tmp/".$$."_customers.csv",
    "/tmp/".$$."_groups.csv",
  ]);

  $s->debug("Done.");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information - Not implemented...");
}

#
# Sync movements
#

sub sync_movements {
  my $s = shift;
  my $to_send = shift || [];

  $s->debug("sync_movements()");

  my $VAR1;
  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  # frontoffice directory
  my $dir = $xml->{dir} || "";
  my $process_current_day = $xml->{process_current_day} || "";

  my $csv = Text::CSV_XS->new({ binary => 1 }) or
    die "Cannot use CSV: ".Text::CSV->error_diag ();

  # directory separator
  my $dsep = $s->directory_separator();
  my $dsn = 'dbi:PgPP:dbname=pos;host=localhost';
  my $dbh = DBI->connect( $dsn, 'postgres', '', { PrintError => 0, RaiseError => 1 } )
    or die "$DBI::errstr";

  my $db = _db();

  my $str = "";
  my $VAR1 = {};

  my $db_update = 0;

  my $rand = strftime("%Y%m%d_%H%M%S", localtime(time)).".".$$;

  my $file_headers  = "/tmp/headers.csv.".$rand;
  my $file_details  = "/tmp/details.csv.".$rand;
  my $file_payments = "/tmp/payments.csv.".$rand;

  unlink $file_headers if -e $file_headers;
  unlink $file_details if -e $file_details;
  unlink $file_payments if -e $file_payments;

  my @to_remove = ();

  my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration',
                                                 }
  );

  eval "$company_config";
  my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $company_config = $s->company_configuration();

  my $act = 'FrontofficeExistentSellDatesFast';
  my $existent_dates = $s->weckoffice_send_data( { action => $act, } );
  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $sth;
  my $business_date = '';
  my $did_data = 0;

  if (open(F, "< /opt/pos/.business_date")) {
    local $/ = undef;
    $business_date = <F>;
    chop($business_date);
    close(F);
  }

  if (! $business_date) {
    die "Can't find business date in /opt/pos/.business_date...";
  }

  # include current day
  delete $existent_dates{$business_date};

  my @files_to_remove = ();

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $dir.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  # add files to be sent with the sales
  for (@$to_send) {
    $zip->addFile( $_ );
  }

  if (-d "/opt/pos/server/data/export") {
    $did_data = 1;

    my @files = read_dir("/opt/pos/server/data/export");

    for (@files) {
      $s->debug("Will add: $_");
      my( $date_tmp ) = $_ =~ /^(.*?)#/;
      delete $existent_dates{$date_tmp};

      $zip->addFile( "/opt/pos/server/data/export/".$_ );
      push @files_to_remove, "/opt/pos/server/data/export/".$_;
    }
  }

  if (-d "/opt/pos/server/data/export") {
    $did_data = 1;

    my @files = read_dir("/opt/pos/server/data/export");

    for (@files) {
      $s->debug("Will add: $_");
      my( $date_tmp ) = $_ =~ /^(.*?)#/;
      delete $existent_dates{$date_tmp};

      $zip->addFile( "/opt/pos/server/data/export/".$_ );
      push @files_to_remove, "/opt/pos/server/data/export/".$_;
    }
  }

  my $last_id_processed = 0;

  # if files were added to this array, don't do this verification
  #if (! scalar @files_to_remove) {
  #  $last_id_processed = _check_last_id_processed($db);
  #}

  my @doc_types_sales = ( 1, 5, 90, 12, 7, 2, 13 );
  my $where_dt = "AND doc_type IN (".join(",", @doc_types_sales).")";

  my $date_from_w1 = $s->get_date_from();
  my $date_from_w2 = $s->get_date_from();

  $s->debug("DATE FROM : ".$s->get_date_from());

  my $date_from_w1 = " AND business_date >= '".$s->get_date_from()."'";
  my $date_from_w2 = " AND sh.business_date >= '".$s->get_date_from()."'";

  $sth = $dbh->prepare(qq{
    SELECT business_date, SUM(value)
    FROM sales_headers
    WHERE 1=1$date_from_w1 $where_dt
    GROUP BY business_date
    ORDER BY business_date
  });

  $sth->execute();

  my %ddates;

  while (my $r = $sth->fetchrow_arrayref) {
    $ddates{$r->[0]} = $r->[1];
  }

  foreach my $date (sort keys %ddates) {
    if (! $process_current_day) {
      if ($date eq $business_date) {
        $s->debug("DATE $date IS TODAY, IGNORING...");
        next;
      }
    }

    $s->debug("DATE TO ANALYZE: $date, NETBO: ".int($existent_dates{$date}*1).", POS: ".int($ddates{$date}*1));

    if (exists $existent_dates{$date}) {
      # old method
      if ($existent_dates{$date} == 1) {
        if ($company_config->{import_sells_date_dupplicate} eq 'ignore' && exists $existent_dates{$date}) {
          print STDERR "######OLD> WILL IGNORE DATE - $date\n";
          next;
        }
      }
      # new method
      else {
          #$company_config->{import_sells_date_dupplicate} eq 'ignore' &&
        if (
          exists $existent_dates{$date} &&
          int($existent_dates{$date}*1) == int($ddates{$date}*1)
        ) {
          print STDERR "######NEW> WILL IGNORE DATE - $date\n";
          next;
        }
      }
    }

    $s->debug("WILL PROCESS DATE: $date");

    $did_data = 1;

    #
    # Headers
    #

    $sth = $dbh->prepare(qq{
      SELECT *
      FROM sales_headers
      WHERE business_date = ?$date_from_w1
    });

    $sth->execute($date);

    open(F, ">> $file_headers") or die "can't write to $file_headers - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    #
    # Details
    #

    $sth = $dbh->prepare(qq{
      SELECT sp.*
      FROM sales_details sp
      LEFT JOIN sales_headers sh ON (sp.header = sh.id)
      WHERE sh.business_date = ?$date_from_w2
    });

    $sth->execute($date);

    open(F, ">> $file_details") or die "can't write to $file_details - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    #
    # Payments
    #

    $sth = $dbh->prepare(qq{
      SELECT sp.*
      FROM sales_payments sp
      LEFT JOIN sales_headers sh ON (sp.header = sh.id)
      WHERE sh.business_date = ?$date_from_w2
    });

    $sth->execute($date);

    open(F, ">> $file_payments") or die "can't write to $file_payments - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();
  }

  if ($did_data) {
    if (-e $file_headers) {
      $zip->addFile( $file_headers );
      push @files_to_remove, $file_headers;
    }

    if (-e $file_details) {
      $zip->addFile( $file_details );
      push @files_to_remove, $file_details;
    }

    if (-e $file_payments) {
      $zip->addFile( $file_payments );
      push @files_to_remove, $file_payments;
    }

    my $prefix_doc_type = $s->vfh( $xml, 'prefix_doc_type' );
    my $prefix_op_type = $s->vfh( $xml, 'prefix_op_type' );

    if (defined $prefix_doc_type || defined $prefix_op_type) {
      my $misc_prefixes_file = $s->tmp_dir() . $dsep . "misc_prefixes.txt";

      open(F, "> $misc_prefixes_file") or die "Can't write to '$misc_prefixes_file': $!\n";

      print F "prefix_doc_type=$prefix_doc_type\n" if defined $prefix_doc_type;
      print F "prefix_op_type=$prefix_op_type\n" if defined $prefix_op_type;

      close(F);

      $zip->addFile( $misc_prefixes_file );
    }

    die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                });
    };

    $s->own_remove($zipFile) if -e $zipFile;
    #system("rm -r $n_tmp_dir");

    if ($@) {
      for (@files_to_remove) {
        unlink $_;
      }

      die "Error sending file: $@";
    }
    else {
      if ($last_id_processed) {
        if (open(F, "> /tmp/last_sold_id.txt")) {
          print F "$last_id_processed\n";
          close(F);
        }
      }

      for (@files_to_remove) {
        unlink $_;
      }
    }
  }

  for (@$to_send) {
    $s->debug("Will remove $_");
    unlink $_;
  }
}

#
# Internal methods
#

sub _db {
  my $dsn = 'dbi:PgPP:dbname=pos;host=localhost';
  my $dbh = DBI->connect( $dsn, 'postgres', '', { PrintError => 0, RaiseError => 1 } )
    or die "$DBI::errstr";

  my $db = WO::DB->new( dbh => $dbh );

  my $dbh_pos = $db->dbh();
  my $sth_pos = $dbh_pos->prepare("PRAGMA cache_size = 200000") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA default_synchronous = OFF") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA synchronous = OFF") or die($dbh_pos->errstr());

  return $db;
}



1;











