package WO::Client::Integrators::PHC;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use DBI;
use WO::DB::ODBC;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use POSIX qw( strftime );
use Encode qw( encode decode );
use File::Basename qw( fileparse fileparse_set_fstype );
use File::Remove qw( remove );
use File::Slurp qw( read_file );

&fileparse_set_fstype('MSWin32');

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;
  $s->debug("sync_fo_information()");
}

#
# Brings weckOFFICE information to FO
#

sub _check_ask_date_script {
  my $s = shift;

  $s->debug("_check_ask_date_script()");

  my $ds = $s->directory_separator();

  my $f = $Bin.$ds."ask_dates.vbs";

  my $date_from = undef;
  my $date_to = undef;

  if (-e $Bin.$ds."date_from.txt") {
    $date_from = read_file($Bin.$ds."date_from.txt");
    chop($date_from);
  }

  if (-e $Bin.$ds."date_to.txt") {
    $date_to = read_file($Bin.$ds."date_to.txt");
    chop($date_to);
  }

  my $dateFromStr = defined $date_from ? '"'.$date_from.'"' : "dateStr";
  my $dateToStr = defined $date_to ? '"'.$date_to.'"' : "dateStr";

  if (! -e $f) {
    my $str =<<END;
Function RegExpValidator(patrn, strng)
    Dim regEx, Match, Matches, RetStr
    Set regEx = new RegExp ' Create a regular expression.
    regEx.Pattern = patrn ' Set pattern.
    regEx.IgnoreCase = True' Set case insensitivity.
    regEx.Global = false' Set global applicability.
    Set Matches = regEx.Execute(strng)' Execute search.
    if Matches.count > 0 then
        RetStr = true
    else
        RetStr = false
    end if
    set Matches = nothing
    set regEx = nothing
    RegExpValidator = RetStr
End Function

function ValidDate(dtDate, sErrorText)
    dim sPatrn, sMonth, sDay, sYear, aDate, bValid, bIsLeap, bPatrnMatch
    dtDate = trim(dtDate)
    sPatrn = "(\\d{4})(\\-)(\\d{2})(\\-)(\\d{2})"
    bPatrnMatch = RegExpValidator(sPatrn,dtDate) '// is the format ok?
    bValid = true
    if bPatrnMatch = true then
        aDate = split(dtDate,"-")
        if ubound(aDate) = 2 then
            sYear = aDate(0)
            sMonth = aDate(1)
            sDay = aDate(2)
            
            if len(sYear) = 4 then
                if sYear < 1753 then ' //dates less than January 1 1753 cause an exception in sql server
                    bValid = false
                    sErrorText = "Year must be greater than 1752"
                else
                    if sMonth < 1 or sMonth > 12 then  '// check month range
                        bValid = false
                        sErrorText = "Month must be between 1 and 12."
                    else
                        if sDay < 1 or sDay > 31 then
                            bValid = false  
                            sErrorText = "Day must be between 1 and 31."
                        else
                            if ((sMonth=4 or sMonth=6 or sMonth=9 or sMonth=11) and sDay=31) then
                                bValid = false
                                sErrorText ="Month " & sMonth & " does not have 31 days!"
                            else
                                if sMonth = 2 then '// check for february 29th
                                    If (sYear Mod 4 = 0 And sYear Mod 100 <> 0) Or sYear Mod 400 = 0 Then
                                        bIsLeap = True
                                    Else
                                        bIsLeap = False
                                    End If
                                    'Response.Write bIsLeap
                                    'Response.Write sDay
                                    
                                    if sDay > 29 or (sDay = 29 and not bIsLeap) then
                                        bValid = false
                                        sErrorText = "February " & sYear & " does not have " & sDay & " days!"
                                    end if
                                end if
                            end if
                        end if
                    end if
                end if
            else
                bValid = false
                sErrorText = "Ano tem que ter 4 digitos"
            end if
        else
            bValid = false
            sErrorText = "Formato para datas deve ser yyyy-mm-dd"
        end if        
    else
        bValid = false
        sErrorText = "Formato para datas deve ser yyyy-mm-dd"  
    end if
    
    ValidDate = bValid
end function

Dim Stuff, myFSO, WriteStuff, dateStamp, dateStr, dateOk, error

dateOk = True

dateStamp = Date()
dateStr = DatePart("yyyy", dateStamp) & "-" & DatePart("m", dateStamp) & "-" & DatePart("d", dateStamp)

Do
  dateFrom = InputBox("De:", "Data - De", $dateFromStr)
  dateOK = ValidDate(dateFrom,erro)
  if (dateFrom = "") Then WScript.Quit
  if (dateOk = False) Then MsgBox("Data Invalida " & erro)
Loop While (dateOK = False)

dateOk = True

Do
  dateTo = InputBox("Ate:", "Data - Ate", $dateToStr)
  dateOK = ValidDate(dateTo,erro) 
  if (dateTo = "") Then WScript.Quit
  if (dateOk = False) Then MsgBox("Data Invalida " & erro)

  if (dateTo < dateFrom) Then dateOK = False
  if (dateTo < dateFrom) Then MsgBox("Data - Ate maior que Data - De")

Loop While (dateOK = False)

Set myFSO = CreateObject("Scripting.FileSystemObject")
Set WriteStuff = myFSO.OpenTextFile("$Bin/date_from.txt", 2, True)
WriteStuff.WriteLine(dateFrom)
WriteStuff.Close

Set WriteStuff = myFSO.OpenTextFile("$Bin/date_to.txt", 2, True)
WriteStuff.WriteLine(dateTo)
WriteStuff.Close

SET WriteStuff = Nothing
SET myFSO = Nothing
END

    open(F, "> $f");
    print F $str;
    close(F);
  }
}

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  $s->_check_ask_date_script();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $csv = Text::CSV_XS->new({ binary => 1 });
  
  my $xml = $s->info();

  # directory separator
  my $dsep = $s->directory_separator();

  # temporary folder
  my $tmp_dir = $s->tmp_dir();

  # folder where the data will be stored to
  my $dir = $s->vfh($xml, 'dir');
    
  $s->abort("Missing 'dir' parameter...") unless defined $dir;

  #my $file_sale_headers = $dir.$dsep."sale_headers.csv";
  #my $file_sale_details = $dir.$dsep."sale_details.csv";
      
  my $str;
  my $VAR1;

  my $script = $Bin.$dsep."ask_dates.vbs";

  system($script) if -e $script;

  my $date_from = undef;
  my $date_to = undef;

  if (-e $Bin.$dsep."date_from.txt") {
    $date_from = read_file($Bin.$dsep."date_from.txt");
    chop($date_from);
    $date_from = undef unless $date_from =~ /^\d\d\d\d\-\d\d\-\d\d$/;
  }

  if (-e $Bin.$dsep."date_to.txt") {
    $date_to = read_file($Bin.$dsep."date_to.txt");
    chop($date_to);
    $date_to = undef unless $date_to =~ /^\d\d\d\d\-\d\d\-\d\d$/;
  }

  $s->own_remove($script);

  #
  # Finatial Closings
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPHC_GetFinancialClosings', date_from => $date_from, date_to => $date_to } );
  };
  
  $s->abort($@) if $@;
  eval $str;

  my %done_sales = ();
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    #open(F, "> $file_sale_headers") or $s->abort("Can't write to $file_sale_headers: $!");

    my $status = $csv->combine('warehouse', 'date', 'code', 'seq_number', 'tax', 'doc_number', 'value1', 'value2');
    #print F $csv->string()."\n";

    foreach my $r (@$VAR1) {
      if (! defined $r->{date}) {
        $s->dump($r);
      }

      if (! exists $done_sales{$r->{date}}) {
        $done_sales{$r->{date}} = 1;
        open(M, "> $dir/fechofin_".$r->{date}.".txt");
        print M "warehouse,date,code,seq_number,tax,doc_number,value1,value2\r\n";
        close(M);
      }

      my $status = $csv->combine($r->{store}, $r->{date}, $r->{code}, $r->{seq_number}, $r->{tax}, $r->{doc_number}, $r->{value1}, $r->{value2});
      #print F $csv->string()."\n";

      &write_to_file( $dir."/fechofin_".$r->{date}.".txt", $csv->string() );
    }
    
    #close(F);
  }

  #
  # Sale Headers
  #

  %done_sales = ();

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPHC_GetSaleHeaders', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    #open(F, "> $file_sale_headers") or $s->abort("Can't write to $file_sale_headers: $!");

    my $status = $csv->combine('doc_id', 'warehouse', 'date', 'doc_type', 'doc_type_name', 'customer_code', 'serie');
    #print F $csv->string()."\n";

    foreach my $r (@$VAR1) {
      if (! defined $r->{date}) {
  $s->dump($r);
}
      if (! exists $done_sales{$r->{date}}) {
        $done_sales{$r->{date}} = 1;
        open(M, "> $dir/vendas_".$r->{date}."_headers.txt");
        print M "doc_id,warehouse,date,doc_type,doc_type_name,customer_code,serie\r\n";
        close(M);
      }

      my $status = $csv->combine($r->{key}, $r->{store}, $r->{date}, $r->{doc_type}, $r->{doc_type_name}, $r->{client_fo_id}, $r->{serie});
      #print F $csv->string()."\n";

      &write_to_file( $dir."/vendas_".$r->{date}."_headers.txt", $csv->string() );
    }
    
    #close(F);
  }

  %done_sales = ();

  #
  # Sale Details
  #
  
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPHC_GetSaleDetails', date_from => $date_from, date_to => $date_to } );
  };
  
  $s->abort($@) if $@;
  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    #open(F, "> $file_sale_details") or $s->abort("Can't write to $file_sale_details: $!");

    # 'doc_id', 'product', 'store', 'sold_v', 'sold_q', 'vat_rate', 'cost_v'
    my $status = $csv->combine('doc_id', 'product_code', 'product_name', 'warehouse', 'sold_v', 'sold_q', 'vat_v', 'vat_rate', 'cost_v');
    #print F $csv->string()."\n";
    
    foreach my $r (@$VAR1) {
      my( $date ) = $r->{key} =~ /^\d+#(.*?)#/;

      if (! exists $done_sales{$date}) {
        $done_sales{$date} = 1;
        open(M, ">> $dir/vendas_".$date."_details.txt");
        print M "doc_id,product_code,product_name,warehouse,sold_v,sold_q,vat_v,vat_rate,cost_v";
        close(M);
      }

      my $vat_rate = $r->{sold_v} > 0 ? sprintf("%.2f", ($r->{vat_v} / $r->{sold_v})*100) : 0;
      my $pcm = exists $r->{pcm} && defined $r->{pcm} ? $r->{pcm} : 0;

      my $status = $csv->combine(
        $r->{key}, $r->{product_code}, $r->{product_name}, $r->{store}, sprintf("%.3f", $r->{sold_v}),
        $r->{sold_q}, sprintf("%.3f", $r->{vat_v}), $vat_rate, sprintf("%.3f", $pcm),
      );

      #print F $csv->string()."\n";
      &write_to_file( $dir."/vendas_".$date."_details.txt", $csv->string() );
    }
            
    #close(F);
  }

  #
  # Sale Payments
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPHC_GetSalePayments', date_from => $date_from, date_to => $date_to } );
  };
  
  $s->abort($@) if $@;
  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    #open(F, "> $file_sale_details") or $s->abort("Can't write to $file_sale_details: $!");

    #doc_id,product,store,sold_v,sold_q,vat_rate,cost_v
    #my $status = $csv->combine('doc_id', 'warehouse', 'pmethod_code', 'pmethod_name', 'value');
    #print F $csv->string()."\n";
    
    foreach my $r (@$VAR1) {
      my( $date ) = $r->{key} =~ /^\d+#(.*?)#/;

      if (! exists $done_sales{$date}) {
        $done_sales{$date} = 1;
        open(M, ">> $dir/vendas_".$date."_pmethods.txt");
        print M "doc_id,warehouse,pmethod_code,pmethod_name,value";
        close(M);
      }

      my $status = $csv->combine(
        $r->{key}, $r->{store}, $r->{pmethod_code}, $r->{pmethod_name}, sprintf("%.3f", $r->{value}),
      );

      #print F $csv->string()."\n";
      &write_to_file( $dir."/vendas_".$date."_pmethods.txt", $csv->string() );
    }
            
    #close(F);
  }

  #
  # Documents
  #
  
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPHC_GetDocuments', date_from => $date_from, date_to => $date_to } );
  };
  
  $s->abort($@) if $@;
  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'HASH') {
    $VAR1->{headers} ||= [];
    $VAR1->{details} ||= [];
    
    #my $file_headers = $dir.$dsep."document_headers.csv";
    #my $file_details = $dir.$dsep."document_details.csv";
    
    #open(H, "> $file_headers") or $s->abort("Can't write to $file_headers: $!");
    #open(D, "> $file_details") or $s->abort("Can't write to $file_details: $!");

    my %dates = ();

    my $status = $csv->combine('doc_id', 'warehouse', 'date', 'doc_type', 'doc_type_name', 'supplier_code', 'supplier_name', 'supplier_fiscal_number', 'supplier_docref', 'serie', 'discount', 'payment_date');
    #print H $csv->string()."\n";

    $status = $csv->combine('doc_id', 'product_code', 'product_name', 'price', 'quantity', 'discount', 'vat_rate', 'pcm', 'extra_cost');
    #print D $csv->string()."\n";

    my %done_headers = ();
    my %done_details = ();
    
    foreach my $h (@{ $VAR1->{headers} }) {
      $h->{discount} ||= 0;
      $h->{date_of_payment} ||= "";
      $h->{supplier_ncontrib} ||= "";

      if (! exists $done_headers{$h->{date}}) {
        $done_headers{$h->{date}} = 1;
        open(M, "> $dir/compras_".$h->{date}."_headers.txt");
        print M "doc_id,warehouse,date,doc_type,doc_type_name,supplier_code,supplier_name,supplier_fiscal_number,supplier_docref,serie,discount,payment_date\r\n";
        close(M);
      }

      $dates{$h->{internal_doc_code}} = $h->{date};

      $h->{supplier_ncontrib} =~ s/\s//g;

      $status = $csv->combine($h->{internal_doc_code}, $h->{store_code}, $h->{date}, $h->{doc_type}, $h->{doc_type_name}, $h->{supplier_code}, $h->{supplier_name}, $h->{supplier_ncontrib}, $h->{supplier_docref},
        $h->{serie}, sprintf("%.3f", $h->{discount}), $h->{date_of_payment});
      #print H $csv->string()."\n";
      &write_to_file( $dir."/compras_".$h->{date}."_headers.txt", $csv->string() );
    }

    foreach my $d (@{ $VAR1->{details} }) {
      my $discount = 0;

      if ($d->{discount} > 0) {
      #  $discount = ($d->{discount} / $d->{buy_price}) * 100;
        $discount = $d->{discount};
        #print STDERR "#####> ".$d->{discount}."\n";
      }

      if (! exists $done_details{$dates{$d->{internal_doc_code}}}) {
        $done_details{$dates{$d->{internal_doc_code}}} = 1;
        open(M, ">> $dir/compras_".$dates{$d->{internal_doc_code}}."_details.txt");
        print M "doc_id,product_code,product_name,price,quantity,discount,vat_rate,pcm,extra_cost\r\n";
        close(M);
      }
    
      $status = $csv->combine($d->{internal_doc_code}, $d->{article_code}, $d->{article_name}, sprintf("%.3f", $d->{buy_price}),
        sprintf("%.2f", $d->{quantity}), sprintf("%.3f", $discount), sprintf("%.2f", $d->{tax}), sprintf("%.3f", $d->{pcm}),
        sprintf("%.3f", $d->{xtra_cost}));
      #print D $csv->string()."\n";
      &write_to_file( $dir."/compras_".$dates{$d->{internal_doc_code}}."_details.txt", $csv->string() );
    }
    
    #close(H);
    #close(D);
  }

}

sub write_to_file {
  my( $fname, $cnt ) = @_;

  open(N, ">> $fname");
  print N $cnt."\r\n";
  close(N);
}

1;


