package WO::Client::Integrators::PMS;

use strict;
use warnings;
use File::Slurp qw( read_dir );
use File::Copy qw( move );
use POSIX qw( strftime );

use WO::Client::Integrators::rTOS::Table::ChargeAccount;

use base qw ( WO::Client::Common );

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $pms_dir = $xml->{pms_folder};
  my $till_dir = $xml->{till_folder};
  my $export_dir = $xml->{till_folder}.$dsep."export";
  my $import_dir = $xml->{till_folder}.$dsep."import";

  if (! -d $pms_dir) {
    die "pms folder '$pms_dir' doesn't exists...";
  }

  if (! -d $till_dir) {
    die "till folder '$till_dir' doesn't exists...";
  }

  if (! -d $export_dir) {
    die "till export folder '$export_dir' doesn't exists...";
  }

  if (! -d $import_dir) {
    die "till import folder '$import_dir' doesn't exists...";
  }

  my $inhouse_file = $pms_dir.$dsep."InHouse.csv";

  if (-e $inhouse_file) {
    $s->_handle_inhouse_file($inhouse_file, $import_dir);
  }

  if (-d $export_dir) {
    $s->_handle_export_files($export_dir, $pms_dir);
  }
}

sub _handle_export_files {
  my $s = shift;
  my $d = shift;
  my $pms_dir = shift;

  # directory separator
  my $dsep = $s->directory_separator();

  $s->debug("_handle_export_files -> $d");

  my @files = &read_dir($d);

  my $outf = "$pms_dir".$dsep."bleep2rm.txt";

  foreach my $f (@files) {
    if ($f =~ /^journal/i) {
      my $fp = $d.$dsep.$f;

      $s->debug("Will process $f and append results to $outf");

      open(PMS, ">> $outf") or die "Can't write to $outf: $!";

      if (open(F, "< $fp")) {
        while (<F>) {
          chop;

          my @t = split /,/;

          # tender line
          next unless $t[1] == 11;

          my( $payed_value, $customer, $check_no, $terminal_id ) = ( $t[21], $t[23], $t[9], $t[3] );

          #next unless $check_no eq '10147';
          next unless $customer > 0;

          # room_number, terminal_id, check_no, amount
          #$s->debug("#### $payed_value - $customer - $check_no - $terminal_id");
          print PMS "$customer,$terminal_id,$check_no,$payed_value\n";
        }
        close(F);

        close(PMS);

        #my $done = $d.$dsep."done";
        #mkdir($done) unless -d $done;
        #&move( $fp, $done );
      } else {
        $s->debug("Can't open $fp: $!");
      }
    }    
  }
}

sub _handle_inhouse_file {
  my $s = shift;
  my $f = shift;
  my $import_dir = shift;

  # directory separator
  my $dsep = $s->directory_separator();

  $s->debug("_handle_inhouse_file -> $f");

  if (open(F, "< $f")) {
    my $cl = WO::Client::Integrators::rTOS::Table::ChargeAccount->new();
    my $did_records = 0;

    while (<F>) {
      chop;
      my @t = split /,/;

      my $room_number = $t[-2];
      my $foolio_number = $t[-1];

      my $lidx = scalar(@t)-3;

      my @all = @t[ 0 .. $lidx ];
      my $name = join(" ", reverse @all);
      $name =~ s/^\s+//;

      #$s->debug("##> $room_number, $foolio_number, $name");

      my $h = {
        recordid         => $room_number,#$idx,
        accountnumber    => $room_number,
#        accountnumber    => $foolio_number,
        name             => substr($name,0,18),
#        address1         => "",
#        city             => "",
#        zipcode          => "",
#        phonenumber      => "", 
        creditlimit      => 999999,
#        state            => 0,
#        disable          => 0,
#        decliningbalance => "",
      };

      $cl->add_row( values => $h );

      $did_records = 1;
    }
    close(F);

    if ($did_records) {
      my $year = int(strftime("%Y", localtime(time)));
      my $mon  = int(strftime("%m", localtime(time)));
      my $day  = int(strftime("%d", localtime(time)));
      my $hour = int(strftime("%H", localtime(time)));
      my $min  = int(strftime("%M", localtime(time)));
      my $sec  = int(strftime("%S", localtime(time)));

      my $out_file = $day.$mon.$year.$hour.$min.$sec.".upd"; # 235200716835.upd

      my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));
      $s->own_remove($tmp_file) if -e $tmp_file;

      $s->debug("Will write data to a tmp file: $tmp_file");

      $cl->dump_records( to_file => $tmp_file );

      my $full_p = $import_dir.$dsep.$out_file;
      my $txt_file = $full_p.".txt";

      $s->debug("Will move file to $full_p");

      $s->own_remove($full_p) if -e $full_p;
      $s->own_remove($txt_file) if -e $txt_file;

      &move( $tmp_file, $txt_file );
      &move( $txt_file, $full_p );
    }

    $s->own_remove($f);
  } else {
    $s->debug("Warning, couldn't open $f: $!");
  }
}

1;




