package WO::Client::Integrators::POSPg;

use strict;
no warnings;

use base qw ( WO::Client::Common );

use WO::DB;
use DBI;

use Encode qw( decode encode from_to );

use Text::CSV_XS;
use File::Copy qw( move copy );
use File::Spec;
use File::Slurp qw( read_file read_dir );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Basename;
use File::Find qw( find );
use Data::Dumper;
use POSIX qw( strftime );


#
# FO to weckOFFICE
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $csv = Text::CSV_XS->new({ binary => 1 }) or
    die "Cannot use CSV: ".Text::CSV->error_diag ();

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # frontoffice directory
  my $dir = $xml->{dir} || "";
  my $keep_files = $xml->{keep_files} || "0";

  if (! -d $dir) {
    die "Frontoffice dir '$dir' don't exists, where is the frontoffice?";
  }

  # directory separator
  my $dsep = $s->directory_separator();
  my $dsn = 'dbi:PgPP:dbname=pos;host=localhost';
  my $dbh = DBI->connect( $dsn, 'postgres', '', { PrintError => 0, RaiseError => 1 } )
    or die "$DBI::errstr";

  my $db = WO::DB->new( dbh => $dbh );

  my $dbh_pos = $db->dbh();
  my $sth_pos = $dbh_pos->prepare("PRAGMA cache_size = 200000") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA default_synchronous = OFF") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA synchronous = OFF") or die($dbh_pos->errstr());

  my $str = "";
  my $VAR1 = {};

  my $db_update = 0;

  #
  # Keyboard
  #
  my %areas = ();
  my $areas_counter = 1;

  my $config_text = "";
  open(FH,"</opt/pos/etc/config.ini");
  while (my $line = <FH>) {
    $config_text .= $line;
  }
  close FH;

  $config_text =~ s/(.*?)(\<PrintingZones.*)/$2/ims;
  for ( split(/\n/, $config_text) ) {
    $areas{$areas_counter++} = $1 if ( $_ =~ /Name\s+(.*)/ );
  }

  #$s->{manual} = 1;

  if ($s->manual() && $xml->{sync_families}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetFirstPOSFamilies', });
    };

    if ($@) {
      die("Error: $@");
    }

    $str = &my_decode($str);
    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $inserted = 0;
      my $updated = 0;

      $s->debug("RECEIVED ".scalar(@$VAR1)." GROUP RECORDS FROM SERVER...");

      my %fam_control = map { $_->{id} => 1 } $db->select("SELECT id, name FROM groups");

      $db->do(qq{ UPDATE groups SET deleted = 1 });

      my $ins_sth = $db->dbh->prepare(qq{ INSERT INTO groups (id, name, button_bg_color, parent, priority, print_priority, deleted) VALUES(?,?,?,?,?,?,0)});
      my $upd_sth = $db->dbh->prepare(qq{ UPDATE groups SET name = ?, button_bg_color = ?, parent = ?, priority = ?, print_priority = ?, deleted = 0 WHERE id = ?});

      foreach my $p (@$VAR1) {
        $p->{family_fo_id} ||= "";
        next unless $p->{family_fo_id};
        next unless $p->{family_fo_id} =~ /^\d+$/;

        $p->{fo_seetings}->{mart_cboth} ||= "0";

        if ($p->{fo_seetings}->{mfam_cboth}) {
          $p->{fo_seetings}->{mfam_cboth} = "&" . $p->{fo_seetings}->{mfam_cboth} . "&";
        }

        $db_update = 1;

        if (exists $fam_control{ $p->{family_fo_id} }) {
          $upd_sth->execute(
            $p->{fo_seetings}->{_nbot},
            $p->{fo_seetings}->{mfam_cboth},
            $p->{top_family_fo_id} || 0,
            10000 - $p->{vorder},
            $p->{mfam_print_priority} || 0,
            $p->{family_fo_id} || 0,
          );

          $updated++;
        } else {
          $ins_sth->execute(
            $p->{family_fo_id},
            $p->{fo_seetings}->{_nbot},
            $p->{fo_seetings}->{mfam_cboth},
            $p->{top_family_fo_id} || 0,
            10000 - $p->{vorder},
            $p->{mfam_print_priority} || 0,
          );
          $inserted++;
        }
      }

      $s->debug("Updated $updated groups...");
      $s->debug("Inserted $inserted groups...");

      #if (! $@) {
        my $rec = $db->select("SELECT MAX(id) AS id FROM groups");
        my $next_id = $rec->[0]->{id};
        if (defined $next_id) {
          $db->do("SELECT setval('groups_seq', ".$rec->[0]->{id}.")");
        }
      #}
      #else {
      #  print STDERR $@;
      #}

      # hide top group without departments
      my @parent_groups = $db->select("SELECT id, name FROM groups WHERE parent = 0");

      foreach my $g (@parent_groups) {
        my @depts = $db->select("SELECT id, name FROM groups WHERE parent = ? AND deleted = 0", $g->{id});

        if (! scalar @depts) {
          $db->do("UPDATE groups SET deleted = 1 WHERE id = ?", $g->{id});
        }
      }
    }
  }

  if ($s->manual() && $xml->{sync_articles}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetFirstPOSArticles', } );
    };

    if ($@) {
      die("Error: $@");
    }

    $str = &my_decode($str);
    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $inserted = 0;
      my $updated = 0;

      $s->debug("RECEIVED ".scalar(@$VAR1)." PRODUCT RECORDS FROM SERVER...");

      my $has_credit_field = 0;
      my %credit_products = ();

      #my $prods = $db->select("SELECT id, name, vat_isention, button_bg_color FROM products");
      my $prods = $db->select("SELECT * FROM products");
      my %prod_control = ();
      my %vat_isentions;

      foreach my $p (@$prods) {
        $prod_control{$p->{id}} = 1;
        $has_credit_field = 1 if exists $p->{credit};

        $p->{vat_isention} ||= "";
        $p->{button_bg_color} ||= "";

        if ($p->{vat_isention}) {
          $vat_isentions{$p->{id}} = $p->{vat_isention};
        }
      }

      $db->do(qq{ UPDATE products SET deleted = 1});

      $db->do(qq{DELETE FROM product_groups});

      my $insert_query = qq{
        INSERT INTO products 
        (id,name,button_text,sell_price1,sell_price2,sell_price3,sell_price4,sell_price5,sell_vat1,sell_vat2,is_weight,ask_price,barcode,printing_zone,tare,is_menu,is_extra,priority,print_priority, is_message, parent_product, vat_isention, time_config, cost_price, deleted, require_extras, qtty_factor,button_bg_color,sell_price_menu,image,require_extra_data,which_extras,handy_plu,product_group) 
        VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)
      };

      my $update_query = qq{
        UPDATE products
        SET name = ?, button_text = ?, sell_price1 = ?, sell_price2 = ?, sell_price3 = ?,
        sell_price4 = ?, sell_price5 = ?, sell_vat1 = ?, sell_vat2 = ?, 
        is_weight = ?, ask_price = ?, barcode = ?, printing_zone = ?, tare = ?, is_menu = ?, is_extra = ?,
        priority = ?, print_priority = ?, is_message = ?, parent_product = ?, vat_isention = ?, time_config = ?, cost_price = ?, 
        deleted = ?, require_extras = ?, qtty_factor = ?, button_bg_color = ?, sell_price_menu = ?, image = ?, require_extra_data = ?, which_extras = ?,
        handy_plu = ?, product_group = ?
        WHERE id = ?
      };

      my $ins_sth = $db->dbh->prepare($insert_query);

      my $upd_sth = $db->dbh->prepare($update_query);

      foreach my $p (@$VAR1) {
        $p->{article_fo_id} ||= "";
        next unless $p->{article_fo_id};
        next unless $p->{article_fo_id} =~ /^\d+$/;

        $db_update = 1;

        $p->{article_name} ||= "";
        $p->{article_price1} ||= 0;
        $p->{article_price2} ||= 0;
        $p->{article_price3} ||= 0;
        $p->{article_price4} ||= 0;
        $p->{article_price5} ||= 0;

        #if ($p->{article_id} == 171953) {
        #  print STDERR Dumper($p);
        #}

        $p->{article_tax0} ||= 0;
        $p->{article_tax1} ||= 0;

        $p->{fo_seetings}->{mart_credit} ||= 0;

        if ($has_credit_field && $p->{fo_seetings}->{mart_credit}) {
          $credit_products{ $p->{article_fo_id} } = 1;
        }

        $p->{fo_seetings}->{mart_cboth} ||= "0";

        if ($p->{fo_seetings}->{mart_cboth}) {
          $p->{fo_seetings}->{mart_cboth} = "&" . $p->{fo_seetings}->{mart_cboth}."&";
        }

        $p->{fo_seetings}->{_nbot} ||= "";
        $p->{fo_seetings}->{_ncur} ||= "";

        my $product_name = $p->{fo_seetings}->{_ncur} || $p->{article_name};
        my $button_text = $p->{fo_seetings}->{_nbot} || $p->{article_name};

        #if (exists $button_bg_colors{$p->{article_fo_id}}) {
        #  $p->{fo_seetings}->{_cbot} = $button_bg_colors{$p->{article_fo_id}};
        #}

        if (exists $vat_isentions{$p->{article_fo_id}}) {
          $p->{fo_seetings}->{mart_vat_isention} = $vat_isentions{$p->{article_fo_id}};
        }

        $product_name =~ s/\t//g;
        $button_text =~ s/\t//g;

        if (exists $prod_control{$p->{article_fo_id}}) {
          $upd_sth->execute(
            $product_name, 
            substr($button_text, 0, 50),
            $p->{article_price1}, 
            $p->{article_price2},
            $p->{article_price3}, 
            $p->{article_price4}, 
            $p->{article_price5}, 
            $p->{article_tax0}, 
            $p->{article_tax1},
            $p->{fo_seetings}->{mart_is_weight} || 0,
            $p->{fo_seetings}->{mart_ask_price} || 0,
            $p->{fo_seetings}->{mart_barcode} || 0,
            defined $areas{$p->{fo_seetings}->{mart_printing_zone}} ? $areas{$p->{fo_seetings}->{mart_printing_zone}} : '',
            $p->{fo_seetings}->{mart_tare} || 0,
            $p->{fo_seetings}->{mart_is_menu} || 0,
            $p->{fo_seetings}->{mart_is_extra} || 0,
            10000 - $p->{fo_seetings}->{_vord},
            $p->{fo_seetings}->{mart_print_priority} || 0,
            $p->{fo_seetings}->{mart_is_message} || 0,
            $p->{fo_seetings}->{mart_parent_product} || 0,
            $p->{fo_seetings}->{mart_vat_isention} || '',
            $p->{fo_seetings}->{mart_time_config} || 0,
            $p->{fo_seetings}->{mart_cost_price} || 0,
            0, 
            $p->{fo_seetings}->{mart_require_extras} || 0,
            $p->{fo_seetings}->{mart_qtty_factor} || 0,
            $p->{fo_seetings}->{mart_cboth},
            $p->{fo_seetings}->{mart_price_menu} || 0,
            $p->{image} || "",
            $p->{require_extra_data} || 0,
            $p->{which_extras} || "",
            substr($p->{handy_plu}, 0, 50) || 0,
            $p->{product_group} || "",
            $p->{article_fo_id}
          );

          $updated++;
        } else {
          $ins_sth->execute(
            $p->{article_fo_id},
            $product_name,
            substr($button_text, 0, 50),
            $p->{article_price1},
            $p->{article_price2},
            $p->{article_price3},
            $p->{article_price4},
            $p->{article_price5},
            $p->{article_tax0},
            $p->{article_tax1},
            $p->{fo_seetings}->{mart_is_weight} || 0,
            $p->{fo_seetings}->{mart_ask_price} || 0,
            $p->{fo_seetings}->{mart_barcode} || 0,
            defined $areas{$p->{fo_seetings}->{mart_printing_zone}} ? $areas{$p->{fo_seetings}->{mart_printing_zone}} : '',
            $p->{fo_seetings}->{mart_tare} || 0,
            $p->{fo_seetings}->{mart_is_menu} || 0,
            $p->{fo_seetings}->{mart_is_extra} || 0,
            10000 - $p->{fo_seetings}->{_vord},
            $p->{fo_seetings}->{mart_print_priority} || 0,
            $p->{fo_seetings}->{mart_is_message} || 0,
            $p->{fo_seetings}->{mart_parent_product} || 0,
            $p->{fo_seetings}->{mart_vat_isention} || 0,
            $p->{fo_seetings}->{mart_time_config} || 0,
            $p->{fo_seetings}->{mart_cost_price} || 0,
            0,
            $p->{fo_seetings}->{mart_require_extras} || 0,
            $p->{fo_seetings}->{mart_qtty_factor} || 0,
            $p->{fo_seetings}->{mart_cboth},
            $p->{fo_seetings}->{mart_price_menu} || 0,
            $p->{image} || "",
            $p->{require_extra_data} || 0,
            $p->{which_extras} || "",
            substr($p->{handy_plu}, 0, 50) || 0,
            $p->{product_group} || "",
          );
          $inserted++;
          $prod_control{$p->{article_fo_id}} = 1;
        }

        $db->do(qq{ INSERT INTO product_groups (product_id,group_id) VALUES(?,?)}, $p->{article_fo_id}, $p->{family_fo_id});
      }

      if (scalar %credit_products) {
        if (! $has_credit_field) {
          $s->debug("RECEIVED CREDIT PRODUCTS BUT credit FIELD DOESN'T EXISTS...");
        }
        else {
          foreach my $id (keys %credit_products) {
            $s->debug("CREDIT - $id");
            $db->do("UPDATE products SET credit = 1 WHERE id = ?", $id);
          }
        }
      }

      $s->debug("Updated $updated products...");
      $s->debug("Inserted $inserted products...");

      #if (! $@) {
        my $rec = $db->select("SELECT MAX(id) AS id FROM products");
        my $next_id = $rec->[0]->{id};
        if (defined $next_id) {
          $db->do("SELECT setval('products_seq', ".$rec->[0]->{id}.")");
        }
      #}
      #else {
      #  print STDERR $@;
      #}
    }
  }

  #print STDERR "VOU MORRER!!!!\n";
  #die "here";

  if ($s->manual() && $xml->{sync_menus}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetMenuArticles', } );
    };

    if ($@) {
      die("Error: $@");
    }

    $str = &my_decode($str);
    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      if (scalar @$VAR1) {
        $s->debug("RECEIVED ".scalar(@$VAR1)." MENU RECORDS FROM SERVER...");

        $db->do(qq{UPDATE products SET is_menu = 0});
        $db->do(qq{DELETE FROM product_menus});

        my $sth_ins = $db->dbh->prepare(qq{
          INSERT INTO product_menus (product_menu_id, product_id, step_number, extra_price) VALUES(?,?,?,?)
        });

        my $sth_upd = $db->dbh->prepare(qq{
          UPDATE products SET is_menu = 1 WHERE id = ?
        });

        foreach my $p (@$VAR1) {
          $p->{fo_id} ||= "";
          next unless $p->{fo_id};
          next unless $p->{fo_id} =~ /^\d+$/;

          $p->{composite_fo_id} ||= "";
          next unless $p->{composite_fo_id};
          next unless $p->{composite_fo_id} =~ /^\d+$/;

          $db_update = 1;
          $sth_ins->execute($p->{fo_id}, $p->{composite_fo_id}, $p->{option}-1, $p->{xtra_cost});
          $sth_upd->execute($p->{fo_id});
        } 
      }
    }
  }

  if ($s->manual() && $xml->{sync_clients}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetStoreClients', } );
    };

    if ($@) {
      die("Error: $@");
    }

    $str = &my_decode($str);
    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $inserted = 0;
      my $updated = 0;

      my %nifs_control = ();

      $s->debug("RECEIVED ".scalar(@$VAR1)." CUSTOMER RECORDS FROM SERVER...");

      my %cust_control = map { $_->{id} => 1 } $db->select("SELECT id, name FROM customers");
      #$db->do(qq{UPDATE customers SET deleted = 1});

      my $ins_sth = $db->dbh->prepare(qq{ 
        INSERT INTO customers
        (id,name,card_number,nif,discount,address,postal_code,city,deleted)
        VALUES(?,?,?,?,?,?,?,?,?)
      });
      my $upd_sth = $db->dbh->prepare(qq{ 
        UPDATE customers
        SET name = ?, card_number = ?, nif = ?, discount = ?,
        address = ?, postal_code = ?, city = ?, deleted = ?
        WHERE id = ?
      });

      foreach my $p (@$VAR1) {
        $p->{fo_id} ||= "";
        next unless $p->{fo_id};
        next unless $p->{fo_id} =~ /^\d+$/;

        $db_update = 1;

        my $nif = $p->{nif} || "";

        if ($nif) {
          if (exists $nifs_control{$nif} && $nifs_control{$nif} >= 1) {
            $nif .= "-".$nifs_control{$nif};
          }
        }

        $p->{fo_id} ||= "";
        next unless $p->{fo_id};
        next unless $p->{fo_id} =~ /^\d+$/;

        $p->{name} =~ s/\t//g;

        if (exists $cust_control{1*$p->{fo_id}}) {
          $upd_sth->execute(
            $p->{name},
            $p->{card_number},
            $nif,
            $p->{discount} || 0,
            $p->{address},
            $p->{postal_code},
            $p->{localidade},
            0,
            $p->{fo_id}
          );
        } else {
          $ins_sth->execute(
            $p->{fo_id},
            $p->{name},
            $p->{card_number},
            $nif,
            $p->{discount} || 0,
            $p->{address},
            $p->{postal_code},
            $p->{localidade},
            0
          );
        }

        if ($nif) {
          $nifs_control{$p->{nif}}++;
        }
      }

      #if (! $@) {
        my $rec = $db->select("SELECT MAX(id) AS id FROM customers");
        my $next_id = $rec->[0]->{id};
        if (defined $next_id) {
          $db->do("SELECT setval('customers_seq', ".$rec->[0]->{id}.")");
        }
      #}
      #else {
      #  print STDERR $@;
      #}
    }
  }

  if ($s->manual() && $xml->{sync_users}) {
    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetWaitresses', } );
    };

    if ($@) {
      die("Error: $@");
    }

    $str = &my_decode($str);
    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $inserted = 0;
      my $updated = 0;

      $s->debug("RECEIVED ".scalar(@$VAR1)." CLERK RECORDS FROM SERVER...");

      my %clerk_control = map { $_->{id} => 1 } $db->select("SELECT id FROM clerks");

      $db->do(qq{ UPDATE clerks SET deleted = 1 });
      eval { 
      my $ins_sth = $db->dbh->prepare(qq{ 
        INSERT INTO clerks
        (
         id,login,passwd,can_session_totals,can_daily_simple,can_daily_totals,can_do_returns,
         used_for_training,can_change_prices,can_use_discounts,can_open_drawer,deleted,default_area,
         can_close_day,can_transfer_products,can_void_products,can_edit_tables,cashier,can_do_open_discounts,
         can_transfer_tables,cant_use_others_tables,cant_close_checks,open_blocked_areas,see_historical_sales,skin,can_login,
         drawer,commission_group,tables,blocked_stations,consumption_discount,cant_print_bill_internal,cant_print_bill,cant_suspend,signon_key
        )
        VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,'PT',?,?,?,?,?,?,?,?,?,?)
      });
      my $upd_sth = $db->dbh->prepare(qq{ 
        UPDATE clerks
        SET 
        login = ?,passwd = ?,can_session_totals = ?,can_daily_simple = ?,can_daily_totals = ?,can_do_returns = ?,
        used_for_training = ?,can_change_prices = ?,can_use_discounts = ?,can_open_drawer = ?,deleted = ?,default_area = ?,
        can_close_day = ?,can_transfer_products = ?,can_void_products = ?,can_edit_tables = ?,cashier = ?,can_do_open_discounts = ?,
        can_transfer_tables = ?,cant_use_others_tables = ?,cant_close_checks = ?,open_blocked_areas = ?,see_historical_sales = ?,skin='PT',can_login=?,
        drawer = ?, commission_group = ?, tables = ?, blocked_stations = ?, consumption_discount = ?, cant_print_bill_internal = ?,
        cant_print_bill = ?, cant_suspend = ?, signon_key = ?
        WHERE id = ?
      });

      foreach my $p (@$VAR1) {
        $p->{fo_id} ||= "";
        $p->{real_fo_id} ||= "";
        $p->{fo_id} = $p->{real_fo_id} if $p->{real_fo_id};

        if (! $p->{fo_id}) {
          $s->debug("EMPTY fo_id, SKIPPING...");
          next;
        }

        if ($p->{fo_id} !~ /^\d+$/) {
          $s->debug("fo_id HAS AN INVALID VALUE - ".$p->{fo_id});
          next;
        }

        $db_update = 1;

=pod
        if ($p->{fo_access_type} eq '1' || $p->{fo_access_type} eq '6') {
          $p->{fo_settings}->{can_daily_simple} = 1;
          $p->{fo_settings}->{can_daily_totals} = 1;
          $p->{fo_settings}->{can_close_day} = 1;
        }

        if ($p->{fo_access_type} eq '6') {
          $p->{fo_settings}->{can_use_discounts} = 1;
          $p->{fo_settings}->{can_do_open_discounts} = 1;
        }

        if ($p->{fo_access_type} eq '3') {
          $p->{fo_settings}->{used_for_training} = 1;
        }
=cut

        $p->{fo_login} = substr($p->{fo_login}, 0, 19);
        $p->{fo_passwd} = substr($p->{fo_passwd}, 0, 19);

        $p->{fo_settings} ||= {};

        if (! exists $p->{fo_settings}->{can_login}) {
          $p->{fo_settings}->{can_login} = 1;
        }

        $p->{fo_login} =~ s/\t//g;
        $p->{fo_login} = $p->{login} unless $p->{fo_login};

        if (exists $clerk_control{1*$p->{fo_id}}) {
          $upd_sth->execute(
            $p->{fo_login},
            $p->{fo_passwd},
            $p->{fo_settings}->{can_session_totals} || 0,
            $p->{fo_settings}->{can_daily_simple} || 0,
            $p->{fo_settings}->{can_daily_totals} || 0,
            $p->{fo_settings}->{can_do_returns} || 0,
            $p->{fo_settings}->{used_for_training} || 0,
            $p->{fo_settings}->{can_change_prices} || 0,
            $p->{fo_settings}->{can_use_discounts} || 0,
            $p->{fo_settings}->{can_open_drawer} || 0,
            $p->{fo_settings}->{deleted} || 0,
            $p->{fo_settings}->{default_area} || 0,
            $p->{fo_settings}->{can_close_day} || 0,
            $p->{fo_settings}->{can_transfer_products} || 0,
            $p->{fo_settings}->{can_void_products} || 0,
            $p->{fo_settings}->{can_edit_tables} || 0,
            $p->{fo_settings}->{cashier} || 0,
            $p->{fo_settings}->{can_do_open_discounts} || 0,
            $p->{fo_settings}->{can_transfer_tables} || 0,
            $p->{fo_settings}->{cant_use_others_tables} || 0,
            $p->{fo_settings}->{cant_close_checks} || 0,
            $p->{fo_settings}->{open_blocked_areas} || 0,
            $p->{fo_settings}->{see_historical_sales} || 0,
            $p->{fo_settings}->{can_login} || 0,
            $p->{fo_settings}->{drawer} || "",
            $p->{fo_settings}->{commission_group} || "",
            $p->{fo_settings}->{tables} || "",
            $p->{fo_settings}->{blocked_stations},
            $p->{fo_settings}->{consumption_discount} || 0,
            $p->{fo_settings}->{cant_print_bill_internal} || 0,
            $p->{fo_settings}->{cant_print_bill} || 0,
            $p->{fo_settings}->{cant_suspend} || 0,
            $p->{fo_settings}->{signon_key} || '',
            $p->{fo_id}
          );
        } else {
          $ins_sth->execute(
            $p->{fo_id},
            $p->{fo_login},
            $p->{fo_passwd},
            $p->{fo_settings}->{can_session_totals} || 0,
            $p->{fo_settings}->{can_daily_simple} || 0,
            $p->{fo_settings}->{can_daily_totals} || 0,
            $p->{fo_settings}->{can_do_returns} || 0,
            $p->{fo_settings}->{used_for_training} || 0,
            $p->{fo_settings}->{can_change_prices} || 0,
            $p->{fo_settings}->{can_use_discounts} || 0,
            $p->{fo_settings}->{can_open_drawer} || 0,
            $p->{fo_settings}->{deleted} || 0,
            $p->{fo_settings}->{default_area} || 0,
            $p->{fo_settings}->{can_close_day} || 0,
            $p->{fo_settings}->{can_transfer_products} || 0,
            $p->{fo_settings}->{can_void_products} || 0,
            $p->{fo_settings}->{can_edit_tables} || 0,
            $p->{fo_settings}->{cashier} || 0,
            $p->{fo_settings}->{can_do_open_discounts} || 0,
            $p->{fo_settings}->{can_transfer_tables} || 0,
            $p->{fo_settings}->{cant_use_others_tables} || 0,
            $p->{fo_settings}->{cant_close_checks} || 0,
            $p->{fo_settings}->{open_blocked_areas} || 0,
            $p->{fo_settings}->{see_historical_sales} || 0,
            $p->{fo_settings}->{can_login} || 0,
            $p->{fo_settings}->{drawer} || "",
            $p->{fo_settings}->{commission_group} || "",
            $p->{fo_settings}->{tables} || "",
            $p->{fo_settings}->{blocked_stations},
            $p->{fo_settings}->{consumption_discount} || 0,
            $p->{fo_settings}->{cant_print_bill_internal} || 0,
            $p->{fo_settings}->{cant_print_bill} || 0,
            $p->{fo_settings}->{cant_suspend} || 0,
            $p->{fo_settings}->{signon_key} || "",
          );
        }
      }
      };

      if (! $@) {
        my $rec = $db->select("SELECT MAX(id) AS id FROM clerks");
        my $next_id = $rec->[0]->{id};
        if (defined $next_id) {
          $db->do("SELECT setval('clerks_seq', ".$rec->[0]->{id}.")");
        }
      }
      else {
        print STDERR $@;
      }
    }
  }

  $xml->{use_update_warning} ||= 0;
  $xml->{date_from} ||= "";

  if ($db_update == 1 && $xml->{use_update_warning}) {
    system("/opt/pos/common/addons/netbo/received_data.sh &");
  }

  #
  # Sales
  #

  $xml->{sync_sells} ||= 0;

  if ($xml->{sync_sells}) {
    $s->sync_movements();
  }

  $s->debug("Done.");
}

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # frontoffice directory
  my $dir = $xml->{dir} || "";
  my $keep_files = $xml->{keep_files} || "0";

  my $dsep = $s->directory_separator();
  my $dsn = 'dbi:PgPP:dbname=pos;host=localhost';
  my $dbh = DBI->connect( $dsn, 'postgres', '', { PrintError => 0, RaiseError => 1 } )
    or die "$DBI::errstr";

  my $db = WO::DB->new( dbh => $dbh );

  my $dbh_pos = $db->dbh();
  my $sth_pos = $dbh_pos->prepare("PRAGMA cache_size = 200000") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA default_synchronous = OFF") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA synchronous = OFF") or die($dbh_pos->errstr());

  my $zip = Archive::Zip->new();
  my $zipFile = $dir.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  my @to_remove = ();

  if ($xml->{sync_families}) {
    my $csv = Text::CSV_XS->new({ binary => 1 }) or die "Cannot use CSV: ".Text::CSV->error_diag ();
    my $file_families  = $s->tmp_dir()."/families.csv";

    unlink $file_families if -e $file_families;

    my @to_remove = ();

    my $sth = $dbh->prepare(qq{
      SELECT id, name, button_color, parent, priority, print_priority, deleted
      FROM groups
    });

    $sth->execute();

    open(F, ">> $file_families") or die "can't write to $file_families - $!";

    my $status = $csv->combine(qw(id name button_color parent priority print_priority deleted));

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();
 
    $zip->addFile( $file_families );
  }

  if ($xml->{sync_articles}) {
    my $csv = Text::CSV_XS->new({ binary => 1 }) or die "Cannot use CSV: ".Text::CSV->error_diag ();
    my $file_articles  = $s->tmp_dir()."/articles.csv";

    unlink $file_articles if -e $file_articles;

    my @to_remove = ();

    my $sth = $dbh->prepare(qq{
      SELECT id,name,button_text,sell_price1,sell_price2,sell_price3,sell_price4,sell_price5,sell_vat1,sell_vat2,is_weight,ask_price,barcode,printing_zone,tare,is_menu,is_extra,priority,print_priority, is_message, parent_product, vat_isention, time_config, cost_price, deleted, require_extras, qtty_factor,button_color,sell_price_menu,image,require_extra_data,which_extras,handy_plu,product_group
      FROM products
    });

    $sth->execute();

    open(F, ">> $file_articles") or die "can't write to $file_articles - $!";

    my $status = $csv->combine(qw(id name button_text sell_price1 sell_price2 sell_price3 sell_price4 sell_price5 sell_vat1 sell_vat2 is_weight ask_price barcode printing_zone tare is_menu is_extra priority print_priority  is_message  parent_product  vat_isention  time_config  cost_price  deleted  require_extras  qtty_factor button_color sell_price_menu image require_extra_data which_extras handy_plu product_group));

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    $zip->addFile( $file_articles );
  }

  if ($xml->{sync_users}) {
    my $csv = Text::CSV_XS->new({ binary => 1 }) or die "Cannot use CSV: ".Text::CSV->error_diag ();
    my $file_clerks = $s->tmp_dir()."/clerks.csv";

    unlink $file_clerks if -e $file_clerks;

    my @to_remove = ();

    my $sth = $dbh->prepare(qq{
      SELECT id,login,passwd,can_session_totals,can_daily_simple,can_daily_totals,can_do_returns,
      used_for_training,can_change_prices,can_use_discounts,can_open_drawer,deleted,default_area,
      can_close_day,can_transfer_products,can_void_products,can_edit_tables,cashier,can_do_open_discounts,
      can_transfer_tables,cant_use_others_tables,cant_close_checks,open_blocked_areas,see_historical_sales,skin,can_login,
      drawer,commission_group,tables,blocked_stations,consumption_discount,cant_print_bill_internal,cant_print_bill,cant_suspend
      FROM clerks
    });

    $sth->execute();

    open(F, ">> $file_clerks") or die "can't write to $file_clerks - $!";

    my $status = $csv->combine(qw(id login passwd can_session_totals can_daily_simple can_daily_totals can_do_returns used_for_training can_change_prices can_use_discounts can_open_drawer deleted default_area can_close_day can_transfer_products can_void_products can_edit_tables cashier can_do_open_discounts can_transfer_tables cant_use_others_tables cant_close_checks open_blocked_areas see_historical_sales skin can_login drawer commission_group tables blocked_stations consumption_discount cant_print_bill_internal cant_print_bill cant_suspend));

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    $zip->addFile( $file_clerks );
  }

  if ($xml->{sync_customers}) {
    my $csv = Text::CSV_XS->new({ binary => 1 }) or die "Cannot use CSV: ".Text::CSV->error_diag ();
    my $file_customers  = $s->tmp_dir()."/customers.csv";

    unlink $file_customers if -e $file_customers;

    my @to_remove = ();

    my $sth = $dbh->prepare(qq{
      SELECT id,name,card_number,nif,discount,address,postal_code,city,deleted
      FROM products
    });

    $sth->execute();

    open(F, ">> $file_customers") or die "can't write to $file_customers - $!";

    my $status = $csv->combine(qw(id name card_number nif discount address postal_code city deleted));

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    $zip->addFile( $file_customers );
  }

  die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

  eval {
    $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                file_upload => $zipFile,
                              });
  };

  $s->own_remove($zipFile) if -e $zipFile;

  if ($@) {
    die "Error sending file: $@";
  } else {
    if (! $keep_files) {
      for (@to_remove) {
        unlink $_;
      }
    }
  }

  #
  # Sales
  #

  $s->sync_movements();

  $s->debug("Done.");
}

#
# Sync movements
#

sub sync_movements {
  my $s = shift;
  my $remove = shift;
  my $date_from = shift;

  $s->debug("sync_movements()");

  my $VAR1;
  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  # frontoffice directory
  my $dir = $xml->{dir} || "";
  my $process_current_day = $xml->{process_current_day} || "";

  my $csv = Text::CSV_XS->new({ binary => 1 }) or
    die "Cannot use CSV: ".Text::CSV->error_diag ();

  # directory separator
  my $dsep = $s->directory_separator();
  my $dsn = 'dbi:PgPP:dbname=pos;host=localhost';
  my $dbh = DBI->connect( $dsn, 'postgres', '', { PrintError => 0, RaiseError => 1 } )
    or die "$DBI::errstr";

  my $db = WO::DB->new( dbh => $dbh );

  my $dbh_pos = $db->dbh();
  my $sth_pos = $dbh_pos->prepare("PRAGMA cache_size = 200000") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA default_synchronous = OFF") or die($dbh_pos->errstr());
  $sth_pos = $dbh_pos->prepare("PRAGMA synchronous = OFF") or die($dbh_pos->errstr());

  my $str = "";
  my $VAR1 = {};

  my $db_update = 0;

  my $rand = strftime("%Y%m%d_%H%M%S", localtime(time)).".".$$;

  my $file_headers  = "/root/headers.csv.".$rand;
  my $file_details  = "/root/details.csv.".$rand;
  my $file_payments = "/root/payments.csv.".$rand;

  unlink $file_headers if -e $file_headers;
  unlink $file_details if -e $file_details;
  unlink $file_payments if -e $file_payments;

  my @to_remove = ();

  my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration',
                                                 }
  );

  eval "$company_config";
  my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $company_config = $s->company_configuration();

  my $act = 'FrontofficeExistentSellDatesFast';
  my $existent_dates = $s->weckoffice_send_data( { action => $act, } );
  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $sth;
  my $business_date = '';
  my $did_data = 0;

  if (open(F, "< /opt/pos/.business_date")) {
    local $/ = undef;
    $business_date = <F>;
    chop($business_date);
    close(F);
  }

  if (! $business_date) {
    die "Can't find business date in /opt/pos/.business_date...";
  }

  # include current day
  delete $existent_dates{$business_date};

  my @files_to_remove = ();

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $dir.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  if (-d "/opt/pos/server/data/export") {
    $did_data = 1;

    my @files = read_dir("/opt/pos/server/data/export");

    for (@files) {
      $s->debug("Will add: $_");
      my( $date_tmp ) = $_ =~ /^(.*?)#/;
      delete $existent_dates{$date_tmp};

      $zip->addFile( "/opt/pos/server/data/export/".$_ );
      push @files_to_remove, "/opt/pos/server/data/export/".$_;
    }
  }

  if (-d "/opt/pos/server/data/export") {
    $did_data = 1;

    my @files = read_dir("/opt/pos/server/data/export");

    for (@files) {
      $s->debug("Will add: $_");
      my( $date_tmp ) = $_ =~ /^(.*?)#/;
      delete $existent_dates{$date_tmp};

      $zip->addFile( "/opt/pos/server/data/export/".$_ );
      push @files_to_remove, "/opt/pos/server/data/export/".$_;
    }
  }

  my $last_id_processed = 0;

  # if files were added to this array, don't do this verification
  #if (! scalar @files_to_remove) {
  #  $last_id_processed = _check_last_id_processed($db);
  #}

  my @doc_types_sales = ( 1, 5, 90, 12, 7, 2, 13 );
  my $where_dt = "AND doc_type IN (".join(",", @doc_types_sales).")";

  my $date_from_w1 = $s->get_date_from();
  my $date_from_w2 = $s->get_date_from();

  $s->debug("DATE FROM : ".$s->get_date_from());

  my $date_from_w1 = " AND business_date >= '".$s->get_date_from()."'";
  my $date_from_w2 = " AND sh.business_date >= '".$s->get_date_from()."'";

  $sth = $dbh->prepare(qq{
    SELECT business_date, SUM(value)
    FROM sales_headers
    WHERE 1=1$date_from_w1 $where_dt
    GROUP BY business_date
    ORDER BY business_date
  });

  $sth->execute();

  my %ddates;

  while (my $r = $sth->fetchrow_arrayref) {
    $ddates{$r->[0]} = $r->[1];
  }

  foreach my $date (sort keys %ddates) {
    if (! $process_current_day) {
      if ($date eq $business_date) {
        $s->debug("DATE $date IS TODAY, IGNORING...");
        next;
      }
    }

    $s->debug("DATE TO ANALYZE: $date, NETBO: ".int($existent_dates{$date}*1).", POS: ".int($ddates{$date}*1));

    if (exists $existent_dates{$date}) {
      # old method
      if ($existent_dates{$date} == 1) {
        if ($company_config->{import_sells_date_dupplicate} eq 'ignore' && exists $existent_dates{$date}) {
          print STDERR "######OLD> WILL IGNORE DATE - $date\n";
          next;
        }
      }
      # new method
      else {
          #$company_config->{import_sells_date_dupplicate} eq 'ignore' &&
        if (
          exists $existent_dates{$date} &&
          int($existent_dates{$date}*1) == int($ddates{$date}*1)
        ) {
          print STDERR "######NEW> WILL IGNORE DATE - $date\n";
          next;
        }
      }
    }

    $s->debug("WILL PROCESS DATE: $date");

    $did_data = 1;

    #
    # Headers
    #

    $sth = $dbh->prepare(qq{
      SELECT *
      FROM sales_headers
      WHERE business_date = ?$date_from_w1
    });

    $sth->execute($date);

    open(F, ">> $file_headers") or die "can't write to $file_headers - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    #
    # Details
    #

    $sth = $dbh->prepare(qq{
      SELECT sp.*
      FROM sales_details sp
      LEFT JOIN sales_headers sh ON (sp.header = sh.id)
      WHERE sh.business_date = ?$date_from_w2
    });

    $sth->execute($date);

    open(F, ">> $file_details") or die "can't write to $file_details - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    #
    # Payments
    #

    $sth = $dbh->prepare(qq{
      SELECT sp.*
      FROM sales_payments sp
      LEFT JOIN sales_headers sh ON (sp.header = sh.id)
      WHERE sh.business_date = ?$date_from_w2
    });

    $sth->execute($date);

    open(F, ">> $file_payments") or die "can't write to $file_payments - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();
  }

  if ($did_data) {
    if (-e $file_headers) {
      $zip->addFile( $file_headers );
      push @files_to_remove, $file_headers;
    }

    if (-e $file_details) {
      $zip->addFile( $file_details );
      push @files_to_remove, $file_details;
    }

    if (-e $file_payments) {
      $zip->addFile( $file_payments );
      push @files_to_remove, $file_payments;
    }

    my $prefix_doc_type = $s->vfh( $xml, 'prefix_doc_type' );
    my $prefix_op_type = $s->vfh( $xml, 'prefix_op_type' );

    if (defined $prefix_doc_type || defined $prefix_op_type) {
      my $misc_prefixes_file = $s->tmp_dir() . $dsep . "misc_prefixes.txt";

      open(F, "> $misc_prefixes_file") or die "Can't write to '$misc_prefixes_file': $!\n";

      print F "prefix_doc_type=$prefix_doc_type\n" if defined $prefix_doc_type;
      print F "prefix_op_type=$prefix_op_type\n" if defined $prefix_op_type;

      close(F);

      $zip->addFile( $misc_prefixes_file );
    }

    die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                });
    };

    $s->own_remove($zipFile) if -e $zipFile;
    #system("rm -r $n_tmp_dir");

    if ($@) {
      die "Error sending file: $@";
    }
    else {
      if ($last_id_processed) {
        if (open(F, "> /tmp/last_sold_id.txt")) {
          print F "$last_id_processed\n";
          close(F);
        }
      }

      for (@files_to_remove) {
        unlink $_;
      }
    }
  }
}

sub common_sync_fo_and_sync_bk {
  my $s = shift;
  my( $xml, $zip, $to_remove ) = @_;

  $s->debug("common_sync_fo_and_sync_bk()");

  my $dsep = $s->directory_separator();

  my $consumption_price_number = $s->vfh( $xml, 'consumption_price_number' );

  if (defined $consumption_price_number) {
    my $consumption_price_number_file = $s->tmp_dir() . $dsep . "consumption_price_number.txt";

    $s->debug("####> $consumption_price_number_file - $consumption_price_number");

    open(F, "> $consumption_price_number_file") or die "Can't write to '$consumption_price_number_file': $!\n";
    print F "$consumption_price_number\n";
    close(F);

    $zip->addFile( $consumption_price_number_file );

    push @$to_remove, $consumption_price_number_file;
  }

  #
  # use sale locals to map things
  #

  $xml->{map_sale_terminals} ||= undef;

  if (defined $xml->{map_sale_terminals}) {
    my $file_map_sale_terminals = $s->tmp_dir() . $dsep . "map_sale_terminals.txt";

    my @mappings = ();

    if (exists $xml->{map_sale_terminals}->{map}) {
      if (ref $xml->{map_sale_terminals}->{map} eq 'HASH') {
        push @mappings, $xml->{map_sale_terminals}->{map};
      } elsif (ref $xml->{map_sale_terminals}->{map} eq 'ARRAY') {
        @mappings = @{ $xml->{map_sale_terminals}->{map} };
      }
    }

    if (scalar @mappings) {
      open(F, "> $file_map_sale_terminals") or die "Can't write to '$file_map_sale_terminals': $!\n";
      print F Dumper(\@mappings);
      close(F);
      $zip->addFile( $file_map_sale_terminals );
      push @$to_remove, $file_map_sale_terminals;
    }
  }
}

sub my_decode {
  my $w = shift;

  eval {
    my $octets = decode("iso-8859-1",$w);
    $w = encode("utf8",$octets);
  };

  if ($@) {
    print STDERR "ENCODING ERROR: #$w#\n";
  }

  return $w;
}

sub _check_last_id_processed {
  my $db = shift;

  # read last sales id from hdd
  my $last_id = "";

  {
    local $/ = undef;
    if (open(F, "< /tmp/last_sold_id.txt")) {
      $last_id = <F>;
      $last_id =~ s/\n//g;
      close(F);
    }
  }

  my $max_id = $db->select(qq{ SELECT MAX(id) AS id FROM sales_headers })->[0]->{id};
  $max_id ||= 0;

  #print STDERR "##### max_id: $max_id, last_id: $last_id\n";

  if ($last_id && $max_id && $last_id eq $max_id) {
    print STDERR "LAST SOLD ID IS THE SAME AS THE LAST PROCESSED ID, ABORTING...\n";
    print STDERR "REMOVE /tmp/last_sold_id.txt TO SKIP THIS VERIFICATION...\n";
    exit;
  }

  return $max_id;
}

1;

