package WO::Client::Integrators::PingWinBO;

use strict;
no warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use WO::DB::ODBC;
use Text::CSV_XS;
use FindBin qw( $Bin );
use File::Slurp qw( read_file );
use File::Basename qw( fileparse fileparse_set_fstype );

&fileparse_set_fstype('MSWin32');

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  my $sql_server_merc_db = $xml->{sql_server_merc_db};
  my $sql_server_merc_db_prefix = $xml->{sql_server_merc_db_prefix};

  my $days = $xml->{back_days} || 19;

  my $day2sec = 24*60*60;
  my $date_from = strftime("%Y-%m-%d", localtime(time()-$days*($day2sec)));
  print STDERR "### $date_from\n";

  my $check_dates_fast = $xml->{check_dates_fast} || 0;

  my $file_doctypes         = $tmp_dir.'/__doctypes__.txt';
  my $file_pmethods         = $tmp_dir.'/__pmethods__.txt';

  my $file_stores           = $tmp_dir.'/__stores__.txt';
  my $file_sale_families    = $tmp_dir.'/__sale_families__.txt';
  my $file_sale_articles    = $tmp_dir.'/__sale_articles__.txt';
  my $file_locals           = $tmp_dir.'/__fo_locals__.txt';
  my $file_users            = $tmp_dir.'/__fo_users__.txt';
  my $file_clients          = $tmp_dir.'/__clients__.txt';
  my $file_store_clients    = $tmp_dir.'/__store_clients__.txt';
  my $file_units            = $tmp_dir.'/__units__.txt';

  my $file_headers          = $tmp_dir.'/__headers__.txt';
  my $file_details          = $tmp_dir.'/__details__.txt';
  my $file_footers          = $tmp_dir.'/__footers__.txt';

  &File::Remove::remove($file_doctypes)       if -e $file_doctypes;
  &File::Remove::remove($file_pmethods)       if -e $file_pmethods;
  &File::Remove::remove($file_stores)         if -e $file_stores;
  &File::Remove::remove($file_sale_families)  if -e $file_sale_families;
  &File::Remove::remove($file_sale_articles)  if -e $file_sale_articles;
  &File::Remove::remove($file_locals)         if -e $file_locals;
  &File::Remove::remove($file_users)          if -e $file_users;
  &File::Remove::remove($file_clients)        if -e $file_clients;
  &File::Remove::remove($file_store_clients)  if -e $file_store_clients;
  &File::Remove::remove($file_units)          if -e $file_units;
  &File::Remove::remove($file_headers)        if -e $file_headers;
  &File::Remove::remove($file_details)        if -e $file_details;
  &File::Remove::remove($file_footers)        if -e $file_footers;

  my $csv = Text::CSV_XS->new({ binary => 1 });

  my @zip_files;

  if ($xml->{sync_sales}) {
    my $dsn = "dbi:ODBC:".$xml->{DSN};
    my $dbh = DBI->connect($dsn, 'SYSDBA', 'masterkey' ) or die("Can't connect to database: $@");
    my $db = WO::DB::ODBC->new( dbh => $dbh );

    my $VAR1 = {};

    my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration' } );
    eval "$company_config";
    my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

    my $sth = undef;

    # GET DOCTYPES
    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.CODE, a.DESCRIPTION, a.STOCK_SIGNAL
      FROM DOCCONFIG a
      WHERE a.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_doctypes") or die "Can't write to $file_doctypes: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_doctypes;

    # GET PMETHODS
    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.PARENT_ID, a.CODE, a.DESCRIPTION
      FROM PAYMETHOD a
      WHERE a.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_pmethods") or die "Can't write to $file_pmethods: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_pmethods;

    # GET STORES
    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.CODE, a.DESCRIPTION
      FROM STORE a
      WHERE a.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_stores") or die "Can't write to $file_stores: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_stores;

    # GET FAMILIES
    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.PARENT_ID, a.CODE, a.DESCRIPTION
      FROM FAMILY a
      WHERE a.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_sale_families") or die "Can't write to $file_sale_families: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_sale_families;

    # GET LOCALS

    $sth = $db->dbh()->prepare(qq{
      SELECT a.STORE_ID, a.LOCAL_ID, l.DESCRIPTION
      FROM STORE_LOCAL a
      LEFT JOIN LOCAL l ON (a.LOCAL_ID=l.ID)
      WHERE a.DELETED = 0 AND l.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_locals") or die "Can't write to $file_locals: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_locals;

    # GET ARTICLES

    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.FAMILY_ID, a.CODE, a.DESCRIPTION
      FROM PRODUCT a
      WHERE a.FORSALE = 1 AND a.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_sale_articles") or die "Can't write to $file_sale_articles: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_sale_articles;

    #GET USERS

    $sth = $db->dbh()->prepare(qq{
      SELECT a.ENTITY_ID, a.INTERNAL_NAME, a.IDENTIFICATION, a.PASSWD, a.LOGIN_NAME
      FROM EMPLOYEE a
      WHERE a.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_users") or die "Can't write to $file_users: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_users;

    #GET CLIENTS

    $sth = $db->dbh()->prepare(qq{
      SELECT a.ENTITY_ID, e.DESCRIPTION, e.FISCALNAME
      FROM CLIENT a
      LEFT JOIN ENTITY e ON (a.ENTITY_ID = e.ID)
      WHERE a.DELETED = 0 AND e.DELETED = 0
    });
    $sth->execute();

    open(F, "> $file_clients") or die "Can't write to $file_clients: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_clients;

    # GET SALE HEADERS

    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.STORE_ID, a.DOCCONFIG_ID, b.PREFIX, a.DOC_NUMBER, 
      a.ENTITY_ID, a.ENTITY_NAME, a.ENTITY_TAXNUM,
      a.EMPLOYEE_ID, a.CLOCK_DATE, a.DOC_DATE, a.TOTAL * -1 * d.STOCK_SIGNAL, a.TAXINCLUDED, a.DOC_PREFIX, a.LOCAL_ID
      FROM DOCHEADER a
      LEFT JOIN DOCCONFIG d ON (a.DOCCONFIG_ID = d.ID)
      LEFT JOIN DOCSERIES b ON (a.DOCSERIES_ID = b.ID)
      LEFT JOIN DOCSTATUS c ON (a.DOCSTATUS_ID = c.ID)
      WHERE a.DELETED = 0 AND c.DELETED = 0 AND c.ISCLOSED = 1 AND c.ISCANCELED = 0
      AND a.DOC_DATE >= ?
    });
    $sth->execute($date_from);

    open(F, "> $file_headers") or die "Can't write to $file_headers: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_headers;

    # GET SALE FOOTERS

    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.DOCHEADER_ID, a.PAYMETHOD_ID, a.PAYVALUE * -1 * d.STOCK_SIGNAL, a.DISCOUNTVALUE
      FROM DOCFOOTER a
      LEFT JOIN DOCHEADER b ON (a.DOCHEADER_ID = b.ID)
      LEFT JOIN DOCCONFIG d ON (b.DOCCONFIG_ID = d.ID)
      WHERE a.DELETED = 0 AND a.DOCHEADER_ID IN (
        SELECT a.ID
        FROM DOCHEADER a
        LEFT JOIN DOCSTATUS c ON (a.DOCSTATUS_ID = c.ID)
        WHERE a.DELETED = 0 AND c.DELETED = 0 AND c.ISCLOSED = 1 AND c.ISCANCELED = 0 AND a.DOC_DATE >= ?
      ) ORDER BY a.DOCHEADER_ID
    });
    $sth->execute($date_from);

    open(F, "> $file_footers") or die "Can't write to $file_footers: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_footers;

    # GET SALE DETAILS

    $sth = $db->dbh()->prepare(qq{
      SELECT a.ID, a.DOCHEADER_ID, a.LINE_NUMBER, a.PRODUCT_ID, a.PRODUCT_CODE, a.DESCRIPTION, 
      a.QNT * -1 * CASE WHEN a.STK_SIGNAL = 0 THEN -1 ELSE a.STK_SIGNAL END, a.UNIT_ID, a.PRICE, 
      a.DISCOUNT_VALUE, a.TOTAL, a.GROSS_TOTAL, a.EMPLOYEE_ID, b.TAXRATE
      FROM DOCMOVDETAIL a
      LEFT JOIN DOCMOVDETAILTAX b ON (a.ID = b.DOCMOVDETAIL_ID) 
      WHERE a.DELETED = 0 AND a.DOCHEADER_ID IN (
        SELECT a.ID
        FROM DOCHEADER a
        LEFT JOIN DOCSTATUS c ON (a.DOCSTATUS_ID = c.ID)
        WHERE a.DELETED = 0 AND c.DELETED = 0 AND c.ISCLOSED = 1 AND c.ISCANCELED = 0 AND a.DOC_DATE >= ?
      ) ORDER BY a.DOCHEADER_ID
    });
    $sth->execute($date_from);

    open(F, "> $file_details") or die "Can't write to $file_details: $@";

    while( my $row = $sth->fetchrow_arrayref() ) {
      my $status = $csv->combine(@$row);
      my $line = $csv->string();

      print F $line."\n";
    }

    close(F);

    push @zip_files, $file_details;

    $dbh->disconnect;
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $s->tmp_dir().$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData', file_upload => $zipFile, });
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      for (@zip_files) {
        $s->debug("Will delete file '$_'");
        $s->own_remove($_);
      }
    }
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  $s->sync_fo_information();
}

sub write_to_file {
  my( $fname, $cnt ) = @_;

  open(N, ">$fname") or die $!;
  print N $cnt."\r\n";
  close(N);
}

1;

