package WO::Client::Integrators::Primavera;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use DBI;
use WO::DB::ODBC;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use POSIX qw( strftime );
use Encode qw( encode decode );
use File::Basename qw( fileparse fileparse_set_fstype );
use File::Remove qw( remove );

&fileparse_set_fstype('MSWin32');

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;
  $s->debug("sync_fo_information()");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $csv = Text::CSV_XS->new({ binary => 1 });
  
  my $xml = $s->info();

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();
  
  my $sales_group_by = $s->vfh($xml, 'sales_group_by');
  my $integration_software = $s->vfh($xml, 'integration_software');
  my $integration_db = $s->vfh($xml, 'integration_db');
  my $integration_login = $s->vfh($xml, 'integration_login');
  my $integration_pw = $s->vfh($xml, 'integration_pw');

  my $integrate_days_sales = $s->vfh($xml, 'integrate_days_sales');
  my $integrate_days_purchases = $s->vfh($xml, 'integrate_days_purchases');

  my $sync_sales = $s->vfh($xml, 'sync_sales');
  my $sync_purchases = $s->vfh($xml, 'sync_purchases');

  my $odbc_name = $s->vfh($xml, 'odbc_name');
  my $odbc_login = $s->vfh($xml, 'odbc_login');
  my $odbc_passwd = $s->vfh($xml, 'odbc_passwd');

  $s->abort("Missing 'integration_software' parameter...") unless defined $integration_software;
  $s->abort("Missing 'integration_db' parameter...") unless defined $integration_db;
  $s->abort("Missing 'integration_login' parameter...") unless defined $integration_login;
  $s->abort("Missing 'integration_pw' parameter...") unless defined $integration_pw;
  $s->abort("Missing 'odbc_name' parameter...") unless defined $odbc_name;
  $s->abort("Missing 'odbc_login' parameter...") unless defined $odbc_login;
  $s->abort("Missing 'odbc_passwd' parameter...") unless defined $odbc_passwd;
  $s->abort("Can't find integration software in $integration_software") unless -e $integration_software;

  my $dbh = DBI->connect('dbi:ODBC:'.$xml->{odbc_name}, $xml->{odbc_login}, $xml->{odbc_passwd})
    or die("Can't connect to the ODBC! source name: ".$xml->{odbc_name}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

  my $db = WO::DB::ODBC->new( dbh => $dbh );

  my( $file_name, $dir, $file_suffix ) = &fileparse($integration_software, ());
  $s->debug( "$integration_software => $file_name, $dir, $file_suffix\n" );
  $file_name =~ s/\.exe$/.txt/i;

  my $integration_log = $dir.$dsep.$file_name;
####  my $integration_log = $dir.$file_name;

  if (-e $integration_log) {
    #system("mkdir -p \"$dir.$dsep"."log_history\"");
    system("move \"$integration_log\" \"$dir.$dsep"."log_history$dsep$file_name.".time.".txt\"");
  }

  $s->_write_to_log($integration_log, &strftime("%Y/%m/%d %H:%M:%S", localtime(time)));
  $s->_write_to_log($integration_log, "\n\n");

  my %doc_mappings = ();
  my $dmap = $xml->{doc_mappings};

  foreach my $k (keys %$dmap) {
    my $i = $dmap->{$k};

    if (ref($i) eq 'HASH') {
      $i->{serie} ||= "";

      if (exists $i->{when_document} && $i->{doc_type}) {
        $doc_mappings{$k}->{ $i->{when_document} } = { doc_type => $i->{doc_type}, serie => $i->{serie} };
      }
      elsif (exists $i->{doc_type}) {
        $doc_mappings{$k}->{"default"} = { doc_type => $i->{doc_type}, serie => $i->{serie} };
      }
      else {
        $s->debug("WARNING, error in mapping (doc_type required)");
      }
    }
    elsif (ref($i) eq 'ARRAY') {
      foreach my $mapping (@$i) {
        $mapping->{serie} ||= "";

        if (exists $mapping->{when_document} && $mapping->{when_document} && exists $mapping->{doc_type} && $mapping->{doc_type}) {
          $doc_mappings{$k}->{ $mapping->{when_document} } = { doc_type => $mapping->{doc_type}, serie => $mapping->{serie} };
        }
        elsif (exists $mapping->{doc_type} && $mapping->{doc_type}) {
          $doc_mappings{$k}->{"default"} = { doc_type => $mapping->{doc_type}, serie => $mapping->{serie} };
        }
        else {
          $s->debug("WARNING, error in mapping (doc_type required)");
        }
      }
    }
  }

  my %vat_mappings = ();
  my $vmap = $xml->{vat_mappings};

  foreach my $k (keys %$vmap) {
    my $i = $vmap->{$k};

    if (ref($i) eq 'HASH') {
      if (exists $i->{when_value} && $i->{code}) {
        $vat_mappings{$k}->{ $i->{when_value} } = { code => $i->{code} };
      }
      elsif (exists $i->{when_value}) {
        $vat_mappings{$k}->{"default"} = { code => $i->{code} };
      }
      else {
        $s->debug("WARNING, error in mapping (code required)");
      }
    }
    elsif (ref($i) eq 'ARRAY') {
      foreach my $mapping (@$i) {
        if (exists $mapping->{when_value} && exists $mapping->{code} && $mapping->{code}) {
          $vat_mappings{$k}->{ $mapping->{when_value} } = { code => $mapping->{code} };
        }
        elsif (exists $mapping->{code} && $mapping->{code}) {
          $vat_mappings{$k}->{"default"} = { code => $mapping->{code} };
        }
        else {
          $s->debug("WARNING, error in mapping (code required)");
        }
      }
    }
  }

  my $vats = $db->select("SELECT Iva FROM Iva ORDER BY Iva");
  my $warehouses = $db->select("SELECT Armazem FROM Armazens ORDER BY Armazem");

  my $VAR1;
  my $str;
  
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPrimavera_GetProducts', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $out_file = $dir.$dsep."products.csv";

    open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

    my $status = $csv->combine('code', 'name', 'warehouse', 'vat');
    print F $csv->string()."\n";
    
    foreach my $a (@$VAR1) {
      $a->{name} =~ s/,/./g;
      my $status = $csv->combine( "W".$a->{fo_id}, &_encode_str($a->{name}), $warehouses->[0]->{Armazem}, $vats->[0]->{Iva} );
      no warnings;
      print F $csv->string()."\n";
    }

    close(F);
  }

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPrimavera_GetSuppliers', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $out_file = $dir.$dsep."suppliers.csv";

    open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

    my $status = $csv->combine('code', 'name', 'fiscal_number');
    print F $csv->string()."\n";
    print F "WF999999,Fornecedor Generico,999999999\n";

    foreach my $a (@$VAR1) {
      $a->{ncontrib} =~ s/\s//g;

      if ($a->{ncontrib} !~ /^\d+$/) {
        $s->debug("FORNECEDOR ".$a->{fo_id}." / ".$a->{name}." SEM NUM CONTRIB OU INVALIDO: ".$a->{ncontrib});
        $s->_write_to_log($integration_log, "FORNECEDOR ".$a->{fo_id}." / ".$a->{name}." SEM NUM CONTRIB OU INVALIDO: ".$a->{ncontrib});
        next;
      }

      $a->{name} =~ s/,/./g;
      my $status = $csv->combine( "WF".$a->{fo_id}, &_encode_str($a->{name}), $a->{ncontrib} );
      print F $csv->string()."\n";
    }

    close(F);
  }

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPrimavera_GetCustomers', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $out_file = $dir.$dsep."customers.csv";

    open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

    my $status = $csv->combine('code', 'name', 'fiscal_number');
    print F $csv->string()."\n";
    print F "WC999999,Cliente Generico,999999999\n";

    foreach my $a (@$VAR1) {
      $a->{ncontrib} =~ s/\s//g;

      if ($a->{ncontrib} !~ /^\d+$/) {
        $s->debug("CLIENTE ".$a->{fo_id}." / ".$a->{name}." SEM NUM CONTRIB OU INVALIDO: ".$a->{ncontrib});
        $s->_write_to_log($integration_log, "FORNECEDOR ".$a->{fo_id}." / ".$a->{name}." SEM NUM CONTRIB OU INVALIDO: ".$a->{ncontrib});
        next;
      }

      $a->{name} =~ s/,/./g;
      my $status = $csv->combine( "WC".$a->{fo_id}, &_encode_str($a->{name}), $a->{ncontrib} );
      print F $csv->string()."\n";
    }

    close(F);
  }

  #my $cache = $s->_read_cache($dir, 'documents');

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPrimavera_GetDocuments', integrate_days_purchases => $integrate_days_purchases } );
  };
  
  $s->abort($@) if $@;
  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'HASH') {
    my $out_file_headers = $dir.$dsep."document_headers.csv";
    my $out_file_details = $dir.$dsep."document_details.csv";

    my $headers = {};

    open(H, "> $out_file_headers") or $s->abort("Can't write to $out_file_headers: $!");
    open(D, "> $out_file_details") or $s->abort("Can't write to $out_file_details: $!");

    my $status = $csv->combine('doc_id', 'date', 'doc_type', 'supplier_docref', 'supplier', 'serie', 'discount', 'foreign_id', 'payment_date');
    print H $csv->string()."\n";
    $status = $csv->combine('doc_id', 'product', 'store', 'price', 'quantity', 'discount', 'vat_rate', 'pcm', 'extra_cost', 'vat_code');
    print D $csv->string()."\n";

    my %md5s = ();
    my %to_skip = ();

=pod
    # Headers
    foreach my $h (@{ $VAR1->{headers} }) {
      $md5s{ $h->{internal_doc_code} } .= $s->_hash_to_md5( $h );
    }

    # Details
    foreach my $d (@{ $VAR1->{details} }) {
      $md5s{ $d->{internal_doc_code} } .= $s->_hash_to_md5( $d );
    }
=cut

    # Headers
    foreach my $h (@{ $VAR1->{headers} }) {
      $headers->{ $h->{internal_doc_code} } = $h;

      my $doc_type = $h->{doc_type};
      my $date = $h->{date};
      my $sup_doc_ref = exists $h->{supplier_docref} ? $h->{supplier_docref} : $h->{internal_doc_code};
      my $sup_code = exists $h->{supplier_code} ? $h->{supplier_code} : "999999";
      
      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'documents', $doc_type, $date );
      next if $primavera_doc_type eq '#IGNORE#';

      # add record to cache
      #$cache->{$h->{internal_doc_code}} = $md5s{ $h->{internal_doc_code} };

      $h->{discount} ||= 0;
      $h->{doc_fo_id} ||= "";

#9,5

      #my $prim_doc = $db->select("SELECT NumDoc, Filial FROM CabecCompras WHERE Observacoes LIKE ?", $primavera_doc_type."#".$primavera_serie."#".$h->{internal_doc_code});
      my $prim_doc = $db->select("SELECT NumDoc, Filial FROM CabecCompras WHERE Observacoes LIKE ?", $h->{doc_fo_id}."#".$primavera_serie);
      my $prim_id = scalar @$prim_doc ? $prim_doc->[0]->{NumDoc}."#".$prim_doc->[0]->{Filial} : "";




      my( $y, $m, $d ) = $date =~ /^(\d+)\-(\d+)\-(\d+)$/;
      my( $y2, $m2, $d2 ) = ( $y, $m, $d );

      if (exists $h->{date_of_payment} && $h->{date_of_payment}) {
        ( $y2, $m2, $d2 ) = $h->{date_of_payment} =~ /^(\d+)\-(\d+)\-(\d+)$/;
      }

      my $status = $csv->combine( $h->{internal_doc_code}, "$d-$m-$y", $primavera_doc_type, $sup_doc_ref, "WF".$sup_code, $primavera_serie, sprintf("%.6f", $h->{discount}), $prim_id, "$d2-$m2-$y2", $h->{doc_fo_id} );
      print H $csv->string()."\n";
    }

    # Details
    foreach my $d (@{ $VAR1->{details} }) {
      #next if exists $to_skip{ $d->{internal_doc_code} };

      my $headers = $headers->{$d->{internal_doc_code}};

      my $doc_type = $headers->{doc_type};
      my $date = $headers->{date};
      
      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'documents', $doc_type, $date );      
      next if $primavera_doc_type eq '#IGNORE#';

      my $xtra_cost = exists $d->{xtra_cost} ? $d->{xtra_cost} : 0;

      my $vat = $d->{tax}*1;
      my $vat_prim_code = exists $vat_mappings{vat}->{$vat} ? $vat_mappings{vat}->{$vat}->{code} : 999999;

      #if (! exists $vat_mappings{vat}->{$vat}) {
      #  print STDERR "########## #$vat#\n";
      #}

      my $discount = 0;

      if ($d->{discount} > 0) {
        $discount = ($d->{discount} / $d->{buy_price}) * 100;
      }
     
      my $status = $csv->combine(
        $d->{internal_doc_code}, "W".$d->{article_code}, $headers->{store_code}, sprintf("%.6f", $d->{buy_price}),
        sprintf("%.4f", $d->{quantity}), sprintf("%.6f", $discount), sprintf("%.4f", $d->{tax}),
        sprintf("%.6f", $d->{pcm}), sprintf("%.6f", $xtra_cost), $vat_prim_code, 
      );
      
      print D $csv->string()."\n";
    }
    
    close(H);
    close(D);
  }

  #$s->_save_cache($dir, 'documents', $cache);
           
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsPrimavera_GetSales', integrate_days_sales => $integrate_days_sales } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my %sales = ();
    my %series = ();
    my %sales_by_cust = ();
    my %sales_by_cust2 = ();
    
    # Sales collect
    for (@$VAR1) {
      my $fam_code = 'NETBO';
      my $tax_value = $_->{tax_value}*1;

      my $orig_fam_code = $fam_code;

      my $date = $_->{date};
      my $movement_type = $_->{movement_type};
      my $store = $_->{store};
      my $client_fo_id = $_->{client_fo_id};
      my $value = $_->{sold_v};
      my $qtty  = $_->{sold_q};

      $sales_by_cust{ $client_fo_id }->{$movement_type} += $value;

      if (defined $_->{subfamily_code}) {
        $fam_code = $_->{subfamily_code};
      }
      elsif (defined $_->{family_code}) {
        $fam_code = $_->{family_code};
      }
      else {
        if (defined $sales_group_by) {
          if ($sales_group_by eq 'top_families') {
            $fam_code = $_->{'top_family_name'};
          }
          elsif ($sales_group_by eq 'sub_families') {
            $fam_code = $_->{'family_name'};
          }
        }
      }

#      if ($_->{consumpt_q} > 0) {
#      #  $value = $_->{consumpt_v};
#      #  $qtty  = $_->{consumpt_q};
#        $movement_type = 'POS_AUTO_CONSUMPTION';
#      }

#      if ($_->{waste_q} > 0) {
#      #  $value = $_->{waste_v};
#      #  $qtty  = $_->{waste_q};
#        $movement_type = 'POS_VOID_WITH_WASTE';
#      }

      $fam_code = $orig_fam_code unless defined $fam_code;
      my $primavera_product = $fam_code."_".$tax_value;
      my $pcm = $qtty*$_->{pcm};

      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'sales', $movement_type, $date );
      my $kk = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client_fo_id;
      $series{ $kk } = $primavera_serie;

#      print STDERR "###### $movement_type - $primavera_doc_type (".$value.") - consumpt_q: ".$_->{consumpt_q}." - waste_q: ".$_->{waste_q}."\n";

      #$sales_by_cust2{ $client_fo_id }->{$primavera_doc_type} += $value;
      $sales_by_cust2{ $client_fo_id }->{$movement_type} += $value;
      
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{sold_v} += sprintf("%.6f", $value);
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{sold_q} += sprintf("%.6f", $qtty);
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{vat} = $tax_value*1;
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{vat_v} += sprintf("%.6f", $_->{vat_v});
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{cost_v} += sprintf("%.6f", $pcm);
    }


    $s->dump(\%sales_by_cust);
    $s->dump(\%sales_by_cust2);

    my $out_file_sale_headers = $dir.$dsep."sale_headers.csv";
    my $out_file_sale_details = $dir.$dsep."sale_details.csv";
    
    open(H, "> $out_file_sale_headers") or $s->abort("Can't write to $out_file_sale_headers $!");
    open(D, "> $out_file_sale_details") or $s->abort("Can't write to $out_file_sale_details $!");

    my $status = $csv->combine('doc_id', 'date', 'doc_type', 'customer_code', 'serie', 'foreign_id');
    print H $csv->string()."\n";
    $status = $csv->combine('doc_id', 'product', 'store', 'sold_v', 'sold_q', 'vat_rate', 'cost_v', 'vat_code');
    print D $csv->string()."\n";
    
    my %done_headers = ();
    my %done_headers_tmp = ();
    my %md5s = ();
    my %to_skip = ();

=pod
    $cache = $s->_read_cache($dir, 'sales');

    # Sales process
    foreach my $store (sort keys %sales) {
      foreach my $date (sort keys %{ $sales{ $store } }) {
        foreach my $mov_type (sort keys %{ $sales{ $store }->{$date} }) {
          foreach my $client (sort keys %{ $sales{ $store }->{$date}->{$mov_type} }) {            
            my $primavera_doc_type = $mov_type;
            my $kk = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client;
            my $primavera_serie = $series{ $kk };
            next if $primavera_doc_type eq '#IGNORE#';

            my $key = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client . "#" . $primavera_serie;

            # Headers
            if (! exists $done_headers_tmp{ $key }) {
              $md5s{ $key } .= &md5_hex($key . $date . $primavera_doc_type . $client . $primavera_serie);
              $done_headers_tmp{$key} = 1;
            }

            # Details
            foreach my $product (sort keys %{ $sales{ $store }->{$date}->{$mov_type}->{$client} }) {
              my $i = $sales{ $store }->{$date}->{$mov_type}->{$client}->{$product};
              $md5s{ $key } .= $s->_hash_to_md5( $i );
            }
          }
        }
      }
    }
=cut

    %sales_by_cust = ();

    # Sales process
    foreach my $store (sort keys %sales) {
      foreach my $date (sort keys %{ $sales{ $store } }) {
        foreach my $mov_type (keys %{ $sales{ $store }->{$date} }) {
          foreach my $client (keys %{ $sales{ $store }->{$date}->{$mov_type} }) {            
            my $primavera_doc_type = $mov_type;
            my $kk = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client;
            my $primavera_serie = $series{ $kk };

            foreach my $product (keys %{ $sales{ $store }->{$date}->{$mov_type}->{$client} }) {
              my $i = $sales{ $store }->{$date}->{$mov_type}->{$client}->{$product};
              $sales_by_cust{ $client }->{$mov_type} += $i->{sold_v};
            }

            next if $primavera_doc_type eq '#IGNORE#';

            my $key = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client . "#" . $primavera_serie;

            # Headers
            if (! exists $done_headers{ $key }) {

=pod
              # cache
              if (exists $cache->{ $key } && $md5s{ $key } eq $cache->{ $key }) {
                $s->debug("Document ".$key." exists in cache and didn't change, skipping...");
                $to_skip{ $key } = 1;
                next;
              }
=cut

              # add record to cache
              #$cache->{$key} = $md5s{ $key };

              my $prim_doc = $db->select("SELECT NumDoc, Filial FROM CabecDoc WHERE DocsOriginais LIKE '$key'");
              my $prim_id = scalar @$prim_doc ? $prim_doc->[0]->{NumDoc}."#".$prim_doc->[0]->{Filial} : "";

              my( $y, $m, $d ) = $date =~ /^(\d+)\-(\d+)\-(\d+)$/;

              my $status = $csv->combine(
                $key, "$d-$m-$y", $primavera_doc_type, "WC".$client, $primavera_serie, $prim_id
              );
        
              print H $csv->string()."\n";

              $done_headers{$key} = 1;
            }

            #next if exists $to_skip{ $key };

            # Details
            foreach my $product (keys %{ $sales{ $store }->{$date}->{$mov_type}->{$client} }) {
              my $i = $sales{ $store }->{$date}->{$mov_type}->{$client}->{$product};
              #$sales_by_cust{ $client }->{$mov_type} += $i->{sold_v};

              my $sold_q = $i->{sold_q} ? $i->{sold_q} : 1;

              my $vat = $i->{vat}*1;
              my $vat_prim_code = exists $vat_mappings{vat}->{$vat} ? $vat_mappings{vat}->{$vat}->{code} : 999999;

              open(PROD, ">> ".$dir.$dsep."products.csv");
              my $status = $csv->combine( &_encode_str("W".$product), &_encode_str("W".$product), $warehouses->[0]->{Armazem}, $vats->[0]->{Iva} );
              print PROD $csv->string()."\n";
              close(PROD);

              my $value = $i->{sold_q} > 0 ? $i->{sold_v}/$i->{sold_q} : $i->{sold_v};

              $status = $csv->combine(
                $key, &_encode_str("W".$product), $store, sprintf("%.6f", $value),
                sprintf("%.4f", $i->{sold_q}), sprintf("%.4f", $i->{vat}),
                sprintf("%.6f", $i->{cost_v}), $vat_prim_code
              );
        
              print D $csv->string()."\n";
            }
          }
        }
      }
    }  

    $s->dump(\%sales_by_cust);

    #$s->_save_cache($dir, 'sales', $cache);
    
    close(D);
    close(H);
  }

  # Show integration log
  if (-e $integration_log) {
    my $cmd = "\"$integration_software\" \"$integration_db\" \"$integration_login\" \"$integration_pw\"";

    $sync_sales = "" unless defined $sync_sales;
    $sync_purchases = "" unless defined $sync_purchases;

    if ($sync_sales && ! $sync_purchases) {
      $cmd .= " vendas";
    }
    elsif (! $sync_sales && $sync_purchases) {
      $cmd .= " compras";
    }

    $s->debug("Will execute: $cmd");

    system($cmd);
    my $alert_prog = $integration_software.".vbs";
    $s->_create_vbs_script($alert_prog);
    system("\"$alert_prog\" \"$integration_log\"");
  }
}

#
# Internal functions
#

sub _doc_type_and_series {
  my $self = shift;
  my $doc_mappings = shift;
  my $key = shift;
  my $doc_type = shift;
  my $date = shift;
  
  # doc type
  my $primavera_doc_type = "";

  if (! exists $doc_mappings->{$key}->{$doc_type} && ! exists $doc_mappings->{$key}->{"default"}) {
    $self->abort("Can't used movement_type $doc_type because we don't have a doc_mapping for it (or a default)");
  }
  elsif (! exists $doc_mappings->{$key}->{$doc_type} && exists $doc_mappings->{$key}->{"default"}) {
    $primavera_doc_type = $doc_mappings->{$key}->{"default"}->{doc_type};
  }
  elsif (exists $doc_mappings->{$key}->{$doc_type}) {
    $primavera_doc_type = $doc_mappings->{$key}->{$doc_type}->{doc_type};
  }

  # serie
  my $primavera_serie = "";

  if (exists $doc_mappings->{$key}->{$doc_type} && $doc_mappings->{$key}->{$doc_type}->{serie}) {
    $primavera_serie = $doc_mappings->{$key}->{$doc_type}->{serie};
  }
  else {
    ( $primavera_serie ) = $date =~ /^(\d\d\d\d)/;
  }

  return ( $primavera_doc_type, $primavera_serie );
}

sub _sqlserver_id {
  my $s = shift;
  my $db = shift;

  my $id = $db->select(qq{
    DECLARE \@NI UNIQUEIDENTIFIER
    SET \@NI=NEWID()
    SELECT \@NI AS id
  });

  return $id->[0]->{id};
}

sub _get_serie_number {
  my $s = shift;
  my( $db, $series_table, $primavera_doc_type, $primavera_serie ) = @_;

  my $series_info_tmp = $db->select(qq{
    SELECT Ordem, Numerador
    FROM $series_table
    WHERE TipoDoc = ? AND Serie = ?
  }, $primavera_doc_type, $primavera_serie);

  if (! scalar @$series_info_tmp) {
    $s->abort("Serie $primavera_serie doesn't exists for document type $primavera_doc_type, please create it...");
  }

  my $serie_info = $series_info_tmp->[0];

  my $numerador_old = $serie_info->{Numerador};
  $numerador_old = 0 unless defined $numerador_old;
  my $numerador = $numerador_old+1;

  $db->do(qq{
    UPDATE $series_table SET Numerador = ? WHERE TipoDoc = ? AND Serie = ?
  }, $numerador, $primavera_doc_type, $primavera_serie);

  return ( $numerador, $serie_info->{Ordem} );
}

sub _database_insert {
  my $s = shift;
  my %a = ( db => undef, table => undef, row => {}, @_ );

  $s->debug("Will insert record in table $a{table}");

  my $query = "INSERT INTO ".$a{table}." (";

  my @values = ();

  foreach my $key (keys %{ $a{row} }) {
    push @values, $a{row}->{$key};
    $query .= "$key, ";
  }

  my $qm = "?, " x scalar(@values);

  $query =~ s/, $/) VALUES ($qm/;
  $query =~ s/, $/)/;

  $a{db}->do($query, @values);
}

sub _read_cache {
  my $s = shift;
  my $d = shift;
  my $w = shift;

  my $dsep = $s->directory_separator();

  my $file = $d.$dsep.$w.".cache";
  my $cache = {};

  if (open(F, "< ".$file)) {
    while (<F>) {
      chop;
      my( $k, $v ) = $_ =~ /^(.*?)=(.*)$/;
      $cache->{$k} = $v;
    }

    close(F);
  }

  return $cache;
}

sub _save_cache {
  my $s = shift;
  my $d = shift;
  my $w = shift;
  my $i = shift;

  my $dsep = $s->directory_separator();

  my $file = $d.$dsep.$w.".cache";

  if (open(F, "> ".$file)) {
    foreach my $k (keys %$i) {
      print F $k."=".$i->{$k}."\n";
    }

    close(F);
  }

  return 1;
}

sub _hash_to_md5 {
  my $s = shift;
  my $h = shift;

  my $str = "";

  foreach my $k (sort keys %$h) {
    my $v = exists $h->{$k} && defined $h->{$k} ? $h->{$k} : "";
    $str .= $v;
  }

  return md5_hex($str);
}

sub _write_to_log {
  my $self = shift;
  my( $file, $msg ) = @_;

  open(F, ">> $file") or die "Can't write to $file: $!";
  print F $msg."\n";
  close(F);
}

sub _encode_str {
  my $w = shift;

  my $str = encode("iso-8859-1", $w);

  #print STDERR "#### #$str#\n";
  return $w;
  #return encode("iso-8859-1", decode("utf8", $w));
  #return encode("latin1",decode("utf8", $w));
  #return encode("utf8", $w);

  #return $w;

  #my $tt = encode("iso-8859-15", $w);
  #print STDERR "### #$w# #$tt#\n";

  #return $tt;

  #return encode("iso-8859-15", $w);
}

sub _create_vbs_script {
  my $self = shift;
  my $script = shift;

  if (! -e $script) {
    open(F, "> $script") or $self->abort("Can't write to $script: $!");

    print F "Sub ShowFile(ByVal sFile)\n".
            "Dim shell\n".
            "Set shell = CreateObject(\"WScript.Shell\")\n".
            "shell.Run Chr(34) & sFile & Chr(34), 1, false\n".
            "Set shell = Nothing\n".
            "End Sub\n".
            "ShowFile WScript.Arguments(0)\n";

    #print F 'strComputer = msgbox(WScript.Arguments(0))'."\n";
    close(F);
  }
}

1;

