package WO::Client::Integrators::RMDA;

use strict;
use warnings;

use base qw ( WO::Client::Common );

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;
  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  $s->debug("Encoded FO type is: $fo_type");

  my $csv = Text::CSV_XS->new({ binary => 1, sep_char => ',', always_quote => 1, quote_char => '"' });

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  my $dir = $s->vfh($xml, 'dir');

  if (! defined $dir) {
    $s->abort("Missing the 'dir' parameter...");
  }

  my $str;
  my $VAR1;

  $s->debug("Will call BackofficeGetWarehouses");

  eval {
    $str = $s->weckoffice_send_data( { action => 'BackofficeGetWarehouses' } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    open(U, "> $dir/".$xml->{weckoffice}->{company}."_Warehouses.txt") or die "Error writing to file: $!";

    for (sort { $a->{name} cmp $b->{name} } @$VAR1) {
      $_->{name} =~ s/,/ /g;
      my @cols = ( $_->{id}, $_->{fo_id}, $_->{name} );
      my $status = $csv->combine(@cols);
      my $line = $csv->string();
      print U $line."\n";
    }

    close(U);
  }

  $s->debug("Will call BackofficeGetUnits");

  eval {
    $str = $s->weckoffice_send_data( { action => 'BackofficeGetUnits' } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    open(U, "> $dir/".$xml->{weckoffice}->{company}."_Units.txt") or die "Error writing to file: $!";

    for (sort { $a->{abrev} cmp $b->{abrev} } @$VAR1) {
      $_->{name} =~ s/,/ /g;
      my @cols = ( $_->{id}, $_->{name}, $_->{abrev}, $_->{factor}*1 );
      my $status = $csv->combine(@cols);
      my $line = $csv->string();
      print U $line."\n";
    }

    close(U);
  }

  $s->debug("Will call BackofficeGetTemplates");

  eval {
    $str = $s->weckoffice_send_data( { action => 'BackofficeGetTemplates' } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'HASH') {
    open(U, "> $dir/".$xml->{weckoffice}->{company}."_Templates.txt") or die "Error writing to file: $!";

    foreach my $template_id (sort { $VAR1->{$a}->{name} cmp $VAR1->{$b}->{name} } keys %$VAR1) {
      my @cols = ( $VAR1->{$template_id}->{name} );

      foreach my $a (@{ $VAR1->{$template_id}->{articles} }) {
        push @cols, $a->{id};
      }

      my $status = $csv->combine(@cols);
      my $line = $csv->string();
      print U $line."\n";
    }

    close(U);
  }

  $s->debug("Will call BackofficeGetRmdaArticles");

  eval {
    $str = $s->weckoffice_send_data( { action => 'BackofficeGetRmdaArticles' } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

=pod
          {
            'subfamily_name' => '----------',
            'subfamily_code' => '----------',
            'is_merc' => 0,
            'family_code' => '----------',
            'family_name' => '----------',
            'sale_unit_factor' => undef,
            'last_buy' => undef,
            'affect_stock' => 1,
            'id' => 32797,
            'is_menu' => 0,
            'stocktake_unit' => undef,
            'is_sale' => 0,
            'date_updated' => '2009-10-28 17:29:34.268014',
            'name' => 'TESTE LOTES (G)',
            'barcode' => undef,
            'barcode2' => undef,
            'fo_id' => undef,
            'sale_unit' => undef,
            'default_unit' => undef,
            'default_unit_factor' => undef,
            'stocktake_unit_factor' => undef,
            'expires' => 0,
            'is_prod' => 0
          }
=cut

#"1230016","Bife Pojador 170 Cong. Goedegebbur (pg)","UNI"

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    open(F, "> $dir/".$xml->{weckoffice}->{company}."_Products.txt") or die "Error writing to file: $!";

    my %unit_control = ();

    my @units = ();

    for (sort { $a->{fo_id} cmp $b->{fo_id} } @$VAR1) {
      next unless defined $_->{fo_id};

      my $unit = "";

      if (defined $_->{stocktake_unit}) {
        $unit = $_->{stocktake_unit_abrev};
      } elsif (defined $_->{default_unit}) {
        $unit = $_->{default_unit_abrev};
      } else {
        $unit = "Un";
      }

      if ($unit && ! exists $unit_control{ $unit }) {
        $unit_control{ $unit } = 1;
        push @units, $unit;
      }

      my $barcode1 = $_->{barcode};
      my $barcode2 = $_->{barcode2};

      $barcode1 =~ s/\s+//g;
      $barcode2 =~ s/\s+//g;

      my @cols = ();

      $_->{name} =~ s/\n//g;
      $_->{name} =~ s/\r//g;
      $_->{name} =~ s/,/ /g;

      $barcode1 = "" unless defined $barcode1;
      $barcode2 = "" unless defined $barcode2;
      $barcode1 = "" if $barcode1 eq '0';
      $barcode2 = "" if $barcode2 eq '0';

      if ($barcode1) {
        push @cols, ( $barcode1, $_->{name}, $unit, $_->{fo_id}, $_->{id} );
        my $status = $csv->combine(@cols);
        my $line = $csv->string();
        print F $line."\n";
      }

      if ($barcode2) {
        push @cols, ( $barcode2, $_->{name}, $unit, $_->{fo_id}, $_->{id} );
        my $status = $csv->combine(@cols);
        my $line = $csv->string();
        print F $line."\n";
      }

      #my $fo_id = defined $_->{fo_id} ? $_->{fo_id} : "";
    }

    close(F);
  }

  $s->debug("Will call BackofficeGetStocksByWarehouse");

  eval {
    $str = $s->weckoffice_send_data( { action => 'BackofficeGetStocksByWarehouse' } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'HASH') {
    open(U, "> $dir/".$xml->{weckoffice}->{company}."_StkTeo.txt") or die "Error writing to file: $!";

    foreach my $store_id (sort keys %$VAR1) {
      foreach my $a (sort { $a->{article_id} <=> $b->{article_id} } @{$VAR1->{$store_id}->{articles}}) {
        my @cols = ( $a->{article_id}, $store_id, $a->{curr_stock} );
        my $status = $csv->combine(@cols);
        my $line = $csv->string();
        print U $line."\n";
      }
    }

    close(U);
  }

  $s->debug("Done");

}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;
  $s->debug("sync_bk_information()");
  $s->debug("Done.");
}


1;

