package WO::Client::Integrators::SageNEXT;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use DBI;
use WO::DB::ODBC;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use POSIX qw( strftime );
use Encode qw( encode decode );
use File::Basename qw( fileparse fileparse_set_fstype );
use File::Remove qw( remove );

&fileparse_set_fstype('MSWin32');

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;
  $s->debug("sync_fo_information()");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");
  
  my $xml = $s->info();

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();
  
  my $sales_group_by = $s->vfh($xml, 'sales_group_by');
  my $integration_software = $s->vfh($xml, 'integration_software');
  

  $s->abort("Missing 'integration_software' parameter...") unless defined $integration_software;
  $s->abort("Can't find integration software in $integration_software") unless -e $integration_software;
  
  my( $file_name, $dir, $file_suffix ) = &fileparse($integration_software, ());
  $file_name =~ s/\.exe$/.txt/i;
  
  # read configuration file
  
  open(F, "< ".$dir.$dsep."siNextAPI.ini") or die "Can't open configuration file: ".$dir.$dsep."siNextAPI.ini - $!";
  my @config = ();
  
  while (<F>) {
    chop;
    push @config, $_;
  }
  
  close(F);
  
  my %supplier_nifs = ();
  my %customer_nifs = ();
  
  my $REGIVA = 'NAC';
  
  my $sage_serie  = $config[10];
  my $sage_sector = $config[11];
  my $separator = $config[4] || ";";

  my $file_products  = $config[5];
  my $file_entities  = $config[6];
  my $file_document_headers = $config[8];
  my $file_document_details = $config[9];

  $s->abort("Missing file for products in siNextAPI.ini") unless defined $file_products;
  $s->abort("Missing file for entities in siNextAPI.ini") unless defined $file_entities;
  $s->abort("Missing file for document headers in siNextAPI.ini") unless defined $file_document_headers;
  $s->abort("Missing file for document details in siNextAPI.ini") unless defined $file_document_details;
  
  my $csv = Text::CSV_XS->new({ binary => 1, sep_char => $separator });

  my $alert_prog = $dir."alert.vbs";

  if (! -e $alert_prog) {
    open(F, "> $alert_prog") or $s->abort("Can't write to $alert_prog: $!");
    print F 'strComputer = msgbox("Artigos sem descritor: " + WScript.Arguments(0))'."\n";
    close(F);
  }
  
  my $VAR1;
  my $str;
  
  #UNIDADES - CODIGO E DESCR
  
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetProducts', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  $s->debug("Will get products...");
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $out_file = $dir.$dsep.$file_products;

    open(F, "> $out_file") or $s->abort("Can't write to $out_file: $!");

    # TODO: o IVA tem de vir do campo auxiliar 2 dos artigos -  o regime de iva
    my $status = $csv->combine(
      'CODART', 'DESCR', 'GRUPO', 'FAMILIA', 'SUBFAM', 'PVPSI1', 'PVPCI1', 'PVPSI2',
      'PVPCI2', 'PVPSI3', 'PVPCI3', 'IVA', 'QTCX', 'INACTIVO',
    );
    
    print F $csv->string()."\n";
    
    foreach my $a (@$VAR1) {
      $a->{name} =~ s/,/./g;
      
      my $status = $csv->combine(
        $a->{fo_id}, &_encode_str($a->{name}), '00000', $a->{family_name}, $a->{subfamily_name}, 0, 0, 0, 0, 0, 0, $a->{integration1}, 0, 0,
      );
      
      print F $csv->string()."\n";
    }

    close(F);
  }
  
  $s->debug("Will get suppliers...");
  
  my $done_suppliers = 0;

  open(F, "> ".$dir.$dsep.$file_entities) or die "Can't write to ".$dir.$dsep.$file_entities.": $!";
  close(F);

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetSuppliers', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
    
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $out_file = $dir.$dsep.$file_entities;

    open(F, ">> $out_file") or $s->abort("Can't write to $out_file: $!");

    my $status = $csv->combine(
      'TPTERC', 'TERCEIRO', 'VENDEDOR', 'NOME', 'NOME1', 'CONTACTO', 'MORADA1', 'MORADA2',
      'CODPOSTAL', 'MORADA3', 'NIF', 'TEL', 'FAX', 'TELEMOVEL', 'OBS', 'LINPRECO',
      'REGIVA', 'DIASPAG', 'TIPOCCREDITO', 'PLAFOND', 'NRDIASCC', 'MOEDA', 'ZONA', 'PAIS',
      'INACTIVO',
    );
    
    print F $csv->string()."\n";
    
    foreach my $a (@$VAR1) {
      $done_suppliers = 1;
      
      $a->{name} =~ s/,/./g;
      
      $supplier_nifs{ $a->{fo_id} } = $a->{ncontrib};
      
      my $status = $csv->combine(
        2, $a->{fo_id}, '000', $a->{name}, '', $a->{telephone}, $a->{address}, '', '', $a->{localidade}, $a->{ncontrib}, '',
        $a->{fax}, $a->{cellular}, $a->{obs}, 1, $REGIVA, 0, 2, 0, 0, 'EUR', '000', '000', 0,
      );
      
      print F $csv->string()."\n";
    }

    close(F);
  }
  
  $s->debug("Will get customers...");

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetCustomers', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
    
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $out_file = $dir.$dsep.$file_entities;

    open(F, ">> $out_file") or $s->abort("Can't write to $out_file: $!");
    
    if (! $done_suppliers) {
      my $status = $csv->combine(
        'TPTERC', 'TERCEIRO', 'VENDEDOR', 'NOME', 'NOME1', 'CONTACTO', 'MORADA1', 'MORADA2',
        'CODPOSTAL', 'MORADA3', 'NIF', 'TEL', 'FAX', 'TELEMOVEL', 'OBS', 'LINPRECO',
        'REGIVA', 'DIASPAG', 'TIPOCCREDITO', 'PLAFOND', 'NRDIASCC', 'MOEDA', 'ZONA', 'PAIS',
        'INACTIVO',
      );
      
      print F $csv->string()."\n";
    }

    foreach my $a (@$VAR1) {
      $a->{name} =~ s/,/./g;
      
      $customer_nifs{ $a->{fo_id} } = $a->{ncontrib};
      
      my $status = $csv->combine(
        1, $a->{fo_id}, '000', $a->{name}, '', $a->{telephone}, $a->{address}, '', $a->{postal_code},
        $a->{localidade}, $a->{ncontrib}, '', $a->{fax}, $a->{cellular}, $a->{obs}, 1, $REGIVA,
        0, 2, 0, 0, 'EUR', '000', '000', 0,
      );
      
      print F $csv->string()."\n";
    }

    close(F);
  }
  
  # add the generic customer to the list of customers
  my $out_file = $dir.$dsep.$file_entities;
  open(F, ">> $out_file") or $s->abort("Can't write to $out_file: $!");
  my $status = $csv->combine(
    1, 999999, '', 'Cliente Generico', '', '', '', '', '', '', 123123123, '', '', '', '', 1,
    $REGIVA, 0, 2, 0, 0, 'EUR', '000', '000', 0,
  );
  print F $csv->string()."\n";
  close(F);
  
  $s->debug("Will get purchases...");

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetDocuments', } );
  };
  
  $s->abort($@) if $@;
  
  eval $str;
  
  my @product_errors = ();
  
  if (defined $VAR1 && ref($VAR1) eq 'HASH') {    
    my $headers_f = $dir.$dsep.$file_document_headers;
    my $details_f = $dir.$dsep.$file_document_details;
    
    my $headers = {};
    
    $VAR1->{headers} ||= [];
    $VAR1->{details} ||= [];
    
    open(F, "> $headers_f") or $s->abort("Can't write to $headers_f: $!");
    open(D, "> $details_f") or $s->abort("Can't write to $details_f: $!");
    
    if (scalar @{ $VAR1->{headers} } && scalar @{ $VAR1->{details} }) {
      my $status = $csv->combine(
        'TPTERC', 'TERCEIRO', 'VENDEDOR', 'CLIMORADA', 'NIF', 'TPDOC', 'NUMDOC', 'SERIE',
        'ANO', 'SECTOR', 'PROCESSO', 'VNUMERO', 'DATA', 'DATAVENC', 'OBS', 'REGIVA',
        'MODPGT', 'DESCCAB',
      );
      
      print F $csv->string()."\n"; 
      
      $status = $csv->combine(
        'TPDOC', 'NUMDOC', 'SERIE', 'ANO', 'NUMLIN', 'CODART', 'DESCR', 'QUANT',
        'PV', 'DESC1', 'DESC2', 'UNIDADE', 'IVA', 'ARMAZEM', 'TPDOC_ORI', 'NUMDOC_ORI',
        'SERIE_ORI', 'ANO_ORI', 'QTDORI', 'VALORI',
      );
      
      print D $csv->string()."\n"; 
    }

=pod
$VAR1 = {
          'date' => '2010-03-01',
          'supplier_docref' => '1220755',
          'date_of_payment' => '2010-03-31',
          'store_code' => '54',
          'total_iva' => '1.20',
          'supplier_code' => '534',
          'store_name' => 'Armazem Transp. Teppan',
          'code' => 'W0000002253',
          'franchised' => undef,
          'total_payed' => '0.00',
          'total_to_pay' => '25.29',
          'discount' => '0.000000',
          'doc_type_name' => 'Nota Crdito F',
          'total_p_iva' => '21=0.000000|0=0.000000|12=0.000000|5=1.20|20=0.000000',
          'supplier_name' => 'VANIBRU - Comrcio Produtos Alimentares, Lda',
          'doc_type' => '200',
          'internal_doc_code' => 'R1714'
        };
=cut
        
    # Headers
    foreach my $h (@{ $VAR1->{headers} }) {
      my( $year ) = $h->{date} =~ /^(\d\d\d\d)/;
      
      $h->{discount} ||= 0;

      my $status = $csv->combine(
        2, $h->{supplier_code}, '000', '', $supplier_nifs{$h->{supplier_code}}, $h->{doc_type}, $h->{internal_doc_code}, $sage_serie, $year, $sage_sector,
        $h->{code}, $h->{supplier_docref}, $h->{date}, $h->{date_of_payment}, '', $REGIVA, '000', sprintf("%.4f", $h->{discount}),
      );

      $headers->{$h->{internal_doc_code}} = $h;
      
      print F $csv->string()."\n";
    }

=pod
$VAR1 = {
          'article_id' => '1170',
          'pcm' => '0.44344562580645161290',
          'subfamily_name' => 'Marisco',
          'xtra_cost' => '0.0000',
          'subfamily_code' => '205',
          'family_code' => '200',
          'family_name' => 'Pratos',
          'unit_name' => 'Emb.800.GR',
          'f_qtty' => '16000',
          'quantity' => '20',
          'tax' => '20',
          'barcode2' => '1',
          'discount' => '0.312',
          'article_name' => 'Camaro Black Tiger 16/20',
          'internal_doc_code' => 'R922',
          'article_code' => '2729',
          'buy_price' => '6.24'
        };
=cut

    my $lines = {};

    # Details
    foreach my $d (@{ $VAR1->{details} }) {
      my $h = $headers->{$d->{internal_doc_code}};
      
      if (! defined $h->{date}) {
        print STDERR "#####> ".$d->{internal_doc_code}." - ".$h->{date}."\n";
        $s->dump($h);
      }
      
      
      my( $year ) = $h->{date} =~ /^(\d\d\d\d)/;
      
      $lines->{$h->{code}}++;

      my $discount = $d->{discount} > 0 ? ($d->{discount} / $d->{buy_price}) * 100 : 0;
      
      if (! defined $d->{integration1} || ! $d->{integration1}) {
        push @product_errors, $d->{article_code};
        next;
      }
      
      my $status = $csv->combine(
        $h->{doc_type}, $h->{code}, $sage_serie, $year, $lines->{$h->{code}}, $d->{article_code},
        $d->{article_name}, sprintf("%.4f", $d->{quantity}), sprintf("%.4f", $d->{buy_price}),
        sprintf("%.4f", $discount), 0, $d->{unit_name}, $d->{tax}, $h->{store_code}, '', '', '',
        '', '', '',
      );
      
      print D $csv->string()."\n";
    }

    close(F);
    close(D);
  }
  
  if (scalar @product_errors) {
    my $str = join(", ", @product_errors);
    system("\"$alert_prog\" \"$str\"");
    #die "terminei...\n";
  }

  $s->debug("Will get sales...");
  
  die "here";
  
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetSales', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
=pod
$VAR1 = {
          'movement_type' => '3',
          'article_id' => '2168',
          'pcm' => 0,
          'date' => '2010-06-14',
          'client_fo_id' => '999999',
          'subfamily_code' => undef,
          'family_code' => undef,
          'family_name' => undef,
          'waste_q' => '0',
          'consumpt_q' => '0',
          'sold_q' => '1.000',
          'top_family_name' => undef,
          'sold_v' => '5.35714285714285714286',
          'store' => '8',
          'consumpt_v' => '0',
          'tax_value' => '12.00',
          'waste_v' => '0',
          'vat_v' => '0.64285714285714285714'
        };
=cut
    
    my $sale_headers = ();
    my $sale_details = ();
    
    # Sales collect
    for (@$VAR1) {
      #$s->dump($_);
=pod   
      my $tax_value = $_->{tax_value}*1;

      my $orig_fam_code = $fam_code;

      my $date = $_->{date};
      my $movement_type = $_->{movement_type};
      my $store = $_->{store};
      my $client_fo_id = $_->{client_fo_id};
      my $value = $_->{sold_v};
      my $qtty  = $_->{sold_q};

      $sales_by_cust{ $client_fo_id }->{$movement_type} += $value;

   
      if (defined $_->{subfamily_code}) {
        $fam_code = $_->{subfamily_code};
      }
      elsif (defined $_->{family_code}) {
        $fam_code = $_->{family_code};
      }
      else {
        if (defined $sales_group_by) {
          if ($sales_group_by eq 'top_families') {
            $fam_code = $_->{'top_family_name'};
          }
          elsif ($sales_group_by eq 'sub_families') {
            $fam_code = $_->{'family_name'};
          }
        }
      }

#      if ($_->{consumpt_q} > 0) {
#      #  $value = $_->{consumpt_v};
#      #  $qtty  = $_->{consumpt_q};
#        $movement_type = 'POS_AUTO_CONSUMPTION';
#      }

#      if ($_->{waste_q} > 0) {
#      #  $value = $_->{waste_v};
#      #  $qtty  = $_->{waste_q};
#        $movement_type = 'POS_VOID_WITH_WASTE';
#      }

      $fam_code = $orig_fam_code unless defined $fam_code;
      my $primavera_product = $fam_code."_".$tax_value;
      my $pcm = $qtty*$_->{pcm};

      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'sales', $movement_type, $date );
      my $kk = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client_fo_id;
      $series{ $kk } = $primavera_serie;

#      print STDERR "###### $movement_type - $primavera_doc_type (".$value.") - consumpt_q: ".$_->{consumpt_q}." - waste_q: ".$_->{waste_q}."\n";

      #$sales_by_cust2{ $client_fo_id }->{$primavera_doc_type} += $value;
      $sales_by_cust2{ $client_fo_id }->{$movement_type} += $value;
      
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{sold_v} += sprintf("%.4f", $value);
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{sold_q} += sprintf("%.4f", $qtty);
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{vat} = $tax_value*1;
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{vat_v} += sprintf("%.4f", $_->{vat_v});
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{cost_v} += sprintf("%.4f", $pcm);
=cut    
    }
    
    #$s->dump(\%sales_by_cust);
    #$s->dump(\%sales_by_cust2);
  }

  die "here\n";
  
=pod

  #my $cache = $s->_read_cache($dir, 'documents');

  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetDocuments', } );
  };
  
  $s->abort($@) if $@;
  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'HASH') {
    my $out_file_headers = $dir.$dsep."document_headers.csv";
    my $out_file_details = $dir.$dsep."document_details.csv";
    
    my $headers = {};

    open(H, "> $out_file_headers") or $s->abort("Can't write to $out_file_headers: $!");
    open(D, "> $out_file_details") or $s->abort("Can't write to $out_file_details: $!");

    my $status = $csv->combine('doc_id', 'date', 'doc_type', 'supplier_docref', 'supplier', 'serie', 'discount', 'payment_date');
    print H $csv->string()."\n";
    $status = $csv->combine('doc_id', 'product', 'store', 'price', 'quantity', 'discount', 'vat_rate', 'pcm', 'extra_cost', 'vat_code');
    print D $csv->string()."\n";

    my %md5s = ();
    my %to_skip = ();

    # Headers
    foreach my $h (@{ $VAR1->{headers} }) {
      $headers->{ $h->{internal_doc_code} } = $h;

      my $doc_type = $h->{doc_type};
      my $date = $h->{date};
      my $sup_doc_ref = exists $h->{supplier_docref} ? $h->{supplier_docref} : $h->{internal_doc_code};
      my $sup_code = exists $h->{supplier_code} ? $h->{supplier_code} : "999999";
      
      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'documents', $doc_type, $date );
      next if $primavera_doc_type eq '#IGNORE#';

      # add record to cache
      #$cache->{$h->{internal_doc_code}} = $md5s{ $h->{internal_doc_code} };

      $h->{discount} ||= 0;

      my( $y, $m, $d ) = $date =~ /^(\d+)\-(\d+)\-(\d+)$/;
      my( $y2, $m2, $d2 ) = ( $y, $m, $d );

      if (exists $h->{date_of_payment} && $h->{date_of_payment}) {
        ( $y2, $m2, $d2 ) = $h->{date_of_payment} =~ /^(\d+)\-(\d+)\-(\d+)$/;
      }
      
      my $status = $csv->combine( $h->{internal_doc_code}, "$d-$m-$y", $primavera_doc_type, $sup_doc_ref, "WF".$sup_code, $primavera_serie, sprintf("%.4f", $h->{discount}), "$d2-$m2-$y2" );
      print H $csv->string()."\n";
    }

    # Details
    foreach my $d (@{ $VAR1->{details} }) {
      #next if exists $to_skip{ $d->{internal_doc_code} };

      my $headers = $headers->{$d->{internal_doc_code}};

      my $doc_type = $headers->{doc_type};
      my $date = $headers->{date};
      
      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'documents', $doc_type, $date );      
      next if $primavera_doc_type eq '#IGNORE#';

      my $xtra_cost = exists $d->{xtra_cost} ? $d->{xtra_cost} : 0;

      my $vat = $d->{tax}*1;
      my $vat_prim_code = exists $vat_mappings{vat}->{$vat} ? $vat_mappings{vat}->{$vat}->{code} : 999999;

      #if (! exists $vat_mappings{vat}->{$vat}) {
      #  print STDERR "########## #$vat#\n";
      #}

      my $discount = 0;

      if ($d->{discount} > 0) {
        $discount = ($d->{discount} / $d->{buy_price}) * 100;
      }
     
      my $status = $csv->combine(
        $d->{internal_doc_code}, "W".$d->{article_code}, $headers->{store_code}, sprintf("%.4f", $d->{buy_price}),
        sprintf("%.4f", $d->{quantity}), sprintf("%.4f", $discount), sprintf("%.4f", $d->{tax}),
        sprintf("%.4f", $d->{pcm}), sprintf("%.4f", $xtra_cost), $vat_prim_code, 
      );
      
      print D $csv->string()."\n";
    }
    
    close(H);
    close(D);
  }

  #$s->_save_cache($dir, 'documents', $cache);
           
  eval {
    $str = $s->weckoffice_send_data( { action => 'NetBO_IntegratorsSAGE_GetSales', } );
  };
  
  $s->abort($@) if $@;
  eval $str;
  
  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my %sales = ();
    my %series = ();
    my %sales_by_cust = ();
    my %sales_by_cust2 = ();
    
    # Sales collect
    for (@$VAR1) {
      my $fam_code = 'NETBO';
      my $tax_value = $_->{tax_value}*1;

      my $orig_fam_code = $fam_code;

      my $date = $_->{date};
      my $movement_type = $_->{movement_type};
      my $store = $_->{store};
      my $client_fo_id = $_->{client_fo_id};
      my $value = $_->{sold_v};
      my $qtty  = $_->{sold_q};

      $sales_by_cust{ $client_fo_id }->{$movement_type} += $value;

      if (defined $_->{subfamily_code}) {
        $fam_code = $_->{subfamily_code};
      }
      elsif (defined $_->{family_code}) {
        $fam_code = $_->{family_code};
      }
      else {
        if (defined $sales_group_by) {
          if ($sales_group_by eq 'top_families') {
            $fam_code = $_->{'top_family_name'};
          }
          elsif ($sales_group_by eq 'sub_families') {
            $fam_code = $_->{'family_name'};
          }
        }
      }

#      if ($_->{consumpt_q} > 0) {
#      #  $value = $_->{consumpt_v};
#      #  $qtty  = $_->{consumpt_q};
#        $movement_type = 'POS_AUTO_CONSUMPTION';
#      }

#      if ($_->{waste_q} > 0) {
#      #  $value = $_->{waste_v};
#      #  $qtty  = $_->{waste_q};
#        $movement_type = 'POS_VOID_WITH_WASTE';
#      }

      $fam_code = $orig_fam_code unless defined $fam_code;
      my $primavera_product = $fam_code."_".$tax_value;
      my $pcm = $qtty*$_->{pcm};

      my( $primavera_doc_type, $primavera_serie ) = $s->_doc_type_and_series( \%doc_mappings, 'sales', $movement_type, $date );
      my $kk = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client_fo_id;
      $series{ $kk } = $primavera_serie;

#      print STDERR "###### $movement_type - $primavera_doc_type (".$value.") - consumpt_q: ".$_->{consumpt_q}." - waste_q: ".$_->{waste_q}."\n";

      #$sales_by_cust2{ $client_fo_id }->{$primavera_doc_type} += $value;
      $sales_by_cust2{ $client_fo_id }->{$movement_type} += $value;
      
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{sold_v} += sprintf("%.6f", $value);
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{sold_q} += sprintf("%.6f", $qtty);
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{vat} = $tax_value*1;
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{vat_v} += sprintf("%.6f", $_->{vat_v});
      $sales{ $store }->{ $date }->{$primavera_doc_type}->{ $client_fo_id }->{ $primavera_product }->{cost_v} += sprintf("%.6f", $pcm);
    }


    $s->dump(\%sales_by_cust);
    $s->dump(\%sales_by_cust2);

    my $out_file_sale_headers = $dir.$dsep."sale_headers.csv";
    my $out_file_sale_details = $dir.$dsep."sale_details.csv";
    
    open(H, "> $out_file_sale_headers") or $s->abort("Can't write to $out_file_sale_headers $!");
    open(D, "> $out_file_sale_details") or $s->abort("Can't write to $out_file_sale_details $!");

    my $status = $csv->combine('doc_id', 'date', 'doc_type', 'customer_code', 'serie');
    print H $csv->string()."\n";
    $status = $csv->combine('doc_id', 'product', 'store', 'sold_v', 'sold_q', 'vat_rate', 'cost_v', 'vat_code');
    print D $csv->string()."\n";
    
    my %done_headers = ();
    my %done_headers_tmp = ();
    my %md5s = ();
    my %to_skip = ();

    %sales_by_cust = ();

    # Sales process
    foreach my $store (sort keys %sales) {
      foreach my $date (sort keys %{ $sales{ $store } }) {
        foreach my $mov_type (keys %{ $sales{ $store }->{$date} }) {
          foreach my $client (keys %{ $sales{ $store }->{$date}->{$mov_type} }) {            
            my $primavera_doc_type = $mov_type;
            my $kk = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client;
            my $primavera_serie = $series{ $kk };

            foreach my $product (keys %{ $sales{ $store }->{$date}->{$mov_type}->{$client} }) {
              my $i = $sales{ $store }->{$date}->{$mov_type}->{$client}->{$product};
              $sales_by_cust{ $client }->{$mov_type} += $i->{sold_v};
            }

            next if $primavera_doc_type eq '#IGNORE#';

            my $key = $store . "#" . $date . "#" . $primavera_doc_type . "#". $client . "#" . $primavera_serie;

            # Headers
            if (! exists $done_headers{ $key }) {
              # add record to cache
              #$cache->{$key} = $md5s{ $key };

              my( $y, $m, $d ) = $date =~ /^(\d+)\-(\d+)\-(\d+)$/;

              my $status = $csv->combine(
                $key, "$d-$m-$y", $primavera_doc_type, "WC".$client, $primavera_serie
              );
        
              print H $csv->string()."\n";

              $done_headers{$key} = 1;
            }

            #next if exists $to_skip{ $key };

            # Details
            foreach my $product (keys %{ $sales{ $store }->{$date}->{$mov_type}->{$client} }) {
              my $i = $sales{ $store }->{$date}->{$mov_type}->{$client}->{$product};
              #$sales_by_cust{ $client }->{$mov_type} += $i->{sold_v};

              my $sold_q = $i->{sold_q} ? $i->{sold_q} : 1;

              my $vat = $i->{vat}*1;
              my $vat_prim_code = exists $vat_mappings{vat}->{$vat} ? $vat_mappings{vat}->{$vat}->{code} : 999999;

              open(PROD, ">> ".$dir.$dsep."products.csv");
              my $status = $csv->combine( &_encode_str("W".$product), &_encode_str("W".$product) );
              print PROD $csv->string()."\n";
              close(PROD);

              my $value = $i->{sold_q} > 0 ? $i->{sold_v}/$i->{sold_q} : $i->{sold_v};

              $status = $csv->combine(
                $key, &_encode_str("W".$product), $store, sprintf("%.6f", $value),
                sprintf("%.4f", $i->{sold_q}), sprintf("%.4f", $i->{vat}),
                sprintf("%.6f", $i->{cost_v}), $vat_prim_code
              );
        
              print D $csv->string()."\n";
            }
          }
        }
      }
    }  

    $s->dump(\%sales_by_cust);

    #$s->_save_cache($dir, 'sales', $cache);
    
    close(D);
    close(H);
  }

  # Show integration log
  if (-e $integration_log) {
    my $cmd = "\"$integration_software\"";

    $s->debug("Will execute: $cmd");

    system($cmd);
    my $alert_prog = $integration_software.".vbs";
    $s->_create_vbs_script($alert_prog);
    system("\"$alert_prog\" \"$integration_log\"");
  }
  
=cut
}

#
# Internal functions
#

sub _doc_type_and_series {
  my $self = shift;
  my $doc_mappings = shift;
  my $key = shift;
  my $doc_type = shift;
  my $date = shift;
  
  # doc type
  my $primavera_doc_type = "";

  if (! exists $doc_mappings->{$key}->{$doc_type} && ! exists $doc_mappings->{$key}->{"default"}) {
    $self->abort("Can't used movement_type $doc_type because we don't have a doc_mapping for it (or a default)");
  }
  elsif (! exists $doc_mappings->{$key}->{$doc_type} && exists $doc_mappings->{$key}->{"default"}) {
    $primavera_doc_type = $doc_mappings->{$key}->{"default"}->{doc_type};
  }
  elsif (exists $doc_mappings->{$key}->{$doc_type}) {
    $primavera_doc_type = $doc_mappings->{$key}->{$doc_type}->{doc_type};
  }
      
  # serie
  my $primavera_serie = "";

  if (exists $doc_mappings->{$key}->{$doc_type} && $doc_mappings->{$key}->{$doc_type}->{serie}) {
    $primavera_serie = $doc_mappings->{$key}->{$doc_type}->{serie};
  }
  else {
    ( $primavera_serie ) = $date =~ /^(\d\d\d\d)/;
  }

  return ( $primavera_doc_type, $primavera_serie );
}

sub _sqlserver_id {
  my $s = shift;
  my $db = shift;

  my $id = $db->select(qq{
    DECLARE \@NI UNIQUEIDENTIFIER
    SET \@NI=NEWID()
    SELECT \@NI AS id
  });

  return $id->[0]->{id};
}

sub _get_serie_number {
  my $s = shift;
  my( $db, $series_table, $primavera_doc_type, $primavera_serie ) = @_;

  my $series_info_tmp = $db->select(qq{
    SELECT Ordem, Numerador
    FROM $series_table
    WHERE TipoDoc = ? AND Serie = ?
  }, $primavera_doc_type, $primavera_serie);

  if (! scalar @$series_info_tmp) {
    $s->abort("Serie $primavera_serie doesn't exists for document type $primavera_doc_type, please create it...");
  }

  my $serie_info = $series_info_tmp->[0];

  my $numerador_old = $serie_info->{Numerador};
  $numerador_old = 0 unless defined $numerador_old;
  my $numerador = $numerador_old+1;

  $db->do(qq{
    UPDATE $series_table SET Numerador = ? WHERE TipoDoc = ? AND Serie = ?
  }, $numerador, $primavera_doc_type, $primavera_serie);

  return ( $numerador, $serie_info->{Ordem} );
}

sub _database_insert {
  my $s = shift;
  my %a = ( db => undef, table => undef, row => {}, @_ );

  $s->debug("Will insert record in table $a{table}");

  my $query = "INSERT INTO ".$a{table}." (";

  my @values = ();

  foreach my $key (keys %{ $a{row} }) {
    push @values, $a{row}->{$key};
    $query .= "$key, ";
  }

  my $qm = "?, " x scalar(@values);

  $query =~ s/, $/) VALUES ($qm/;
  $query =~ s/, $/)/;

  $a{db}->do($query, @values);
}

sub _read_cache {
  my $s = shift;
  my $d = shift;
  my $w = shift;

  my $dsep = $s->directory_separator();

  my $file = $d.$dsep.$w.".cache";
  my $cache = {};

  if (open(F, "< ".$file)) {
    while (<F>) {
      chop;
      my( $k, $v ) = $_ =~ /^(.*?)=(.*)$/;
      $cache->{$k} = $v;
    }

    close(F);
  }

  return $cache;
}

sub _save_cache {
  my $s = shift;
  my $d = shift;
  my $w = shift;
  my $i = shift;

  my $dsep = $s->directory_separator();

  my $file = $d.$dsep.$w.".cache";

  if (open(F, "> ".$file)) {
    foreach my $k (keys %$i) {
      print F $k."=".$i->{$k}."\n";
    }

    close(F);
  }

  return 1;
}

sub _hash_to_md5 {
  my $s = shift;
  my $h = shift;

  my $str = "";

  foreach my $k (sort keys %$h) {
    my $v = exists $h->{$k} && defined $h->{$k} ? $h->{$k} : "";
    $str .= $v;
  }

  return md5_hex($str);
}

sub _write_to_log {
  my $self = shift;
  my( $file, $msg ) = @_;

  open(F, ">> $file") or die "Can't write to $file: $!";
  print F $msg."\n";
  close(F);
}

sub _encode_str {
  my $w = shift;

  my $str = encode("iso-8859-1", $w);

  #print STDERR "#### #$str#\n";
  return $w;
  #return encode("iso-8859-1", decode("utf8", $w));
  #return encode("latin1",decode("utf8", $w));
  #return encode("utf8", $w);

  #return $w;

  #my $tt = encode("iso-8859-15", $w);
  #print STDERR "### #$w# #$tt#\n";

  #return $tt;

  #return encode("iso-8859-15", $w);
}

sub _create_vbs_script {
  my $self = shift;
  my $script = shift;

  if (! -e $script) {
    open(F, "> $script") or $self->abort("Can't write to $script: $!");

    print F "Sub ShowFile(ByVal sFile)\n".
            "Dim shell\n".
            "Set shell = CreateObject(\"WScript.Shell\")\n".
            "shell.Run Chr(34) & sFile & Chr(34), 1, false\n".
            "Set shell = Nothing\n".
            "End Sub\n".
            "ShowFile WScript.Arguments(0)\n";

    #print F 'strComputer = msgbox(WScript.Arguments(0))'."\n";
    close(F);
  }
}

1;




