package WO::Client::Integrators::Servirail;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime ceil );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use Text::CSV_XS;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use Data::Dumper;
# este modulo no est incluido na verso standard do client
use Spreadsheet::ParseExcel::Simple;

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  $s->debug("Encoded FO type is: $fo_type");

  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();

  my $sync_schedules = $s->vfh($xml, 'sync_schedules');
  my $dir_schedules = $s->vfh($xml, 'dir_schedules');

  if (defined $sync_schedules) {
    $s->abort("Missing 'dir_schedules' parameter...") unless defined $dir_schedules;

    my %user_trips;

    my $total_files = 0;

    &find( sub {
      return unless /xls$/i;

      my( $year, $month ) = $_ =~ /^(\d\d\d\d)_(\d\d)/;

      if (! defined $year || ! defined $month) {
        return;
      }

      $total_files++;
    }, $dir_schedules );

    my $counter = 0;

    &find( sub {
      return unless /xls$/i;

      $counter++;

      my( $year, $month ) = $_ =~ /^(\d\d\d\d)_(\d\d)/;

      if (! defined $year || ! defined $month) {
        return;
      }

      return unless $counter == $total_files || $counter == $total_files-1;

      $s->debug("Will process file: $_ (year: $year, month: $month)");
      
      my $xls = Spreadsheet::ParseExcel::Simple->read($File::Find::name);

      my $look_for_clerks = 0;
      my $fetch_clerk_next_row = 0;
      my $number_of_days = 0;
      my $current_user = "";

      foreach my $sheet ($xls->sheets) {
        while ($sheet->has_data) {  
          my @data = $sheet->next_row;
             
          if ($data[0] eq uc('NOME')) {
            $number_of_days = 0;

            for (my $i=2; $i<scalar(@data); $i++) {
              last unless $data[$i];
              $number_of_days++;
            }
            
            $look_for_clerks = 1;
          }

          if ($look_for_clerks && $data[0] =~ /^\d\d\d\d$/) {
            $current_user = $data[0];

            my $trips = &_get_trip_codes( $number_of_days, \@data );

            for (my $i=0; $i<scalar @$trips; $i++) {
              next unless $trips->[$i];

              my $day = $i+1;
              $day = "0".$day unless length($day) == 2;

              my $date = $year.'-'.$month.'-'.$day;
              $user_trips{ $current_user }->{dates}->{$date} = 1;

              push @{ $user_trips{ $current_user }->{trips} }, {
                date => $date,
                trip => $trips->[$i],
              };
            }

            $fetch_clerk_next_row = 1;
          }
          elsif ($fetch_clerk_next_row) {
            my $trips = &_get_trip_codes( $number_of_days, \@data );

            for (my $i=0; $i<scalar @$trips; $i++) {
              next unless $trips->[$i];

              my $day = $i+1;
              $day = "0".$day unless length($day) == 2;

              my $date = $year.'-'.$month.'-'.$day;
              #$user_trips{ $current_user }->{dates}->{$date} = 1;

              push @{ $user_trips{ $current_user }->{trips} }, {
                date => $date,
                trip => $trips->[$i],
              };
            }

            $fetch_clerk_next_row = 0;
          }
        }
         
        #last;
      }
    }, $dir_schedules );

    if (scalar keys %user_trips) {
      $s->debug("Will send content to server...");

      eval {
        my $cnt = $s->weckoffice_send_data( { 
            action => 'ServirailClerksSchedule',
            info => $s->shrink(\%user_trips),
        });
      };
    }

    #$s->dump(\%user_trips);
    #$s->dump($user_trips{"2035"});
  }
}

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  $s->debug("Encoded FO type is: $fo_type");

  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();
}

sub _get_trip_codes {
  my( $nd, $data ) = @_;

  my $c = 0;

  my @trips = ();

  for (my $i=2; $i<scalar(@$data); $i++) {
    last if $c++ == $nd;
    my $d = $data->[$i];
    $d =~ s/\W//g;
    $d =~ s/[a-z]//ig;
    push @trips, $d;
  }

  return \@trips;
}

1;

