package WO::Client::Integrators::UMA;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime ceil );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Spec;
use DBI;
use Text::CSV_XS;
use FindBin qw( $Bin );
use Digest::MD5 qw( md5_hex );
use Data::Dumper;

my $SIMULATE = 0;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $tmp_dir = $s->tmp_dir();
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  # executes unless a manual execution order was issued
  #return unless $s->manual();

  $s->debug("sync_bk_information()");

  my $tmp_dir = $s->tmp_dir();

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $out_xml_file = $tmp_dir.$dsep."out.xml";
  unlink($out_xml_file) if -e $out_xml_file;

  my $date_from = exists $xml->{date_from} ? $xml->{date_from} : "";
  my $integra_client = exists $xml->{integra_client} ? $xml->{integra_client} : "";

  $s->abort("You must define an 'integra_client' in your config XML") unless $integra_client;

  if (-e $ENV{USERPROFILE}."/DocsNaoIntegrados.xml") {
    $s->own_remove($ENV{USERPROFILE}."/DocsNaoIntegrados.xml");
  }

  my $alert_prog = $integra_client.".vbs";

  if (! -e $alert_prog) {
    open(F, "> $alert_prog") or $s->abort("Can't write to $alert_prog: $!");
    print F 'strComputer = msgbox("Artigos sem mapeamento de cdigo de barras: " + WScript.Arguments(0))'."\n";
    close(F);
  }

  my $str;
  my $VAR1;

  eval {
    $str = $s->weckoffice_send_data( { action => 'UMAGetSales', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $xml = "<NewDataSet>\n";

    my %control;

    foreach my $doc (@$VAR1) {
      my $art_id = '';

      if ($doc->{tax} == 5) {
        $art_id = '7211';
      }
      elsif ($doc->{tax} == 12) {
        $art_id = '7213';
      }
      elsif ($doc->{tax} == 21) {
        $art_id = '7212';
      }

      next if $art_id eq '';

      $control{ $doc->{date} }++;

      my $date_tmp = $doc->{date};
      $date_tmp =~ s/\-//g;

      if ($control{ $doc->{date} } == 1) {
        $xml .= "\t<Doc>\n";
        $xml .= "\t\t<Campo2>V</Campo2>\n";
        $xml .= "\t\t<Campo3></Campo3>\n";
        $xml .= "\t\t<Campo4></Campo4>\n";
        $xml .= "\t\t<Campo5></Campo5>\n";
        $xml .= "\t\t<ref>$date_tmp</ref>\n";
        $xml .= "\t\t<armid>".$doc->{store_fo_id}."</armid>\n";
        $xml .= "\t\t<tpdocid></tpdocid>\n";
        $xml .= "\t\t<docnum>$date_tmp</docnum>\n";
        $xml .= "\t\t<data>".$doc->{date}."</data>\n";
        $xml .= "\t\t<entid>0</entid>\n";
        $xml .= "\t\t<Campo1></Campo1>\n";
        $xml .= "\t\t<NomeJuridicoEnt></NomeJuridicoEnt>\n";
        $xml .= "\t\t<ContribEnt></ContribEnt>\n";
        $xml .= "\t\t<MoradaEnt></MoradaEnt>\n";
        $xml .= "\t\t<LocalidadeEnt></LocalidadeEnt>\n";
        $xml .= "\t\t<percdescent1>0</percdescent1>\n";
        $xml .= "\t\t<percdescent2>0</percdescent2>\n";
        $xml .= "\t</Doc>\n";
      }

      #my $wo_tax = $doc->{sum} * ($doc->{tax} / 100);
      #$s->debug("##### ".$doc->{tax}." -> $wo_tax -> ".$doc->{sum});

      $xml .= "\t<LnDoc>\n";
        $xml .= "\t\t<tpdocid></tpdocid>\n";
        $xml .= "\t\t<docnum>$date_tmp</docnum>\n";
        $xml .= "\t\t<ln>".$control{ $doc->{date} }."</ln>\n";
        $xml .= "\t\t<artid>".$art_id."</artid>\n";
        $xml .= "\t\t<qt>1</qt>\n";
        $xml .= "\t\t<precounitinp>".$doc->{sum}."</precounitinp>\n";
        $xml .= "\t\t<percdescArt1>0</percdescArt1>\n";
        $xml .= "\t\t<percdescArt2>0</percdescArt2>\n";
      $xml .= "\t</LnDoc>\n";      
    }

    $xml .= "</NewDataSet>\n";

    my $out_xml_file='c:\StatusDoc_VENDAS.xml';

    open(F, "> $out_xml_file") or die "Can't write to $out_xml_file: $!\n";
    print F $xml;
    close(F);

    $s->debug("Will call: \"$integra_client\" 1 \"$out_xml_file\"");

    my $res = 0;

    if ($SIMULATE == 0) {
      $res = system("\"$integra_client\" 1 \"$out_xml_file\"");
      $s->debug("Result: $res");

      my $cmd = "move /Y \"$ENV{USERPROFILE}\\StatusDoc.xml\" \"$ENV{HOMEPATH}\\StatusDoc_VENDAS.xml\"";
      #$s->debug($cmd);
      system($cmd);
      #unlink($out_xml_file) if -e $out_xml_file;
    }
  }

  eval {
    $str = $s->weckoffice_send_data( { action => 'UMAGetDocuments', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my %invalids;
    my %idx;

    my %receptions;
    my %devolutions;
    my %transfers;

    #
    # Receptions and Devolutions
    #

    my $full_id_list;

    foreach my $doc (@$VAR1) {
      my $ref;
      my $key = "";

      # this is a reception
      if (exists $doc->{reception_id}) {
        $ref = \%receptions;
        $key = 'reception_id';
      }
      # this is a devolution
      elsif (exists $doc->{devolution_id}) {
        $ref = \%devolutions;
        $key = 'devolution_id';
      }
      # this is a devolution
      elsif (exists $doc->{transfer_id}) {
        $ref = \%transfers;
        $key = 'transfer_id';
      }

      next unless $key ne '';

      # header
      if (! exists $ref->{ $doc->{$key} }) {
        $doc->{global_discount_percent} ||= 0;
        $doc->{global_discount} ||= 0;

        #if ($doc->{global_discount} > 0 && $doc->{global_discount_percent} == 0) {
        if ($doc->{global_discount} > 0) {
          $doc->{global_discount_percent} = sprintf("%0.2f", ($doc->{global_discount} / $doc->{total_pay}) * 100);
        }

        $doc->{cache_key} = $doc->{$key};

        if ($key eq 'reception_id') {
          $doc->{cache_key} = "R".$doc->{$key};
        }
        elsif ($key eq 'devolution_id') {
          $doc->{cache_key} = "D".$doc->{$key};
        }
        elsif ($key eq 'transfer_id') {
          $doc->{cache_key} = "T".$doc->{$key};
        }

        $full_id_list .= $doc->{cache_key}.",";

        my $h = {
          cache_key => $doc->{cache_key},
	        armid => $doc->{store_fo_id},
          tpdocid => $doc->{doc_code},
          docnum => $doc->{$key},
          data => $doc->{date},
          entid => $doc->{supplier_id} || "",
          Campo1 => $doc->{supplier_docref} || "",
          Campo3 => $doc->{total_pay} || 0,
          Campo4 => $doc->{total_iva} || 0,
          NomeJuridicoEnt => $doc->{sup_name} || "",
          ContribEnt => $doc->{sup_ncontrib} || "",
          MoradaEnt => $doc->{sup_address} || "",
          LocalidadeEnt => $doc->{sup_city} || "",
          CodPostalEnt => $doc->{sup_postal_code} || "",
          percdescent1 => 0,
          percdescent2 => 0,
          key => $key,
          ArmDestino => $doc->{dest_store_fo_id} || "",
        };

        $h->{lines} = [];

        if ($doc->{global_discount} > 0 && defined $key && $key eq 'reception_id') {
          push @{ $h->{lines} }, {
            tpdocid => $doc->{doc_code},
            docnum => $doc->{$key},
            ln => ++$idx{$key}->{$doc->{$key}},
            artid => 'DESC',
            qt => -1,
            precounit => $doc->{global_discount},
            percdescArt1 => 0,
            percdescArt2 => 0,
          };
        }
        # as doacoes, s tm um artigo
        elsif ($h->{tpdocid} eq 'D01') {
          push @{ $h->{lines} }, {
            tpdocid => $doc->{doc_code},
            docnum => $doc->{$key},
            ln => ++$idx{$key}->{$doc->{$key}},
            artid => '380',
            qt => 1,
            precounit => $doc->{total_pay},
            percdescArt1 => 0,
            percdescArt2 => 0,
          };
        }

        $ref->{ $doc->{$key} } = $h;
      }

      next unless defined $doc->{barcode2} && $doc->{barcode2} ne '';

      if ($doc->{barcode2} eq '0') {
        $invalids{ $doc->{article_fo_id} } = 1 if $key eq 'reception_id';
        next;
      }

      $doc->{line_discount} ||= 0;
      $doc->{line_discount_percent} ||= 0;
      $doc->{unit_factor} ||= 1;
      $doc->{xtra_cost} ||= 0;
      $doc->{buy_price} ||= 0;
      $doc->{doc_code} ||= 0;

      #my $buy_price = $doc->{buy_price} + $doc->{xtra_cost};
      my $buy_price = $doc->{buy_price};

      #if ($doc->{line_discount} > 0 && $doc->{line_discount_percent} == 0) {
      if ($doc->{line_discount} > 0) {
        $doc->{line_discount_percent} = sprintf("%0.2f", ($doc->{line_discount} / $doc->{buy_price}) * 100);
      }

      my $unit_price = ($buy_price - $doc->{line_discount} + $doc->{xtra_cost})*$doc->{quantity};

      # nao adicionamos os artigos se estivermos perante uma doao
      if ($doc->{doc_code} ne 'D01') {
        push @{ $ref->{ $doc->{$key} }->{lines} }, {
          tpdocid => $doc->{doc_code},
          docnum => $doc->{$key},
          ln => ++$idx{$key}->{$doc->{$key}},
          artid => $doc->{barcode2},
          qt => 1,
          precounit => $unit_price,
          percdescArt1 => 0,
          percdescArt2 => 0,
        };
      }
    }

    #
    # Handle invalids
    #

    if (scalar keys %invalids) {
      my $str = join(", ", keys %invalids);

      #use Data::Dumper;
      #open(F, "> c:/t.txt");
      #print F Dumper($str);
      #close(F);

      system("\"$alert_prog\" \"$str\"");
      #die "terminei...\n";
    }

    $full_id_list =~ s/,$//;

    my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));
    open(F, "> $tmp_file") or die "Can't write to $tmp_file: $!\n";
    print F $full_id_list;
    close(F);
    my $res = system("\"$integra_client\" 4 \"$tmp_file\"");
    $s->own_remove($tmp_file);

    my @all_missing;

    if (-e $ENV{USERPROFILE}."/DocsNaoIntegrados.xml" && -s $ENV{USERPROFILE}."/DocsNaoIntegrados.xml") {
      if (open(F, "< ".$ENV{USERPROFILE}."/DocsNaoIntegrados.xml")) {
        local $/ = undef;
        my $cnt = <F>;
        $cnt =~ s/\n//g;
        $cnt =~ s/\r//g;
        $cnt =~ s/\s//g;
        close(F);
        ( @all_missing ) = $cnt =~ /<DocsNaoIntegrados><Ref>(.*?)<\/Ref>/g;
      }
    }

    #die "here\n";

    my %all_missing = map { $_ => 1 } @all_missing;

    #%receptions = ();
    #%devolutions = ();

    #my $receptions_to_send  = $s->handle_cache( 'receptions', \%all_missing, \%receptions );
    #my $devolutions_to_send = $s->handle_cache( 'devolutions', \%all_missing, \%devolutions );
    my $receptions_to_send = \%receptions;
    my $devolutions_to_send = \%devolutions;
    my $transfers_to_send = \%transfers;

    my @k1 = keys %receptions;
    my @k2 = keys %devolutions;
    my @k3 = keys %transfers;

    my @kk1 = keys %$receptions_to_send;
    my @kk2 = keys %$devolutions_to_send;
    my @kk3 = keys %$transfers_to_send;

    $s->debug("####### total recepcoes: ".scalar(@k1)." | depois cache: ".scalar(@kk1));
    $s->debug("####### total devolucoes: ".scalar(@k2)." | depois cache: ".scalar(@kk2));
    $s->debug("####### total transferencias: ".scalar(@k3)." | depois cache: ".scalar(@kk3));

    my @docs;

    if (scalar keys %$receptions_to_send) {
      push @docs, values %$receptions_to_send;
    }

    if (scalar keys %$devolutions_to_send) {
      push @docs, values %$devolutions_to_send;
    }

    if (scalar keys %$transfers_to_send) {
      push @docs, values %$transfers_to_send;
    }

    if (scalar @docs) {
      my $xml = "<NewDataSet>\n";
      my $c = 0;
      my $file_counter = 0;

      foreach my $d (@docs) {
        #next unless $d->{data} =~ /^2008\-02/;
        #next unless $d->{entid} == 24;

        $d->{tpdocid} ||= "0";
        $d->{armid} ||= "";
        $d->{docnum} ||= "";
        $d->{data} ||= "";
        $d->{entid} ||= "0";
        $d->{Campo1} ||= "";
        $d->{NomeJuridicoEnt} ||= "";
        $d->{ContribEnt} ||= "";
        $d->{MoradaEnt} ||= "";
        $d->{LocalidadeEnt} ||= "";
        $d->{CodPostalEnt} ||= "";

        # ignora VD's fornecedor
        if ($d->{tpdocid} eq '302') {
          $s->debug("VOU IGNORAR VD DE FORNECEDOR (doc num: ".$d->{docnum}.")");
          next;
        }

        $xml .= "\t<Doc>\n";

        if ($d->{key} eq 'reception_id') {
          $xml .= "\t\t<Campo2>R</Campo2>\n";
          $xml .= "\t\t<ref>R".$d->{docnum}."</ref>\n";
        }
        elsif ($d->{key} eq 'transfer_id') {
          $xml .= "\t\t<Campo2>T</Campo2>\n";
          $xml .= "\t\t<ref>T".$d->{docnum}."</ref>\n";
        }
        # mark as being a devolution
        elsif ($d->{key} eq 'devolution_id') {
          if ($d->{tpdocid} eq 'D01') {
            $xml .= "\t\t<Campo2>DOACAO</Campo2>\n";
          } else {
            $xml .= "\t\t<Campo2>D</Campo2>\n";
          }

          $xml .= "\t\t<ref>D".$d->{docnum}."</ref>\n";
        }
        else {
          next;
        }

      	$xml .= "\t\t<armid>".$d->{armid}."</armid>\n";
        $xml .= "\t\t<ArmDestino>".$d->{ArmDestino}."</ArmDestino>\n";
        $xml .= "\t\t<tpdocid>".$d->{tpdocid}."</tpdocid>\n";
    		$xml .= "\t\t<docnum>".$d->{docnum}."</docnum>\n";
	    	$xml .= "\t\t<data>".$d->{data}."</data>\n";
		    $xml .= "\t\t<entid>".$d->{entid}."</entid>\n";
        $xml .= "\t\t<Campo1>".$d->{Campo1}."</Campo1>\n";
  	  	$xml .= "\t\t<NomeJuridicoEnt>".$d->{NomeJuridicoEnt}."</NomeJuridicoEnt>\n";
	  	  $xml .= "\t\t<ContribEnt>".$d->{ContribEnt}."</ContribEnt>\n";
  		  $xml .= "\t\t<MoradaEnt>".$d->{MoradaEnt}."</MoradaEnt>\n";
    		$xml .= "\t\t<LocalidadeEnt>".$d->{LocalidadeEnt}."</LocalidadeEnt>\n";
	    	$xml .= "\t\t<percdescent1>".$d->{percdescent1}."</percdescent1>\n";
        $xml .= "\t\t<percdescent2>".$d->{percdescent2}."</percdescent2>\n";
        $xml .= "\t\t<Campo3>".$d->{Campo3}."</Campo3>\n";
        $xml .= "\t\t<Campo4>".$d->{Campo4}."</Campo4>\n";
        $xml .= "\t\t<Campo5></Campo5>\n";
        $xml .= "\t</Doc>\n";

        foreach my $line (@{ $d->{lines} || [] }) {
          $xml .= "\t<LnDoc>\n";
            $xml .= "\t\t<tpdocid>".$line->{tpdocid}."</tpdocid>\n";
            $xml .= "\t\t<docnum>".$line->{docnum}."</docnum>\n";
            $xml .= "\t\t<ln>".$line->{ln}."</ln>\n";
            $xml .= "\t\t<artid>".$line->{artid}."</artid>\n";
            $xml .= "\t\t<qt>".$line->{qt}."</qt>\n";
            $xml .= "\t\t<precounitinp>".$line->{precounit}."</precounitinp>\n";
            $xml .= "\t\t<percdescart1>".$line->{percdescArt1}."</percdescart1>\n";
            $xml .= "\t\t<percdescart2>".$line->{percdescArt2}."</percdescart2>\n";
          $xml .= "\t</LnDoc>\n";
        }

        #if ($d->{docnum} == 537) {
        #  #$s->debug($xml);
        #  #die "here\n";
        #  #$s->debug("################################# $file_counter");
        #}

        if ($c == 100000) {
          $xml .= "</NewDataSet>\n";

          $file_counter++;

          $out_xml_file='c:\StatusDoc_'.$file_counter.'.xml';

          open(F, "> $out_xml_file") or die "Can't write to $out_xml_file: $!\n";
          print F $xml;
          close(F);

          my $res = 0;

          if ($SIMULATE == 0) {
            $s->debug("Will call: \"$integra_client\" 1 \"$out_xml_file\"");
            $res = system("\"$integra_client\" 1 \"$out_xml_file\"");
            $s->debug("Result: $res");

            my $cmd = "move /Y \"$ENV{USERPROFILE}\\StatusDoc.xml\" \"$ENV{HOMEPATH}\\StatusDoc_".$file_counter.".xml\"";
            #$s->debug($cmd);
            system($cmd);
          }  
          
          #unlink($out_xml_file) if -e $out_xml_file;

          $xml = "<NewDataSet>\n";

          $c = 0;
        }

        $c++;
      }

      #die "here";

      if ($xml && $xml !~ /^<NewDataSet>\n$/) {
        $xml .= "</NewDataSet>\n";

        $file_counter++;

        $out_xml_file='c:\StatusDoc_'.$file_counter.'.xml';

        open(F, "> $out_xml_file") or die "Can't write to $out_xml_file: $!\n";
        print F $xml;
        close(F);

        $s->debug("Will call: \"$integra_client\" 1 \"$out_xml_file\"");

        my $res = 0;

        if ($SIMULATE == 0) {
          $res = system("\"$integra_client\" 1 \"$out_xml_file\"");
          $s->debug("Result: $res");

          my $cmd = "move /Y \"$ENV{USERPROFILE}\\StatusDoc.xml\" \"$ENV{HOMEPATH}\\StatusDoc_".$file_counter.".xml\"";
          #$s->debug($cmd);
          system($cmd);
        }
        #unlink($out_xml_file) if -e $out_xml_file;
      }
    }
  }
}

sub process_xml {
  my $s = shift;
}

sub handle_cache {
  my( $s, $k_file, $missing, $data ) = @_;

#  foreach my $key (sort { $a <=> $b } keys %$data) {
#    $s->debug("#####> $key");
#    $s->dump($data->{$key});
#    die "here";
#  }

  my %final_docs;

  my $md5_toks = &read_cache($k_file);

  foreach my $key (sort { $a <=> $b } keys %$data) {
    my $lines = delete $data->{$key}->{lines} || [];

    my $k = $data->{$key}->{cache_key};

    # check integraclient result
    if (exists $missing->{$k}) {
      $final_docs{$key} = $data->{$key};
      $final_docs{$key}->{lines} = $lines;

      my @v = values %{ $data->{$key} };
      my $str = join(',', @v);

      for my $l (@$lines) {
        foreach my $key (sort { $a cmp $b } keys %$l) {
          $str .= ",".$l->{$key};
        }
      }

      my $md5_str = &md5_hex($str);
      $md5_toks->{$k} = $md5_str;
    }
    # check local cache
    else {
      #$s->dump($data->{$key});

      my @v = values %{ $data->{$key} };
      my $str = join(',', @v);

      for my $l (@$lines) {
        foreach my $key (sort { $a cmp $b } keys %$l) {
          $str .= ",".$l->{$key};
        }
      }

      my $md5_str = &md5_hex($str);

      if (! exists $md5_toks->{$k}) {
        #$s->debug("DOC $key is new in cache...");
        $md5_toks->{$k} = $md5_str;
        $final_docs{$key} = $data->{$key};
        $final_docs{$key}->{lines} = $lines;
      } elsif (exists $md5_toks->{$k} && $md5_toks->{$k} ne $md5_str) {
        $md5_toks->{$k} = $md5_str;
        #$s->debug("$k | DOC $key is in cache but has changed, will reprocess...");
        $final_docs{$key} = $data->{$key};
        $final_docs{$key}->{lines} = $lines;
      } else {
        #$s->debug("$k | DOC $k is in cache and didn't change... ignoring...");
      }
    }
  }

  &write_cache($k_file, $md5_toks);

  return \%final_docs;
}

sub read_cache {
  my $k = shift;

  my %r = ();

  if (-e $Bin."/$k.cache") {
    my $VAR1;
    local $/ = undef;

    open(F, "< ".$Bin."/$k.cache") or die "erro reading $k.cache: $!";
    my $str = <F>;
    eval "$str";
    if (! $@) {
      return $VAR1;
    }
    close(F);
  }

  return \%r;
}

sub write_cache {
  my( $k, $r ) = @_;

  open(F, "> ".$Bin."/$k.cache") or die "error writing $k.cache: $!";
  print F Dumper($r);
  close(F);
}

1;

__END__
<NewDataSet>
	<Doc>
		<tpdocid>300</tpdocid>
		<docnum>139</docnum>
		<data>2006-10-30</data>
		<entid>FORN_10</entid>
		<NomeJuridicoEnt></NomeJuridicoEnt>
		<ContribEnt></ContribEnt>
		<MoradaEnt></MoradaEnt>
		<LocalidadeEnt></LocalidadeEnt>
		<CodPostalEnt></CodPostalEnt>
	</Doc>
	<LnDoc>
		<tpdocid>300</tpdocid>
		<docnum>139</docnum>
		<ln>1</ln>
		<artid>W0000000624</artid>
		<qt>30</qt>
		<precounit>0.402</precounit>
	</LnDoc>

