package WO::Client::Integrators::UMinho;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use File::Remove qw( remove );
use File::Copy qw( move copy );
use File::Slurp qw( write_file read_file read_dir );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );

use Data::Dumper;
use WO::DB::ODBC;
use DBI;

#
# Sends FO info back to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  # service xml
  my $xml = $s->info();

  $xml->{odbc_name}   ||= "";
  $xml->{odbc_login}  ||= "";
  $xml->{odbc_passwd} ||= "";

  #my @warehouses = $db->select("SELECT Armazem, Telefone, Localidade, Morada, Descricao, CpLocalidade, Fax, Cp FROM Armazens");
  #$s->dump(\@warehouses);
  #my @res = $db->select("SELECT NEWID() AS id");
  #$s->dump(\@res);
  #die("AquI");

  my $file_warehouses       = 'c:/__warehouses__.txt';
  my $file_suppliers        = 'c:/__suppliers__.txt';
  my $file_supp_art         = 'c:/__supp_art__.txt';
  my $file_taxes            = 'c:/__taxes__.txt';
  my $file_units            = 'c:/__units__.txt';
  my $file_articles         = 'c:/__articles__.txt';
  my $file_doc_types1       = 'c:/__doc_types1__.txt';
  my $file_doc_types2       = 'c:/__doc_types2__.txt';
  my $file_doc_types3       = 'c:/__doc_types3__.txt';
  my $file_pay_cond         = 'c:/__pay_cond__.txt';
  my $file_clients          = 'c:/__clients__.txt';
  my $file_transfers        = 'c:/__transfers__.txt';
  my $file_users            = 'c:/__users__.txt';
  my $file_offers_in        = 'c:/__offers_in__.txt';
  my $file_breaks           = 'c:/__breaks__.txt';
  my $file_inventories      = 'c:/__inventories__.txt';
  my $file_receptions       = 'c:/__receptions__.txt';
  my $file_families         = 'c:/__families__.txt';
  my $file_pcms             = 'c:/__pcms__.txt';

=pod
  my $file_stores           = 'c:/__stores__.txt';       # done
  my $file_store_warehouses = 'c:/__store_warehouses__.txt';       # done
  my $file_store_products   = 'c:/__store_products__.txt';
  my $file_families         = 'c:/__families__.txt';     # done
  my $file_articles         = 'c:/__articles__.txt';     # done
  my $file_units            = 'c:/__units__.txt';        # done
  my $file_supp_art         = 'c:/__supp_art__.txt';     # done
  my $file_prod_part        = 'c:/__prod_part__.txt';
  my $file_prod_price       = 'c:/__prod_price__.txt';
  my $file_fiscal_regions   = 'c:/__fiscal_regions__.txt';
  my $file_inv_headers      = 'c:/__inventory_headers__.txt';
  my $file_inv_details      = 'c:/__inventory_details__.txt';
  my $file_buy_headers      = 'c:/__reception_headers__.txt';
  my $file_buy_details      = 'c:/__reception_details__.txt';
  my $file_buy_footers      = 'c:/__reception_footers__.txt';
  my $file_store_clients    = 'c:/__store_clients__.txt';
=cut

  &remove($file_warehouses)  if -e $file_warehouses;
  &remove($file_suppliers)   if -e $file_suppliers;
  &remove($file_supp_art)    if -e $file_supp_art;
  &remove($file_taxes)       if -e $file_taxes;
  &remove($file_units)       if -e $file_units;
  &remove($file_articles)    if -e $file_articles;
  &remove($file_doc_types1)  if -e $file_doc_types1;
  &remove($file_doc_types2)  if -e $file_doc_types2;
  &remove($file_doc_types3)  if -e $file_doc_types3;
  &remove($file_pay_cond)    if -e $file_pay_cond;
  &remove($file_clients)     if -e $file_clients;
  &remove($file_transfers)   if -e $file_transfers;
  &remove($file_users)       if -e $file_users;
  &remove($file_offers_in)   if -e $file_offers_in;
  &remove($file_breaks)      if -e $file_breaks;
  &remove($file_inventories) if -e $file_inventories;
  &remove($file_receptions)  if -e $file_receptions;
  &remove($file_families)    if -e $file_families;
  &remove($file_pcms)    if -e $file_pcms;

=pod
  &remove($file_stores)  if -e $file_stores;
  &remove($file_store_warehouses)  if -e $file_store_warehouses;
  &remove($file_store_products)  if -e $file_store_products;
  
  &remove($file_users)      if -e $file_users;
  &remove($file_prod_part)  if -e $file_prod_part;
  &remove($file_prod_price) if -e $file_prod_price;
  &remove($file_fiscal_regions) if -e $file_fiscal_regions;
  &remove($file_inv_headers) if -e $file_inv_headers;
  &remove($file_inv_details) if -e $file_inv_details;
  &remove($file_buy_headers) if -e $file_buy_headers;
  &remove($file_buy_details) if -e $file_buy_details;
  &remove($file_buy_footers) if -e $file_buy_footers;
  &remove($file_store_clients) if -e $file_store_clients;
=cut

  $xml->{sync_warehouses} = exists $xml->{sync_warehouses} ? $xml->{sync_warehouses} : 1;
  $xml->{sync_suppliers} = exists $xml->{sync_suppliers} ? $xml->{sync_suppliers} : 1;
  $xml->{sync_supplier_articles} = exists $xml->{sync_supplier_articles} ? $xml->{sync_supplier_articles} : 1;
  $xml->{sync_taxes} = exists $xml->{sync_taxes} ? $xml->{sync_taxes} : 1;
  $xml->{sync_units} = exists $xml->{sync_units} ? $xml->{sync_units} : 1;
  $xml->{sync_articles} = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_document_types} = exists $xml->{sync_document_types} ? $xml->{sync_document_types} : 1;
  $xml->{sync_payment_types} = exists $xml->{sync_payment_types} ? $xml->{sync_payment_types} : 1;
  $xml->{sync_clients} = exists $xml->{sync_clients} ? $xml->{sync_clients} : 1;
  $xml->{sync_transfers} = exists $xml->{sync_transfers} ? $xml->{sync_transfers} : 1;
  $xml->{sync_offers_in} = exists $xml->{sync_offers_in} ? $xml->{sync_offers_in} : 1;
  $xml->{sync_breaks} = exists $xml->{sync_breaks} ? $xml->{sync_breaks} : 1;
  $xml->{sync_inventories} = exists $xml->{sync_inventories} ? $xml->{sync_inventories} : 1;
  $xml->{sync_receptions} = exists $xml->{sync_receptions} ? $xml->{sync_receptions} : 1;
  $xml->{sync_families} = exists $xml->{sync_families} ? $xml->{sync_families} : 1;
  $xml->{integrate_weckoffice_data} = exists $xml->{integrate_weckoffice_data} ? $xml->{integrate_weckoffice_data} : 0;

=pod
  $xml->{sync_users} = exists $xml->{sync_users} ? $xml->{sync_users} : 1;
  $xml->{sync_article_prices} = exists $xml->{sync_article_prices} ? $xml->{sync_article_prices} : 1;
  $xml->{sync_sells} = exists $xml->{sync_sells} ? $xml->{sync_sells} : 1;
  $xml->{sync_composite_articles} = exists $xml->{sync_composite_articles} ? $xml->{sync_composite_articles} : 1;
  $xml->{sync_stores} = exists $xml->{sync_stores} ? $xml->{sync_stores} : 1;
=cut

  #
  # Integrate data in Primavera
  #

  if ($xml->{integrate_weckoffice_data}) {
    require WO::DB::Access;

    $xml->{integrate_days} ||= 0;
    $xml->{integrate_from} ||= 0;
    $xml->{integrate_to}   ||= 0;

    $xml->{database_pri_integrator} ||= "";
    $xml->{sync_stocks_information} ||= "";
    $xml->{sync_sells_information} ||= "";
    $xml->{sync_articles} ||= "";
    $xml->{sync_clients} ||= "";
    $xml->{sync_warehouses} ||= "";

    $s->debug("##> ".$xml->{weckoffice}->{store}. " | from: ".$xml->{integrate_from}." -> to: ".$xml->{integrate_to}." -> days: ".$xml->{integrate_days});

    my %doc_mappings = ();
    $xml->{doc_mappings} ||= "";

    if ($xml->{doc_mappings}) {
      my @toks = split /,/, $xml->{doc_mappings};
      %doc_mappings = map { /^(.*)=(\d+)$/; $1 => $2; } @toks;
    }

    if (! $xml->{database_pri_integrator} || ! -e $xml->{database_pri_integrator}) {
      die("Missing 'database_pri_integrator' or the file it points to don't exists...");
    }

    my $dsn2 = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$xml->{database_pri_integrator};
    my $dbh2 = DBI->connect($dsn2) or die("Can't connect to database: $@");
    my $db_pri = WO::DB::Access->new( dbh => $dbh2 );

    my $res = [];

    if ($xml->{sync_warehouses}) {
    #
    # Get Warehouses
    #

    $s->debug("Will get 'Warehouses'");

    $s->debug("*> BackofficeGetWarehouses()");

    eval {
      $res = $s->weckoffice_send_data( { action => 'BackofficeGetWarehouses',
                                       }
                                     );
    };

    if ($@) {
      die("Error sending file: $@");
    } else {
      my $VAR1;
      eval "$res";
      $VAR1 ||= [];

      if (scalar @$VAR1) {
        $db_pri->do("DELETE FROM Armazens");

        my %conversions = ();

        for (@$VAR1) {
          my $active = $_->{can_sell} == 1 ? -1 : 0;

          $conversions{ $_->{stock_store_fo_id} } = 1 if $_->{stock_store_fo_id} && $_->{can_sell};

          $db_pri->do(
            "INSERT INTO Armazens (Codigo, Descricao, Conversao, Activo) VALUES (?, ?, ?, ?)",
            $_->{fo_id}, $_->{name}, $_->{stock_store_fo_id} || $_->{fo_id}, $active
          );

        }

        for (keys %conversions) {
          $db_pri->do("UPDATE Armazens SET Activo = -1 WHERE Codigo = ?", $_);
        }
      }
    }
    }

    if ($xml->{sync_clients}) {
    #
    # Get Clients
    #

    $s->debug("*> BackofficeGetClients()");
    eval {
      $res = $s->weckoffice_send_data( { action => 'BackofficeGetClients',
                                       }
                                     );
    };

    if ($@) {
      die("Error sending file: $@");
    } else {
      my $VAR1;
      eval "$res";
      $VAR1 ||= [];

      $s->debug("Will insert 'Clients' -> ".scalar @$VAR1);

      if (scalar @$VAR1) {
        $db_pri->do("DELETE FROM Clientes");

        my %control = ();

        for (@$VAR1) {
          $_->{obs} ||= "";
          my $fo_id = $_->{obs} ? $_->{obs} : $_->{fo_id};
          $fo_id = '0' unless defined $fo_id && $fo_id ne '';

          next if exists $control{ $fo_id };
          $db_pri->do("INSERT INTO Clientes (Codigo, Nome) VALUES (?, ?)", $fo_id, $_->{name});
          $control{ $fo_id } = 1;
        }
      }
    }
    }

    if ($xml->{sync_articles}) {
    #
    # Get Articles
    #

    $s->debug("*> BackofficeGetArticlesWithoutStoreCheck()");

    eval {
      $res = $s->weckoffice_send_data( { action => 'BackofficeGetArticlesWithoutStoreCheck',
                                       }
                                     );
    };

    if ($@) {
      die("Error sending file: $@");
    } else {
      my $VAR1;
      eval "$res";
      $VAR1 ||= [];

      $s->debug("Will insert 'Articles' -> ".scalar(@$VAR1));

      if (scalar @$VAR1) {
        $db_pri->do("DELETE FROM Artigos");

        for (@$VAR1) {
          $db_pri->do("INSERT INTO Artigos (Codigo, Descricao) VALUES (?, ?)", $_->{fo_id}, $_->{name});
        }
      }
    }
    }

    #
    # Get Stocks
    #

    if ($xml->{sync_stocks_information}) {
      my @exported_stocks = $db_pri->select("SELECT Documento, DataDoc, Armazem FROM Movstocks");
      my %exported_stocks = map { $_->{Documento}.$_->{DataDoc}.$_->{Armazem} => 1 } @exported_stocks;

      $s->debug("*> BackofficeStocksBySellMovements()");

      eval {
        $res = $s->weckoffice_send_data( { action => 'BackofficeStocksBySellMovements',
                                           days => $xml->{integrate_days},
                                           from => $xml->{integrate_from},
                                           to   => $xml->{integrate_to},
                                         }
                                       );
      };

      if ($@) {
        die("Error sending file: $@");
      } else {
        my $VAR1;
        eval "$res";
        $VAR1 ||= [];

        $s->debug("Will insert 'Stocks' -> ".scalar(@$VAR1));

        my $query = qq{
          INSERT INTO MovStocks
            (Documento, NumDoc, DataDoc, Artigo, Armazem, Qtd, CustoUnitario, CustoTotal)
          VALUES (?, ?, ?, ?, ?, ?, ?, ?)
        };

        for (@$VAR1) {
          my $is_offer = $_->{is_offer} || 0;

          my $doc;

          my $cost_to_use = $is_offer ? $_->{cost_pcu} : $_->{cost_pcm};

          if ($is_offer && exists $doc_mappings{OFFERS}) {
            $doc = $doc_mappings{OFFERS};
          } elsif (exists $doc_mappings{$_->{movement_type}}) {
            $doc = $doc_mappings{$_->{movement_type}};
          } else {
            $doc = $_->{movement_type};
          }

          $_->{date} = $_->{date}." 00:00:00";
          next if exists $exported_stocks{$doc.$_->{date}.$_->{store_fo_id}};

          my $qtd = sprintf("%.4f", $_->{f_qtty}); $qtd =~ s/\./,/g;
          my $price = sprintf("%.6f", $_->{price}); $price =~ s/\./,/g;
          my $cost = sprintf("%.6f", $cost_to_use); $cost =~ s/\./,/g;

          $db_pri->do(
            $query,
            $doc, $_->{fo_doc_number}, $_->{date}, $_->{article_fo_id}, $_->{store_fo_id}, $qtd, $price, $cost
          );
        }
      }
    }

    #
    # Sells information
    #

    if ($xml->{sync_sells_information}) {
      my @exported_sales = $db_pri->select("SELECT Documento, DataVnd, Armazem FROM Vendas");
      my %exported_sales = map { $_->{Documento}.$_->{DataVnd}.$_->{Armazem} => 1 } @exported_sales;

      my @exported_cco = $db_pri->select("SELECT Documento, DataDoc, Armazem, NumDoc FROM VendasCCO");
      my %exported_cco = map { $_->{Documento}.$_->{DataDoc}.$_->{Armazem}.$_->{NumDoc} => 1 } @exported_cco;

      my @exported_pendentes = $db_pri->select("SELECT TipoDoc, DataDoc, Armazem, NumDoc FROM Pendentes");
      my %exported_pendentes = map { $_->{TipoDoc}.$_->{DataDoc}.$_->{Armazem}.$_->{NumDoc} => 1 } @exported_pendentes;

      #
      # Get FTC's
      #

      $s->debug("*> BackofficeSalesByMovementType(2,5)");

      eval {
        $res = $s->weckoffice_send_data( { action => 'BackofficeSalesByMovementType',
                                           movement_types => "2,5",
                                           days => $xml->{integrate_days},
                                           from => $xml->{integrate_from},
                                           to   => $xml->{integrate_to},
                                         }
                                       );
      };

      if ($@) {
        die("Error sending file: $@");
      } else {
        my $VAR1;
        eval "$res";
        $VAR1 ||= [];

        my $query_cco = qq{
          INSERT INTO VendasCCO
            (Cliente, Documento, NumDoc, DataDoc, Mesa, Descricao, Quantidade, Total, TipoLinha, NumClientes, Funcionario, HoraI,
             HoraF, Exportado, CodPedido, Armazem)
          VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
        };

        my $query_pendentes = qq{
          INSERT INTO Pendentes
            (TipoEntidade, Entidade, TipoDoc, NumDoc, Cambio, DataDoc, DataVenc, Moeda, ValorTotal, ValorPendente, Exportado, Armazem)
          VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
        };

        my $query_vendas = qq{
          INSERT INTO Vendas
            (Documento, NumDoc, DataVnd, Artigo, TipoLinha, Quantidade, Total, Custo, NumClientes, Funcionario, HoraI, HoraF, TaxaIva,
             ConsumoInterno, Mesa, ExportCBL, Armazem, IdPostoPRI, Exportado, Contador)
          VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
        };

        my @info = ();
        my @info_cco = ();
        my %info_pendentes = ();

        for (@$VAR1) {
          my $key = $_->{movement_type}.$_->{article_fo_id}.$_->{date}.$_->{fo_doc_number};

          my $info1 = {};
          my $info2 = {};
          my $info3 = {};

          $info1->{Documento}      = exists $doc_mappings{$_->{movement_type}} ? $doc_mappings{$_->{movement_type}} : $_->{movement_type};
          $info1->{NumDoc}         = $_->{fo_doc_number};
          $info1->{DataVnd}        = $_->{date}." 00:00:00";
          $info1->{Artigo}         = $_->{article_fo_id};
          $info1->{TipoLinha}      = 3;
          $info1->{Quantidade}     = sprintf("%.4f", $_->{quantity});
          $info1->{Total}          = sprintf("%.6f", ($_->{price} - $_->{offer} - $_->{discount}));
          $info1->{Custo}          = $_->{article_pcm};
          $info1->{NumClientes}    = 1;
          $info1->{Funcionario}    = $_->{user_login};
          $info1->{HoraI}          = $_->{hour};
          $info1->{HoraF}          = $_->{hour};
          $info1->{TaxaIva}        = int($_->{tax_value});
          $info1->{ConsumoInterno} = $_->{movement_type} && $_->{movement_type} == 4 ? 1 : 0;
          $info1->{Mesa}           = 1;
          $info1->{ExportCBL}      = 0;
          $info1->{Armazem}        = $_->{store_fo_id};
          $info1->{IdPostoPRI}     = 0;
          $info1->{Exportado}      = 0;
          $info1->{Contador}       = 0;

          push @info, $info1;

          my $client_fo_id = $_->{client_obs} ? $_->{client_obs} : $_->{client_fo_id};

          $info2->{Cliente}     = $client_fo_id;
          $info2->{Documento}   = exists $doc_mappings{$_->{movement_type}} ? $doc_mappings{$_->{movement_type}} : $_->{movement_type};
          $info2->{NumDoc}      = $_->{fo_doc_number};
          $info2->{DataDoc}     = $_->{date}." 00:00:00";
          $info2->{Descricao}   = $_->{article_name};
          $info2->{Quantidade}  = sprintf("%.4f", $_->{quantity});
          $info2->{Total}       = sprintf("%.6f", ($_->{price} - $_->{offer} - $_->{discount}));
          $info2->{Funcionario} = $_->{user_login};
          $info2->{HoraI}       = $_->{hour};
          $info2->{HoraF}       = $_->{hour};
          $info2->{Exportado}   = 0;
          $info2->{TipoLinha}   = 3;
          $info2->{CodPedido}   = 0;
          $info2->{Mesa}        = 1;
          $info2->{NumClientes} = 1;
          $info2->{Armazem}     = $_->{store_fo_id};

          push @info_cco, $info2;

          my $key_pendentes = $_->{fo_doc_number}.$_->{date};

          $info_pendentes{$key_pendentes}->{TipoEntidade} = "C";
          $info_pendentes{$key_pendentes}->{TipoDoc}      = exists $doc_mappings{$_->{movement_type}} ? $doc_mappings{$_->{movement_type}} : $_->{movement_type};
          $info_pendentes{$key_pendentes}->{NumDoc}       = $_->{fo_doc_number};
          $info_pendentes{$key_pendentes}->{DataDoc}      = $_->{date}." 00:00:00";
          $info_pendentes{$key_pendentes}->{DataVenc}     = $_->{date}." 00:00:00";
          $info_pendentes{$key_pendentes}->{Exportado}    = 0;
          $info_pendentes{$key_pendentes}->{Armazem}      = $_->{store_fo_id};
          $info_pendentes{$key_pendentes}->{Cambio}       = "1";
          $info_pendentes{$key_pendentes}->{Moeda}        = "EUR";
          $info_pendentes{$key_pendentes}->{Entidade}     = $client_fo_id;
          $info_pendentes{$key_pendentes}->{ValorTotal}    += (($_->{price} - $_->{offer} - $_->{discount}) * $_->{quantity});
          $info_pendentes{$key_pendentes}->{ValorPendente} += (($_->{price} - $_->{offer} - $_->{discount}) * $_->{quantity});
        }

        $s->debug("Will insert 'Vendas' -> ".scalar(@info));

        for (@info) {
          next if exists $exported_sales{$_->{Documento}.$_->{DataVnd}.$_->{Armazem}};

          my $Quantidade = sprintf("%.4f", $_->{Quantidade}); $Quantidade =~ s/\./,/;
          my $Total = sprintf("%.6f", $_->{Total}) || 0; $Total =~ s/\./,/;
          my $Custo = sprintf("%.6f", $_->{Custo}) || 0; $Custo =~ s/\./,/;

          $db_pri->do(
            $query_vendas,
            $_->{Documento}, $_->{NumDoc}, $_->{DataVnd}, $_->{Artigo}, $_->{TipoLinha}, $Quantidade, $Total,
            $Custo, $_->{NumClientes}, $_->{Funcionario}, $_->{HoraI}, $_->{HoraF}, $_->{TaxaIva},
            $_->{ConsumoInterno}, $_->{Mesa}, $_->{ExportCBL}, $_->{Armazem}, $_->{IdPostoPRI}, $_->{Exportado}, $_->{Contador}
          );
        }

        $s->debug("Will insert 'VendasCCO' -> ".scalar(@info_cco));

        for (@info_cco) {
          next if exists $exported_cco{$_->{Documento}.$_->{DataDoc}.$_->{Armazem}.$_->{NumDoc}};

          my $Quantidade = sprintf("%.4f", $_->{Quantidade}); $Quantidade =~ s/\./,/;
          my $Total = sprintf("%.6f", $_->{Total}) || 0; $Total =~ s/\./,/;

          $db_pri->do(
            $query_cco,
            $_->{Cliente}, $_->{Documento}, $_->{NumDoc}, $_->{DataDoc}, $_->{Mesa}, $_->{Descricao}, $Quantidade,
            $Total, $_->{TipoLinha}, $_->{NumClientes}, $_->{Funcionario}, $_->{HoraI}, $_->{HoraF}, $_->{Exportado}, $_->{CodPedido},
            $_->{Armazem}
          );

        }

        $s->debug("Will insert 'Pendentes' -> ".scalar(values(%info_pendentes)));

        for (values %info_pendentes) {
          next if exists $exported_pendentes{$_->{TipoDoc}.$_->{DataDoc}.$_->{Armazem}.$_->{NumDoc}};

          my $ValorPendente = sprintf("%.6f", $_->{ValorPendente}); $ValorPendente =~ s/\./,/;
          my $ValorTotal = sprintf("%.6f", $_->{ValorTotal}); $ValorTotal =~ s/\./,/;

          $db_pri->do(
            $query_pendentes,
            $_->{TipoEntidade}, $_->{Entidade}, $_->{TipoDoc}, $_->{NumDoc}, $_->{Cambio}, $_->{DataDoc}, $_->{DataVenc},
            $_->{Moeda}, $ValorTotal, $ValorPendente, $_->{Exportado}, $_->{Armazem}
          );
        }
      }

      #
      # Auto-CS
      #

      $s->debug("*> BackofficeSalesByMovementType(4)");

      eval {
        $res = $s->weckoffice_send_data( { action => 'BackofficeSalesByMovementType',
                                           movement_types => "4",
                                           days => $xml->{integrate_days},
                                           from => $xml->{integrate_from},
                                           to   => $xml->{integrate_to},
                                         }
                                       );
      };

      if ($@) {
        die("Error sending file: $@");
      } else {
        my $VAR1;
        eval "$res";
        $VAR1 ||= [];

        my $query = qq{
          INSERT INTO Vendas
            (Documento, NumDoc, DataVnd, Artigo, TipoLinha, Quantidade, Total, Custo, NumClientes, Funcionario, HoraI, HoraF, TaxaIva,
             ConsumoInterno, Mesa, ExportCBL, Armazem, IdPostoPRI, Exportado, Contador)
          VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
        };

        my @info;

        for (@$VAR1) {
          push @info, {
            Documento      => exists $doc_mappings{$_->{movement_type}} ? $doc_mappings{$_->{movement_type}} : $_->{movement_type},
            NumDoc        => $_->{fo_doc_number},
            DataVnd        => $_->{date}." 00:00:00",
            Artigo        => $_->{article_fo_id},
            TipoLinha     => 3,
            Quantidade    => sprintf("%.4f", $_->{quantity}),
            Total         => sprintf("%.6f", ($_->{price} - $_->{offer} - $_->{discount})),
            Custo        => $_->{article_pcm},
            NumClientes    => 1,
            Funcionario   => $_->{user_login},
            HoraI          => $_->{hour},
            HoraF          => $_->{hour},
            TaxaIva        => int($_->{tax_value}),
            ConsumoInterno => 1,
            Mesa           => 1,
            ExportCBL      => 0,
            Armazem        => $_->{store_fo_id},
            IdPostoPRI     => 0,
            Exportado      => 0,
            Contador       => 0,
          }
        }

        $s->debug("Will insert 'Vendas' (auto cs, type: 4) -> ".scalar(@$VAR1));

        for (@info) {
          next if exists $exported_sales{$_->{Documento}.$_->{DataVnd}.$_->{Armazem}};

          my $Quantidade = $_->{Quantidade}; $Quantidade =~ s/\./,/;
          my $Total = $_->{Total} || 0; $Total =~ s/\./,/;
          my $Custo = $_->{Custo} || 0; $Custo =~ s/\./,/;

          $db_pri->do(
            $query,
            $_->{Documento}, $_->{NumDoc}, $_->{DataVnd}, $_->{Artigo}, $_->{TipoLinha}, $Quantidade, $Total,
            $Custo, $_->{NumClientes}, $_->{Funcionario}, $_->{HoraI}, $_->{HoraF}, $_->{TaxaIva},
            $_->{ConsumoInterno}, $_->{Mesa}, $_->{ExportCBL}, $_->{Armazem}, $_->{IdPostoPRI}, $_->{Exportado}, $_->{Contador}
          );
        }
      }

      #
      # VD's
      #

      $s->debug("*> BackofficeSalesByMovementType(3,1,6,0)");

      eval {
        $res = $s->weckoffice_send_data( { action => 'BackofficeSalesByMovementType',
                                           movement_types => "3,1,6,0",
                                           days => $xml->{integrate_days},
                                           from => $xml->{integrate_from},
                                           to   => $xml->{integrate_to},
                                         }
                                       );
      };

      if ($@) {
        die("Error sending file: $@");
      } else {
        my $VAR1;
        eval "$res";
        $VAR1 ||= [];

        my $query = qq{
          INSERT INTO Vendas
            (Documento, NumDoc, DataVnd, Artigo, TipoLinha, Quantidade, Total, Custo, NumClientes, Funcionario, HoraI, HoraF, TaxaIva,
             ConsumoInterno, Mesa, ExportCBL, Armazem, IdPostoPRI, Exportado, Contador)
          VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
        };

        my @info = ();

        for (@$VAR1) {
          my $is_offer = $_->{price} > 0 && $_->{offer} eq $_->{price} ? 1 : 0;

          my $doc;

          if ($is_offer && exists $doc_mappings{OFFERS}) {
            $doc = $doc_mappings{OFFERS};
          } elsif (exists $doc_mappings{$_->{movement_type}}) {
            $doc = $doc_mappings{$_->{movement_type}};
          } else {
            $doc = $_->{movement_type};
          }

          push @info, {
            Documento      => $doc,
            NumDoc         => $_->{fo_doc_number},
            DataVnd        => $_->{date}." 00:00:00",
            Artigo         => $_->{article_fo_id},
            TipoLinha      => 3,
            Quantidade     => sprintf("%.4f", $_->{quantity}),
            Total          => sprintf("%.6f", ($_->{price} - $_->{offer} - $_->{discount})),
            Custo          => $_->{article_pcm},
            NumClientes    => 1,
            Funcionario    => $_->{user_login},
            HoraI          => $_->{hour},
            HoraF          => $_->{hour},
            TaxaIva        => int($_->{tax_value}),
            ConsumoInterno => 0,
            Mesa           => 1,
            ExportCBL      => 0,
            Armazem        => $_->{store_fo_id},
            IdPostoPRI     => 0,
            Exportado      => 0,
            Contador       => 0,
          }
        }
    
        $s->debug("Will insert 'Vendas' (types: 3, 1, 6,0) -> ".scalar(@$VAR1));

        for (@info) {
          next if exists $exported_sales{$_->{Documento}.$_->{DataVnd}.$_->{Armazem}};

          my $Quantidade = $_->{Quantidade}; $Quantidade =~ s/\./,/;
          my $Total = $_->{Total} || 0; $Total =~ s/\./,/;
          my $Custo = $_->{Custo} || 0; $Custo =~ s/\./,/;

          $db_pri->do(
            $query,
            $_->{Documento}, $_->{NumDoc}, $_->{DataVnd}, $_->{Artigo}, $_->{TipoLinha}, $Quantidade, $Total,
            $Custo, $_->{NumClientes}, $_->{Funcionario}, $_->{HoraI}, $_->{HoraF}, $_->{TaxaIva},
            $_->{ConsumoInterno}, $_->{Mesa}, $_->{ExportCBL}, $_->{Armazem}, $_->{IdPostoPRI}, $_->{Exportado}, $_->{Contador}
          );
        }
      }
    } # Sells if
  }
  
  #
  # Normal Run mode (when we don't use integrate_weckoffice_data = 1 in the XML file)
  #

  else {
    my $dbh = DBI->connect('dbi:ODBC:'.$xml->{odbc_name}, $xml->{odbc_login}, $xml->{odbc_passwd})
      or die("Can't connect to the ODBC! source name: ".$xml->{odbc_name}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

    my $db = WO::DB::ODBC->new( dbh => $dbh );

    my %users = ();

    if ($xml->{sync_taxes}) {
      my @res = $db->select("SELECT Iva, Descricao, Taxa FROM Iva");
      write_file( $file_taxes, Dumper(\@res) );
    }

    if ($xml->{sync_warehouses}) {
      my @warehouses = $db->select("SELECT Armazem, Telefone, Localidade, Morada, Descricao, CpLocalidade, Fax, Cp FROM Armazens");
      write_file( $file_warehouses, Dumper(\@warehouses) );
    }

    if ($xml->{sync_suppliers}) {
      my @suppliers = $db->select("SELECT Fornecedor, Nome, Morada, Morada, Local, Cp, CpLoc, Tel, Fax, NumContrib, EnderecoWeb FROM Fornecedores");
      open(F, "> $file_suppliers") or die "Can't write to $file_suppliers\n";
      print F Dumper(\@suppliers);
      close(F);
#      write_file( $file_suppliers, Dumper(\@suppliers) );
    }

    my $sub = $db->select_iter(qq{
      SELECT DISTINCT Artigo, PCM, CAST(STR(YEAR(Data)) + '/' + STR(MONTH(Data)) + '/' + STR(DAY(Data)) AS DATETIME) AS Data
      FROM         LinhasSTK
      WHERE     (Artigo IS NOT NULL)
      ORDER BY Artigo, Data
    });
    $s->write_csv_from_iter( $file_pcms, $sub );

    if ($xml->{sync_supplier_articles}) {
      my @res = $db->select("SELECT Artigo, Fornecedor, ReferenciaFor, PrCustoUltimo, UnidadeCompra, CodBarras FROM ArtigoFornecedor");
      open(F, "> $file_supp_art") or die "Can't write to $file_supp_art\n";
      print F Dumper(\@res);
      close(F);
#      write_file( $file_supp_art, Dumper(\@res) );
    }

    if ($xml->{sync_units}) {
      my @res = $db->select("SELECT Unidade, Descricao, Arredondamento FROM Unidades");
      open(F, "> $file_units") or die "Can't write to $file_units\n";
      print F Dumper(\@res);
      close(F);
#      write_file( $file_units, Dumper(\@res) );
    }

    if ($xml->{sync_articles}) {
      my @res = $db->select(qq{
        SELECT Artigo, Descricao, CodBarras, UnidadeBase, Iva, STKMinimo, STKMaximo,
               PCMedio, PCUltimo, Familia, SubFamilia, CDU_IVA1, CDU_IVA2,
               CDU_PrecoPOS, CDU_Talao, CDU_Botoes, CDU_Teclado
        FROM Artigo
      });
      open(F, "> $file_articles") or die "Can't write to $file_articles\n";
      print F Dumper(\@res);
      close(F);
#      write_file( $file_articles, Dumper(\@res) );
    }

    if ($xml->{sync_families}) {
      my @res = $db->select(qq{
        SELECT f.Descricao AS TopDescricao, s.Descricao AS SubDescricao, f.Familia As TopCode, s.SubFamilia AS SubCode,
               s.Ordem
          FROM Familias f, SubFamilias s
         WHERE f.Familia = s.Familia
         ORDER BY s.Ordem
      });
      open(F, "> $file_families") or die "Can't write to $file_families\n";
      print F Dumper(\@res);
      close(F);
#      write_file( $file_families, Dumper(\@res) );
    }

    if ($xml->{sync_document_types}) {
      my @res1 = $db->select("SELECT Documento, Descricao FROM DocumentosCompra");
      my @res2 = $db->select("SELECT Documento, Descricao FROM DocumentosStk");
      my @res3 = $db->select("SELECT Documento, Descricao FROM DocumentosVenda");

      open(F, "> $file_doc_types1") or die "Can't write to $file_doc_types1\n";
      print F Dumper(\@res1);
      close(F);
      open(F, "> $file_doc_types2") or die "Can't write to $file_doc_types2\n";
      print F Dumper(\@res2);
      close(F);
      open(F, "> $file_doc_types3") or die "Can't write to $file_doc_types3\n";
      print F Dumper(\@res3);
      close(F);
#      write_file( $file_doc_types1, Dumper(\@res1) );
#      write_file( $file_doc_types2, Dumper(\@res2) );
#      write_file( $file_doc_types3, Dumper(\@res3) );
    }

    if ($xml->{sync_payment_types}) {
      my @res = $db->select("SELECT CondPag, Descricao, Dias FROM CondPag");
      open(F, "> $file_pay_cond") or die "Can't write to $file_pay_cond\n";
      print F Dumper(\@res);
      close(F);
#      write_file( $file_pay_cond, Dumper(\@res) );
    }

    if ($xml->{sync_clients}) {
      my @res = $db->select("SELECT Cliente, Nome, Fac_Mor, Fac_Local, Fac_Cp, Fac_Cploc, Fac_Tel, Fac_Fax, LimiteCred, TotalDeb, NumContrib FROM Clientes");
      my @res2 = $db->select("SELECT Terceiro AS Cliente, Nome, Morada AS Fac_Mor, Local AS Fac_Local, Cp AS Fac_Cp, Cploc AS Fac_Cploc, Tel AS Fac_Tel, Fax AS Fac_Fax, LimiteCred, TotalDeb, NumContrib FROM OutrosTerceiros WHERE DevCred = 'D'");

      my @final = ( @res, @res2 );

      open(F, "> $file_clients") or die "Can't write to $file_clients\n";
      print F Dumper(\@final);
      close(F);
#      write_file( $file_clients, Dumper(\@final) );
    }

    if ($xml->{sync_offers_in}) {
      my %info = ();
      
      # process offers_in
      my @res = $db->select(qq{
        SELECT cs.TipoDoc, cs.NumDoc, cs.Filial, cs.Serie, ls.Armazem, ls.NumLinha,
               cs.Data, ls.Artigo, ls.Quantidade, ls.EntradaSaida, ls.Unidade, cs.Utilizador,
               ls.TaxaIva, ls.PrecUnit, ls.Arred
          FROM CabecSTK cs, LinhasSTK ls
         WHERE cs.TipoDoc = ls.TipoDoc AND cs.NumDoc = ls.NumDoc AND cs.Filial = ls.Filial
               AND cs.Serie = ls.Serie AND (ls.TipoDoc = 'OFS' OR ls.TipoDoc = 'OFE')
      ORDER BY ls.NumDoc, ls.NumLinha
      });
      
      my $to_w = undef;

      foreach my $r (@res) {
        my $key = $r->{TipoDoc} . $r->{NumDoc} . $r->{Filial} . $r->{Serie};

        $to_w = $r->{Armazem};

        $users{ $r->{Utilizador} } = 1;

        my $info = { to => $to_w,
                     date => $r->{Data},
                     user => $r->{Utilizador},
                     fo_id => $r->{TipoDoc}.$r->{NumDoc},
                     doc_type => $r->{TipoDoc},
                     obs => 'TipoDoc: '.$r->{TipoDoc} . ' | NumDoc: ' . $r->{NumDoc} . ' | Filial: '. $r->{Filial} . ' | Serie: '.$r->{Serie} . ' | Data: '.$r->{Data},
                   };

        $info{ $key }->{info} = $info;

        if (! defined $r->{Artigo}) {
          push @{ $info{ $key }->{errors} }, 'Missing article';
        } elsif (! defined $r->{Armazem}) {
          push @{ $info{ $key }->{errors} }, 'Missing warehouse';
        } else {
          delete $r->{Armazem};
          delete $r->{Utilizador};
          delete $r->{Data};

          next if exists $info{ $key }->{art_check}->{$r->{Artigo}};
          $info{ $key }->{art_check}->{$r->{Artigo}} = 1;
          push @{ $info{ $key }->{articles} }, $r;
        }      
      }

      delete $_->{art_check} for values %info;

      my @offers_in = values %info;
      open(F, "> $file_offers_in") or die "Can't write to $file_offers_in\n";
      print F Dumper(\@offers_in);
      close(F);
#      write_file( $file_offers_in, Dumper(\@offers_in) );
    }

    if ($xml->{sync_breaks}) {
      my %info = ();
      
      # process breaks
      my @res = $db->select(qq{
        SELECT cs.TipoDoc, cs.NumDoc, cs.Filial, cs.Serie, ls.Armazem, ls.NumLinha,
               cs.Data, ls.Artigo, ls.Quantidade, ls.EntradaSaida, ls.Unidade, cs.Utilizador,
               ls.TaxaIva, ls.PrecUnit, ls.Arred
          FROM CabecSTK cs, LinhasSTK ls
         WHERE cs.TipoDoc = ls.TipoDoc AND cs.NumDoc = ls.NumDoc AND cs.Filial = ls.Filial
               AND cs.Serie = ls.Serie AND (ls.TipoDoc = 'ACS' OR ls.TipoDoc = 'CS' OR ls.TipoDoc = 'CSI' OR ls.TipoDoc = 'CSP')
      ORDER BY ls.NumDoc, ls.NumLinha
      });
      
      my $from_w = undef;

      foreach my $r (@res) {
        my $key = $r->{TipoDoc} . $r->{NumDoc} . $r->{Filial} . $r->{Serie};

        $from_w = $r->{Armazem};

        $users{ $r->{Utilizador} } = 1;

        my $info = { from => $from_w,
                     fo_id => $r->{TipoDoc}.$r->{NumDoc},
                     date => $r->{Data},
                     user => $r->{Utilizador},
                     doc_type => $r->{TipoDoc},
                     obs => 'TipoDoc: '.$r->{TipoDoc} . ' | NumDoc: ' . $r->{NumDoc} . ' | Filial: '. $r->{Filial} . ' | Serie: '.$r->{Serie} . ' | Data: '.$r->{Data},
                   };

        $info{ $key }->{info} = $info;

        if (! defined $r->{Artigo}) {
          push @{ $info{ $key }->{errors} }, 'Missing article';
        } elsif (! defined $r->{Armazem}) {
          push @{ $info{ $key }->{errors} }, 'Missing warehouse';
        } else {
          delete $r->{Armazem};
          delete $r->{Utilizador};
          delete $r->{Data};

          next if exists $info{ $key }->{art_check}->{$r->{Artigo}};
          $info{ $key }->{art_check}->{$r->{Artigo}} = 1;
          push @{ $info{ $key }->{articles} }, $r;
        }      
      }

      delete $_->{art_check} for values %info;

      my @breaks = values %info;
      open(F, "> $file_breaks") or die "Can't write to $file_breaks\n";
      print F Dumper(\@breaks);
      close(F);
#      write_file( $file_breaks, Dumper(\@breaks) );
    }

    if ($xml->{sync_inventories}) {
      my %info = ();
      
      # process inventories
      my @res = $db->select(qq{
        SELECT cs.TipoDoc, cs.NumDoc, cs.Filial, cs.Serie, ls.Armazem, ls.NumLinha,
               cs.Data, ls.Artigo, ls.Quantidade, ls.EntradaSaida, ls.Unidade, cs.Utilizador,
               ls.TaxaIva, ls.PrecUnit, ls.Arred
          FROM CabecSTK cs, LinhasSTK ls
         WHERE cs.TipoDoc = ls.TipoDoc AND cs.NumDoc = ls.NumDoc AND cs.Filial = ls.Filial
               AND cs.Serie = ls.Serie AND ls.TipoDoc = 'SI'
      ORDER BY ls.NumDoc, ls.NumLinha
      });
      
      my $to_w = undef;

      foreach my $r (@res) {
        my $key = $r->{TipoDoc} . $r->{NumDoc} . $r->{Filial} . $r->{Serie};

        $to_w = $r->{Armazem};

        $users{ $r->{Utilizador} } = 1;

        my $info = { to => $to_w,
                     fo_id => $r->{TipoDoc}.$r->{NumDoc},
                     date => $r->{Data},
                     user => $r->{Utilizador},
                     doc_type => $r->{TipoDoc},
                     obs => 'TipoDoc: '.$r->{TipoDoc} . ' | NumDoc: ' . $r->{NumDoc} . ' | Filial: '. $r->{Filial} . ' | Serie: '.$r->{Serie} . ' | Data: '.$r->{Data},
                   };

        $info{ $key }->{info} = $info;

        if (! defined $r->{Artigo}) {
          push @{ $info{ $key }->{errors} }, 'Missing article';
        } elsif (! defined $r->{Armazem}) {
          push @{ $info{ $key }->{errors} }, 'Missing warehouse';
        } else {
          delete $r->{Armazem};
          delete $r->{Utilizador};
          delete $r->{Data};

          next if exists $info{ $key }->{art_check}->{$r->{Artigo}};
          $info{ $key }->{art_check}->{$r->{Artigo}} = 1;
          push @{ $info{ $key }->{articles} }, $r;
        }      
      }

      delete $_->{art_check} for values %info;

      my @inventories = values %info;
      open(F, "> $file_inventories") or die "Can't write to $file_inventories\n";
      print F Dumper(\@inventories);
      close(F);
#      write_file( $file_inventories, Dumper(\@inventories) );
    }

    if ($xml->{sync_transfers}) {
      my %info = ();
      
      # process transfers
      my @res = $db->select(qq{
        SELECT cs.TipoDoc, cs.NumDoc, cs.Filial, cs.Serie, ls.Armazem, ls.NumLinha,
               cs.Data, ls.Artigo, ls.Quantidade, ls.EntradaSaida, ls.Unidade, cs.Utilizador,
               ls.Arred
          FROM CabecSTK cs, LinhasSTK ls
         WHERE cs.TipoDoc = ls.TipoDoc AND cs.NumDoc = ls.NumDoc AND cs.Filial = ls.Filial
               AND cs.Serie = ls.Serie AND ls.TipoDoc = 'TRA'
      ORDER BY ls.NumDoc, ls.NumLinha
      });
      
      foreach my $r (@res) {
        my $key = $r->{TipoDoc} . $r->{NumDoc} . $r->{Filial} . $r->{Serie};

        # get info from cache
        my $info = $info{ $key }->{info};
        my $add = 0;

        if (! defined $info) {
          $add = 1;
          $info = { fo_id => $r->{TipoDoc}.$r->{NumDoc},
                    date => $r->{Data},
                    user => $r->{Utilizador},
                    obs => 'TipoDoc: '.$r->{TipoDoc} . ' | NumDoc: ' . $r->{NumDoc} . ' | Filial: '. $r->{Filial} . ' | Serie: '.$r->{Serie} . ' | Data: '.$r->{Data},
                  };
        }

        if ($r->{EntradaSaida} eq 'S') {
          $info->{from} = $r->{Armazem};
        } elsif ($r->{EntradaSaida} eq 'E') {
          $info->{to} = $r->{Armazem};
        }

        $users{ $r->{Utilizador} } = 1;

        $info{ $key }->{info} = $info if $add == 1;

        if (! defined $r->{Artigo}) {
          push @{ $info{ $key }->{errors} }, 'Missing article';
        } elsif (! defined $r->{Armazem}) {
          push @{ $info{ $key }->{errors} }, 'Missing warehouse';
        } else {
          delete $r->{Armazem};
          delete $r->{Utilizador};
          delete $r->{Data};

          next if exists $info{ $key }->{art_check}->{$r->{Artigo}};
          $info{ $key }->{art_check}->{$r->{Artigo}} = 1;
          push @{ $info{ $key }->{articles} }, $r;
        }      
      }

      delete $_->{art_check} for values %info;

      my @transfers = values %info;
      open(F, "> $file_transfers") or die "Can't write to $file_transfers\n";
      print F Dumper(\@transfers);
      close(F);
#      write_file( $file_transfers, Dumper(\@transfers) );
    }

    if ($xml->{sync_receptions}) {
      my %info = ();

      # process receptions
      my @res = $db->select(qq{
        SELECT CabecCompras.TipoDoc, CabecCompras.NumDoc, CabecCompras.Entidade, CabecCompras.DataDoc, CabecCompras.TotalMerc, 
               CabecCompras.TotalIva, CabecCompras.TotalDesc, CabecCompras.TotalOutros, CabecCompras.TotalDespesasAdicionais, CabecCompras.Moeda, 
               CabecCompras.Id, CabecCompras.CDU_ExpPRI, LinhasCompras.NumLinha, LinhasCompras.Artigo, LinhasCompras.Desconto1, 
               LinhasCompras.Desconto2, LinhasCompras.Desconto3, LinhasCompras.TaxaIva, LinhasCompras.Quantidade, LinhasCompras.PrecUnit, 
               LinhasCompras.DataDoc, LinhasCompras.Armazem, LinhasCompras.CustoAdicionalManual, LinhasCompras.DescontoComercial,
               LinhasCompras.PrecoLiquido, LinhasCompras.IdCabecCompras, LinhasCompras.Unidade, CabecCompras.Utilizador, CabecCompras.Filial,
               CabecCompras.Serie, CabecCompras.DataDoc, CabecCompras.CondPag, CabecCompras.DataVencimento, CabecCompras.NumDocExterno,
               CabecCompras.Arredondamento, CabecCompras.ArredondamentoIva, CabecCompras.Id
          FROM CabecCompras INNER JOIN LinhasCompras ON CabecCompras.Id = LinhasCompras.IdCabecCompras
         WHERE ((CabecCompras.TipoDoc = 'GTR') OR (CabecCompras.TipoDoc = 'GTV') OR (CabecCompras.TipoDoc = 'PRO'))
      ORDER BY LinhasCompras.NumLinha
      });

      my $to_w = undef;

      foreach my $r (@res) {
        my $key = $r->{TipoDoc} . $r->{NumDoc} . $r->{Filial} . $r->{Serie};

        $to_w = $r->{Serie};

        $users{ $r->{Utilizador} } = 1;

        my $info = { to => $to_w,
                     supplier        => $r->{Entidade},
                     supplier_docref => $r->{NumDocExterno},
                     date            => $r->{DataDoc},
                     user            => $r->{Utilizador},
                     doc_type        => $r->{TipoDoc},
                     pay_cond        => $r->{CondPag},
                     pay_date        => $r->{DataVencimento},
                     total_pay       => $r->{TotalMerc},
                     total_tax       => $r->{TotalIva},
                     total_discount  => $r->{TotalDesc},
                     round           => $r->{Arredondamento},
                     round_tax       => $r->{ArredondamentoIva},
                     fo_id           => $r->{Id},
                     obs             => 'TipoDoc: '.$r->{TipoDoc} . ' | NumDoc: ' . $r->{NumDoc} . ' | Filial: '. $r->{Filial} . ' | Serie: '.$r->{Serie},
                   };

        $info{ $key }->{info} = $info;

        if (! defined $r->{Artigo}) {
          push @{ $info{ $key }->{errors} }, 'Missing article';
        } elsif (! defined $r->{Armazem}) {
          push @{ $info{ $key }->{errors} }, 'Missing warehouse';
        } else {
          delete $r->{Utilizador};
          delete $r->{DataDoc};
          delete $r->{DataVencimento};
          delete $r->{Entidade};
          delete $r->{CondPag};
          delete $r->{TotalMerc};
          delete $r->{TotalIva};
          delete $r->{TotalDesc};
          delete $r->{Arredondamento};
          delete $r->{ArredondamentoIva};

  #        next if exists $info{ $key }->{art_check}->{$r->{Artigo}};
  #        $info{ $key }->{art_check}->{$r->{Artigo}} = 1;
          push @{ $info{ $key }->{articles} }, $r;
        }      
      }

      #delete $_->{art_check} for values %info;

      my @receptions = values %info;

      $s->debug("##### NEW METHOD!!!!!");

      open(F, "> $file_receptions") or die "Can't write to $file_receptions\n";
      print F Dumper(\@receptions);
      close(F);
      #write_file( $file_receptions, Dumper(\@receptions) );
    }

    if (scalar keys %users) {
      my @users = keys %users;
      my @t = ();

      for (@users) {
        push @t, { login => $_ };
      }

      open(F, "> $file_users") or die "Can't write to $file_users\n";
      print F Dumper(\@t);
      close(F);
#      write_file( $file_users, Dumper(\@t) );
    }

    my $fo_type = $s->encode_fo_type();

    # directory separator
    my $dsep = System::Util->directory_separator();

    # build ZIP file
    my $zip = Archive::Zip->new();

    # zip file
    my $zipFile = 'c:'.$dsep.$fo_type.'_FO_DATA.zip';
    &remove( $zipFile ) if -e $zipFile;

    $zip->addFile( $file_warehouses ) if -e $file_warehouses;
    $zip->addFile( $file_suppliers ) if -e $file_suppliers;
    $zip->addFile( $file_supp_art ) if -e $file_supp_art;
    $zip->addFile( $file_taxes ) if -e $file_taxes;
    $zip->addFile( $file_units ) if -e $file_units;
    $zip->addFile( $file_doc_types1 ) if -e $file_doc_types1;
    $zip->addFile( $file_doc_types2 ) if -e $file_doc_types2;
    $zip->addFile( $file_doc_types3 ) if -e $file_doc_types3;
    $zip->addFile( $file_pay_cond ) if -e $file_pay_cond;
    $zip->addFile( $file_clients ) if -e $file_clients;
    $zip->addFile( $file_articles ) if -e $file_articles;
    $zip->addFile( $file_transfers ) if -e $file_transfers;
    $zip->addFile( $file_users ) if -e $file_users;
    $zip->addFile( $file_offers_in ) if -e $file_offers_in;
    $zip->addFile( $file_breaks ) if -e $file_breaks;
    $zip->addFile( $file_inventories ) if -e $file_inventories;
    $zip->addFile( $file_receptions ) if -e $file_receptions;
    $zip->addFile( $file_families ) if -e $file_families;
    $zip->addFile( $file_pcms ) if -e $file_pcms;

=pod
    $zip->addFile( $file_inv_headers ) if -e $file_inv_headers;
    $zip->addFile( $file_inv_details ) if -e $file_inv_details;
    $zip->addFile( $file_buy_headers ) if -e $file_buy_headers;
    $zip->addFile( $file_buy_details ) if -e $file_buy_details;
    $zip->addFile( $file_buy_footers ) if -e $file_buy_footers;
    $zip->addFile( $file_stores ) if -e $file_stores;
    $zip->addFile( $file_store_products ) if -e $file_store_products;
    $zip->addFile( $file_prod_part ) if -e $file_prod_part;
    $zip->addFile( $file_prod_price ) if -e $file_prod_price;
    $zip->addFile( $file_users ) if -e $file_users;
    $zip->addFile( $file_fiscal_regions ) if -e $file_fiscal_regions;
    $zip->addFile( $file_store_clients ) if -e $file_store_clients;
    $zip->addFile( $file_store_warehouses ) if -e $file_store_warehouses;
=cut  

    die('write error') unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    #
    # Send ZIP file to the server
    #

    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die("Error sending file: $@");
    } else {
      &remove($file_warehouses) if -e $file_warehouses;
      &remove($file_suppliers) if -e $file_suppliers;
      &remove($file_supp_art) if -e $file_supp_art;
      &remove($file_taxes) if -e $file_taxes;
      &remove($file_units) if -e $file_units;
      &remove($file_doc_types1) if -e $file_doc_types1;
      &remove($file_doc_types2) if -e $file_doc_types2;
      &remove($file_doc_types3) if -e $file_doc_types3;
      &remove($file_pay_cond) if -e $file_pay_cond;
      &remove($file_clients) if -e $file_clients;
      &remove($file_articles) if -e $file_articles;
      &remove($file_transfers) if -e $file_transfers;
      &remove($file_users) if -e $file_users;
      &remove($file_offers_in) if -e $file_offers_in;
      &remove($file_breaks) if -e $file_breaks;
      &remove($file_inventories) if -e $file_inventories;
      &remove($file_receptions) if -e $file_receptions;
      &remove($file_families) if -e $file_families;
      &remove($file_pcms) if -e $file_pcms;


=pod
      &remove($file_inv_headers) if -e $file_inv_headers;
      &remove($file_inv_details) if -e $file_inv_details;
      &remove($file_buy_headers) if -e $file_buy_headers;
      &remove($file_buy_details) if -e $file_buy_details;
      &remove($file_buy_footers) if -e $file_buy_footers;
      &remove($file_stores) if -e $file_stores;
      &remove($file_store_products) if -e $file_store_products;
      &remove($file_prod_part) if -e $file_prod_part;
      &remove($file_prod_price) if -e $file_prod_price;
      &remove($file_users) if -e $file_users;
      &remove($file_fiscal_regions) if -e $file_fiscal_regions;
      &remove($file_store_clients) if -e $file_store_clients;
      &remove($file_store_warehouses) if -e $file_store_warehouses;
=cut

      &remove($zipFile) if -e $zipFile;
    }
  }

  $s->debug("Done.");
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information");

  my $xml = $s->info();

  my $dir = $xml->{dir};
}

1;

