package WO::Client::Integrators::iECR::Online;

use strict;
no warnings;

use base qw ( WO::Client::Common );

use WO::DB;
use DBI;

use Encode qw( decode encode from_to );

use Text::CSV_XS;
use File::Copy qw( move copy );
use File::Spec;
use File::Slurp qw( read_file read_dir );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Basename;
use File::Find qw( find );
use Data::Dumper;
use POSIX qw( strftime );


#
# FO to weckOFFICE
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  $s->sync_movements();

  $s->debug("Done.");
}

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  $s->sync_movements();

  $s->debug("Done.");
}

#
# Sync movements
#

sub sync_movements {
  my $s = shift;
  my $remove = shift;
  my $date_from = shift;

  $s->debug("sync_movements()");

  my $VAR1;
  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  # frontoffice directory
  my $dir = $xml->{dir} || "";
  my $process_current_day = $xml->{process_current_day} || "";

  my $csv = Text::CSV_XS->new({ binary => 1 }) or
    die "Cannot use CSV: ".Text::CSV->error_diag ();

  # directory separator
  my $dsep = $s->directory_separator();
  my $dsn = 'dbi:PgPP:dbname=iecr;host=localhost';
  my $dbh = DBI->connect( $dsn, 'postgres', '', { PrintError => 0, RaiseError => 1 } )
    or die "$DBI::errstr";

  my $db = WO::DB->new( dbh => $dbh );

  my $dbh_iecr = $db->dbh();
  my $sth_iecr = $dbh_iecr->prepare("PRAGMA cache_size = 200000") or die($dbh_iecr->errstr());
  $sth_iecr = $dbh_iecr->prepare("PRAGMA default_synchronous = OFF") or die($dbh_iecr->errstr());
  $sth_iecr = $dbh_iecr->prepare("PRAGMA synchronous = OFF") or die($dbh_iecr->errstr());

  my $str = "";
  my $VAR1 = {};

  my $db_update = 0;

  my $rand = strftime("%Y%m%d_%H%M%S", localtime(time)).".".$$;

  my $file_headers  = "/root/headers.csv.".$rand;
  my $file_details  = "/root/details.csv.".$rand;
  my $file_payments = "/root/payments.csv.".$rand;

  unlink $file_headers if -e $file_headers;
  unlink $file_details if -e $file_details;
  unlink $file_payments if -e $file_payments;

  my @to_remove = ();

  my $company_config = $s->weckoffice_send_data( { action => 'CompanyConfiguration',
                                                 }
  );

  eval "$company_config";
  my %company_config = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $company_config = $s->company_configuration();

  my $act = 'FrontofficeExistentSellDatesFast';
  my $existent_dates = $s->weckoffice_send_data( { action => $act, } );
  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my $sth;
  my $business_date = '';
  my $did_data = 0;

  if (open(F, "< /opt/iECR/.business_date")) {
    local $/ = undef;
    $business_date = <F>;
    chop($business_date);
    close(F);
  }

  if (! $business_date) {
    die "Can't find business date in /opt/iECR/.business_date...";
  }

  # include current day
  delete $existent_dates{$business_date};

  my @files_to_remove = ();

  # Prepare ZIP file
  my $zip = Archive::Zip->new();
  my $zipFile = $dir.$dsep.$fo_type.'_FO_DATA.zip';
  $s->own_remove( $zipFile ) if -e $zipFile;

  if (-d "/opt/iECR/server/data/export") {
    $did_data = 1;

    my @files = read_dir("/opt/iECR/server/data/export");

    for (@files) {
      $s->debug("Will add: $_");
      my( $date_tmp ) = $_ =~ /^(.*?)#/;
      delete $existent_dates{$date_tmp};

      $zip->addFile( "/opt/iECR/server/data/export/".$_ );
      push @files_to_remove, "/opt/iECR/server/data/export/".$_;
    }
  }

  my $last_id_processed = 0;

  # if files were added to this array, don't do this verification
  #if (! scalar @files_to_remove) {
  #  $last_id_processed = _check_last_id_processed($db);
  #}

  my @doc_types_sales = ( 1, 5, 90, 12, 7, 2, 13 );
  my $where_dt = "AND doc_type IN (".join(",", @doc_types_sales).")";

  my $date_from_w1 = $s->get_date_from();
  my $date_from_w2 = $s->get_date_from();

  $s->debug("DATE FROM : ".$s->get_date_from());

  my $date_from_w1 = " AND business_date >= '".$s->get_date_from()."'";
  my $date_from_w2 = " AND sh.business_date >= '".$s->get_date_from()."'";

  $sth = $dbh->prepare(qq{
    SELECT business_date, SUM(value)
    FROM sales_headers
    WHERE 1=1$date_from_w1 $where_dt
    GROUP BY business_date
    ORDER BY business_date
  });

  $sth->execute();

  my %ddates;

  while (my $r = $sth->fetchrow_arrayref) {
    $ddates{$r->[0]} = $r->[1];
  }

  foreach my $date (sort keys %ddates) {
    if (! $process_current_day) {
      if ($date eq $business_date) {
        $s->debug("DATE $date IS TODAY, IGNORING...");
        next;
      }
    }

    $s->debug("DATE TO ANALYZE: $date, NETBO: ".int($existent_dates{$date}*1).", POS: ".int($ddates{$date}*1));

    if (exists $existent_dates{$date}) {
      # old method
      if ($existent_dates{$date} == 1) {
        if ($company_config->{import_sells_date_dupplicate} eq 'ignore' && exists $existent_dates{$date}) {
          print STDERR "######OLD> WILL IGNORE DATE - $date\n";
          next;
        }
      }
      # new method
      else {
        if (
          exists $existent_dates{$date} &&
          int($existent_dates{$date}*1) == int($ddates{$date}*1)
        ) {
          #print STDERR "######NEW> WILL IGNORE DATE - $date\n";
          next;
        }
      }
    }

    $s->debug("WILL PROCESS DATE: $date");

    $did_data = 1;

    #
    # Headers
    #

    $sth = $dbh->prepare(qq{
      SELECT *
      FROM sales_headers
      WHERE business_date = ?$date_from_w1
    });

    $sth->execute($date);

    open(F, ">> $file_headers") or die "can't write to $file_headers - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    #
    # Details
    #

    $sth = $dbh->prepare(qq{
      SELECT sp.*
      FROM sales_details sp
      LEFT JOIN sales_headers sh ON (sp.header = sh.id)
      WHERE sh.business_date = ?$date_from_w2
    });

    $sth->execute($date);

    open(F, ">> $file_details") or die "can't write to $file_details - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();

    #
    # Payments
    #

    $sth = $dbh->prepare(qq{
      SELECT sp.*
      FROM sales_payments sp
      LEFT JOIN sales_headers sh ON (sp.header = sh.id)
      WHERE sh.business_date = ?$date_from_w2
    });

    $sth->execute($date);

    open(F, ">> $file_payments") or die "can't write to $file_payments - $!";

    while (my $r = $sth->fetchrow_arrayref) {
      my $status = $csv->combine(@$r);
      my $line = $csv->string();
      print F $line."\n";
    }

    close(F);

    $sth->finish();
  }

  if ($did_data) {
    if (-e $file_headers) {
      $zip->addFile( $file_headers );
      push @files_to_remove, $file_headers;
    }

    if (-e $file_details) {
      $zip->addFile( $file_details );
      push @files_to_remove, $file_details;
    }

    if (-e $file_payments) {
      $zip->addFile( $file_payments );
      push @files_to_remove, $file_payments;
    }

    my $prefix_doc_type = $s->vfh( $xml, 'prefix_doc_type' );
    my $prefix_op_type = $s->vfh( $xml, 'prefix_op_type' );

    if (defined $prefix_doc_type || defined $prefix_op_type) {
      my $misc_prefixes_file = $s->tmp_dir() . $dsep . "misc_prefixes.txt";

      open(F, "> $misc_prefixes_file") or die "Can't write to '$misc_prefixes_file': $!\n";

      print F "prefix_doc_type=$prefix_doc_type\n" if defined $prefix_doc_type;
      print F "prefix_op_type=$prefix_op_type\n" if defined $prefix_op_type;

      close(F);

      $zip->addFile( $misc_prefixes_file );
    }

    die "write error" unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                });
    };

    $s->own_remove($zipFile) if -e $zipFile;
    #system("rm -r $n_tmp_dir");

    if ($@) {
      die "Error sending file: $@";
    }
    else {
      if ($last_id_processed) {
        if (open(F, "> /tmp/last_sold_id.txt")) {
          print F "$last_id_processed\n";
          close(F);
        }
      }

      for (@files_to_remove) {
        unlink $_;
      }
    }
  }
}

sub common_sync_fo_and_sync_bk {
  my $s = shift;
  my( $xml, $zip, $to_remove ) = @_;

  $s->debug("common_sync_fo_and_sync_bk()");

  my $dsep = $s->directory_separator();

  my $consumption_price_number = $s->vfh( $xml, 'consumption_price_number' );

  if (defined $consumption_price_number) {
    my $consumption_price_number_file = $s->tmp_dir() . $dsep . "consumption_price_number.txt";

    $s->debug("####> $consumption_price_number_file - $consumption_price_number");

    open(F, "> $consumption_price_number_file") or die "Can't write to '$consumption_price_number_file': $!\n";
    print F "$consumption_price_number\n";
    close(F);

    $zip->addFile( $consumption_price_number_file );

    push @$to_remove, $consumption_price_number_file;
  }

  #
  # use sale locals to map things
  #

  $xml->{map_sale_terminals} ||= undef;

  if (defined $xml->{map_sale_terminals}) {
    my $file_map_sale_terminals = $s->tmp_dir() . $dsep . "map_sale_terminals.txt";

    my @mappings = ();

    if (exists $xml->{map_sale_terminals}->{map}) {
      if (ref $xml->{map_sale_terminals}->{map} eq 'HASH') {
        push @mappings, $xml->{map_sale_terminals}->{map};
      } elsif (ref $xml->{map_sale_terminals}->{map} eq 'ARRAY') {
        @mappings = @{ $xml->{map_sale_terminals}->{map} };
      }
    }

    if (scalar @mappings) {
      open(F, "> $file_map_sale_terminals") or die "Can't write to '$file_map_sale_terminals': $!\n";
      print F Dumper(\@mappings);
      close(F);
      $zip->addFile( $file_map_sale_terminals );
      push @$to_remove, $file_map_sale_terminals;
    }
  }
}

sub my_decode {
  my $w = shift;

  eval {
    my $octets = decode("iso-8859-1",$w);
    $w = encode("utf8",$octets);
  };

  if ($@) {
    print STDERR "ENCODING ERROR: #$w#\n";
  }

  return $w;
}

sub _check_last_id_processed {
  my $db = shift;

  # read last sales id from hdd
  my $last_id = "";

  {
    local $/ = undef;
    if (open(F, "< /tmp/last_sold_id.txt")) {
      $last_id = <F>;
      $last_id =~ s/\n//g;
      close(F);
    }
  }

  my $max_id = $db->select(qq{ SELECT MAX(id) AS id FROM sales_headers })->[0]->{id};
  $max_id ||= 0;

  #print STDERR "##### max_id: $max_id, last_id: $last_id\n";

  if ($last_id && $max_id && $last_id eq $max_id) {
    print STDERR "LAST SOLD ID IS THE SAME AS THE LAST PROCESSED ID, ABORTING...\n";
    print STDERR "REMOVE /tmp/last_sold_id.txt TO SKIP THIS VERIFICATION...\n";
    exit;
  }

  return $max_id;
}

1;




