package WO::Client::Integrators::rTOS;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use WO::Client::Integrators::rTOS::Table::Generic;
use WO::Client::Integrators::rTOS::Table::Department;
use WO::Client::Integrators::rTOS::Table::Group;
use WO::Client::Integrators::rTOS::Table::EmployeeJobCodes;
use WO::Client::Integrators::rTOS::Table::Employee;
use WO::Client::Integrators::rTOS::Table::JobCodes;
use WO::Client::Integrators::rTOS::Table::Media;
use WO::Client::Integrators::rTOS::Table::PLU;
use WO::Client::Integrators::rTOS::Table::ScreenHeader;
use WO::Client::Integrators::rTOS::Table::Buttons;
use WO::Client::Integrators::rTOS::Table::SystemPrinter;
use WO::Client::Integrators::rTOS::Table::TaxTable;
use WO::Client::Integrators::rTOS::Table::TaxTableTrp;
use WO::Client::Integrators::rTOS::Table::MenuItemModifier;
use WO::Client::Integrators::rTOS::Table::Discount;
use WO::Client::Integrators::rTOS::Table::ChargeAccount;
use WO::Client::Integrators::rTOS::Table::MealPeriod;

use POSIX qw( strftime );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Basename qw( fileparse );
use File::Path qw( mkpath );
use File::Slurp qw( read_dir );




#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $export_dir = $xml->{till_folder}.$dsep."export";
  my $backup_dir = $xml->{backup_dir};
  my $backup_data = $xml->{backup_data} || 0;

  # backup data dir of the till
  if ($backup_data && -d $xml->{till_folder} && $xml->{backup_dir}) {
    mkpath($xml->{backup_dir}) unless -d $xml->{backup_dir};

    $s->debug("Will backup folder ".$xml->{till_folder}." in ".$xml->{backup_dir});
    # build ZIP file
    my $zip = Archive::Zip->new();

    &find( sub {
      return if -d $File::Find::name;
      $zip->addFile( $File::Find::name );
    }, $xml->{till_folder} );

    my $zipFile = $xml->{backup_dir}.$dsep.'till_backup.zip';
    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;
    $s->backup_file( file => $zipFile, num => 4 );
    $s->own_remove( $zipFile );
  }

  # backup two new files
  if ($backup_dir && -e $export_dir.$dsep."salesmix.txt") {
    my $rnd = int(rand(10000000));
    my $dest = $backup_dir.$dsep."salesmix.txt.".$rnd;
    $s->debug("Will move file '".$export_dir.$dsep."salesmix.txt"."' to '$dest'");

    &_move($export_dir.$dsep."salesmix.txt", $dest);
  }

  if ($backup_dir && -e $export_dir.$dsep."financialz1.txt") {
    my $rnd = int(rand(10000000));
    my $dest = $backup_dir.$dsep."financialz1.txt.".$rnd;
    $s->debug("Will move file '".$export_dir.$dsep."financialz1.txt"."' to '$dest'");

    &_move($export_dir.$dsep."financialz1.txt", $dest);
  }

  my @zip_files;

  if (-d $export_dir) {
    &find( sub {
      return if -d $File::Find::name;

      if (/journal\.txt/i || /checkheaders\.txt/i || /^c\.\d+/ || /^j\.\d+/) {
        push @zip_files, { full_path => $File::Find::dir.$dsep.$_, basename => $_ }; 
      }
    }, $export_dir );
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_->{full_path} ) } @zip_files;

    my $zipFile = $s->tmp_dir().$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      mkdir($backup_dir) if $backup_dir && ! -d $backup_dir;

      for (@zip_files) {
        if ($backup_dir) {
          my $rnd = int(rand(10000000));
          my $dest = $backup_dir.$dsep.$_->{basename}.".".$rnd;
          $s->debug("Will backup file '".$_->{full_path}."' to '$dest'");
          if ($^O eq 'linux') {
            my $st = system("cp \"".$_->{full_path}."\" \"".$dest."\"");
          } else {
            my $st = system("copy /Y \"".$_->{full_path}."\" \"$dest\" > NUL 2> NUL");
          }
        }


        $s->debug("Will delete file '".$_->{full_path}."'");
        $s->own_remove($_->{full_path});
      }
    }
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  # executes unless a manual execution order was issued
  #return unless $s->manual();

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  # frontoffice directory
  my $dir = $xml->{dir} || $xml->{dir_import} || "";
  my $images_dir = $xml->{dir_images} || $dir || "";
  #die "Export dir '$dir' don't exists..." unless -d $dir;

  $xml->{use_cache} ||= 0;

  my $cache_prefix = exists $xml->{cache_prefix} ?
    $xml->{weckoffice}->{store} . $xml->{cache_prefix} :
    $xml->{weckoffice}->{store};

  $s->debug("export dir: $dir");

  my $upd_exists = 0;

  find( sub {
    return unless -f $File::Find::name;
    return if $upd_exists;
    $upd_exists = 1 if $_ =~ /upd$/i;
  }, ( $dir ));

  if ($upd_exists) {
    $s->debug("$dir already has an upd file, aborting...");
    return 1;
  }

  $xml->{sync_tables} = exists $xml->{sync_tables} ? $xml->{sync_tables} : 1;

  my $req_manual = $xml->{update_data_requires_manual} || "";

  # validate manual
  if ($req_manual && ! $s->manual()) {
    $s->debug("Will terminate, this FO must be manually executed...");
    return 1;
  }

  my $str;
  my $VAR1;

  # This call is here to validate that the till import folder is
  # reachable. If it isn't, this will croak.
  my @tmp_files = &read_dir($dir);

  #
  # Images
  #

  if ($xml->{sync_images}) {
    # Let's sync the images now

    eval {
      $str = $s->weckoffice_send_data( { action => 'BleepScreenMiniData', } );
    };

    if ($@) {
      die("Error: $@");
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'HASH') {
      my $company_id = $VAR1->{company_id};
      my $keyboard_id = $VAR1->{keyboard_id};

      if ($company_id && $keyboard_id) {
        my $cmd = "\"$ENV{WECK_CLIENT_PATH}".$dsep."bin".$dsep."sync\" --url http://$ENV{WECK_RSYNC_SERVER}:$ENV{WECK_RSYNC_PORT}/FileSync --channel screens-pctools-images --dir \"".$xml->{weckoffice}->{host}."\/$company_id/$keyboard_id\" --destination \"$images_dir\"";
        my $res = system($cmd);
        $s->debug("# sync result: ".($res == 0 ? "SUCCESS" : "ERROR"));
      }
    }
  }

  return unless $xml->{sync_tables};

  #
  # Tables
  #

  my @contents = ();

  #
  # Static Tables
  #

  my %TABLE_CONVERSION = (
#    printertype     => 'PRT',
#    report          => 'RPT',
#    colors          => 'COL',
#    taxbracket      => 'TXB',
#    revenuecenter   => 'REV',
#    scheduler       => 'SCH',
#    roundingtable   => 'RND',
#    roundingentries => 'RNE',
#    systemconfig    => 'SYS',
#    laborsettings   => 'LAB',
#    errorsettings   => 'ERR',
#    checksettings   => 'CHK',
#    networksettings => 'NET',
#    taretable       => 'TAT',
#    cardscheme      => 'STR',
#    trainingconfig  => 'TRN',
#    tax  => 'TAX',
  );

  my @tables = keys %TABLE_CONVERSION;
  my $tbl_str = join(",", @tables);

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetPcToolsTables', tables => $tbl_str } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'HASH') {
    foreach my $tbl (keys %$VAR1) {
      my $t = $TABLE_CONVERSION{$tbl};

      shift(@{$VAR1->{$tbl}->{fields}});

      my $c_gen = WO::Client::Integrators::rTOS::Table::Generic->new(
        cache_prefix => $cache_prefix,
        cache => $xml->{use_cache},
        table => $TABLE_CONVERSION{$tbl},
        fields => $VAR1->{$tbl}->{fields},
      );

      $VAR1->{$tbl}->{values} ||= [];

      for (@{ $VAR1->{$tbl}->{values} }) {
        my $h = {};

        for (my $i=0; $i<scalar @{ $VAR1->{$tbl}->{fields} }; $i++) {
          $h->{ $VAR1->{$tbl}->{fields}->[$i] } = $_->[$i];
        }

        $c_gen->add_row( values => $h );
      }

      #$c_gen->dump_records();
      push @contents, $c_gen;
    }
  }

  #
  # Tax Table
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetTaxes', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  my %taxes_map = ();

  if ($xml->{weckoffice}->{host} =~ /^trp\./) {
    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $cl = WO::Client::Integrators::rTOS::Table::TaxTableTrp->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

      # ignore if class is marked has disabled
      for (@$VAR1) {
        my $h = {
          recordid => $_->{fo_id},
          name => $_->{name},
          rate => $_->{value},
          taxident => $_->{taxident},
          receiptdesc => $_->{receiptname},
          netamount => $_->{netamount},
          adjustsales => $_->{adjustsales},
          method => $_->{method},
          printinclusive => $_->{printinclusive},
          taxontax => $_->{taxontax},
          hashgroup => $_->{hashgroup},
          printhashtotals => $_->{printhashtotals},
          exemptreqsignature => $_->{exemptreqsignature},
          printbarcodetotal => $_->{printbarcodetotal}
        };

        $taxes_map{ $_->{id} } = $_->{fo_id};

        $cl->add_row( values => $h );
      }

      push @contents, $cl;
    }
  } else {
    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $cl = WO::Client::Integrators::rTOS::Table::TaxTable->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

      # ignore if class is marked has disabled
      for (@$VAR1) {
        my $h = {
          recordid => $_->{fo_id},
          name => $_->{name},
          rate => $_->{value},
          taxident => $_->{taxident},
          receiptdesc => $_->{receiptname},
          netamount => $_->{netamount},
          adjustsales => $_->{adjustsales},
          method => $_->{method},
          printinclusive => $_->{printinclusive},
          taxontax => $_->{taxontax},
          hashgroup => $_->{hashgroup},
          printhashtotals => $_->{printhashtotals},
          exemptreqsignature => $_->{exemptreqsignature},
        };

        $taxes_map{ $_->{id} } = $_->{fo_id};

        $cl->add_row( values => $h );
      }

      push @contents, $cl;
    }  
  }

  #
  # Departments
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetFamilies', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  my %department_ids = ();

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::Department->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );
    my $cg = WO::Client::Integrators::rTOS::Table::Group->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my $idx_g = 1;
    my $idx_d = 1;

    my %groups;
    my %group_ids;

    # ignore if class is marked has disabled
    for (@$VAR1) {
      if (defined $_->{top_family_fo_id} && ! exists $groups{ $_->{top_family_fo_id} }) {
        my( $gid ) = $_->{top_family_fo_id} =~ /^.(.*?)$/;
        my $h = {
          groupindex => $idx_g,
          groupname  => $_->{top_family_name},
        };

        $group_ids{ $_->{top_family_fo_id} } = $idx_g;
        $idx_g++;

        $cg->add_row( values => $h );

        $groups{ $_->{top_family_fo_id} } = 1;
      }
    }

    for (@$VAR1) {
      next unless defined $_->{top_family_fo_id};

      my @dept_tax = (0,0,0,0,0,0,0,0);

      foreach my $tax (@{$_->{taxes} || []}) {
        if (defined $taxes_map{$tax} && $taxes_map{$tax}) {
          $dept_tax[$taxes_map{$tax}-1] = 1 if exists($dept_tax[$taxes_map{$tax}-1]);
        }
      }

      my $h = {
        #recordid => $_->{family_fo_id},
        recordid => $idx_d,
        departmentdesc => $_->{family_name},
        prompt => 'Select a '.$_->{family_name},
        course => $_->{course},
        remotecopycount => $_->{rmtcopycount},
        categoryusage => $_->{usage},
        trackusersales => $_->{trackuser},
        tracksystemsales => $_->{tracksystem},
        printonserverslip => $_->{serverslip},
        taxtable1 => $dept_tax[0],
        taxtable2 => $dept_tax[1],
        taxtable3 => $dept_tax[2],
        taxtable4 => $dept_tax[3],
        taxtable5 => $dept_tax[4],
        taxtable6 => $dept_tax[5],
        taxtable7 => $dept_tax[6],
        taxtable8 => $dept_tax[7],
        majorgroup => $group_ids{ $_->{top_family_fo_id} },
      };

      $department_ids{ $_->{family_fo_id} } = $idx_d;

      $cl->add_row( values => $h );

      $idx_d++;
    }

    push @contents, $cg;
    push @contents, $cl;
  }

  #
  # Discounts
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetDiscounts', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::Discount->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my @params = qw(fo_id discount discountname itemdiscount nontaxable recipientid printdelayed percentdiscount servicecharge verifyrecipientid printofficecopy resettotalizer departmentlink securitylevel taxtable1 taxtable2 taxtable3 taxtable4 taxtable5 taxtable6 taxtable7 taxtable8 halo itemlink);

    for (@$VAR1) {
      my $h = {};

      foreach my $param (@params) {
        $h->{$param} = $_->{$param} if (defined $_->{$param});
      }
      $h->{recordid} = $h->{fo_id};

      next if $h->{discountname} eq '';

      $cl->add_row( values => $h );
    }

    push @contents, $cl;
  }

  #
  # Products
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetArticles', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::PLU->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my @params = qw(disabled rolluppricing addonprice openprice scaleable staydownsize printofficecopy tracksystem trackuser printhighlight enablepricelevels limitedqty routeprinting prefixisprice spellit kvscolor designateditem promotype discountlnk enableprice1 enableprice2 enableprice3 enableprice4 enableprice5 enableprice6 enableprice7 enableprice8 activesize1 activesize2 activesize3 activesize4 activesize5 activesize6 activesize7 activesize8 mealcount securitylevel taretableid quantityhalo barcode promopriority discountable prefixprice);

    my %param_trsl = (
      taretableid => 'taretableindex',
      addonprice  => 'additionalprice',
      tracksystem => 'tracksystemsales',
      trackuser => 'trackusersales',
      printhighlight => 'printhighlighted',
      enablepricelevels => 'enablelevelpricing',
      enableprice1 => 'level1active',
      enableprice2 => 'level2active',
      enableprice3 => 'level3active',
      enableprice4 => 'level4active',
      enableprice5 => 'level5active',
      enableprice6 => 'level6active',
      enableprice7 => 'level7active',
      enableprice8 => 'level8active',
      discountlnk => 'discountlink',
      kvscolor => 'color',
      prefixprice => 'prefixisprice'
    );

    my $control = {};

    # ignore if class is marked has disabled
    for (@$VAR1) {
      $_->{article_barcode} ||= "";

      if (exists $control->{ $_->{article_fo_id} }) {
        $s->debug("Product with code ".$_->{article_fo_id}." is duplicated... skipping...");
        next;
      }

      my %diff_barcode = (
        'trp'                  => 1,
        'milligans'            => 1,
        'dragandemo'           => 1,
        'rare'                 => 1,
        'manormeats'           => 1,
        'barbakandeli'         => 1,
        'paymentsensedemo1'    => 1,
        'paymentsensedemo2'    => 1,
        'vrl'                  => 1,
        'psrestaurant'         => 1,
        'levelfour'            => 1,
        'beytug'               => 1,
        'nlgaltd'              => 1,
      );

      my $decimals = defined $xml->{decimalplaces} ? $xml->{decimalplaces} : 2;

      $_->{article_price0} = defined $_->{article_price0} && $_->{article_price0} ? $_->{article_price0} : 0;
      $_->{article_price1} = defined $_->{article_price1} && $_->{article_price1} ? $_->{article_price1} : 0;
      $_->{article_price2} = defined $_->{article_price2} && $_->{article_price2} ? $_->{article_price2} : 0;
      $_->{article_price3} = defined $_->{article_price3} && $_->{article_price3} ? $_->{article_price3} : 0;
      $_->{article_price4} = defined $_->{article_price4} && $_->{article_price4} ? $_->{article_price4} : 0;
      $_->{negative_price} = defined $_->{negative_price} && $_->{negative_price} ? $_->{negative_price} : 0;

      $_->{article_barcode} ||= "";

      my $h = {
        #recordid => undef,
        recordid => $_->{article_fo_id},
        department => $department_ids{ $_->{family_fo_id} },
        itemdesc => $_->{article_name},
        remotedesc => uc($_->{article_name}),
        mealper1price => sprintf("%.".$decimals."f", $_->{article_price0}),
        mealper2price => sprintf("%.".$decimals."f", $_->{article_price1}),
        mealper3price => sprintf("%.".$decimals."f", $_->{article_price2}),
        mealper4price => sprintf("%.".$decimals."f", $_->{article_price3}),
        mealper5price => sprintf("%.".$decimals."f", $_->{article_price4}),
        mealper6price => sprintf("%.".$decimals."f", $_->{article_price4}),
        mealper7price => sprintf("%.".$decimals."f", $_->{article_price4}),
        mealper8price => sprintf("%.".$decimals."f", $_->{article_price4}),
        barcode       => $diff_barcode{$xml->{weckoffice}->{company}} ? $_->{article_barcode} : substr($_->{article_barcode}, 0, length($_->{article_barcode})-1),
      };

      foreach my $param (@params) {
        $h->{$param_trsl{$param} ? $param_trsl{$param} : $param} = $_->{$param} if (defined $_->{$param});
      }

      # fill correct printers
      if (defined $_->{printers} && $_->{printers}) {
        my $i = 0;
        for (split(//,$_->{printers})) {
          $h->{"device".++$i} = $_;
        }
      }

      my @idxs;

      my $enablelevelpricing = 0;

      for my $k (keys %$h) {
        if ($k =~ /^mealper(\d)/ && $h->{$k}) {
          my $v = $h->{$k}*1;
          $enablelevelpricing = 1 if $v > 0;
          push @idxs, { key => $1, value => $v };
        }
      }

      unless ($_->{overrideautopricelevels}) {
        $h->{enablelevelpricing} = $enablelevelpricing;

        for (@idxs) {
          $h->{"level".$_->{key}."active"} = $_->{value} > 0 ? 1 : 0;
        }
      }

      $cl->add_row( values => $h );

      $control->{ $_->{article_fo_id} } = 1;
    }

    push @contents, $cl;
  }

  #
  # Printers
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetPrinters', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::SystemPrinter->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    for (@$VAR1) {
      my $h = {
        recordid => $_->{fo_id},
        printerdesc => $_->{name} ,
        printserverid => $_->{printserverid},
        backupprinterid => $_->{backupprinterid} ,
        promptredirect => $_->{promptredirect} ,
        videodevice => $_->{videodevice} ,
        topmess1ctrl => $_->{topmessageline1largeprint},
        topmess1text => $_->{topmessageline1text},
        topmess2ctrl => $_->{topmessageline2largeprint},
        topmess2text => $_->{topmessageline2text},
        topmess3ctrl => $_->{topmessageline3largeprint},
        topmess3text => $_->{topmessageline3text},
        topmess4ctrl => $_->{topmessageline4largeprint},
        topmess4text => $_->{topmessageline4text},
        topmess5ctrl => $_->{topmessageline5largeprint},
        topmess5text => $_->{topmessageline5text},
        topmess6ctrl => $_->{topmessageline6largeprint},
        topmess6text => $_->{topmessageline6text},
        topmess7ctrl => $_->{topmessageline7largeprint},
        topmess7text => $_->{topmessageline7text},
        topmess8ctrl => $_->{topmessageline8largeprint},
        topmess8text => $_->{topmessageline8text},
        topmess9ctrl => $_->{topmessageline8largeprint},
        topmess9text => $_->{topmessageline9text},
        botmess1ctrl => $_->{bottommessageline1largeprint},
        botmess1text => $_->{bottommessageline1text},
        botmess2ctrl => $_->{bottommessageline2largeprint},
        botmess2text => $_->{bottommessageline2text},
        botmess3ctrl => $_->{bottommessageline3largeprint},
        botmess3text => $_->{bottommessageline3text},
        botmess4ctrl => $_->{bottommessageline4largeprint},
        botmess4text => $_->{bottommessageline4text},
        botmess5ctrl => $_->{bottommessageline5largeprint},
        botmess5text => $_->{bottommessageline5text},
        botmess6ctrl => $_->{bottommessageline6largeprint},
        botmess6text => $_->{bottommessageline6text},
        botmess7ctrl => $_->{bottommessageline7largeprint},
        botmess7text => $_->{bottommessageline7text},
        botmess8ctrl => $_->{bottommessageline8largeprint},
        botmess8text => $_->{bottommessageline8text},
        botmess9ctrl => $_->{bottommessageline9largeprint},
        botmess9text => $_->{bottommessageline9text},
      };

      $cl->add_row( values => $h );
    }

    push @contents, $cl;
  }

  #
  # Modifiers
  #
 
  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetMenuItemModifiers', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::MenuItemModifier->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    for (@$VAR1) {
      my $link = defined $_->{link} ? $_->{link} : '';
      my $h = {
        recordid => $_->{recordid},
        index => $_->{index},
        link => $_->{ismenuitem} ? $link : $department_ids{ $link },
        qty => $_->{qty},
        ismenuitem => $_->{ismenuitem},
      };

      $cl->add_row( values => $h );
    }

    push @contents, $cl;
  }

  #
  # JobCodes 
  #   
  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetJobCodes', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  my %jobcode_map = ();

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::JobCodes->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my $idx = 0;

    for (sort { $a->{fo_id}*1 <=> $b->{fo_id}*1 } @$VAR1) {
      #$s->debug("####### #".$_->{fo_id}."# #".substr($_->{name},0,15)."# #".$_->{recallotheroperatorcheck}."#");

      my $h = {
        recordid             => $_->{fo_id},
        tipped               => $_->{tipped},
        autologout           => $_->{logoutaftertrans},
        requireinitialloan   => $_->{requireinitialloan},
        tipshare             => $_->{tipshare},
        recallotheruserchks  => $_->{recallotheroperatorcheck},
        tenderotheruserchks  => $_->{tenderotheroperatorchecks},
        allowbankloan        => $_->{allowbankloans},
        mainscreen1          => $_->{screen1},
        mainscreen2          => $_->{screen2},
        mainscreen3          => $_->{screen3},
        mainscreen4          => $_->{screen4},
        mainscreen5          => $_->{screen5},
        mainscreen6          => $_->{screen6},
        mainscreen7          => $_->{screen7},
        mainscreen8          => $_->{screen8},
        mgrmodeautologout    => $_->{pgmmodelogouttime},
        posmodeautologout    => $_->{posmodelogouttime},
        recallpaidchks       => $_->{recallpaidchecks},
        drawerassignment     => $_->{drawerassignement},
        mgrsecurity          => $_->{mgtsecurity},
        jobcodename          => substr($_->{name},0,15),
        defaultordertype     => $_->{defaultordertype},
        securitylevel        => $_->{securitylevel},
        basepayrate          => $_->{basepayrate},
        autocreatecheck      => $_->{autocreatecheck},
        tenderchecks         => $_->{tenderchecks},
      };

      $jobcode_map{$_->{id}} = $_->{fo_id};

      $cl->add_row( values => $h );
    }

    push @contents, $cl;
  }


  #
  # Employees
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetWaitresses', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  my %user_map = ();

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl1 = WO::Client::Integrators::rTOS::Table::Employee->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    # ignore if class is marked has disabled
    my $idx = 0;

    for (sort { $a->{fo_id}*1 <=> $b->{fo_id}*1 } @$VAR1) {
      $idx++;

      my @names = split /\s+/, $_->{name};

      my $h1 = {
        recordid       => $_->{fo_id}, #$idx,
        firstname      => $names[0],
        lastname       => join(" ", @names[1 .. scalar(@names)-1]),
        logincode      => $_->{fo_login},
        userpassword   => $_->{fo_passwd},
        cardnumber     => $_->{bi},
        hourlypay      => $_->{hourlypay} ,
        requireclockin => $_->{requireclockin},
        clockinonly    => $_->{clockinonly},
        onetouchlogin  => $_->{onetouchlogin},
        trainingmode   => $_->{trainingmode},
        cashdrawerport => $_->{cashdrawerport},
      };

      $user_map{$_->{id}} = $_->{fo_id}; #$idx;

      $cl1->add_row( values => $h1 );
    }

    push @contents, $cl1;
  }

  #
  # Employee JobCode Assoc
  #
  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetEmpJobCodes', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl1 = WO::Client::Integrators::rTOS::Table::EmployeeJobCodes->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my %userjobcodeindex = ();

    # ignore if class is marked has disabled
    my $idx = 0;

    my @tmp = ();
 
    for (@$VAR1) {
      if (defined $user_map{$_->{user_id}} && $jobcode_map{$_->{jobcode_id}}) {
        $userjobcodeindex{$user_map{$_->{user_id}}} = defined $userjobcodeindex{$user_map{$_->{user_id}}} ? $userjobcodeindex{$user_map{$_->{user_id}}} : 0;

        my $h1 = {
          recordid => $user_map{$_->{user_id}},
          userlink => $user_map{$_->{user_id}},
          jobcodeid => $jobcode_map{$_->{jobcode_id}},
          payrate => $_->{pay_rate} || 0,
          userjobcodeindex => $userjobcodeindex{$user_map{$_->{user_id}}}++,
        };

        push @tmp, $h1;
#        $cl1->add_row( values => $h1 );
      }
    }

    for (sort { $a->{recordid}*1 <=> $b->{recordid}*1 || $a->{userjobcodeindex}*1 <=> $b->{userjobcodeindex}*1 } @tmp) {
      $cl1->add_row( values => $_ );
    }
    push @contents, $cl1;
  } 
 
  #
  # Medias
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetPaymentMethods', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $media = WO::Client::Integrators::rTOS::Table::Media->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my @params = qw(changemedia mediatotalsid indepositlist opensecondarydrawer verifysignature servicetype serviceclass  changedescription  exchangerate numdecimalsinexchangerate currencydecimalplaces securitylevel  overtendertogratuity usedaschange givechange opendrawer printsignature speedtender sellable chargeacct  printofficecopy payinfull promptforpin  printsecondcopyvoucher foreigncurrency printexchangerate  triangulationmethod denomination changehalo discountlink cardschemelink);

    # ignore if class is marked has disabled
    for (@$VAR1) {
      my $h = {
        recordid   => $_->{fo_id},
        medianame  => $_->{name},
      };

      foreach my $param (@params) {
        $h->{$param} = $_->{$param} if (defined $_->{$param});
      }

      next if $h->{medianame} eq '';

      $media->add_row( values => $h );
    }

    push @contents, $media;
  }

  #
  # Charge Accounts (Clients)
  #
  eval {
    #$str = $s->weckoffice_send_data( { action => 'BleepGetStoreClients', } );
    $str = $s->weckoffice_send_data( { action => 'BleepGetStoreClients', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::ChargeAccount->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my $idx = 0;

    for (@$VAR1) {
      next unless $_->{fo_id};
      $idx++;

      my $h = {
        recordid         => $_->{fo_id},#$idx,
        accountnumber    => $_->{card_number} ? substr($_->{card_number},0,18) : substr($_->{r4_accountnumber},0,18),
        name             => substr($_->{name},0,18),
        address1         => substr($_->{address},0,28),
        city             => substr($_->{localidade},0,28),
        zipcode          => substr($_->{postal_code},0,9),
        phonenumber      => substr($_->{telephone},0,18), 
        creditlimit      => $_->{credit_limit},
        state            => $_->{r4_state},
        disable          => $_->{r4_disabled},
        decliningbalance => $_->{r4_decliningbalance},
      };

      #$s->debug("####### #".$_->{user_balance}."#");
      $cl->add_row( values => $h );
    }

    push @contents, $cl if $idx > 0;
  }

  #
  # Meal Periods
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepGetMealPeriods', } );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    my $cl = WO::Client::Integrators::rTOS::Table::MealPeriod->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

    my $idx = 0;
    my $done = 0;

    for (@$VAR1) {
      $idx++;

      $done = 1;

      my $sun_hour = sprintf("%02d", $_->{sun_hour});
      my $sun_min = sprintf("%02d", $_->{sun_min});
      my $mon_hour = sprintf("%02d", $_->{mon_hour});
      my $mon_min = sprintf("%02d", $_->{mon_min});
      my $tue_hour = sprintf("%02d", $_->{tue_hour});
      my $tue_min = sprintf("%02d", $_->{tue_min});
      my $wed_hour = sprintf("%02d", $_->{wed_hour});
      my $wed_min = sprintf("%02d", $_->{wed_min});
      my $thu_hour = sprintf("%02d", $_->{thu_hour});
      my $thu_min = sprintf("%02d", $_->{thu_min});
      my $fri_hour = sprintf("%02d", $_->{fri_hour});
      my $fri_min = sprintf("%02d", $_->{fri_min});
      my $sat_hour = sprintf("%02d", $_->{sat_hour});
      my $sat_min = sprintf("%02d", $_->{sat_min});

      my $h = {
        recordid       => $idx,
        name           => substr($_->{name},0,10),
        sunday         => '2006-01-16 '.$sun_hour.":".$sun_min,
        monday         => '2006-01-16 '.$mon_hour.":".$mon_min,
        tuesday        => '2006-01-16 '.$tue_hour.":".$tue_min,
        wednesday      => '2006-01-16 '.$wed_hour.":".$wed_min,
        thursday       => '2006-01-16 '.$thu_hour.":".$thu_min,
        friday         => '2006-01-16 '.$fri_hour.":".$fri_min,
        saturday       => '2006-01-16 '.$sat_hour.":".$sat_min,
      };

      $cl->add_row( values => $h );
    }

    push @contents, $cl if $done;
  }

  #
  # Screens
  #

  eval {
    $str = $s->weckoffice_send_data( { action => 'BleepScreenData',
                                     }
                                   );
  };

  if ($@) {
    die("Error: $@");
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    # ignore if class is marked has disabled
    my $add_header = 1;

    my $use_cache = $xml->{use_cache};
    my $no_buttons = 1;

    # don't use cache
    if (! $use_cache) {
      my @headers;
      my @buttons;

      my $cl_h = WO::Client::Integrators::rTOS::Table::ScreenHeader->new( cache_prefix => $cache_prefix, cache => $xml->{use_cache} );
      my $cl_b = WO::Client::Integrators::rTOS::Table::Buttons->new( add_header => 0, cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

      for my $header (@$VAR1) {
        my $h = {
          showcheckwindow => $header->{ShowCheckWindow},
          recordid => $header->{RecordID},
          backgroundcolor => $header->{BGroundColor},
          exitafter => $header->{ExitAfter},
          name => $header->{Name},
          exitafter => $header->{ExitAfter},
        };

        $cl_h->add_row( values => $h );

        for (@{ $header->{_buttons} }) {
          next if $_->{Width} == 0 && $_->{Height} == 0;

          $s->buttons_fixup( $_ ); # used by rTOSv412.pm

          my $h = {
            recordid => $header->{RecordID},
            buttonindex => $_->{ButtonIndex},
            height => $_->{Height},
            width => $_->{Width},
            round => $_->{Round},
            itemlink => $_->{ItemLink},
            screenlink => $_->{ScreenLink},
            fontcolor => $_->{FontColor},
            buttoncolor => $_->{ButtonColor},
            caption => $_->{Caption},
            showcaptionflag => $_->{ShowCaptionFlag},
            tlcol => $_->{TLCol},
            buttontype => $_->{ButtonType},
            bitmapimage => $_->{BitmapImage},
            bitmapimageposition => $_->{BitmapImagePosition},
            font => $_->{FontFace},
            fontsize => $_->{FontSize},
            tlrow => $_->{TLRow},
          };

          $cl_b->add_row( values => $h );

          $no_buttons = 0;
        }
      }

      if ($no_buttons) {
        die "Illegal error... no buttons...\n";
      }

      push @contents, $cl_h;
      push @contents, $cl_b;
    }
    # use cache
    else {
############################################################################
      my $no_buttons = 1;
      my $screens_data = {};
      my %screens_export = ();

      for my $header (@$VAR1) {
        my $h = {
          showcheckwindow => $header->{ShowCheckWindow},
          recordid => $header->{RecordID},
          backgroundcolor => $header->{BGroundColor},
          exitafter => $header->{ExitAfter},
          name => $header->{Name},
          exitafter => $header->{ExitAfter},
        };

        $screens_data->{ $header->{RecordID} }->{headers} = $h;

        my $cl_b = WO::Client::Integrators::rTOS::Table::Buttons->new( add_header => 0, cache_prefix => $cache_prefix, cache => $xml->{use_cache} );

        for (@{ $header->{_buttons} }) {
          next if $_->{Width} == 0 && $_->{Height} == 0;

          $s->buttons_fixup( $_ ); # used by rTOSv412.pm

          my $h = {
            recordid => $header->{RecordID},
            buttonindex => $_->{ButtonIndex},
            height => $_->{Height},
            width => $_->{Width},
            round => $_->{Round},
            itemlink => $_->{ItemLink},
            screenlink => $_->{ScreenLink},
            fontcolor => $_->{FontColor},
            buttoncolor => $_->{ButtonColor},
            caption => $_->{Caption},
            showcaptionflag => $_->{ShowCaptionFlag},
            tlcol => $_->{TLCol},
            buttontype => $_->{ButtonType},
            bitmapimage => $_->{BitmapImage},
            bitmapimageposition => $_->{BitmapImagePosition},
            font => $_->{FontFace},
            fontsize => $_->{FontSize},
            tlrow => $_->{TLRow},
          };

          $no_buttons = 0;

          push @{ $screens_data->{ $header->{RecordID} }->{buttons} }, $h;

          my $changed = $cl_b->add_row( values => $h );
        
          if ($changed) {
            $screens_export{ $header->{RecordID} } = 1;
          }          
        }
      }

      foreach my $id (keys %screens_export) {
        my $cl_h = WO::Client::Integrators::rTOS::Table::ScreenHeader->new( add_header => 0, cache => 0 );
        $cl_h->add_row( values => $screens_data->{ $id }->{headers} );

        push @contents, $cl_h;

        foreach my $btn (@{ $screens_data->{ $id }->{buttons} }) {
          my $cl_b = WO::Client::Integrators::rTOS::Table::Buttons->new( add_header => 0, cache => 0 );
          $cl_b->add_row( values => $btn );

          push @contents, $cl_b;
        }
      }

      if ($no_buttons) {
        die "Illegal error... no buttons...\n";
      }

############################################################################
    }
  }

  if (scalar @contents) {
    my $year = int(strftime("%Y", localtime(time())));
    my $mon  = int(strftime("%m", localtime(time())));
    my $day  = int(strftime("%d", localtime(time())));
    my $hour = int(strftime("%H", localtime(time())));
    my $min  = int(strftime("%M", localtime(time())));
    my $sec  = int(strftime("%S", localtime(time())));

    my $out_file = $day.$mon.$year.$hour.$min.$sec.".upd"; # 235200716835.upd

    my $tmp_file = $s->tmp_dir() . $dsep . int(rand(1000000));
    $s->own_remove($tmp_file) if -e $tmp_file;

    my $full_p = $dir.'/'.$out_file;
    $s->own_remove($full_p) if -e $full_p;

    $s->debug("Will write data to a tmp file: $tmp_file");

    for (@contents) {
      $_->dump_records( to_file => $tmp_file );
    }

    $s->debug("Will move file to $full_p");

    my $out_tmp = "$dir".$dsep."..".$dsep."$out_file";
    my $txt_file = $dir.$dsep.$out_file.".txt";

    if (defined $xml->{upd_backup} && $xml->{upd_backup}) {
      &copy($tmp_file, $xml->{upd_backup}.$dsep."$out_file");
    }

    my $st1 = &move( $tmp_file, $txt_file );
    my $st2 = &move( $txt_file, $full_p );
  }
}

sub buttons_fixup {
  my $s = shift;
  my $h = shift;
}

sub backup_file {
  my $s = shift;

  my %a = ( num => 2, file => undef, @_ );

  &File::Basename::fileparse_set_fstype('MSWin32');

  die "Missing 'file' parameter..." unless defined $a{file};

  return undef unless -e $a{file};

  my( $name, $path, $suffix) = fileparse($a{file}, ());

  my $time_str = strftime("%Y%m%d%H%M%S", localtime(time));

  my @collected = ();

  &find( sub {
    return unless -f $File::Find::name;

    if ($File::Find::name ne $a{file} && /$name\.\d+$/) {
      push @collected, $File::Find::name;
    }
  }, ( $path ) );

  &copy( $a{file}, $a{file}.'.'.$time_str );

  my $c = 0;

  for (reverse sort @collected) {
    next if ++$c < $a{num};
    #$s->debug("backup_file -> will remove $_");
    $s->own_remove( $_ );
  }
}

sub _move {
  my( $d, $o ) = @_;

  if ($^O eq 'linux') {
    my $st = system("mv -f \"".$d."\" \"$o\"");
  } else {
    my $st = system("move /Y \"$d\" \"$o\" > NUL 2> NUL");
  }
}

1;




















