package WO::Client::Integrators::rTOS::Mini;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use POSIX qw( strftime );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Copy qw( copy move );
use File::Find qw( find );
use File::Basename qw( fileparse );
use File::Path qw( mkpath );
use File::Slurp qw( read_dir );

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  my $export_dir = $xml->{till_folder}.$dsep."export";
  my $backup_dir = $xml->{backup_dir};
  my $backup_data = $xml->{backup_data} || 0;

  # backup data dir of the till
  if ($backup_data && -d $xml->{till_folder} && $xml->{backup_dir}) {
    mkpath($xml->{backup_dir}) unless -d $xml->{backup_dir};

    $s->debug("Will backup folder ".$xml->{till_folder}." in ".$xml->{backup_dir});
    # build ZIP file
    my $zip = Archive::Zip->new();

    &find( sub {
      return if -d $File::Find::name;
      $zip->addFile( $File::Find::name );
    }, $xml->{till_folder} );

    my $zipFile = $xml->{backup_dir}.$dsep.'till_backup.zip';
    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;
    $s->backup_file( file => $zipFile, num => 4 );
    $s->own_remove( $zipFile );
  }

  # backup two new files
  if ($backup_dir && -e $export_dir.$dsep."salesmix.txt") {
    my $rnd = int(rand(10000000));
    my $dest = $backup_dir.$dsep."salesmix.txt.".$rnd;
    $s->debug("Will move file '".$export_dir.$dsep."salesmix.txt"."' to '$dest'");

    &_move($export_dir.$dsep."salesmix.txt", $dest);
  }

  if ($backup_dir && -e $export_dir.$dsep."financialz1.txt") {
    my $rnd = int(rand(10000000));
    my $dest = $backup_dir.$dsep."financialz1.txt.".$rnd;
    $s->debug("Will move file '".$export_dir.$dsep."financialz1.txt"."' to '$dest'");

    &_move($export_dir.$dsep."financialz1.txt", $dest);
  }

  my @zip_files;

  my %files_to_fetch = (
    'chargeacct.txt' => 1,
    'checkheaders.txt' => 1,
    'department.txt' => 1,
    'employee.txt' => 1,
    'journal.txt' => 1,
    'majorgroup.txt' => 1,
    'plu.txt' => 1,
    'taxtable.txt' => 1,
    'timecards.txt' => 1,
    'button.txt' => 1,
  );

  if (-d $export_dir) {
    &find( sub {
      return if -d $File::Find::name;

      my $lc_file = lc($_);

      if (lc($_) =~ /$lc_file/ || /^c\.\d+/i || /^j\.\d+/i) {
        $s->debug("Will add file: $_");
        push @zip_files, { full_path => $File::Find::dir.$dsep.$_, basename => $_ }; 
      }
    }, $export_dir );
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_->{full_path} ) } @zip_files;

    my $zipFile = $s->tmp_dir().$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      mkdir($backup_dir) if $backup_dir && ! -d $backup_dir;

      for (@zip_files) {
        if ($backup_dir) {
          my $rnd = int(rand(10000000));
          my $dest = $backup_dir.$dsep.$_->{basename}.".".$rnd;
          $s->debug("Will backup file '".$_->{full_path}."' to '$dest'");

          if ($^O eq 'linux') {
            my $st = system("cp \"".$_->{full_path}."\" \"".$dest."\"");
          } else {
            my $st = system("copy /Y \"".$_->{full_path}."\" \"$dest\" > NUL 2> NUL");
          }
        }


        $s->debug("Will delete file '".$_->{full_path}."'");
        $s->own_remove($_->{full_path});
      }
    }
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

#
# Brings weckOFFICE information to FO
#

sub sync_bk_information {
  my $s = shift;

  die "Not implemented...\n";
}

sub buttons_fixup {
  my $s = shift;
  my $h = shift;
}

sub backup_file {
  my $s = shift;

  my %a = ( num => 2, file => undef, @_ );

  &File::Basename::fileparse_set_fstype('MSWin32');

  die "Missing 'file' parameter..." unless defined $a{file};

  return undef unless -e $a{file};

  my( $name, $path, $suffix) = fileparse($a{file}, ());

  my $time_str = strftime("%Y%m%d%H%M%S", localtime(time));

  my @collected = ();

  &find( sub {
    return unless -f $File::Find::name;

    if ($File::Find::name ne $a{file} && /$name\.\d+$/) {
      push @collected, $File::Find::name;
    }
  }, ( $path ) );

  &copy( $a{file}, $a{file}.'.'.$time_str );

  my $c = 0;

  for (reverse sort @collected) {
    next if ++$c < $a{num};
    #$s->debug("backup_file -> will remove $_");
    $s->own_remove( $_ );
  }
}

sub _move {
  my( $d, $o ) = @_;

  if ($^O eq 'linux') {
    my $st = system("mv -f \"".$d."\" \"$o\"");
  } else {
    my $st = system("move /Y \"$d\" \"$o\" > NUL 2> NUL");
  }
}


1;


