package WO::Client::TrayApp;

use strict;
no warnings;
use POSIX qw( strftime );
use FindBin qw( $Bin );
use LWP::UserAgent qw( post );
use Data::Dumper;
use MIME::Base64 qw( decode_base64 encode_base64 );
use Digest::MD5 qw( md5_hex );
use Encode qw(encode decode);

my %controls;

sub Common {
  my $fo_info = shift;

  my $str = "";
  
  # handle temperatures
  if (exists $fo_info->{temperature_file} && $fo_info->{temperature_file}) {
    if (open(F, "< ".$fo_info->{temperature_file})) {
      local $/ = undef;
      my $temp = <F>;
      close(F);

      # these sensors have an ID on the, if we find some that doesn't, we'll use this parameter for that
      $fo_info->{temperature_sensor_id} ||= "";

      # Node 10102241 T=+32.0625
      if ($temp =~ /^Node\s(\d+)\s+T=(.*?)$/) {
        $str .= "TEMP=$1|$2"; # don't use commas (,) here, to send multiple params, use # to separate them
      }
    }
  }
  elsif (exists $fo_info->{temperature_winrest} && $fo_info->{temperature_winrest}) {
    $fo_info->{temperature_winrest_file} ||= "";
    
    if ($fo_info->{temperature_winrest_file} && -f $fo_info->{temperature_winrest_file}) {
      my $f = $fo_info->{temperature_winrest_file};
      system("mv \"$f\" \"".$f.".done\"");
      local $/ = undef;                                               
      if (open(F, "< ".$f.".done")) {
        my $temp = <F>;                                             
        $temp = encode_base64($temp);                           
        close(F);
        unlink $f.".done";
        
        if (defined $temp && $temp) {
          $temp =~ s/\n/@/gs;
          $str .= $str ? "#TEMP_WINREST=$temp" : "TEMP_WINREST=$temp";
        }
      }
    }
  }

  # vpn address
  my $vpn_ip = &__get_vpn_ip();
  $str .= $str ? "#VPN_IP=$vpn_ip" : "VPN_IP=$vpn_ip" if defined $vpn_ip;

  # hdparm validation
  my $hdparm = $^O eq 'MSWin32' ? 'hdparm.exe' : 'hdparm';

  my $fecth_hdparm = 0;

  if ($^O eq 'MSWin32' && ! -e $Bin.'/'.$hdparm) {
    $fecth_hdparm = 1;
  } elsif ($^O eq 'linux') {
    if (! -e '/bin/hdparm' && ! -e '/sbin/hdparm' && ! -e $Bin.'/'.$hdparm) {
      $fecth_hdparm = 1;
    }
  }

  if ($fecth_hdparm) {
    $fo_info->{weckoffice}->{proxy_host} ||= "";
    $fo_info->{weckoffice}->{proxy_port} ||= "";
    $fo_info->{weckoffice}->{proxy_user} ||= "";
    $fo_info->{weckoffice}->{proxy_passwd} ||= "";

    &__download_file(
      server     => $ENV{WECKOFFICE_SERVER},
      port       => $ENV{WECKOFFICE_SERVER_PORT},
      proxy_host => $fo_info->{weckoffice}->{proxy_host},
      proxy_port => $fo_info->{weckoffice}->{proxy_port},
      proxy_user => $fo_info->{weckoffice}->{proxy_user},
      proxy_pass => $fo_info->{weckoffice}->{proxy_passwd},
      dl_path    => '/downloads/weckOFFICE_Client/hdparm/'.$hdparm,
      save_to    => $Bin.'/'.$hdparm,
    );

    if ($^O ne 'MSWin32' && -e $Bin.'/'.$hdparm) {
      system("chmod a+x ".$Bin.'/'.$hdparm);
    }
  }

  my $hdd_id = &__harddrive_id();
  $str .= $str ? "#HDD_ID=$hdd_id" : "HDD_ID=$hdd_id" if defined $hdd_id;
  
  # handle dispense                                                       
  if (exists $fo_info->{dispense_folder} && $fo_info->{dispense_folder}) {
    if (opendir(my $dh, $fo_info->{dispense_folder})) {                   
      my $sent = $fo_info->{dispense_folder}."/dispense.sent";

      while( my $file = readdir($dh) ) {                                  
        my $f = $fo_info->{dispense_folder}."/".$file;                    
                                                                          
        if ($f =~ /\.log$/) {                                             
          system("cat \"$f\" >> \"$sent\"");
          my $new_f = $f."_".int(rand(10000000)).".send";
          system("mv \"$f\" \"$new_f\"");                                        
          local $/ = undef;                                               
          open(F, "< $new_f");                                           
          my $dispense = <F>;                                             
          $dispense = encode_base64($dispense);                           
          close(F);                                                       
          unlink $new_f;

          if (defined $dispense && $dispense) {
            $dispense =~ s/\n/@/gs;
            $str .= $str ? "#DISPENSE=$dispense" : "DISPENSE=$dispense";
          }
        }                                                                 
      }                                                                   
      closedir($dh);                                                      
    }                                                                     
  }     
  
  return $str;
}

sub WO__Client__Integrators__rTOS__XmlOnlineTransactions__today_sells {
  my $fo_info = shift;
  return 0;
}

sub WO__Client__Integrators__ExportAsCSV__today_sells {
  my $fo_info = shift;
  return 0;
}

sub WO__Client__Integrators__KvsTimings__today_sells {
  my $fo_info = shift;
  return 0;
}

sub WO__Client__Integrators__OnlinePOSPg__today_sells {
  my $fo_info = shift;

  my $total = 0;

  if (-e "/root/weckOFFICE/bin/pos_today_sales") {
    $total = `/root/weckOFFICE/bin/pos_today_sales`;
    chop($total);
  }

  return $total;
}

sub WO__Client__Integrators__POSPg__today_sells {
  my $fo_info = shift;

  #&__iecr_instant_sales($fo_info)
  #  if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};

  my $total = 0;

  if (-e "/root/weckOFFICE/bin/pos_today_sales") {
    $total = `/root/weckOFFICE/bin/pos_today_sales`;
    chop($total);
  }

  return $total;
}

sub WO__Client__Integrators__rTOS_20141023__today_sells {
  my $fo_info = shift;

  my $today_total = 0;
  my $dsep = $^O eq 'MSWin32' ? "\\" : '/';

  return unless defined $fo_info->{till_folder};
  my $till_path = $fo_info->{till_folder}.$dsep."export".$dsep;

  if (exists $fo_info->{active} && $fo_info->{active}) {
    my $journal = exists $fo_info->{till_folder} ?
      $fo_info->{till_folder}.$dsep."export".$dsep."journal.txt" : "";
    my $checkheaders = exists $fo_info->{till_folder} ?
      $fo_info->{till_folder}.$dsep."export".$dsep."checkheaders.txt" : "";
    my $financial = exists $fo_info->{till_folder} ?
      $fo_info->{till_folder}.$dsep."export".$dsep."financialrptz1.txt" : "";

    my $time = time;

    my $st=0;

    if ($fo_info->{weckoffice}->{company} ne 'bettyscafetearooms') {
    if (open(F, "< $journal")) {
      if (open(N, "> ".$till_path."j.".$time)) {
        while(<F>) {
          print N $_;
        }
        close(N);
        $st = 1;
      }
      close(F);
      unlink($journal) if $st;
      print STDERR "TrayApp: Error writing to $journal.$time\n" if !$st;
    }
    
    $st=0;

    if (open(F, "< $checkheaders")) {
      if (open(N, "> ".$till_path."c.".$time)) {
        while(<F>) {
          print N $_;
        }
        close(N);
        $st = 1;
      }
      close(F);
      unlink($checkheaders) if $st;
      print STDERR "TrayApp: Error writing to $checkheaders.$time\n" if !$st;
    }
    }

    if (open(F, "< $financial")) {
      while(<F>) {
        my @t = split /,/;

        my $idx1 = 2;
        my $idx2 = 4;

        if (scalar @t == 9) {
          $idx1 += 3;
          $idx2 += 3;
        }

        if ($t[$idx1] eq '"Gross Sales"') {
          $today_total = $t[$idx2];
          last;
        }
      }
      close(F);
      unlink($financial);
    }
  }

  return $today_total;
}

sub WO__Client__Integrators__rTOS__Mini_today_sells {
  my $fo_info = shift;
  return WO__Client__Integrators__rTOS__today_sells($fo_info);
}

sub WO__Client__Integrators__rTOS__today_sells {
  my $fo_info = shift;

  my $today_total = 0;
  my $dsep = $^O eq 'MSWin32' ? "\\" : '/';

  return unless defined $fo_info->{till_folder};
  my $till_path = $fo_info->{till_folder}.$dsep."export".$dsep;

  if (exists $fo_info->{active} && $fo_info->{active}) {
    my $journal = exists $fo_info->{till_folder} ?
      $fo_info->{till_folder}.$dsep."export".$dsep."journal.txt" : "";
    my $checkheaders = exists $fo_info->{till_folder} ?
      $fo_info->{till_folder}.$dsep."export".$dsep."checkheaders.txt" : "";
    my $financial = exists $fo_info->{till_folder} ?
      $fo_info->{till_folder}.$dsep."export".$dsep."financialrptz1.txt" : "";

    my $time = time;

    my $st=0;

    if ($fo_info->{weckoffice}->{company} ne 'bettyscafetearooms') {
    if (open(F, "< $journal")) {
      if (open(N, "> ".$till_path."j.".$time)) {
        while(<F>) {
          print N $_;
        }
        close(N);
        $st = 1;
      }
      close(F);
      unlink($journal) if $st;
      print STDERR "TrayApp: Error writing to $journal.$time\n" if !$st;
    }
    
    $st=0;

    if (open(F, "< $checkheaders")) {
      if (open(N, "> ".$till_path."c.".$time)) {
        while(<F>) {
          print N $_;
        }
        close(N);
        $st = 1;
      }
      close(F);
      unlink($checkheaders) if $st;
      print STDERR "TrayApp: Error writing to $checkheaders.$time\n" if !$st;
    }
    }

    if (open(F, "< $financial")) {
      while(<F>) {
        my @t = split /,/;
        if ($t[2] eq '"Gross Sales"') {
          $today_total = $t[4];
          last;
        }
      }
      close(F);
      unlink($financial);
    }
  }

  return $today_total;
}

sub WO__Client__Integrators__FirstPOS__today_sells {
  my $fo_info = shift;

  if (-e "/var/log/config_actions.log" && -s "/var/log/config_actions.log" > 50000) {
    unlink "/var/log/config_actions.log";
  }

  &__firstpos_instant_sales($fo_info)
    if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};

  my $total = 0;

  $fo_info->{dir} ||= "";

  my $data_dir = $fo_info->{dir}."/dados";

  if (-d $data_dir) {
    my $file = "";
    my %files;

    if (opendir(D, $data_dir)) {
      while(my $f = readdir(D)) {
        next unless $f =~ /^(.*?)\.csv$/i;
        $file = $data_dir."/".$f;
        $files{$1} = $file;
      }

      closedir(D);
    }

    foreach my $k (sort keys %files) {
      $file = $files{$k};
    }

    if (open(FF, "< $file")) {
      while(<FF>) {
        chop; chop;
        my @toks = split /,/;
        my $elems_num = scalar @toks;
        my $mov_type = $elems_num >= 17 ? $toks[14] : $toks[13];
        next unless defined $mov_type;
        next if $mov_type eq '7' || $mov_type eq '8' || $mov_type eq '2';
        my $val = $elems_num >= 17 ? $toks[5] : $toks[4];
        $total += $val/100 if defined $val;
      }
      close(FF);
    }
  }

  return $total;
}

sub WO__Client__Integrators__iECR__Online__today_sells {
  my $fo_info = shift;

  #&__iecr_instant_sales($fo_info)
  #  if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};

  my $total = 0;

  if (-e "/root/weckOFFICE/bin/iecr_today_sales") {
    $total = `/root/weckOFFICE/bin/iecr_today_sales`;
    chop($total);
  }

  return $total;
}

sub WO__Client__Integrators__iECR__Mini__today_sells {
  my $fo_info = shift;

  #&__iecr_instant_sales($fo_info)
  #  if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};

  my $total = 0;

  if (-e "/root/weckOFFICE/bin/iecr_today_sales") {
    $total = `/root/weckOFFICE/bin/iecr_today_sales`;
    chop($total);
  }

  return $total;
}

sub WO__Client__Integrators__iECRPg__today_sells {
  my $fo_info = shift;

  #&__iecr_instant_sales($fo_info)
  #  if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};

  my $total = 0;

  if (-e "/root/weckOFFICE/bin/iecr_today_sales") {
    $total = `/root/weckOFFICE/bin/iecr_today_sales`;
    chop($total);
  }

  return $total;
}

sub WO__Client__Integrators__iECR__today_sells {
  my $fo_info = shift;

  #&__iecr_instant_sales($fo_info)
  #  if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};

  my $total = 0;

  if (-e "/root/weckOFFICE/bin/iecr_today_sales") {
    $total = `/root/weckOFFICE/bin/iecr_today_sales`;
    chop($total);
  }

  return $total;
}

sub WO__Client__Integrators__OnlineWinRest__today_sells {
  my $fo_info = shift;
  return &WO__Client__WinRest__today_sells($fo_info);
}

sub WO__Client__WinRest__today_sells {
  my $fo_info = shift;

  &__winrest_instant_sales($fo_info)
    if exists $fo_info->{use_instant_sales} && $fo_info->{use_instant_sales};
  
  my $d = $fo_info->{dir}."/files/data";
  my $locals_f = $fo_info->{dir}."/files/data/wrsttloc.000";
  my $d_tables = $fo_info->{dir}."/temp";

  return 0 unless defined $d && -d $d;

  my $file_name_headers;
  my $file_name_details;

  if (-e $d."/fdocheader.000") {
    $file_name_headers = $d."/fdocheader.000";
  }
  else {
    # winrest old version
    $file_name_headers = $d."/"."c" . sprintf("%02d", strftime("%y", localtime(time))) .
                              sprintf("%02d", strftime("%m", localtime(time))) .
                              sprintf("%02d", strftime("%d", localtime(time))) .
                              ".000";

    if (! -e $file_name_headers) {
      if (opendir(DIR, $d)) {
        while (my $f = readdir(DIR)) {
          if ($f =~ /^c\d\d\d\d\d\d\.000$/i) {
            $file_name_headers = $d."/".$f;
            last;
          }
        }
        closedir(DIR);
      }
    }
  }

  if (-e $d."/fdocdetail.000") {
    $file_name_details = $d."/fdocdetail.000";
  }
  else {
    # winrest old version
    $file_name_details = $d."/"."d" . sprintf("%02d", strftime("%y", localtime(time))) .
                              sprintf("%02d", strftime("%m", localtime(time))) .
                              sprintf("%02d", strftime("%d", localtime(time))) .
                              ".000";

    if (! -e $file_name_details) {
      if (opendir(DIR, $d)) {
        while (my $f = readdir(DIR)) {
          if ($f =~ /^d\d\d\d\d\d\d\.000$/i) {
            $file_name_details = $d."/".$f;
            last;
          }
        }
        closedir(DIR);
      }
    }
  }

  return 0 unless -e $file_name_headers;

  my %to_ignore = ();
  my $total_w_tax = 0;
  my $total_wo_tax = 0;

  if (open(RR, "< $file_name_headers")) {
    while(<RR>) {
      my $ndoc = &__trim(substr($_, 0, 9));
      my $tdoc = &__trim(substr($_, 57, 1));
      my $estr = &__trim(substr($_, 94, 1));

      if ($estr eq 'T') {
        $to_ignore{ $ndoc } = 1;
      }
      elsif ($tdoc == 4) {
        $to_ignore{ $ndoc } = 1;
      }
    }

    close(RR);
  }

  if (open(RR, "< $file_name_details")) {
    while(<RR>) {
      my $op   = substr($_, 13, 1);
      next unless $op && $op == 1;

      my $ndoc = &__trim(substr($_, 0, 9));
      next if exists $to_ignore{ $ndoc };

      my $qtty = &__trim(substr($_, 47, 8));
      my $pvp  = &__trim(substr($_, 55, 12));
      my $desc = &__trim(substr($_, 67, 12));
      my $ofer = &__trim(substr($_, 79, 12));
      my $iva  = &__trim(substr($_, 91, 6));

      my $t = $qtty * ($pvp-$desc-$ofer);
      my $t2 = $t/(($iva/100)+1);

      $total_w_tax += $t;
      $total_wo_tax += $t2;
    }
  
    close(RR);
  }

  if (! exists $fo_info->{today_total_without_vat}) {
    $fo_info->{today_total_without_vat} = 0;
  }

  my $total = 0;

  if ($fo_info->{today_total_without_vat}) {
    $total = $total_wo_tax;
  }
  else {
    $total = $total_w_tax;
  }

  $fo_info->{use_open_tables} ||= 0;

  if ($fo_info->{use_open_tables} || $fo_info->{weckoffice}->{company} eq 'realindiana') {
    my $autocs_interval = &__extract_autocs_from_locals($locals_f);
    opendir(my $dh, $d_tables) || die "can't open WinREST temp dir: $d_tables: $!";
    my @tables = grep { /^wrst\d\d\d\d\./ && -f "$d_tables/$_" } readdir($dh);
    closedir $dh;

    foreach my $table (@tables) {
      my( $num ) = $table =~ /^wrst(\d+)\./i;
      $num = $num*1;
      next if exists $autocs_interval->{$num};
      
      my $val = &__read_winrest_temp_file("$d_tables/$table");
      $total += $val if defined $val && $val;
    }
  }

  return $total;
}

sub __firstpos_instant_sales {
  my $fo_info = shift;

  my $sales_cache_file = &__firstpos_instant_sales_cache_file($fo_info);
  return unless defined $sales_cache_file;

  if (! exists $controls{"__firstpos_instant_sales"}) {
    $controls{"__firstpos_instant_sales"} = 1;
    &__cleanup_cache_files($sales_cache_file);
    #unlink($sales_cache_file);
  }

  my $content = "";

  my $y = sprintf("%02d", strftime("%y", localtime(time)));
  my $m = sprintf("%02d", strftime("%m", localtime(time)));
  my $d = sprintf("%02d", strftime("%d", localtime(time)));

  my $dir = $fo_info->{dir}."/dados";
  my $fname = $dir.'/2'.$y.$m.$d.".CSV";

  my $short_file = '2'.$y.$m.$d.".CSV";
  my $file = "";

  if (! -e $fname) {
    if (opendir(DIR, $dir)) {
      my @files = ();

      while (my $f = readdir(DIR)) {
        if ($f =~ /csv$/i) {
          push @files, $f;
        }
      }
      closedir(DIR);

      if (scalar @files) {
        $file = $dir."/".$files[-1];
        $short_file = $files[-1];
      }
    }
  } else {
    $file = $fname;
  }

  my %cache;
  my @procs;

  if (open(CACHE, "< $sales_cache_file")) {
    while(<CACHE>) {
      chop;
      $cache{$_} = 1;
    }
    close(CACHE);
  }

  my %done_keys;

  if (open(H, "< $file")) {    
    while(<H>) {
      my @toks = split /,/;
      next unless ! exists $cache{ $toks[0] } && $toks[0] =~ /^z/i;
      $done_keys{ $toks[0] } = 1;

      $content .= $_;
    }
    
    close(H);
  }

  my $final_res = 1;

  if (length($content)) {
    my $st = &__post_sales_instant_data($fo_info, 'instant_sales', "#$short_file#$content");
    $final_res = 0 unless defined $st;
  }

  if ($final_res && length($content) && scalar %done_keys) {
    if (open(CACHE, ">> $sales_cache_file")) {
      for (keys %done_keys) {
        print CACHE $_."\n";
      }
      close(CACHE);
    }      
  }
    
  %cache = ();
}

sub __winrest_instant_sales {
  my $fo_info = shift;

  my $sales_cache_file = &__winrest_instant_sales_cache_file($fo_info);
  return unless defined $sales_cache_file;

  if (! exists $controls{"__winrest_instant_sales"}) {
    $controls{"__winrest_instant_sales"} = 1;
    &__cleanup_cache_files($sales_cache_file);
    #unlink($sales_cache_file);
  }
    
  my $y = sprintf("%02d", strftime("%y", localtime(time)));
  my $m = sprintf("%02d", strftime("%m", localtime(time)));
  my $d = sprintf("%02d", strftime("%d", localtime(time)));
  
  my $dir = $fo_info->{dir}."/files/data";

  my $headers_f  = "fdocheader.000";
  my $details_f  = "fdocdetail.000";
  my $footers_f  = "fdocfooter.000";
  my $cashiers_f = "cashier.000";

  my $tmpdir = '/tmp/isales';
  eval {
    system("/bin/mkdir -p $tmpdir");
    system("/bin/cp $dir/fdoc*000 $tmpdir");
    system("/bin/cp $dir/cashier*000 $tmpdir");

  };

  print STDERR $@;

  print STDERR "Processing INstant Sales\n";

  my $headers  = $dir."/".$headers_f;
  my $details  = $dir."/".$details_f;
  my $footers  = $dir."/".$footers_f;
  my $cashiers = $dir."/".$cashiers_f;

#  my $headers  = $tmpdir."/".$headers_f;
#  my $details  = $tmpdir."/".$details_f;
#  my $footers  = $tmpdir."/".$footers_f;
#  my $cashiers = $tmpdir."/".$cashiers_f;


  $headers  = $dir . "/" . "c" . $y . $m . $d . ".000" unless -e $headers;
  $details  = $dir . "/" . "d" . $y . $m . $d . ".000" unless -e $details;
  $footers  = $dir . "/" . "r" . $y . $m . $d . ".000" unless -e $footers;
  $cashiers = $dir . "/" . "x" . $y . $m . $d . ".000" unless -e $cashiers;

  if (! -e $headers) {
    if (opendir(DIR, $dir)) {
      while (my $f = readdir(DIR)) {
        if ($f =~ /^c(\d\d\d\d\d\d)\.000$/i) {
          $headers  = $dir."/".$f;
          $details  = $dir."/d".$1.".000";
          $footers  = $dir."/r".$1.".000";
          $cashiers = $dir."/r".$1.".000";
          last;
        }
      }
      closedir(DIR);
    }
  }
  
  my %cache;
  my @procs;
  
  if (open(CACHE, "< $sales_cache_file")) {
    while(<CACHE>) {
      chop;
      $cache{$_} = 1;
    }
    close(CACHE);
  }
  
  my $headers_cnt  = "HEADERS\n";
  my $details_cnt  = "DETAILS\n";
  my $footers_cnt  = "FOOTERS\n";
  my $cashiers_cnt = "CASHIERS\n";

  $fo_info->{fb_d_narm_in_details} ||= undef;

  if (defined $fo_info->{fb_d_narm_in_details} && $fo_info->{fb_d_narm_in_details}) {
    $headers_cnt = "fb_d_narm_in_details\nHEADERS\n";
  }

  my $has_cashiers = 0;
  my $send_only_cashiers = 0;

  $fo_info->{instant_sales_include_cashier} ||= undef;

  if (defined $fo_info->{instant_sales_include_cashier}) {
    if (open(N, "< $cashiers")) {

      my $read_cashiers = 0;
      my $cashiers_fsize = -s $cashiers;

      if (exists $controls{"__winrest_cashier_file_size"}) {
        if ($cashiers_fsize != $controls{"__winrest_cashier_file_size"}) {
          $read_cashiers = 1;
        }
      } else {
        $controls{"__winrest_cashier_file_size"} = $cashiers_fsize;
        $read_cashiers = 1;
      }

      if ($read_cashiers) {
        while(<N>) {
          my( $id ) = $_ =~ /^(\d+)/;
          next unless $id && $id eq '19'; # apanha apenas os recibos
          $has_cashiers = 1;
          $cashiers_cnt .= $_;
        }
      }

      close(N);
    }
  }

  if (open(H, "< $headers")) {
    while(<H>) {
      my $l = $_;

      my $proc = substr($l, 0, 9);
      $proc =~ s/^\s+//;

      chop($l);
      my $proc_cache = $l;

      my $footers = &__winrest_get_recs($footers, $proc);
      my $footers_to_key = $footers;
      $footers_to_key =~ s/\r|\n//g;
      
      my $details_cnt_tmp .= &__winrest_get_recs($details, $proc);
      my $hash_key = &md5_hex($details_cnt_tmp);
      
      if (! exists $cache{$proc_cache.$footers_to_key.$hash_key}) {
        $headers_cnt .= $l."\n";
        $details_cnt .= $details_cnt_tmp;
        push @procs, $proc_cache.$footers_to_key.$hash_key;
        $footers_cnt .= $footers;
      }
    }
    
    close(H);
  } elsif ($has_cashiers && $has_cashiers) {
    $send_only_cashiers = 1;
  }
  
  my $final_res = 1;
    
  if (scalar @procs || $send_only_cashiers) {
    my $st;

    if (defined $fo_info->{instant_sales_include_cashier}) {
      $st = &__post_sales_instant_data($fo_info, 'instant_sales', $headers_cnt.$details_cnt.$footers_cnt.$cashiers_cnt);
    } else {
      $st = &__post_sales_instant_data($fo_info, 'instant_sales', $headers_cnt.$details_cnt.$footers_cnt);
    }

    $final_res = 0 unless defined $st;
  }

  if ($final_res && scalar @procs) {
    if (open(CACHE, ">> $sales_cache_file")) {
      for (@procs) {
        print CACHE $_."\n";
      }
      close(CACHE);
    }      
  }
    
  %cache = ();
}

sub __winrest_get_recs {
  my( $file, $p ) = @_;
  
  my $str = "";
  
  if (open(FF, "< $file")) {
    while(<FF>) {
      my $proc = substr($_, 0, 9);
      $proc =~ s/^\s+//;
      $str .= $_ if $proc eq $p;
    }
    
    close(FF);
  }
  
  return $str;
}

sub __post_sales_instant_data {
  my $fo_info = shift;
  my $key = shift;
  my $cnt = shift;
  
  my $ret = undef;

  my $ua = LWP::UserAgent->new( timeout => 5 );
  $ua->default_header('host' => $fo_info->{weckoffice}->{host});

  if (defined $fo_info->{weckoffice}->{proxy_host}) {
    my $port = defined $fo_info->{weckoffice}->{proxy_port} ? ":".$fo_info->{weckoffice}->{proxy_port} : "";
    if ($fo_info->{weckoffice}->{proxy_host} =~ /^http/i) {
      $ua->proxy( ['http'] => $fo_info->{weckoffice}->{proxy_host}.$port );
    } else {
      $ua->proxy( ['http'] => "http://".$fo_info->{weckoffice}->{proxy_host}.$port );
    }
  }

  my @map_sale_locals = ();

  if (exists $fo_info->{map_sale_locals}) {
    my $mappings = $fo_info->{map_sale_locals}->{map};
    
    if (defined $mappings) {
      @map_sale_locals = ref $mappings eq 'HASH' ? ( $mappings ) : @$mappings;
    }
  }
  
  my $r = $ua->post(
    "http://".$ENV{WECKOFFICE_SERVER}.":".$ENV{WECKOFFICE_SERVER_PORT}."/cgi-bin/control_executions.cgi",
                         Content_Type => 'form-data',
                         Content => [
                           'company'         => $fo_info->{weckoffice}->{company},
                           'store'           => $fo_info->{weckoffice}->{store},
                           'login'           => $fo_info->{weckoffice}->{login},
                           'passwd'          => $fo_info->{weckoffice}->{passwd},
                           'host'            => $fo_info->{weckoffice}->{host},
                           'fo_type'         => $fo_info->{type},
                           'extra_id'        => encode_base64($fo_info->{weckoffice}->{extra_id}),
                           'map_sale_locals' => encode_base64(Dumper(\@map_sale_locals)),
                           $key              => $cnt,
                         ],
                   );      
                    
  if (defined $fo_info->{weckoffice}->{proxy_user}) {
    $r->proxy_authorization_basic(
      $fo_info->{weckoffice}->{proxy_user},
      defined $fo_info->{weckoffice}->{proxy_passwd} ? $fo_info->{weckoffice}->{proxy_passwd} : "",
    );
  }
    
  if ($r->code() == 200 && $r->message() eq 'OK') {
    $ret = 1;
  }
      
  return $ret;
}

sub __get_vpn_ip {
  my $vpn_prefix = "10.8";
  my $ip_vpn = undef;
  my $interfaces = "";

  if ($^O eq 'MSWin32') {
    $interfaces = `ipconfig`;
  } else {
    if (-e "/sbin/ifconfig") {
      $interfaces = `/sbin/ifconfig`;
    } elsif (-e "/bin/ifconfig") {
      $interfaces = `/bin/ifconfig`;
    }
  }

  my( @toks ) = $interfaces =~ /$vpn_prefix\.(\d+)\.(\d+)/g;

  while (scalar @toks) {
    my $t1 = shift @toks;
    my $t2 = shift @toks;

    $ip_vpn = $vpn_prefix.".$t1.$t2";

    last;
  }

  return $ip_vpn;
}

sub __cleanup_cache_files {
  my $file = shift;

  opendir(my $dh, $Bin) || die "can't opendir $Bin: $!";

  while (my $f = readdir($dh)) {
    my $full_p = $Bin.'/'.$f;
    unlink $full_p if $f =~ /instant_sales_cache/ && $full_p ne $file;
    next;
  }

  closedir $dh;
}

sub __winrest_instant_sales_cache_file {
  my $fo_info = shift;

  my $date_f = $fo_info->{dir}.'/files/data/wrstdata.000';

  if (-e $date_f) {
    if (open(F, "< $date_f")) {
      local $/ = undef;
      my $cnt = <F>;
      close(F);

      my( $m, $d, $a ) = $cnt =~ /^(\d+)\/(\d+)\/(\d+)/;

      return "$Bin/.winrest_instant_sales_cache_".$a.$m.$d;
    }
  }

  return undef;
}

sub __firstpos_instant_sales_cache_file {
  my $fo_info = shift;

  my $data_dir = $fo_info->{dir}.'/dados';

  if (opendir(D, $data_dir)) {
    my $max = -1;

    while(my $f = readdir(D)) {
      next unless $f =~ /^(\d+)\.csv$/i;
      $max = $1 if $1 > $max;
    }

    closedir(D);

    if ($max > 0) {
      my( $y, $m, $d ) = $max =~ /^.(\d\d)(\d\d)(\d\d)/;
      $y = '20'.$y;

      return "$Bin/.firstpos_instant_sales_cache_".$y.$m.$d;
    }
  }

  return undef;
}

sub __download_file {
  my %a = (
    server     => undef,
    port       => undef,
    proxy_host => undef,
    proxy_port => undef,
    proxy_user => undef,
    proxy_pass => undef,
    dl_path    => undef,
    save_to    => undef,
    @_,
  );

  die "Missing 'server' parameter..."  unless defined $a{server};
  die "Missing 'port' parameter..."    unless defined $a{port};
  die "Missing 'dl_path' parameter..." unless defined $a{dl_path};
  die "Missing 'save_to' parameter..." unless defined $a{save_to};

  my $ua = LWP::UserAgent->new( timeout => 10 );

  if (defined $a{proxy_host} && $a{proxy_host}) {
    my $port = defined $a{proxy_port} ? ":".$a{proxy_port} : "";
    if ($a{proxy_host} =~ /^http/i) {
      $ua->proxy( ['http'] => $a{proxy_host}.$port );
    } else {
      $ua->proxy( ['http'] => "http://".$a{proxy_host}.$port );
    }
  }

  my $r = $ua->get(
    "http://".$a{server}.":".$a{port}.$a{dl_path},
    ":content_file" => $a{save_to},
  );

  if (defined $a{proxy_user}) {
    $r->proxy_authorization_basic($a{proxy_user}, defined $a{proxy_pass} ? $a{proxy_pass} : "");
  }

  if ($r->code() eq '200') {
    return 1;
  } else {
    unlink $a{save_to} if -e $a{save_to};
    return 0;
  }
}

sub __harddrive_id {
  my $hdparm = $^O eq 'MSWin32' ? 'hdparm.exe' : 'hdparm';
  my $hdp_path = $Bin.'/'.$hdparm;

  my $r_serial = undef;

  if ($^O eq 'MSWin32') {
    my @disks = ( 'a', 'b', 'c', 'd' );

    for (@disks) {
      my $out_f = $Bin.'/hdd.txt';
      my $prg = '"'.$hdp_path.'" -i /dev/hd'.$_.' > "'.$out_f.'" 2> nul';
      my $res = system("$prg");

      if ($res == 0) {
        local $/=undef;
        open(F, "< ".$out_f);
        my $cnt = <F>;
        close(F);
        my( $serial ) = $cnt =~ /SerialNo=(.*)/;
        unlink($out_f);

        if ($serial) {
          $serial =~ s/^\s+//;
          $serial =~ s/\s+$//;
        }

        $r_serial = $serial;
        last;
      }
    }
  }
  elsif ($^O eq 'linux') {
    my @devs = ();
    
    # WinREST
    if (-e "/sbin/init2") {
      push @devs, '/dev/discs/disc0/disc';
      push @devs, '/dev/discs/disc1/disc';
      push @devs, '/dev/discs/disc2/disc';
      push @devs, '/dev/discs/disc3/disc';
      push @devs, '/dev/discs/disc4/disc';
    } else {
      my @lines;

      if (-e '/bin/mount') {
        @lines = `/bin/mount`;
      } elsif (-e '/sbin/mount') {
        @lines = `/sbin/mount`;
      }
      
      for (@lines) {
        if (/^(\/.*?)\s/) {
          push @devs, $1;
        }
      }
    }

    my $hd_parm = undef;

    $hd_parm = '/bin/hdparm' if -e '/bin/hdparm' && ! defined $hd_parm;
    $hd_parm = '/sbin/hdparm' if -e '/sbin/hdparm' && ! defined $hd_parm;
    $hd_parm = $Bin.'/hdparm' if -e $Bin.'/hdparm' && ! defined $hd_parm;

    if (defined $hd_parm && scalar @devs) {
      for (@devs) {
        my $out_f = $Bin.'/hdd.txt';
        my $prg = '"'.$hd_parm.'" -i '.$_.' > "'.$out_f.'" 2> /dev/null';
        my $res = system("$prg");
        if ($res == 0) {
          local $/=undef;
          open(F, "< ".$out_f);
          my $cnt = <F>;
          close(F);
          my( $serial ) = $cnt =~ /SerialNo=(.*)/;
          unlink($out_f);

          if ($serial) {
            $serial =~ s/^\s+//;
            $serial =~ s/\s+$//;
          }

          $r_serial = $serial;
          last;
        }
      }
    }
  }

  return $r_serial;
}

sub __trim {
  my $w = shift;
  $w =~ s/^\s+//;
  $w =~ s/\s+$//;
  return $w;
}

sub __read_winrest_temp_file {
  my $fname = shift;

  my @lines = ();

  open(FH,"<$fname");

  my $buff = "";

  while(<FH>) {
    $buff .= $_;
  }

  close FH;

  return 0 if ( length($buff) == 200);

  return unpack('d', pack('h*', scalar(reverse(&__invert_bytes(substr($buff,156,8))))));
}

sub __invert_bytes {
  my $buffer = shift;

  (my $hex = join("", reverse split(//,$buffer))) =~ s/(.|\n)/sprintf("%02lx", ord $1)/eg;
  
  return $hex;
}

sub __extract_autocs_from_locals {
  my $f = shift;
  
  my $r = {};
  
  if (open(F, "< $f")) {
    while(<F>) {
      next unless /4096/;
      my $tables = substr($_, 47, 32);
      $tables =~ s/\s+$//;
      my @toks = split /,/, $tables;
      for (@toks) {
        if (/^(\d+)\:(\d+)$/) {
          for ($1 .. $2) {
            $r->{$_} = 1;
          }
        } else {
          $r->{$_} = 1;
        }
      }
    }
    close(F);
  }
  
  return $r;
}

1;


