package WO::Client::WinRest::Store;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use DBI;
use WO::DB::FireBird;
use Data::Dumper;
use File::Slurp qw( write_file read_file read_dir );
use POSIX;
use File::Remove;
use File::Copy qw( move );
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # frontoffice directory
  my $gdb_file = $xml->{gdb_file} || "";

  # directory separator
  my $dsep = $s->directory_separator();

  if (! -e $gdb_file) {
    die("GDB file '$gdb_file' don't exists...");
  }

  my $history_days_limit = $xml->{history_days_limit} || "";
  my $date_from;

  if ($history_days_limit) {
    my $time_back = 60*60*(24*$history_days_limit); # 1 month
    $date_from = strftime("%Y/%m/%d", localtime(time()-$time_back));
  }

  my $file_stores           = $s->tmp_dir() . $dsep . '__stores__.txt';       # done
  my $file_warehouses       = $s->tmp_dir() . $dsep . '__warehouses__.txt';       # done
  my $file_store_warehouses = $s->tmp_dir() . $dsep . '__store_warehouses__.txt';       # done
  my $file_store_products   = $s->tmp_dir() . $dsep . '__store_products__.txt';
  my $file_suppliers        = $s->tmp_dir() . $dsep . '__suppliers__.txt';    # done
  my $file_families         = $s->tmp_dir() . $dsep . '__families__.txt';     # done
  my $file_articles         = $s->tmp_dir() . $dsep . '__articles__.txt';     # done
  my $file_units            = $s->tmp_dir() . $dsep . '__units__.txt';        # done
  my $file_supp_art         = $s->tmp_dir() . $dsep . '__supp_art__.txt';     # done
  my $file_users            = $s->tmp_dir() . $dsep . '__users__.txt';        # done
  my $file_pay_cond         = $s->tmp_dir() . $dsep . '__pay_cond__.txt';     # done
  my $file_doc_types        = $s->tmp_dir() . $dsep . '__doc_types__.txt';    # done
  my $file_prod_part        = $s->tmp_dir() . $dsep . '__prod_part__.txt';
  my $file_prod_price       = $s->tmp_dir() . $dsep . '__prod_price__.txt';
  my $file_taxes            = $s->tmp_dir() . $dsep . '__taxes__.txt';
  my $file_fiscal_regions   = $s->tmp_dir() . $dsep . '__fiscal_regions__.txt';
  my $file_inv_headers      = $s->tmp_dir() . $dsep . '__inventory_headers__.txt';
  my $file_inv_details      = $s->tmp_dir() . $dsep . '__inventory_details__.txt';
  my $file_buy_headers      = $s->tmp_dir() . $dsep . '__reception_headers__.txt';
  my $file_buy_details      = $s->tmp_dir() . $dsep . '__reception_details__.txt';
  my $file_buy_footers      = $s->tmp_dir() . $dsep . '__reception_footers__.txt';
  my $file_clients          = $s->tmp_dir() . $dsep . '__clients__.txt';
  my $file_store_clients    = $s->tmp_dir() . $dsep . '__store_clients__.txt';

  &File::Remove::remove($file_stores)  if -e $file_stores;
  &File::Remove::remove($file_warehouses)  if -e $file_warehouses;
  &File::Remove::remove($file_store_warehouses)  if -e $file_store_warehouses;
  &File::Remove::remove($file_store_products)  if -e $file_store_products;
  &File::Remove::remove($file_suppliers)  if -e $file_suppliers;
  &File::Remove::remove($file_families)   if -e $file_families;
  &File::Remove::remove($file_articles)   if -e $file_articles;
  &File::Remove::remove($file_units)      if -e $file_units;
  &File::Remove::remove($file_supp_art)   if -e $file_supp_art;
  &File::Remove::remove($file_users)      if -e $file_users;
  &File::Remove::remove($file_pay_cond)   if -e $file_pay_cond;
  &File::Remove::remove($file_doc_types)   if -e $file_doc_types;
  &File::Remove::remove($file_prod_part)  if -e $file_prod_part;
  &File::Remove::remove($file_prod_price) if -e $file_prod_price;
  &File::Remove::remove($file_taxes)      if -e $file_taxes;
  &File::Remove::remove($file_fiscal_regions) if -e $file_fiscal_regions;
  &File::Remove::remove($file_inv_headers) if -e $file_inv_headers;
  &File::Remove::remove($file_inv_details) if -e $file_inv_details;
  &File::Remove::remove($file_buy_headers) if -e $file_buy_headers;
  &File::Remove::remove($file_buy_details) if -e $file_buy_details;
  &File::Remove::remove($file_buy_footers) if -e $file_buy_footers;
  &File::Remove::remove($file_clients) if -e $file_clients;
  &File::Remove::remove($file_store_clients) if -e $file_store_clients;

  my $dsn = 'dbi:InterBase:db='.$gdb_file.';host=localhost;ib_dialect=3';
  my $dbh = DBI->connect($dsn, 'SYSDBA', 'masterkey' ) or die("Can't connect to database: $@");

  my $db = WO::DB::FireBird->new( dbh => $dbh );

  $xml->{sync_users} = exists $xml->{sync_users} ? $xml->{sync_users} : 1;
  $xml->{sync_articles} = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_article_prices} = exists $xml->{sync_article_prices} ? $xml->{sync_article_prices} : 1;
  $xml->{sync_families} = exists $xml->{sync_families} ? $xml->{sync_families} : 1;
  $xml->{sync_sells} = exists $xml->{sync_sells} ? $xml->{sync_sells} : 1;
  $xml->{sync_payment_types} = exists $xml->{sync_payment_types} ? $xml->{sync_payment_types} : 1;
  $xml->{sync_units} = exists $xml->{sync_units} ? $xml->{sync_units} : 1;
  $xml->{sync_suppliers} = exists $xml->{sync_suppliers} ? $xml->{sync_suppliers} : 1;
  $xml->{sync_composite_articles} = exists $xml->{sync_composite_articles} ? $xml->{sync_composite_articles} : 1;
  $xml->{sync_supplier_articles} = exists $xml->{sync_supplier_articles} ? $xml->{sync_supplier_articles} : 1;
  $xml->{sync_inventories} = exists $xml->{sync_inventories} ? $xml->{sync_inventories} : 1;
  $xml->{sync_receptions} = exists $xml->{sync_receptions} ? $xml->{sync_receptions} : 1;
  $xml->{sync_document_types} = exists $xml->{sync_document_types} ? $xml->{sync_document_types} : 1;
  $xml->{sync_clients} = exists $xml->{sync_clients} ? $xml->{sync_clients} : 1;
  $xml->{sync_stores} = exists $xml->{sync_stores} ? $xml->{sync_stores} : 1;
  $xml->{sync_taxes} = exists $xml->{sync_taxes} ? $xml->{sync_taxes} : 1;

  my %info = ();

  if ($xml->{sync_clients}) {
    $s->debug("sync_clients");

    # clients
    my @clients = $db->select("SELECT * FROM CLIENT");
    $s->debug("\t".scalar(@clients)." records (clients)...");
    write_file( $file_clients, Dumper(\@clients) );

    # store_clients
    my @store_clients = $db->select("SELECT * FROM STORE_CLIENT");
    $s->debug("\t".scalar(@store_clients)." records (store_clients)...");
    write_file( $file_store_clients, Dumper(\@store_clients) );
  }

  # inventories
  if ($xml->{sync_inventories}) {
    $s->debug("sync_inventories");

    my @inv_headers = ();

    if ($history_days_limit) {
      @inv_headers = $db->select("SELECT * FROM INVENTORYHEADER WHERE INVENTORY_DATE >= ?", $date_from);
    } else {
      @inv_headers = $db->select("SELECT * FROM INVENTORYHEADER");
    }

    $s->debug("\t".scalar(@inv_headers)." headers...");

    write_file( $file_inv_headers, Dumper(\@inv_headers) );
    undef @inv_headers;

    my @inv_details = ();

    if ($history_days_limit) {
      @inv_details = $db->select(qq{
        SELECT d.*
        FROM INVENTORYDETAIL d
        LEFT JOIN INVENTORYHEADER h ON (h.CODE = d.CODE AND h.STORE_CODE = d.STORE_CODE)
        WHERE h.INVENTORY_DATE >= ?
      }, $date_from);
    } else {
      @inv_details = $db->select(qq{
        SELECT d.*
        FROM INVENTORYDETAIL d
        LEFT JOIN INVENTORYHEADER h ON (h.CODE = d.CODE AND h.STORE_CODE = d.STORE_CODE)
      });
    }

    $s->debug("\t".scalar(@inv_details)." details...");

    write_file( $file_inv_details, Dumper(\@inv_details) );
    undef @inv_details;
  }

  if ($xml->{sync_document_types}) {
    $s->debug("sync_document_types");

    # document types
    my @documents = $db->select("SELECT * FROM DOCREFERENCE");
    $s->debug("\t".scalar(@documents)." records...");
    write_file( $file_doc_types, Dumper(\@documents) );
  }

  if ($xml->{sync_articles}) {
    $s->debug("sync_articles");

    # articles
    my @articles = $db->select("SELECT * FROM PRODUCT");
    $s->debug("\t".scalar(@articles)." records...");
    write_file( $file_articles, Dumper(\@articles) );
  }

  if ($xml->{sync_receptions}) {
    $s->debug("sync_receptions");

    my $sub;

    # headers
    my @buys_headers;

    if ($history_days_limit) {
      @buys_headers = $db->select("SELECT * FROM BDOCHEADER WHERE DOC_DATE >= ?", $date_from);
    } else {
      @buys_headers = $db->select("SELECT * FROM BDOCHEADER");
    }

    $s->debug("\t".scalar(@buys_headers)." headers...");

    write_file( $file_buy_headers, Dumper(\@buys_headers) );
    undef @buys_headers;

    # details
    my @buys_details = ();

    if ($history_days_limit) {
      @buys_details = $db->select(qq{
        SELECT d.*
        FROM BMOVDOCDETAIL d
        LEFT JOIN BDOCHEADER h ON (d.STORE_CODE = h.STORE_CODE AND d.DOC_CODE = h.DOC_CODE AND d.DOC_NUMBER = h.DOC_NUMBER)
        WHERE h.DOC_DATE >= ?
      }, $date_from);
    } else {
      @buys_details = $db->select(qq{
        SELECT d.*
        FROM BMOVDOCDETAIL d
        LEFT JOIN BDOCHEADER h ON (d.STORE_CODE = h.STORE_CODE AND d.DOC_CODE = h.DOC_CODE AND d.DOC_NUMBER = h.DOC_NUMBER)
      });
    }

    $s->debug("\t".scalar(@buys_details)." details...");
 
    write_file( $file_buy_details, Dumper(\@buys_details) );
    undef @buys_details;

    # footers
    my @buys_footers = ();

    if ($history_days_limit) {
      @buys_footers = $db->select(qq{
        SELECT f.*
        FROM BMOVDOCFOOTER f
        LEFT JOIN BDOCHEADER h ON (f.STORE_CODE = h.STORE_CODE AND f.DOC_CODE = h.DOC_CODE AND f.DOC_NUMBER = h.DOC_NUMBER)
        WHERE h.DOC_DATE >= ?
      }, $date_from);
    } else {
      @buys_footers = $db->select(qq{
        SELECT f.*
        FROM BMOVDOCFOOTER f
        LEFT JOIN BDOCHEADER h ON (f.STORE_CODE = h.STORE_CODE AND f.DOC_CODE = h.DOC_CODE AND f.DOC_NUMBER = h.DOC_NUMBER)
        WHERE h.DOC_DATE >= ?
      });
    }

    $s->debug("\t".scalar(@buys_footers)." footers...");

    write_file( $file_buy_footers, Dumper(\@buys_footers) );
    undef @buys_footers;
  }

  # article_prices
  if ($xml->{sync_article_prices}) {
    $s->debug("sync_article_prices");

    my @store_products = $db->select(qq{
        SELECT sp.store_code, pp.product_code, pp.price_wo_tax
        FROM store_product sp, productprice pp
        WHERE sp.product_code = pp.product_code AND sp.checked is not null
    });

    $s->debug("\t".scalar(@store_products)." records...");

    write_file( $file_store_products, Dumper(\@store_products) );
  }

  if ($xml->{sync_suppliers}) {
    $s->debug("sync_suppliers");

    # suppliers
    my @suppliers = $db->select("SELECT * FROM SUPLIER");
    $s->debug("\t".scalar(@suppliers)." records...");
    write_file( $file_suppliers, Dumper(\@suppliers) );
  }

  if ($xml->{sync_families}) {
    $s->debug("sync_families");

    # families
    my @families = $db->select("SELECT * FROM FAMILY");
    my @fo_families;
    eval { @fo_families = $db->select("SELECT * FROM FO_FAMILY"); };
    map { $_->{fo_family} = 1 } @fo_families;
    my @all = ( @families, @fo_families );

    $s->debug("\t".scalar(@all)." records...");

    write_file( $file_families, Dumper(\@all) );
  }

  if ($xml->{sync_units}) {
    $s->debug("sync_units");

    # units
    my @units_tmp = $db->select("SELECT * FROM UNIT");
    $s->debug("\t".scalar(@units_tmp)." records...");
    write_file( $file_units, Dumper(\@units_tmp) );
  }

  if ($xml->{sync_supplier_articles}) {
    $s->debug("sync_supplier_articles");

    # supplier articles
    my @supp_art = $db->select("SELECT * FROM PRODUCT_SUPLIER");
    $s->debug("\t".scalar(@supp_art)." records...");
    write_file( $file_supp_art, Dumper(\@supp_art) );
  }

  if ($xml->{sync_users}) {
    $s->debug("sync_users");

    # users
    my @users = $db->select("SELECT * FROM EMPLOYEE");
    $s->debug("\t".scalar(@users)." records...");
    write_file( $file_users, Dumper(\@users) );
  }

  if ($xml->{sync_payment_types}) {
    $s->debug("sync_payment_types");

    # payment conditions
    my @pay_cond = $db->select("SELECT * FROM PAYMENTCOND");
    $s->debug("\t".scalar(@pay_cond)." records...");
    write_file( $file_pay_cond, Dumper(\@pay_cond) );
  }

  if ($xml->{sync_composite_articles}) {
    $s->debug("sync_composite_articles");

    # product parts
    my @prod_part = $db->select("SELECT * FROM PRODUCTPART");
    $s->debug("\t".scalar(@prod_part)." records...");
    write_file( $file_prod_part, Dumper(\@prod_part) );
  }

  if ($xml->{sync_taxes}) {
    $s->debug("sync_taxes");

    #
    # Get fiscal regions, if present
    #

    my @fiscal_regions = ();

    eval {
      @fiscal_regions = $db->select("SELECT * FROM FISCALREGION");
    };

    if (! $@) {
      $s->debug("WILL USE FISCAL REGIONS...");

      write_file( $file_fiscal_regions, Dumper(\@fiscal_regions) );

      my @taxes = $db->select("SELECT * FROM FISCALREGION_TAX");
      $s->debug("\t".scalar(@taxes)." records...");
      write_file( $file_taxes, Dumper(\@taxes) );
    } else {
      $s->debug("THIS STORE VERSION, DON'T HAVE FISCAL REGIONS, WILL USE NUMPARAM...");
      # taxes
      my @taxes = $db->select("SELECT * FROM NUMPARAM WHERE PARAM_TYPE='TIVA' ORDER BY CODE");
      $s->debug("\t".scalar(@taxes)." records...");
      write_file( $file_taxes, Dumper(\@taxes) );
    }
  }

  if ($xml->{sync_stores}) {
    $s->debug("sync_stores");

    # stores
    my @stores = $db->select(qq{
      SELECT * FROM STORE
    });

    my @warehouses = $db->select(qq{  
      SELECT * FROM WAREHOUSE WHERE DELETED = 0
    });

    my @store_warehouses = $db->select(qq{
      SELECT * FROM STORE_WAREHOUSE
    });

    $s->debug("\t".scalar(@stores)." records (stores)...");
    $s->debug("\t".scalar(@warehouses)." records (warehouses)...");
    $s->debug("\t".scalar(@store_warehouses)." records (store_warehouses)...");

    write_file( $file_stores, Dumper(\@stores) );
    write_file( $file_warehouses, Dumper(\@warehouses) );
    write_file( $file_store_warehouses, Dumper(\@store_warehouses) );
  }

  #
  # Prepare ZIP file
  #

  my $zip = Archive::Zip->new();

  $dsep = $s->directory_separator();

  my $zipFile = $s->tmp_dir() . $dsep . $fo_type.'_FO_DATA.zip';
  &File::Remove::remove( $zipFile ) if -e $zipFile;

  #
  # Articles
  #

  $zip->addFile( $file_inv_headers ) if -e $file_inv_headers;
  $zip->addFile( $file_inv_details ) if -e $file_inv_details;
  $zip->addFile( $file_buy_headers ) if -e $file_buy_headers;
  $zip->addFile( $file_buy_details ) if -e $file_buy_details;
  $zip->addFile( $file_buy_footers ) if -e $file_buy_footers;
  $zip->addFile( $file_stores ) if -e $file_stores;
  $zip->addFile( $file_warehouses ) if -e $file_warehouses;
  $zip->addFile( $file_store_products ) if -e $file_store_products;
  $zip->addFile( $file_suppliers ) if -e $file_suppliers;
  $zip->addFile( $file_families ) if -e $file_families;
  $zip->addFile( $file_articles ) if -e $file_articles;
  $zip->addFile( $file_units ) if -e $file_units;
  $zip->addFile( $file_supp_art ) if -e $file_supp_art;
  $zip->addFile( $file_pay_cond ) if -e $file_pay_cond;
  $zip->addFile( $file_doc_types ) if -e $file_doc_types;
  $zip->addFile( $file_prod_part ) if -e $file_prod_part;
  $zip->addFile( $file_prod_price ) if -e $file_prod_price;
  $zip->addFile( $file_users ) if -e $file_users;
  $zip->addFile( $file_taxes ) if -e $file_taxes;
  $zip->addFile( $file_fiscal_regions ) if -e $file_fiscal_regions;
  $zip->addFile( $file_clients ) if -e $file_clients;
  $zip->addFile( $file_store_clients ) if -e $file_store_clients;
  $zip->addFile( $file_store_warehouses ) if -e $file_store_warehouses;

  die('write error') unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

  #
  # Send ZIP file to the server
  #

  eval {
    $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                file_upload => $zipFile,
                              }
                            );
  };

  if ($@) {
    die("Error sending file: $@");
  } else {
    &File::Remove::remove($file_inv_headers) if -e $file_inv_headers;
    &File::Remove::remove($file_inv_details) if -e $file_inv_details;
    &File::Remove::remove($file_buy_headers) if -e $file_buy_headers;
    &File::Remove::remove($file_buy_details) if -e $file_buy_details;
    &File::Remove::remove($file_buy_footers) if -e $file_buy_footers;
    &File::Remove::remove($file_stores) if -e $file_stores;
    &File::Remove::remove($file_warehouses) if -e $file_warehouses;
    &File::Remove::remove($file_store_products) if -e $file_store_products;
    &File::Remove::remove($file_suppliers) if -e $file_suppliers;
    &File::Remove::remove($file_families) if -e $file_families;
    &File::Remove::remove($file_articles) if -e $file_articles;
    &File::Remove::remove($file_units) if -e $file_units;
    &File::Remove::remove($file_supp_art) if -e $file_supp_art;
    &File::Remove::remove($file_pay_cond) if -e $file_pay_cond;
    &File::Remove::remove($file_doc_types) if -e $file_doc_types;
    &File::Remove::remove($file_prod_part) if -e $file_prod_part;
    &File::Remove::remove($file_prod_price) if -e $file_prod_price;
    &File::Remove::remove($file_taxes) if -e $file_taxes;
    &File::Remove::remove($file_users) if -e $file_users;
    &File::Remove::remove($file_fiscal_regions) if -e $file_fiscal_regions;
    &File::Remove::remove($file_clients) if -e $file_clients;
    &File::Remove::remove($file_store_clients) if -e $file_store_clients;
    &File::Remove::remove($file_store_warehouses) if -e $file_store_warehouses;
    &File::Remove::remove($zipFile) if -e $zipFile;
  }

  $s->debug("Done.");
}

sub _round {
  my $v = shift || 0;
  return int($v + 0.5) > $v ? POSIX::ceil($v) : POSIX::floor($v);
}

1;

