package WO::Client::WinTouch::Access;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Slurp qw( write_file );
use Data::Dumper;
use FileHandle;
use DBI;
use FindBin qw( $Bin );

use WinTouch::Export::Artigos;
use WinTouch::Export::Familias;
use WinTouch::Export::Grupos;
use WinTouch::Export::Categorias;
use WinTouch::Export::BotoesCategoria;
use WinTouch::Export::Funcionarios;

use WO::DB;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $data_dir = $xml->{data_dir};

  if (! -d $data_dir) {
    $s->debug("Can't find data_dir $data_dir");
    die "Can't find data_dir $data_dir";
  }

  # directory separator
  my $dsep = $s->directory_separator();

  my $file_mestres = $data_dir.'\MstMestres.mdb';
  my $file_vendas = $data_dir.'\AcvVendas.mdb';
  my $file_backoffice = $data_dir.'\bckback.mdb';

  my $has_backoffice_db = -e $file_backoffice ? 1 : 0;
  my $has_mestres_db    = -e $file_mestres ? 1 : 0;

  my $file_inventories       = $Bin.'/__inventories__.txt';
  my $file_warehouses        = $Bin.'/__warehouses__.txt';
  my $file_suppliers         = $Bin.'/__suppliers__.txt';
  my $file_art_suppliers     = $Bin.'/__supplier_articles__.txt';
  my $buys_headers           = $Bin.'/__buys_headers__.txt';
  my $buys_details           = $Bin.'/__buys_details__.txt';
  my $file_groups            = $Bin.'/__groups__.txt';
  my $file_families          = $Bin.'/__families__.txt';
  my $file_articles          = $Bin.'/__articles__.txt';
  my $file_art_warehouses    = $Bin.'/__articles_warehouses__.txt';
  my $file_units             = $Bin.'/__units__.txt';
  my $file_users             = $Bin.'/__users__.txt';
  my $file_pay_cond          = $Bin.'/__pay_cond__.txt';
  my $file_doc_types         = $Bin.'/__doc_types__.txt';
  my $file_taxes             = $Bin.'/__taxes__.txt';
  my $file_article_composite = $Bin.'/__article_composite__.txt';
  my $file_movements         = $Bin.'/__movements__.txt';
  my $file_movement_payments = $Bin.'/__movement_payments__.txt';

  $xml->{sync_users} = exists $xml->{sync_users} ? $xml->{sync_users} : 1;
  $xml->{sync_articles} = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_article_prices} = exists $xml->{sync_article_prices} ? $xml->{sync_article_prices} : 1;
  $xml->{sync_families} = exists $xml->{sync_families} ? $xml->{sync_families} : 1;
  $xml->{sync_sells} = exists $xml->{sync_sells} ? $xml->{sync_sells} : 1;
  $xml->{sync_payment_types} = exists $xml->{sync_payment_types} ? $xml->{sync_payment_types} : 1;
  $xml->{sync_document_types} = exists $xml->{sync_document_types} ? $xml->{sync_document_types} : 1;
  $xml->{sync_units} = exists $xml->{sync_units} ? $xml->{sync_units} : 1;
  $xml->{sync_suppliers} = exists $xml->{sync_suppliers} ? $xml->{sync_suppliers} : 1;
  $xml->{sync_composite_articles} = exists $xml->{sync_composite_articles} ? $xml->{sync_composite_articles} : 1;
  $xml->{sync_supplier_articles} = exists $xml->{sync_supplier_articles} ? $xml->{sync_supplier_articles} : 1;
  $xml->{sync_inventories} = exists $xml->{sync_inventories} ? $xml->{sync_inventories} : 1;
  $xml->{sync_receptions} = exists $xml->{sync_receptions} ? $xml->{sync_receptions} : 1;
  $xml->{sync_warehouses} = exists $xml->{sync_warehouses} ? $xml->{sync_warehouses} : 1;
  $xml->{sync_taxes} = exists $xml->{sync_taxes} ? $xml->{sync_taxes} : 1;

  if ($xml->{sync_sells}) {
    die "Can't find file '$file_vendas'" unless -e $file_vendas;
  }

  #die "Can't find file '$file_mestres'" unless -e $file_mestres;

  my @zip_files = ();

  $s->debug("Will process masters...");

  my $db_mestres = "";

  if ($has_mestres_db) {
    my $dsn = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$file_mestres;
    my $dbh = DBI->connect($dsn) or die "Can't connect to database: $@";

    $dbh->{'LongTruncOk'} = 1;
    $dbh->{'LongReadLen'} = 200;

    $db_mestres = WO::DB->new( dbh => $dbh );
  }

  # wintouch version
  #my @wintouch_version = $db_mestres->select("SELECT * FROM Versao");
  #my $version = $wintouch_version[0]->{Versao};

    $s->own_remove($file_inventories)       if -e $file_inventories;
    $s->own_remove($file_warehouses)        if -e $file_warehouses;
    $s->own_remove($buys_headers)           if -e $buys_headers;
    $s->own_remove($buys_details)           if -e $buys_details;
    $s->own_remove($file_suppliers)         if -e $file_suppliers;
    $s->own_remove($file_art_suppliers)     if -e $file_art_suppliers;
    $s->own_remove($file_groups)            if -e $file_groups;
    $s->own_remove($file_families)          if -e $file_families;
    $s->own_remove($file_articles)          if -e $file_articles;
    $s->own_remove($file_art_warehouses)    if -e $file_art_warehouses;
    $s->own_remove($file_units)             if -e $file_units;
    $s->own_remove($file_users)             if -e $file_users;
    $s->own_remove($file_pay_cond)          if -e $file_pay_cond;
    $s->own_remove($file_doc_types)         if -e $file_doc_types;
    $s->own_remove($file_taxes)             if -e $file_taxes;
    $s->own_remove($file_article_composite) if -e $file_article_composite;

    my $sub;

    if ($has_backoffice_db) {
      my $dsn3 = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$file_backoffice;

      my $dbh3 = DBI->connect($dsn3) or die "Can't connect to database: $@";
      $dbh3->{'LongTruncOk'} = 1;
      $dbh3->{'LongReadLen'} = 200;

      my $db_backoffice = WO::DB->new( dbh => $dbh3 );

      if ($xml->{sync_inventories}) {
        # inventories
        $s->debug("inventories");
        my @inventories = $db_backoffice->select("SELECT * FROM Inventario");
        write_file( $file_inventories, Dumper(\@inventories) );
        push @zip_files, $file_inventories;
        undef @inventories;
        #$sub = $db_backoffice->select_iter("SELECT * FROM Inventario");
        #$s->write_csv_from_iter( $file_inventories, $sub );
        #push @zip_files, $file_inventories;
      }

      if ($xml->{sync_suppliers}) {
        # suppliers
        $s->debug("suppliers");
        my @suppliers = $db_backoffice->select("SELECT * FROM Fornecedores");
        write_file( $file_suppliers, Dumper(\@suppliers) );
        push @zip_files, $file_suppliers;
        undef @suppliers;
        #$sub = $db_backoffice->select_iter("SELECT * FROM Fornecedores");
        #$s->write_csv_from_iter( $file_suppliers, $sub );
        #push @zip_files, $file_suppliers;
      }

      if ($xml->{sync_receptions}) {
        # buys headers
        $s->debug("buys headers");
        my @buys_headers = $db_backoffice->select("SELECT * FROM CabCompras");
        write_file( $buys_headers, Dumper(\@buys_headers) );
        push @zip_files, $buys_headers;
        undef @buys_headers;
        #$sub = $db_backoffice->select_iter("SELECT * FROM CabCompras");
        #$s->write_csv_from_iter( $buys_headers, $sub );
        #push @zip_files, $buys_headers;

        # buys details
        $s->debug("buys details");
        my @buys_details = $db_backoffice->select("SELECT * FROM LinCompras");
        write_file( $buys_details, Dumper(\@buys_details) );
        push @zip_files, $buys_details;
        undef @buys_details;
        #$sub = $db_backoffice->select_iter("SELECT * FROM LinCompras");
        #$s->write_csv_from_iter( $buys_details, $sub );
        #push @zip_files, $buys_details;
      }
    }

    my @tmp;

    if ($xml->{sync_warehouses}) {
      # warehouses
      $s->debug("warehouses");
      my @warehouses = $db_mestres->select("SELECT * FROM Armazens");
      write_file( $file_warehouses, Dumper(\@warehouses) );
      push @zip_files, $file_warehouses;
      undef @warehouses;
    }

    if ($xml->{sync_supplier_articles}) {
      # art_suppliers
      $s->debug("art suppliers");
      my @art_suppliers = ();

      # this is here because some wintouch versions have ArtFornecedores others ArtFornecedor
      @tmp = $db_mestres->select("SELECT * FROM ArtFornecedores");
      push @art_suppliers, @tmp if scalar @tmp;
      @tmp = $db_mestres->select("SELECT * FROM ArtFornecedor");
      push @art_suppliers, @tmp if scalar @tmp;

      write_file( $file_art_suppliers, Dumper(\@art_suppliers) );
      push @zip_files, $file_art_suppliers;
      undef @art_suppliers;
      #$sub = $db_mestres->select_iter("SELECT * FROM ArtFornecedores");
      #$s->write_csv_from_iter( $file_art_suppliers, $sub );
      #push @zip_files, $file_art_suppliers;
    }

    if ($xml->{sync_articles}) {
      # articles
      $s->debug("articles");
      my @articles = $db_mestres->select("SELECT * FROM Artigos");
      write_file( $file_articles, Dumper(\@articles) );
      push @zip_files, $file_articles;
      undef @articles;
      #$sub = $db_mestres->select_iter("SELECT * FROM Artigos");
      #$s->write_csv_from_iter( $file_articles, $sub );
      #push @zip_files, $file_articles;
    }

=pod
      # art_armazem
      $s->debug("art armazem");
      #my @art_armazem = $db_mestres->select("SELECT Artigo, Armazem FROM ArtArmazem");
      #write_file( $file_art_warehouses, Dumper(\@art_armazem) );
      #push @zip_files, $file_art_warehouses;
      #undef @art_armazem;
      $sub = $db_mestres->select_iter("SELECT Artigo, Armazem FROM ArtArmazem");
      $s->write_csv_from_iter( $file_art_warehouses, $sub );
      push @zip_files, $file_art_warehouses;
=cut

    if ($xml->{sync_families}) {
      # groups
      $s->debug("groups");
      my @groups = $db_mestres->select("SELECT * FROM Grupos");
      write_file( $file_groups, Dumper(\@groups) );
      push @zip_files, $file_groups;
      undef @groups;

      # families
      $s->debug("families");
      my @families = $db_mestres->select("SELECT * FROM Familias");
      write_file( $file_families, Dumper(\@families) );
      push @zip_files, $file_families;
      undef @families;
    }

    if ($xml->{sync_units}) {
      # units
      $s->debug("units");
      my @units = $db_mestres->select("SELECT * FROM Unidades");
      write_file( $file_units, Dumper(\@units) );
      push @zip_files, $file_units;
      undef @units;
    }

    if ($xml->{sync_users}) {
      # users
      $s->debug("users");
      my @users = $db_mestres->select("SELECT * FROM Funcionarios");
      write_file( $file_users, Dumper(\@users) );
      push @zip_files, $file_users;
      undef @users;
    }

    if ($xml->{sync_payment_types}) {
      # payment methods
      $s->debug("payment methods");
      my @payment_methods = $db_mestres->select("SELECT * FROM MeiosPagamento");
      write_file( $file_pay_cond, Dumper(\@payment_methods) );
      push @zip_files, $file_pay_cond;
      undef @payment_methods;
    }

    if ($xml->{sync_document_types}) {
      # document types
      $s->debug("document types");
      my @document_types = $db_mestres->select("SELECT * FROM Documentos");
      write_file( $file_doc_types, Dumper(\@document_types) );
      push @zip_files, $file_doc_types;
      undef @document_types;
    }

    if ($xml->{sync_taxes}) {
      # taxes
      $s->debug("taxes");
      my @taxes = $db_mestres->select("SELECT * FROM Iva");
      write_file( $file_taxes, Dumper(\@taxes) );
      push @zip_files, $file_taxes;
      undef @taxes;
    }

    if ($xml->{sync_composite_articles}) {
      # article composition
      $s->debug("article composition");
      my @composite = ();

      @tmp = $db_mestres->select("SELECT * FROM FichasTecnicas");
      push @composite, @tmp if scalar @tmp;
      @tmp = $db_mestres->select("SELECT * FROM FichasTecnicasVenda");
      push @composite, @tmp if scalar @tmp;

      write_file( $file_article_composite, Dumper(\@composite) );
      push @zip_files, $file_article_composite;
      undef @composite;
    }

  if ($xml->{sync_sells}) {
    $s->debug("Will process sells...");

    $s->_sync_sells();
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $Bin.$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      $s->own_remove($file_inventories)       if -e $file_inventories;
      $s->own_remove($file_warehouses)        if -e $file_warehouses;
      $s->own_remove($buys_headers)           if -e $buys_headers;
      $s->own_remove($buys_details)           if -e $buys_details;
      $s->own_remove($file_suppliers)         if -e $file_suppliers;
      $s->own_remove($file_art_suppliers)     if -e $file_art_suppliers;
      $s->own_remove($file_groups)            if -e $file_groups;
      $s->own_remove($file_families)          if -e $file_families;
      $s->own_remove($file_articles)          if -e $file_articles;
      $s->own_remove($file_art_warehouses)    if -e $file_art_warehouses;
      $s->own_remove($file_units)             if -e $file_units;
      $s->own_remove($file_users)             if -e $file_users;
      $s->own_remove($file_pay_cond)          if -e $file_pay_cond;
      $s->own_remove($file_doc_types)         if -e $file_doc_types;
      $s->own_remove($file_taxes)             if -e $file_taxes;
      $s->own_remove($file_article_composite) if -e $file_article_composite;
      $s->own_remove($file_movements)         if -e $file_movements;
      $s->own_remove($file_movement_payments) if -e $file_movement_payments;
      $s->own_remove($zipFile)                if -e $zipFile;
    }

    $s->debug("Done.");
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

#
# weckOFFICE to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $data_dir = $xml->{data_dir} || "";

  if (! -d $data_dir) {
    $s->debug("Can't find data_dir $data_dir");
    die "Can't find data_dir $data_dir";
  }

  # directory separator
  my $dsep = $s->directory_separator();

  # manso
  $s->_sync_sells();

  # manual execution, receive remote data
  if ($s->{manual}) {
  	$s->_receive_remote_data();
  }
  # else, sync sells
  else {
  	$s->_sync_sells();
  }
}

sub _sync_sells {
	my $s = shift;

  $s->debug("_sync_sells()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  my $data_dir = $xml->{data_dir} || "";
  my $warehouse_filter = $xml->{warehouse_filter} || "";

  # directory separator
  my $dsep = $s->directory_separator();

  my $file_mestres = $data_dir.'\MstMestres.mdb';
  my $file_vendas = $data_dir.'\AcvVendas.mdb';
  my $file_backoffice = $data_dir.'\bckback.mdb';

  my $date_from = $xml->{date_from} || undef;

  #
  # Sync Sells
  #

  my $file_movements         = $Bin.'/__movements__.txt';
  my $file_movement_payments = $Bin.'/__movement_payments__.txt';

  my @zip_files = ();

  if ($xml->{sync_sells} && -e $file_vendas) {
    $s->debug("Will process sells...");

    my $check_dates_fast = $xml->{check_dates_fast} || 0;

    $s->own_remove($file_movements)         if -e $file_movements;
    $s->own_remove($file_movement_payments) if -e $file_movement_payments;

    my $dsn = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$file_vendas;
    my $dbh = DBI->connect($dsn) or die "Can't connect to database: $@";
    $dbh->{'LongTruncOk'} = 1;
    $dbh->{'LongReadLen'} = 200;

    my $db_sells = WO::DB->new( dbh => $dbh );

    my $VAR1 = {};

    $s->debug("Will check existant dates...");

    my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDates';
    my $existent_dates = $s->weckoffice_send_data( { action => $act,
                                                   }
                                                 );

    eval "$existent_dates";
    my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

    my %control = ();
    map { $control{$_." 00:00:00"} = 1 } keys %existent_dates;

    # movements
    my $sub;

    # TODO: !!!!!!!!!!!!!!!!! PARA REMOVER !!!!!!!!!!!!!!!!!!!!!!!!
    #$db_sells->do("UPDATE Vendas SET TaxaIva = 12");

    my $q = "";
    my @values;

    push @values, $warehouse_filter if $warehouse_filter;

    if (defined $date_from) {
      push @values, $date_from;

      if ($warehouse_filter) {
        $q = "SELECT * FROM Vendas WHERE Armazem = ? AND DataVnd >= ?";
      } else {
        $q = "SELECT * FROM Vendas WHERE DataVnd >= ?";
      }
    } else {
      if ($warehouse_filter) {
        $q = "SELECT * FROM Vendas WHERE Armazem = ?";
      } else {
        $q = "SELECT * FROM Vendas";
      }
    }

    if (scalar @values) {
      $sub = $db_sells->select_iter($q, @values);
    } else {
      $sub = $db_sells->select_iter($q);
    }

    if ($sub) {
      $s->write_csv_from_iter( $file_movements, $sub, { control_index => 2, control_hash => \%control } );
      push @zip_files, $file_movements;
    }

    $q = "";
    @values = ();

    push @values, $warehouse_filter if $warehouse_filter;

    if (defined $date_from) {
      push @values, $date_from;

      if ($warehouse_filter) {
        $q = "SELECT * FROM MeiosPAgamento WHERE Armazem = ? AND DataMP >= ?";
      } else {
        $q = "SELECT * FROM MeiosPAgamento WHERE DataMP >= ?";
      }
    } else {
      if ($warehouse_filter) {
        $q = "SELECT * FROM MeiosPAgamento WHERE Armazem = ?";
      } else {
        $q = "SELECT * FROM MeiosPAgamento";
      }
    }

    $s->debug("#### $q");
    print STDERR Dumper(\@values);

    if (scalar @values) {
      $sub = $db_sells->select_iter($q, @values);
    } else {
      $sub = $db_sells->select_iter($q);
    }

    if ($sub) {
      $s->write_csv_from_iter( $file_movement_payments, $sub, { control_index => 2, control_hash => \%control } );
      push @zip_files, $file_movement_payments;
    }
  } else {
    $s->debug("Can't sync sells because file $file_vendas don't exists...");
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $Bin.$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      $s->own_remove($file_movements)         if -e $file_movements;
      $s->own_remove($file_movement_payments) if -e $file_movement_payments;
      $s->own_remove($zipFile)                if -e $zipFile;
    }

    $s->debug("Done.");
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

sub _receive_remote_data {
	my $s = shift;

  $s->debug("_receive_remote_data()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  my $data_dir = $xml->{data_dir} || "";

  # directory separator
  my $dsep = $s->directory_separator();

  my $file_mestres = $data_dir.'\MstMestres.mdb';
  my $file_vendas = $data_dir.'\AcvVendas.mdb';
  my $file_backoffice = $data_dir.'\bckback.mdb';

  my $has_backoffice_db = -e $file_backoffice ? 1 : 0;
  my $has_mestres_db    = -e $file_mestres ? 1 : 0;

  my $db_mestres = "";

  if ($has_mestres_db) {
    my $dsn = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$file_mestres;
    my $dbh = DBI->connect($dsn) or die "Can't connect to database: $@";
    $dbh-> {'LongTruncOk'} = 1;
    $dbh-> {'LongReadLen'} = 200;
    $db_mestres = WO::DB->new( dbh => $dbh );
  }

  my @ivas = ();
  my %ivas = ();

  if ($has_mestres_db) {
    @ivas = $db_mestres->select("SELECT Codigo, Taxa FROM Iva");
    %ivas = map { int($_->{Taxa}) => $_->{Codigo} } @ivas;
  }

  my $keyboard_number = $xml->{keyboard_number} || undef;
  my $cache_control = defined $xml->{cache_control} ? $xml->{cache_control} : 1;

  my $max_num = 99;
  $max_num = $keyboard_number if defined $keyboard_number;

#  if (! defined $keyboard_number) {
#    my @res = $db_mestres->select("SELECT MAX(Num) As max FROM BotoesCategoria");
#    $max_num = scalar @res ? $res[0]->{max}+1 : 1;
#  }

  $xml->{sync_articles} = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_families} = exists $xml->{sync_families} ? $xml->{sync_families} : 1;

  $xml->{export_file} ||= "";
  die "Missing 'export_file' parameter..." unless $xml->{export_file};

  $s->own_remove($xml->{export_file}.".ok") if -e $xml->{export_file}.".ok";

  my $VAR1 = undef;
  my $str = undef;

  my $bot = WinTouch::Export::BotoesCategoria->new();
  my $cat = WinTouch::Export::Categorias->new();
  my @buttons = ();
  my @categories = ();

  #
  # Employees
  #

  if ($xml->{sync_users}) {
    $s->debug("Will get remote users...");

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetWaitresses',
                                       }
                                     );
    };

    if ($@) {
      die "Error: $@";
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $users = WinTouch::Export::Funcionarios->new();

      $s->debug("Received ".scalar(@$VAR1). " users...");

      my @users = ();

      for my $rec (@$VAR1) {
        $rec->{name} =~ s/\s+$//;
        push @users, {
          Codigo    => $rec->{fo_id},
          Nome      => $rec->{name},
          Categoria => $rec->{fo_access_type},
          Password  => $rec->{fo_passwd},
          TodosSectores => 1,
          TemSessao     => 1,
          Balcao        => 1,
        };
      }

      $users->records(
        validate_fields => 1,
        data => \@users,
      );

      my $fh = FileHandle->new();

      if ($fh->open(">> ".$xml->{export_file})) {
        $db_mestres->do("DELETE FROM Funcionarios");
        $users->to_fh( fh => $fh, cache_control => 0 );
        $fh->close();
      }
    }
  }

  #
  # Families
  #

  my @families;
  my %top_families;

  eval {
    $str = $s->weckoffice_send_data( { action => 'WinTouchGetFamilies',
                                     }
                                   );
  };

  if ($@) {
    die "Error: $@";
  }

  eval $str;

  if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
    # categories
    for my $rec (@$VAR1) {
      if (! defined $rec->{top_family_id}) {
        my $name = substr($rec->{family_name}, 0, 15);
        $top_families{ uc($name) } = 1;
      }

      push @families, $rec;
    }
  }

  #
  # Articles
  #

  if ($xml->{sync_articles}) {
    $s->debug("Will get remote articles...");

    eval {
      $str = $s->weckoffice_send_data( { action => 'WinTouchGetArticles',
                                       }
                                     );
    };

    if ($@) {
      die "Error: $@";
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $art = WinTouch::Export::Artigos->new();

      my @recs = ();

      $s->debug("Received ".scalar(@$VAR1). " articles...");

      for my $rec (@$VAR1) {
        my $cat_name = substr($rec->{family_name}, 0, 15);

        my $i = {
          Categoria   => $cat_name,
          Artigo      => $rec->{article_fo_id},
          Ordem       => $rec->{vorder} ? $rec->{vorder}+1 : 1,
          Num         => $max_num,
          Label       => $rec->{fo_seetings}->{_nbot},
#          Tipotextura => $rec->{fo_seetings}->{_cbot} || 1,
          Tipotextura => 1,
        };

        $i->{Categoria} = $i->{Categoria}."S" if exists $top_families{ uc($cat_name) };

        push @buttons, $i;

        my $h = {
          Preco1 => defined $rec->{article_price1} ? $rec->{article_price1} : 0,
          Preco2 => defined $rec->{article_price2} ? $rec->{article_price2} : 0,
          Preco3 => defined $rec->{article_price3} ? $rec->{article_price3} : 0,
          Preco4 => defined $rec->{article_price4} ? $rec->{article_price4} : 0,
          Preco5 => defined $rec->{article_price5} ? $rec->{article_price5} : 0,
          Iva    => defined $rec->{article_tax1} ? $ivas{ int($rec->{article_tax1}) } : "",
          Iva1   => defined $rec->{article_tax2} ? $ivas{ int($rec->{article_tax2}) } : "",
          Codigo         => $rec->{article_fo_id} || "",
          Familia        => $rec->{family_fo_id} || "",
          CodBarras      => $rec->{article_barcode} || "",
          Descricao      => $rec->{article_name} || "",
          ArtVenda       => 1,
          CodTerminal    => "",

          AbrevImpressao => defined $rec->{fo_seetings}->{_ncur} ? substr($rec->{fo_seetings}->{_ncur}, 0, 30)  : "",
          AbrevTecla     => defined $rec->{fo_seetings}->{_nbot} ? substr($rec->{fo_seetings}->{_nbot}, 0, 100) : "",
          AbrevMonitor   => defined $rec->{fo_seetings}->{_nbot} ? substr($rec->{fo_seetings}->{_nbot}, 0, 30)  : "",
        };

        # define other fo settings
        foreach my $key (keys %{ $rec->{fo_seetings} }) {
          next if $key eq '_nbot' || $key eq '_ncur' || $key eq '_vord' || $key eq '_cbot';
          $h->{$key} = $rec->{fo_seetings}->{$key};
        }

        delete $h->{TamanhoBotoes} if exists $h->{TamanhoBotoes};

        push @recs, $h;
      }

      $art->records(
        validate_fields => 1,
        data => \@recs,
      );

      my $fh = FileHandle->new();

      if ($fh->open(">> ".$xml->{export_file})) {
        $art->to_fh( fh => $fh, cache_control => $cache_control );
        $fh->close();
      }
    }
  }

  #
  # Families
  #

  if ($xml->{sync_families}) {
    my $fam = WinTouch::Export::Familias->new();
    my $grp = WinTouch::Export::Grupos->new();

    my @info = ();

    $s->debug("Received ".scalar(@families). " families...");

    # categories
    for my $rec (@families) {
      next if defined $rec->{top_family_id};

      my $name = substr($rec->{family_name}, 0, 15);

      my $h = {
        Categoria => $name,
        Descricao => $rec->{family_name},
        Nivel => 0,
        Ordem => $rec->{vorder},
        Num => $max_num,
#          Tipotextura  => $rec->{fo_seetings}->{_cbot} || 1,
        Tipotextura  => 1,
        TamanhoBotoes => 53,
      };

      my $props = $rec->{fo_seetings} || {};

      for my $k (keys %$props) {
        next if $k =~ /^_/ || $k eq 'vorder';
        $h->{$k} = $props->{$k};
      }

      $top_families{ uc($name) } = 1;
      push @categories, $h;
    }

    # sub categories
    for my $rec (@families) {
      next unless defined $rec->{top_family_id};

      my $name = substr($rec->{family_name}, 0, 15);

      my $h = {
        Categoria => $name,
        Descricao => $rec->{family_name},
        Nivel => 1,
        Ordem => $rec->{vorder},
        Num => $max_num,
#          Tipotextura  => $rec->{fo_seetings}->{_cbot} || 1,
        Tipotextura  => 1,
        TamanhoBotoes => 53,
      };

      my $props = $rec->{fo_seetings} || {};

      for my $k (keys %$props) {
        next if $k =~ /^_/ || $k eq 'vorder';
        $h->{$k} = $props->{$k};
      }

      $h->{Categoria} = $h->{Categoria}."S" if exists $top_families{uc($name)};

      push @categories, $h;
    }

    # buttons
    for my $rec (@families) {
      next unless defined $rec->{top_family_id};

      my $top_name = substr($rec->{top_family_name}, 0, 15);
      my $sub_name = substr($rec->{family_name}, 0, 15);

      my $h = {
        Categoria    => $top_name,
        SubCategoria => $sub_name,
        Ordem        => $rec->{vorder},
        Num          => $max_num,
        Label        => $rec->{fo_seetings}->{_nbot},
#          Tipotextura  => $rec->{fo_seetings}->{_cbot} || 1,
        Tipotextura  => 1,
      };

      my $props = $rec->{fo_seetings} || {};

      for my $k (keys %$props) {
        next if $k =~ /^_/ || $k eq 'vorder';
        $h->{$k} = $props->{$k};
      }

      delete $h->{TamanhoBotoes} if exists $h->{TamanhoBotoes};

      $h->{SubCategoria} = $h->{SubCategoria}."S" if defined $rec->{top_family_id} && exists $top_families{uc($sub_name)};

      push @buttons, $h;
    }

    my %fam_control = ();

    # groups (top_families)
    for my $rec (@families) {
      next unless defined $rec->{top_family_name};
      next if exists $fam_control{ $rec->{top_family_name} };

      push @info, {
        Codigo => $rec->{top_family_fo_id},
        Descricao => $rec->{top_family_name},
      };

      $fam_control{ $rec->{top_family_name} } = 1;
    }

    $grp->records(
      validate_fields => 1,
      data => \@info,
    );

    my $fh = FileHandle->new();

    if ($fh->open(">> ".$xml->{export_file})) {
      $grp->to_fh( fh => $fh, cache_control => $cache_control );
      $fh->close();
    }

    # families (sub_families)
    @info = ();

    for my $rec (@families) {
      next unless defined $rec->{top_family_id};

      push @info, {
        Codigo => $rec->{family_fo_id},
        Descricao => $rec->{family_name},
        Grupo => $rec->{top_family_fo_id},
      };
    }

    $fam->records(
      validate_fields => 1,
      data => \@info,
    );

    if ($fh->open(">> ".$xml->{export_file})) {
      $fam->to_fh( fh => $fh, cache_control => $cache_control );
      $fh->close();
    }
  }

  #
  # Buttons
  #

  if (scalar @buttons) {
    $db_mestres->do("DELETE FROM BotoesCategoria WHERE Num = $max_num");
    $db_mestres->do("DELETE FROM BotoesCategoria WHERE Num = -".$max_num);

    $bot->records(
      validate_fields => 1,
      data => \@buttons,
    );

    my $fh = FileHandle->new();

    if ($fh->open(">> ".$xml->{export_file})) {
      $bot->to_fh( fh => $fh );
      $fh->close();
    }
  }

  #
  # Categories
  #

  if (scalar @categories) {
    $db_mestres->do("DELETE FROM Categorias WHERE Num = $max_num");
    $db_mestres->do("DELETE FROM Categorias WHERE Num = -".$max_num);

    $cat->records(
      validate_fields => 1,
      data => \@categories,
    );

    my $fh = FileHandle->new();

    if ($fh->open(">> ".$xml->{export_file})) {
      $cat->to_fh( fh => $fh );
      $fh->close();
    }
  }
}

1;

