package WO::Client::WinTouch::SQL;

use strict;
use warnings;

use base qw ( WO::Client::Common );

use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
use File::Slurp qw( write_file );
use Data::Dumper;
use FileHandle;
use FindBin qw( $Bin );
use POSIX qw(strftime);

use DBI;
use WO::DB::ODBC;

#
# FO to weckOFFICE
#

sub sync_fo_information {
  my $s = shift;

  $s->debug("sync_fo_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  # directory separator
  my $dsep = $s->directory_separator();

  $xml->{odbc_name}   ||= "";
  $xml->{odbc_login}  ||= "";
  $xml->{odbc_passwd} ||= "";

  my $dbh = DBI->connect('dbi:ODBC:'.$xml->{odbc_name}, $xml->{odbc_login}, $xml->{odbc_passwd})
    or die("Can't connect to the ODBC! source name: ".$xml->{odbc_name}." | login: ".$xml->{odbc_login}." | passwd: ".$xml->{odbc_passwd}."\n");

  my $db = WO::DB::ODBC->new( dbh => $dbh );

  my $file_groups            = $Bin.'/__groups__.txt';                # x wgcgrupos
  my $file_families          = $Bin.'/__families__.txt';              # x wgcfamilias
  my $file_users             = $Bin.'/__users__.txt';                 # x wgcvendedores

  my $file_movements         = $Bin.'/__movements__.txt';             # v wsir_vnd_vendas
  my $file_movement_payments = $Bin.'/__movement_payments__.txt';     # v wsir_vnd_meiospagamento
  my $file_movement_details  = $Bin.'/__movement_details__.txt';     # v wsir_vnd_meiospagamento
  my $file_clients           = $Bin.'/__clients__.txt';               # v wsir_mst_dlclientes
  my $file_warehouses        = $Bin.'/__warehouses__.txt';            # v wgcarmazens
  my $file_articles          = $Bin.'/__articles__.txt';              # v wgcartigos
  my $file_payment_types     = $Bin.'/__payment_types__.txt';         # v wgcmeiospagamento
  my $file_document_types    = $Bin.'/__document_types__.txt';        # v wgctiposdocumentos
  my $file_taxes             = $Bin.'/__taxes__.txt';                 # v wgcivas
  my $file_article_composite = $Bin.'/__article_composite__.txt';     # v wgcfichastecnicas
  my $file_units             = $Bin.'/__units__.txt';                 # v wgcunidades

  $s->own_remove($file_warehouses) if -e $file_warehouses;
  $s->own_remove($file_groups) if -e $file_groups;
  $s->own_remove($file_families) if -e $file_families;
  $s->own_remove($file_articles) if -e $file_articles;
  $s->own_remove($file_movements) if -e $file_movements;
  $s->own_remove($file_movement_payments) if -e $file_movement_payments;
  $s->own_remove($file_movement_details) if -e $file_movement_details;
  $s->own_remove($file_payment_types) if -e $file_payment_types;
  $s->own_remove($file_document_types) if -e $file_document_types;
  $s->own_remove($file_taxes) if -e $file_taxes;
  $s->own_remove($file_users) if -e $file_users;
  $s->own_remove($file_article_composite) if -e $file_article_composite;
  $s->own_remove($file_units) if -e $file_units;
  $s->own_remove($file_clients) if -e $file_clients;

  $xml->{sync_warehouses}         = exists $xml->{sync_warehouses} ? $xml->{sync_warehouses} : 1;
  $xml->{sync_families}           = exists $xml->{sync_families} ? $xml->{sync_families} : 1;
  $xml->{sync_articles}           = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_sells}              = exists $xml->{sync_sells} ? $xml->{sync_sells} : 1;
  $xml->{sync_payment_types}      = exists $xml->{sync_payment_types} ? $xml->{sync_payment_types} : 1;
  $xml->{sync_document_types}     = exists $xml->{sync_document_types} ? $xml->{sync_document_types} : 1;
  $xml->{sync_taxes}              = exists $xml->{sync_taxes} ? $xml->{sync_taxes} : 1;
  $xml->{sync_composite_articles} = exists $xml->{sync_composite_articles} ? $xml->{sync_composite_articles} : 1;
  $xml->{sync_units}              = exists $xml->{sync_units} ? $xml->{sync_units} : 1;
  $xml->{sync_clients}            = exists $xml->{sync_clients} ? $xml->{sync_clients} : 1;
  $xml->{sync_users}              = exists $xml->{sync_users} ? $xml->{sync_users} : 1;

  my @zip_files = ();

  if ($xml->{sync_warehouses}) {
    $s->debug("warehouses");
    my @warehouses = $db->select("SELECT * FROM wgcarmazens");
    write_file( $file_warehouses, Dumper(\@warehouses) );
    push @zip_files, $file_warehouses;
    undef @warehouses;
  }

  if ($xml->{sync_taxes}) {
    $s->debug("taxes");
    my @taxes = $db->select("SELECT * FROM wgcivas");
    write_file( $file_taxes, Dumper(\@taxes) );
    push @zip_files, $file_taxes;
    undef @taxes;
  }

  if ($xml->{sync_users}) {
    # users
    $s->debug("users");
    my @users = $db->select("SELECT * FROM wgcvendedores");
    write_file( $file_users, Dumper(\@users) );
    push @zip_files, $file_users;
    undef @users;
  }

  if ($xml->{sync_families}) {
    $s->debug("groups");
    my @groups = $db->select("SELECT * FROM wgcgrupos");
    write_file( $file_groups, Dumper(\@groups) );
    push @zip_files, $file_groups;
    undef @groups;

    $s->debug("families");
    my @families = $db->select("SELECT * FROM wgcfamilias");
    write_file( $file_families, Dumper(\@families) );
    push @zip_files, $file_families;
    undef @families;
  }

  if ($xml->{sync_articles}) {
    $s->debug("articles");
    my @articles = $db->select("SELECT * FROM wgcartigos");
    write_file( $file_articles, Dumper(\@articles) );
    push @zip_files, $file_articles;
    undef @articles;
  }

  if ($xml->{sync_sells}) {
    $s->debug("sells");

    $s->_sync_sells($db, \@zip_files);

=pod
    my @warehouses = $db->select("SELECT * FROM wgcivas");
    write_file( $file_warehouses, Dumper(\@warehouses) );
    push @zip_files, $file_warehouses;
    undef @warehouses;
=cut
  }

  if ($xml->{sync_payment_types}) {
    $s->debug("payment_types");
    my @payment_types = $db->select("SELECT * FROM wgcmeiospagamento");
    write_file( $file_payment_types, Dumper(\@payment_types) );
    push @zip_files, $file_payment_types;
    undef @payment_types;
  }

  if ($xml->{sync_document_types}) {
    $s->debug("document_types");
    my @document_types = $db->select("SELECT * FROM wgctiposdocumentos");
    write_file( $file_document_types, Dumper(\@document_types) );
    push @zip_files, $file_document_types;
    undef @document_types;
  }

  if ($xml->{sync_composite_articles}) {
    $s->debug("composite_articles");
    my @composites = $db->select("SELECT * FROM wgcfichastecnicas");
    write_file( $file_article_composite, Dumper(\@composites) );
    push @zip_files, $file_article_composite;
    undef @composites;
  }

  if ($xml->{sync_units}) {
    $s->debug("units");
    my @units = $db->select("SELECT * FROM wgcunidades");
    write_file( $file_units, Dumper(\@units) );
    push @zip_files, $file_units;
    undef @units;
  }

  if ($xml->{sync_clients}) {
    $s->debug("clients");
    my @clients = $db->select("SELECT * FROM wgcterceiros WHERE fornecedor = ?", 0);
    write_file( $file_clients, Dumper(\@clients) );
    push @zip_files, $file_clients;
    undef @clients;
  }

  if (scalar @zip_files) {
    # build ZIP file
    my $zip = Archive::Zip->new();

    map { $zip->addFile( $_ ) } @zip_files;

    my $zipFile = $Bin.$dsep.$fo_type.'_FO_DATA.zip';
    $s->own_remove( $zipFile ) if -e $zipFile;

    die 'zip: write error' unless $zip->writeToFileNamed( $zipFile ) == AZ_OK;

    $s->debug($zipFile);

    # send zip file to weckoffice
    eval {
      $s->weckoffice_send_data( { action => 'FrontofficeReceiveData',
                                  file_upload => $zipFile,
                                }
                              );
    };

    if ($@) {
      die "Error sending file: $@";
    } else {
      $s->own_remove($file_warehouses) if -e $file_warehouses;
      $s->own_remove($file_groups) if -e $file_groups;
      $s->own_remove($file_families) if -e $file_families;
      $s->own_remove($file_articles) if -e $file_articles;
      $s->own_remove($file_movements) if -e $file_movements;
      $s->own_remove($file_movement_payments) if -e $file_movement_payments;
      $s->own_remove($file_movement_details) if -e $file_movement_details;
      $s->own_remove($file_payment_types) if -e $file_payment_types;
      $s->own_remove($file_document_types) if -e $file_document_types;
      $s->own_remove($file_taxes) if -e $file_taxes;
      $s->own_remove($file_users) if -e $file_users;
      $s->own_remove($file_article_composite) if -e $file_article_composite;
      $s->own_remove($file_units) if -e $file_units;
      $s->own_remove($file_clients) if -e $file_clients;
      $s->own_remove($zipFile) if -e $zipFile;
    }

    $s->debug("Done.");
  } else {
    $s->debug("Nothing to send to weckOFFICE...");
  }
}

#
# weckOFFICE to FO
#

sub sync_bk_information {
  my $s = shift;

  $s->debug("sync_bk_information()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();
  $s->debug("Encoded FO type is: $fo_type");

  my $data_dir = $xml->{data_dir} || "";

  if (! -d $data_dir) {
    $s->debug("Can't find data_dir $data_dir");
    die "Can't find data_dir $data_dir";
  }

  # directory separator
  my $dsep = $s->directory_separator();

  # manual execution, receive remote data
  if ($s->{manual}) {
  	$s->_receive_remote_data();
  }
  # else, sync sells
  else {
  	$s->_sync_sells();
  }
}

sub _sync_sells {
	my $s = shift;
  my $db_sells = shift;
  my $zip_files = shift;

  $s->debug("_sync_sells()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  # directory separator
  my $dsep = $s->directory_separator();

  #
  # Sync Sells
  #

  my $file_movements         = $Bin.'/__movements__.txt';             # x wsir_vnd_vendas and wsir_vnd_meiospagamento
  my $file_movement_details  = $Bin.'/__movement_details__.txt';
  my $file_movement_payments = $Bin.'/__movement_payments__.txt';

  $s->debug("Will process sells...");

  my $check_dates_fast = $xml->{check_dates_fast} || 0;
  my $date_from = $xml->{date_from} || '';
  my $single_warehouse = $xml->{sales_in_single_warehouse} || 0;

  my $VAR1 = {};

  $s->debug("Will check existant dates...");

  my $existent_dates;

  if ($single_warehouse) {
    my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDates';
    $existent_dates = $s->weckoffice_send_data( { action => $act,
                                                   }
                                                 );
  } else {
    my $act = $check_dates_fast ? 'FrontofficeExistentSellDatesFast' : 'FrontofficeExistentSellDatesNoStore';
    $existent_dates = $s->weckoffice_send_data( { action => $act,
                                                   }
                                                 );
  }

  eval "$existent_dates";
  my %existent_dates = defined $VAR1 && ref($VAR1) eq 'HASH' ? %$VAR1 : ();

  my @dates = keys %existent_dates;

  my $where_str = "";
=pod
  if (scalar @dates) {
    $where_str = " WHERE ";

    for (@dates) {
      if (/#/) {
        my @v = split /#/;
        $where_str .= "(l.armazem != ".$v[0]." AND c.datadoc != '".$v[1]."') AND ";
      } else {
        $where_str .= "c.datadoc != '".$_."' AND ";
      }
    }

    $where_str =~ s/ AND $//;
  }
=cut
#  unless ($date_from) {
    my $time_back = (60*60*24)*7; # 90 days
    $date_from = strftime("%Y-%m-%d", localtime(time()-$time_back));
#  }

  print STDERR "### $date_from\n";

#  $date_from = '2012-01-01';

  if ($date_from) {
    $date_from =~ s/\//-/g;

    if ($where_str) {
      $where_str .= " AND c.datadoc >= '".$date_from."'";
    } else {
      $where_str .= " WHERE c.datadoc >= '".$date_from."'";
    }
  }

  $s->debug("## 1 ##> $where_str - $date_from");

  # headers
  my $sub = $db_sells->select_iter(qq{
    SELECT c.*, l.armazem
    FROM wgcdoccab c
    LEFT JOIN (SELECT DISTINCT tipodoc, serie, numdoc, armazem FROM wgcdoclinhas) AS l
    ON (l.tipodoc = c.tipodoc AND l.serie = c.serie AND l.numdoc = c.numdoc)
    $where_str
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_movements, $sub );
    push @$zip_files, $file_movements;
  }

  $s->debug("## 2 ##> $where_str - $date_from");

  # details
  $sub = $db_sells->select_iter(qq{
    SELECT l.*
    FROM wgcdoclinhas l
    LEFT JOIN wgcdoccab c ON (l.tipodoc = c.tipodoc AND l.serie = c.serie AND l.numdoc = c.numdoc)
    $where_str
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_movement_details, $sub );
    push @$zip_files, $file_movement_details;
  }

  $s->debug("## 3 ##> $where_str - $date_from");

  # footers
  $sub = $db_sells->select_iter(qq{
    SELECT p.*, c.datadoc, l.armazem
    FROM wgcpagamentos p
    LEFT JOIN (SELECT DISTINCT tipodoc, serie, numdoc, armazem FROM wgcdoclinhas) AS l
    ON (l.tipodoc = p.tipodoc AND l.serie = p.serie AND l.numdoc = p.numdoc)
    LEFT JOIN wgcdoccab c ON (l.tipodoc = c.tipodoc AND l.serie = c.serie AND l.numdoc = c.numdoc)
    $where_str
  });

  if ($sub) {
    $s->write_csv_from_iter( $file_movement_payments, $sub );
    push @$zip_files, $file_movement_payments;
  }
}

sub _receive_remote_data {
	my $s = shift;

  $s->debug("_receive_remote_data()");

  my $xml = $s->info();

  my $fo_type = $s->encode_fo_type();

  my $data_dir = $xml->{data_dir} || "";

  # directory separator
  my $dsep = $s->directory_separator();

  my $file_mestres = $data_dir.'\MstMestres.mdb';
  my $file_vendas = $data_dir.'\AcvVendas.mdb';
  my $file_backoffice = $data_dir.'\bckback.mdb';

  my $has_backoffice_db = -e $file_backoffice ? 1 : 0;
  my $has_mestres_db    = -e $file_mestres ? 1 : 0;

  my $db_mestres = "";

  if ($has_mestres_db) {
    my $dsn = 'dbi:ODBC:driver=Microsoft Access Driver (*.mdb);dbq='.$file_mestres;
    my $dbh = DBI->connect($dsn) or die "Can't connect to database: $@";
    $db_mestres = WO::DB->new( dbh => $dbh );
  }

  my @ivas = ();
  my %ivas = ();

  if ($has_mestres_db) {
    @ivas = $db_mestres->select("SELECT Codigo, Taxa FROM Iva");
    %ivas = map { int($_->{Taxa}) => $_->{Codigo} } @ivas;
  }

  my $keyboard_number = $xml->{keyboard_number} || undef;

  my $max_num = 99;
  $max_num = $keyboard_number if defined $keyboard_number;

#  if (! defined $keyboard_number) {
#    my @res = $db_mestres->select("SELECT MAX(Num) As max FROM BotoesCategoria");
#    $max_num = scalar @res ? $res[0]->{max}+1 : 1;
#  }

  $xml->{sync_articles} = exists $xml->{sync_articles} ? $xml->{sync_articles} : 1;
  $xml->{sync_families} = exists $xml->{sync_families} ? $xml->{sync_families} : 1;

  $xml->{export_file} ||= "";
  die "Missing 'export_file' parameter..." unless $xml->{export_file};

  $s->own_remove($xml->{export_file}.".ok") if -e $xml->{export_file}.".ok";

  my $VAR1 = undef;
  my $str = undef;

  my $bot = WinTouch::Export::BotoesCategoria->new();
  my $cat = WinTouch::Export::Categorias->new();
  my @buttons = ();
  my @categories = ();

  #
  # Employees
  #

  if ($xml->{sync_users}) {
    $s->debug("Will get remote users...");

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetWaitresses',
                                       }
                                     );
    };

    if ($@) {
      die "Error: $@";
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $users = WinTouch::Export::Funcionarios->new();

      $s->debug("Received ".scalar(@$VAR1). " users...");

      my @users = ();

      for my $rec (@$VAR1) {
        push @users, {
          Codigo    => $rec->{fo_id},
          Nome      => $rec->{name},
          Categoria => $rec->{fo_access_type},
          Password  => $rec->{fo_passwd},
          TodosSectores => 1,
          TemSessao     => 1,
          Balcao        => 1,
        };
      }

      $users->records(
        validate_fields => 1,
        data => \@users,
      );

      my $fh = FileHandle->new();

      if ($fh->open(">> ".$xml->{export_file})) {
        $db_mestres->do("DELETE FROM Funcionarios");
        $users->to_fh( fh => $fh, cache_control => 0 );
        $fh->close();
      }
    }
  }

  #
  # Articles
  #

  if ($xml->{sync_articles}) {
    $s->debug("Will get remote articles...");

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetArticlesWithoutParamsInheritance',
                                       }
                                     );
    };

    if ($@) {
      die "Error: $@";
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $art = WinTouch::Export::Artigos->new();

      my @recs = ();

      $s->debug("Received ".scalar(@$VAR1). " articles...");

      for my $rec (@$VAR1) {
        my $i = {
          Categoria   => substr($rec->{family_name}, 0, 15),
          Artigo      => $rec->{article_fo_id},
          Ordem       => $rec->{vorder} ? $rec->{vorder}+1 : 1,
          Num         => $max_num,
          Label       => $rec->{fo_seetings}->{_nbot},
#          Tipotextura => $rec->{fo_seetings}->{_cbot} || 1,
          Tipotextura => 1,
        };

        push @buttons, $i;

        my $h = {
          Preco1 => defined $rec->{article_price1} ? $rec->{article_price1} : 0,
          Preco2 => defined $rec->{article_price2} ? $rec->{article_price2} : 0,
          Preco3 => defined $rec->{article_price3} ? $rec->{article_price3} : 0,
          Preco4 => defined $rec->{article_price4} ? $rec->{article_price4} : 0,
          Preco5 => defined $rec->{article_price5} ? $rec->{article_price5} : 0,
          Iva    => defined $rec->{article_tax1} ? $ivas{ int($rec->{article_tax1}) } : "",
          Iva1   => defined $rec->{article_tax2} ? $ivas{ int($rec->{article_tax2}) } : "",
          Codigo         => $rec->{article_fo_id} || "",
          Familia        => $rec->{family_fo_id} || "",
          CodBarras      => $rec->{article_barcode} || "",
          Descricao      => $rec->{article_name} || "",
          ArtVenda       => 1,
          CodTerminal    => "",

          AbrevImpressao => defined $rec->{fo_seetings}->{_ncur} ? substr($rec->{fo_seetings}->{_ncur}, 0, 30)  : "",
          AbrevTecla     => defined $rec->{fo_seetings}->{_nbot} ? substr($rec->{fo_seetings}->{_nbot}, 0, 100) : "",
          AbrevMonitor   => defined $rec->{fo_seetings}->{_nbot} ? substr($rec->{fo_seetings}->{_nbot}, 0, 30)  : "",
        };

        # define other fo settings
        foreach my $key (keys %{ $rec->{fo_seetings} }) {
          next if $key eq '_nbot' || $key eq '_ncur' || $key eq '_vord' || $key eq '_cbot';
          $h->{$key} = $rec->{fo_seetings}->{$key};
        }

        push @recs, $h;
      }

      $art->records(
        validate_fields => 1,
        data => \@recs,
      );

      my $fh = FileHandle->new();

      if ($fh->open(">> ".$xml->{export_file})) {
        $art->to_fh( fh => $fh, cache_control => 1 );
        $fh->close();
      }
    }
  }

  #
  # Families
  #

  if ($xml->{sync_families}) {
    $s->debug("Will get remote families...");

    eval {
      $str = $s->weckoffice_send_data( { action => 'BackofficeGetFamiliesWithoutParamsInheritance',
                                       }
                                     );
    };

    if ($@) {
      die "Error: $@";
    }

    eval $str;

    if (defined $VAR1 && ref($VAR1) eq 'ARRAY') {
      my $fam = WinTouch::Export::Familias->new();
      my $grp = WinTouch::Export::Grupos->new();

      my @info = ();

      $s->debug("Received ".scalar(@$VAR1). " families...");

      # categories
      for my $rec (@$VAR1) {
        my $h = {
          Categoria => substr($rec->{family_name}, 0, 15),
          Descricao => $rec->{family_name},
          Nivel => defined $rec->{top_family_id} ? 1 : 0,
          Ordem => $rec->{vorder},
          Num => $max_num,
#          Tipotextura  => $rec->{fo_seetings}->{_cbot} || 1,
          Tipotextura  => 1,
          TamanhoBotoes => 53,
        };

        my $props = $rec->{fo_seetings} || {};

        for my $k (keys %$props) {
          next if $k =~ /^_/ || $k eq 'vorder';
          $h->{$k} = $props->{$k};
        }

        push @categories, $h;
      }

      # sub categories
      for my $rec (@$VAR1) {
        next unless defined $rec->{top_family_id};

        my $h = {
          Categoria    => substr($rec->{top_family_name}, 0, 15),
          SubCategoria => substr($rec->{family_name}, 0, 15),
          Ordem        => $rec->{vorder},
          Num          => $max_num,
          Label        => $rec->{fo_seetings}->{_nbot},
#          Tipotextura  => $rec->{fo_seetings}->{_cbot} || 1,
          Tipotextura  => 1,
        };

        my $props = $rec->{fo_seetings} || {};

        for my $k (keys %$props) {
          next if $k =~ /^_/ || $k eq 'vorder';
          $h->{$k} = $props->{$k};
        }

        push @buttons, $h;
      }

      my %fam_control = ();

      # groups (top_families)
      for my $rec (@$VAR1) {
        next unless defined $rec->{top_family_name};
        next if exists $fam_control{ $rec->{top_family_name} };

        push @info, {
          Codigo => $rec->{top_family_fo_id},
          Descricao => $rec->{top_family_name},
        };

        $fam_control{ $rec->{top_family_name} } = 1;
      }

      $grp->records(
        validate_fields => 1,
        data => \@info,
      );

      my $fh = FileHandle->new();

      if ($fh->open(">> ".$xml->{export_file})) {
        $grp->to_fh( fh => $fh, cache_control => 1 );
        $fh->close();
      }

      # families (sub_families)
      @info = ();

      for my $rec (@$VAR1) {
        next unless defined $rec->{top_family_id};

        push @info, {
          Codigo => $rec->{family_fo_id},
          Descricao => $rec->{family_name},
          Grupo => $rec->{top_family_fo_id},
        };
      }

      $fam->records(
        validate_fields => 1,
        data => \@info,
      );

      if ($fh->open(">> ".$xml->{export_file})) {
        $fam->to_fh( fh => $fh, cache_control => 1 );
        $fh->close();
      }
    }
  }

  #
  # Buttons
  #

  if (scalar @buttons) {
    $db_mestres->do("DELETE FROM BotoesCategoria WHERE Num = $max_num");
    $db_mestres->do("DELETE FROM BotoesCategoria WHERE Num = -".$max_num);

    $bot->records(
      validate_fields => 1,
      data => \@buttons,
    );

    my $fh = FileHandle->new();

    if ($fh->open(">> ".$xml->{export_file})) {
      $bot->to_fh( fh => $fh );
      $fh->close();
    }
  }

  #
  # Categories
  #

  if (scalar @categories) {
    $db_mestres->do("DELETE FROM Categorias WHERE Num = $max_num");
    $db_mestres->do("DELETE FROM Categorias WHERE Num = -".$max_num);

    $cat->records(
      validate_fields => 1,
      data => \@categories,
    );

    my $fh = FileHandle->new();

    if ($fh->open(">> ".$xml->{export_file})) {
      $cat->to_fh( fh => $fh );
      $fh->close();
    }
  }
}

1;




